using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class CBusqueda : UserControl {

        private IEColeccion mColeccionOrig = null;
        private IEColeccion mColeccion = null;
        private PropertyDescriptorCollection pdcol = null;

        public CBusqueda() {
            InitializeComponent();
        }

        public CBusqueda(IEColeccion col): this() {
            this.Coleccion = col;
        }

        private void CBusqueda_Load(object sender, EventArgs e) {
            dtpFechaBuscar.Visible = false;
        }

        private void btReestablecer_Click(object sender, EventArgs e) {
            if (txtBusqueda.Visible) {
                txtBusqueda.Text = "";
            }
            IFiltroBuscar fFiltro = this.ParentForm as IFiltroBuscar;
            if (fFiltro != null) {
                fFiltro.Aplicar(mColeccionOrig);
            }
        }

        private void cbBuscarPor_SelectionChangeCommitted(object sender, EventArgs e) {
            //establecer que control se activa segn el valor seleccionado del combobox.
            EstablecerControl((PropertyDescriptor)cbBuscarPor.SelectedItem);
            if (!txtBusqueda.Visible) {
                txtBusqueda.Text = "";
            }
        }

        private void EstablecerControl(PropertyDescriptor pd) {
            if (pd.PropertyType.IsEnum) {
                cbEnum.Location = txtBusqueda.Location;
                txtBusqueda.Visible = false;
                dtpFechaBuscar.Visible = false;
                cbEnum.Visible = true;
                EnumConvert ec = new EnumConvert();
                Collection<KeyValuePair<Enum, string>> col = ec.GetCollectionDescription(pd.PropertyType);
                cbEnum.DataSource = col;
                cbEnum.DisplayMember = "Value";
                cbEnum.ValueMember = "Key";
            } else if (pd.PropertyType.ToString().ToUpper().Contains("DATETIME")) {
                if (cbBuscarPor.Text.ToUpper().Contains("HORA") && !cbBuscarPor.Text.ToUpper().Contains("FECHA")) {
                    dtpFechaBuscar.Format = DateTimePickerFormat.Time;
                } else {
                    dtpFechaBuscar.Format = DateTimePickerFormat.Short;
                }
                txtBusqueda.Visible = false;
                dtpFechaBuscar.Visible = true;
                dtpFechaBuscar.Anchor = AnchorStyles.Left | AnchorStyles.Right;
                cbEnum.Visible = false;
            } else {
                txtBusqueda.Visible = true;
                dtpFechaBuscar.Visible = false;
                cbEnum.Visible = false;
            }
        }

        private void AplicarFiltro() {
            int numFiltros = mColeccion.Filtros.Count;
            try {
                PropertyDescriptor pd = (PropertyDescriptor)cbBuscarPor.SelectedItem;
                if (txtBusqueda.Visible) {
                    //mirar si es numero o texto
                    if (pd.PropertyType.Name.ToUpper().Contains("INT")) {
                        int valor = Convert.ToInt32(txtBusqueda.Text);
                        mColeccion.Filtros.Add(pd, FiltroSimple.TipoOperador.Igual, valor);
                    } else if (pd.PropertyType.Name.ToUpper().Contains("DOUBLE")) {
                        double valor = Convert.ToDouble(txtBusqueda.Text);
                        mColeccion.Filtros.Add(pd, FiltroSimple.TipoOperador.MayorIgual, valor);
                    } else {//es un string
                        string valor = txtBusqueda.Text;
                        mColeccion.Filtros.Add(pd, FiltroSimple.TipoOperador.Como, valor);
                    }
                } else if (dtpFechaBuscar.Visible) {
                    DateTime valor = dtpFechaBuscar.Value.Date;
                    mColeccion.Filtros.Add(pd, FiltroFecha.TipoOperador.MayorIgual, valor);
                } else {
                    object valor = cbEnum.SelectedValue;
                    mColeccion.Filtros.Add(pd, FiltroSimple.TipoOperador.Como, valor);
                }
                mColeccion.Cargar();
                mColeccion.Filtros.Remove(pd.Name);
                IFiltroBuscar fFiltro = this.ParentForm as IFiltroBuscar;
                if (fFiltro != null) {
                    fFiltro.Aplicar(mColeccion);
                }
            } catch (FormatException ex) {
                MessageBox.Show(ex.Message, "ERROR FORMATO", MessageBoxButtons.OK, MessageBoxIcon.Error);
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR FORMATO", MessageBoxButtons.OK, MessageBoxIcon.Error);
            } finally {
                if (mColeccion.Filtros.Count > numFiltros) {
                    mColeccion.Filtros.RemoveAt(mColeccion.Filtros.Count - 1);
                }
            }
        }

        private void txtBusqueda_TextChanged(object sender, EventArgs e) {
            if (String.IsNullOrEmpty(txtBusqueda.Text)) {
                IFiltroBuscar fFiltro = this.ParentForm as IFiltroBuscar;
                if (fFiltro != null) {
                    fFiltro.Aplicar(mColeccionOrig);
                } 
            } else {
                AplicarFiltro();
            }
        }

        private void dtpFechaBuscar_ValueChanged(object sender, EventArgs e) {
            AplicarFiltro();
        }

        private void cbEnum_SelectionChangeCommitted(object sender, EventArgs e) {
            AplicarFiltro();
        }

        public IEColeccion Coleccion {
            set {
                if (value != null) {
                    mColeccion = value;
                    mColeccionOrig = value.Clone() as IEColeccion;
                    //los descriptores se copian
                    ITypedList itl = value as ITypedList;
                    if (itl != null) {
                        pdcol = itl.GetItemProperties(null);
                        foreach (PropertyDescriptor pd in pdcol) {
                            mColeccionOrig.ColumnaMostrar(pd);
                        }
                    }
                    //los filtros tambin hay que copiarlos.
                    IFiltro[] array = new IFiltro[value.Filtros.Count];
                    value.Filtros.CopyTo(array, 0);
                    for (int cont = 0; cont < array.Length; cont++) {
                        mColeccionOrig.Filtros.Add(array[cont]);
                    }
                    
                    //recogemos las columnas.
                    cbBuscarPor.Items.Clear();
                    foreach (PropertyDescriptor pd in pdcol) {
                        if (!(mColeccion.Titulo != null && mColeccion.Titulo.ToUpper().Contains("INSCRITOS") && pd.PropertyType.IsClass)) {
                            cbBuscarPor.Items.Add(pd);
                        }
                    }
                    //cbBuscarPor.DataSource = pdcol;
                    cbBuscarPor.DisplayMember = "DisplayName";
                    //segn el combobox decir que tipo de control se activa.
                    if (cbBuscarPor.SelectedItem != null) {
                        EstablecerControl((PropertyDescriptor)cbBuscarPor.SelectedItem);
                    }
                }
            }
        }
    }
}
