using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FCampeonatoPuntuable : Form, IFichaEntidad, IFichaEntidadCol {

        ECampeonatoPuntuable obj = null;
        EColeccion<ECampeonatoPuntuable> mColeccion = null;

        public FCampeonatoPuntuable() {
            InitializeComponent();
            EnumConvert enumConvert = new EnumConvert();
            Collection<KeyValuePair<Enum, string>> colkvp = enumConvert.GetCollectionDescription(typeof(TipoCalculoPuntos));
            cbTipoCalculoPuntos.DataSource = colkvp;
            cbTipoCalculoPuntos.DisplayMember = "Value";
            cbTipoCalculoPuntos.ValueMember = "Key";
        }

        private void FCampeonatoPuntuable_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.Carrera.EsNuevo) {
                    MessageBox.Show("No se puede guardar por que la carrera no est almacenada an.");
                } else {
                    if (obj.EstaSucio) {
                        DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Campeonato Puntuable", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            obj.Guardar();
                            MessageBox.Show("El campeonato puntuable se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            if (btCancelar.Visible) {
                                btCancelar.Visible = lblCancelar.Visible = false;
                            }
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            ECarrera carrera = null;
            if (obj.EsNuevo) {
                carrera = obj.Carrera;
            }
            obj.Deshacer();
            if (obj.EsNuevo) {
                obj.Carrera = carrera;
            }
            this.Cargar(obj);
        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("CampeonatoPuntuable.");
            }
            if (obj != null) {
                bind.Clear();
            }
            obj = entidad as ECampeonatoPuntuable;
            if (obj == null) {
                throw new InvalidCastException("La entidad debe ser de tipo ECampeonatoPuntuable.");
            }
            //agregar las tablas fija si procede.
            CalculadorTablaFija calcTabla = obj.CalculadorPuntos as CalculadorTablaFija;
            if (calcTabla != null) {
                lbTabla.Visible = btAgregar.Visible = btQuitar.Visible = lblPuesto1.Visible = lblPuntos1.Visible = nudPuesto.Visible = nudPuntos.Visible = true;
                lbTabla.Items.Clear();
                EColeccion<ETablaFija> col = calcTabla.Tabla;
                col.ApplySort(ETablaFija.NPPuesto, ListSortDirection.Ascending);
                foreach (ETablaFija tfija in col) {
                    lbTabla.Items.Add(tfija);
                }
            } else {//no se ve.
                lbTabla.Visible = btAgregar.Visible = btQuitar.Visible = lblPuesto1.Visible = lblPuntos1.Visible = nudPuesto.Visible = nudPuntos.Visible = false;
            }
            bind.Add(obj);
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            obj_PropertyChanged(null, new PropertyChangedEventArgs(ECampeonatoPuntuable.NPCampeonato));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(ECampeonatoPuntuable.NPCampeonato)) {
                pe = obj.ErrorDePropiedad(ECampeonatoPuntuable.NPCampeonato);
                if (pe != null) {
                    errProv.SetError(btCampeonato, pe.Message);
                } else {
                    errProv.SetError(btCampeonato, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new ECampeonatoPuntuable());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(Nibi.Negocio.Colecciones.IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin CampeonatoPuntuable.");
            }
            mColeccion = col as EColeccion<ECampeonatoPuntuable>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonatoPuntuable.");
            }
            if (pos < 0 || pos >= mColeccion.Count) {
                throw new OverflowException("La posicin no se encuentra entre los valores mnimos y mximos de la coleccin");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin CampeonatoPuntuable.");
            }
            mColeccion = col as EColeccion<ECampeonatoPuntuable>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonatoPuntuable.");
            }
            obj = (ECampeonatoPuntuable)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        private void btCampeonato_Click(object sender, EventArgs e) {
            EColeccion<ECampeonato> col = ECampeonato.GetCampeonatos(mColeccion);
            if (col != null) {
                col.ColumnaMostrarRango(new string[] {ECampeonato.NPNombre, ECampeonato.NPAlias });
                FDialogo fd = new FDialogo(col);
                DialogResult dr = fd.ShowDialog();
                if (dr == DialogResult.OK) {
                    ECampeonato esb = fd.ExtraerEleccion() as ECampeonato;
                    if (esb != null) {
                        obj.Campeonato = esb;
                    } else {
                        MessageBox.Show("Tiene que elegir un campeonato.");
                    }
                }
            }
        }

        private void btAgregar_Click(object sender, EventArgs e) {
            try {
                CalculadorTablaFija calcTabla = obj.CalculadorPuntos as CalculadorTablaFija;
                ETablaFija tfija = calcTabla.Tabla.AddNew();
                tfija.Puesto = Int32.Parse(nudPuesto.Value.ToString());
                tfija.Puntos = Int32.Parse(nudPuntos.Value.ToString());
                if (calcTabla.TablaConPuestosDuplicados()) {
                    calcTabla.Tabla.CancelNew(calcTabla.Tabla.IndexOf(tfija));
                    MessageBox.Show("No se puede insertar por que ya hay un puesto agregado.\nQutelo primero.");
                } else {
                    lbTabla.Items.Clear();
                    EColeccion<ETablaFija> col = calcTabla.Tabla;
                    col.ApplySort(ETablaFija.NPPuesto, ListSortDirection.Ascending);
                    foreach (ETablaFija tf in col) {
                        lbTabla.Items.Add(tf);
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btQuitar_Click(object sender, EventArgs e) {
            try {
                CalculadorTablaFija calcTabla = obj.CalculadorPuntos as CalculadorTablaFija;
                if (!calcTabla.Tabla.Contains(lbTabla.SelectedItem as ETablaFija)) {
                    throw new Exception("Elija un elemento de la lista.");
                }
                ETablaFija tfija = lbTabla.SelectedItem as ETablaFija;
                calcTabla.Tabla.Remove(tfija);
                lbTabla.Items.Remove(tfija);
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void cbTipoCalculoPuntos_SelectionChangeCommitted(object sender, EventArgs e) {
            TipoCalculoPuntos tipoPuntos = (TipoCalculoPuntos)cbTipoCalculoPuntos.SelectedValue;
            if (tipoPuntos == TipoCalculoPuntos.TFija) {
                lbTabla.Visible = btAgregar.Visible = btQuitar.Visible = lblPuesto1.Visible = lblPuntos1.Visible = nudPuesto.Visible = nudPuntos.Visible = true;
            } else {
                lbTabla.Visible = btAgregar.Visible = btQuitar.Visible = lblPuesto1.Visible = lblPuntos1.Visible = nudPuesto.Visible = nudPuntos.Visible = false;
            }
        }
    }
}