using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using Nibi.Negocio.Colecciones;

namespace Forms {
    public partial class FClase : Form, IFichaEntidad, IFichaEntidadCol {

        private ECampeonato obj = null;
        private EColeccion<ECampeonato> mColeccion = null;

        #region Constructores

        public FClase() {
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }

        public FClase(ELicenciaOficial lof) : this() {
            Cargar(lof);
        }

        public FClase(IEColeccion col) : this() {
            CrearNuevo(col);
        }
        public FClase(IEColeccion col, int pos) : this() {
            Cargar(col, pos);
        }
#endregion

        private void FClase_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {//poner transparente
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FClase_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.Campeonato.EsNuevo) {
                    MessageBox.Show("No se puede guardar por que el grupo al que pertenece no est almacenado an.");
                } else {
                    if (obj.EstaSucio) {
                        DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Clase", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            obj.Guardar();
                            MessageBox.Show("La clase se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                    }
                }
                
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            ECampeonato grupo = null;
            if (obj.EsNuevo) {
                grupo = obj.Campeonato;
            }
            obj.Deshacer();
            if (obj.EsNuevo) {
                obj.Campeonato = grupo;
            }
            this.Cargar(obj);
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if (var == null) {
                throw new ArgumentNullException("Clase");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ECampeonato;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonato(clase).");
            }
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ECampeonato.NPNombre));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ECampeonato.NPCilindradaMaxima));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ECampeonato.NPCilindradaMinima));
        }

        void  obj_PropertyChanged(object sender, PropertyChangedEventArgs e){
            PropertyException pe = null;
            if (e.PropertyName.Equals(ECampeonato.NPNombre)) {
                pe = obj.ErrorDePropiedad(ECampeonato.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            } else if (e.PropertyName.Equals(ECampeonato.NPCilindradaMaxima)) {
                pe = obj.ErrorDePropiedad(ECampeonato.NPCilindradaMaxima);
                if (pe != null) {
                    errProv.SetError(txtCilindradaMaxima, pe.Message);
                } else {
                    errProv.SetError(txtCilindradaMaxima, "");
                }
            } else if (e.PropertyName.Equals(ECampeonato.NPCilindradaMinima)) {
                pe = obj.ErrorDePropiedad(ECampeonato.NPCilindradaMinima);
                if (pe != null) {
                    errProv.SetError(txtCilindradaMinima, pe.Message);
                } else {
                    errProv.SetError(txtCilindradaMinima, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new ECampeonato());
            obj.Nivel = 2;
        }

        public void SetGrupo(ECampeonato grupo) {
            ECampeonato clase = obj as ECampeonato;
            if (clase == null) {
                throw new InvalidCastException("No se ha podido extraer la clase de este formulario.\nTal vez sea por no haber creado una clase.");
            }
            clase.Campeonato = grupo;
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Clase");
            }
            mColeccion = col as EColeccion<ECampeonato>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonato(clase).");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Clase");
            }
            mColeccion = col as EColeccion<ECampeonato>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonato(clase).");
            }
            obj = (ECampeonato)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion
    }
}