using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;

namespace Forms {
    public partial class FCronoClasificacion : Form, IFichaEntidad {

        private EClasificacionCarrera obj = null;
        private bool EstaCargado = false;

        public FCronoClasificacion() {
            InitializeComponent();
        }

        public FCronoClasificacion(EClasificacionCarrera obj) : this() {
            this.obj = obj;
        }

        private void FCronoClasificacion_Load(object sender, EventArgs e) {
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
            if (!EstaCargado) {
                Cargar(obj);
            }
        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("La entidad no puede estar vaca.");
            }
            obj = entidad as EClasificacionCarrera;
            if (obj == null) {
                throw new InvalidCastException("La entidad debe ser de tipo EClasificacionCarrera.");
            }
            //limpiamos los bind
            bind.Clear();
            //
            bind.Add(obj);
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            EstaCargado = true;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EClasificacionCarrera.NPHoraLLegada));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EClasificacionCarrera.NPHoraSalida));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EClasificacionCarrera.NPMinutosLLegada));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EClasificacionCarrera.NPMinutosSalida));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EClasificacionCarrera.NPSegundosLLegada));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EClasificacionCarrera.NPSegundosSalida));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EClasificacionCarrera.NPMilesimasLLegada));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EClasificacionCarrera.NPMilesimasSalida));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EClasificacionCarrera.NPMotivoAbandono));

            //EstaCerrada.
            txtHoraLLegada.ReadOnly = txtHoraSalida.ReadOnly = txtMilesimasLLegada.ReadOnly = txtMilesimasSalida.ReadOnly =
                txtMinutosLLegada.ReadOnly = txtMinutosSalida.ReadOnly = txtSegundosLLegada.ReadOnly = txtSegundosSalida.ReadOnly = 
                txtMotivoAbandono.ReadOnly = this.obj.PilotoCarrera.Carrera.EstaCerrada;
            chbAbandono.Enabled = !obj.PilotoCarrera.Carrera.EstaCerrada;
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(EClasificacionCarrera.NPHoraLLegada)) {
                pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPHoraLLegada);
                if (pe != null) {
                    errProv.SetError(txtHoraLLegada, pe.Message);
                } else {
                    errProv.SetError(txtHoraLLegada, "");
                    pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPTiempoRealLLegada);
                    if (pe != null) {
                        errProv.SetError(lblTiempoLLegada, pe.Message);
                    } else {
                        errProv.SetError(lblTiempoLLegada, "");
                        errProv.SetError(lblTiempoSalida, "");
                    }
                }
            } else if (e.PropertyName.Equals(EClasificacionCarrera.NPMinutosLLegada)) {
                pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPMinutosLLegada);
                if (pe != null) {
                    errProv.SetError(txtMinutosLLegada, pe.Message);
                } else {
                    errProv.SetError(txtMinutosLLegada, "");
                    pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPTiempoRealLLegada);
                    if (pe != null) {
                        errProv.SetError(lblTiempoLLegada, pe.Message);
                    } else {
                        errProv.SetError(lblTiempoLLegada, "");
                        errProv.SetError(lblTiempoSalida, "");
                    }
                }
            } else if (e.PropertyName.Equals(EClasificacionCarrera.NPSegundosLLegada)) {
                pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPSegundosLLegada);
                if (pe != null) {
                    errProv.SetError(txtSegundosLLegada, pe.Message);
                } else {
                    errProv.SetError(txtSegundosLLegada, "");
                    pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPTiempoRealLLegada);
                    if (pe != null) {
                        errProv.SetError(lblTiempoLLegada, pe.Message);
                    } else {
                        errProv.SetError(lblTiempoLLegada, "");
                        errProv.SetError(lblTiempoSalida, "");
                    }
                }
            } else if (e.PropertyName.Equals(EClasificacionCarrera.NPMilesimasLLegada)) {
                pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPMilesimasLLegada);
                if (pe != null) {
                    errProv.SetError(txtMilesimasLLegada, pe.Message);
                } else {
                    errProv.SetError(txtMilesimasLLegada, "");
                    pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPTiempoRealLLegada);
                    if (pe != null) {
                        errProv.SetError(lblTiempoLLegada, pe.Message);
                    } else {
                        errProv.SetError(lblTiempoLLegada, "");
                        errProv.SetError(lblTiempoSalida, "");
                    }
                }
            } else if (e.PropertyName.Equals(EClasificacionCarrera.NPMotivoAbandono)) {
                pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPMotivoAbandono);
                if (pe != null) {
                    errProv.SetError(txtMotivoAbandono, pe.Message);
                } else {
                    errProv.SetError(txtMotivoAbandono, "");
                }
            }
            else if (e.PropertyName.Equals(EClasificacionCarrera.NPHoraSalida)) {
                pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPHoraSalida);
                if (pe != null) {
                    errProv.SetError(txtHoraSalida, pe.Message);
                } else {
                    errProv.SetError(txtHoraSalida, "");
                    pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPTiempoRealSalida);
                    if (pe != null) {
                        errProv.SetError(lblTiempoSalida, pe.Message);
                    } else {
                        errProv.SetError(lblTiempoSalida, "");
                        errProv.SetError(lblTiempoLLegada, "");
                    }
                }
            } else if (e.PropertyName.Equals(EClasificacionCarrera.NPMinutosSalida)) {
                pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPMinutosSalida);
                if (pe != null) {
                    errProv.SetError(txtMinutosSalida, pe.Message);
                } else {
                    errProv.SetError(txtMinutosSalida, "");
                    pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPTiempoRealSalida);
                    if (pe != null) {
                        errProv.SetError(lblTiempoSalida, pe.Message);
                    } else {
                        errProv.SetError(lblTiempoSalida, "");
                        errProv.SetError(lblTiempoLLegada, "");
                    }
                }
            }else if (e.PropertyName.Equals(EClasificacionCarrera.NPSegundosSalida)) {
                pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPSegundosSalida);
                if (pe != null) {
                    errProv.SetError(txtSegundosSalida, pe.Message);
                } else {
                    errProv.SetError(txtSegundosSalida, "");
                    pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPTiempoRealSalida);
                    if (pe != null) {
                        errProv.SetError(lblTiempoSalida, pe.Message);
                    } else {
                        errProv.SetError(lblTiempoSalida, "");
                        errProv.SetError(lblTiempoLLegada, "");
                    }
                }
            } else if (e.PropertyName.Equals(EClasificacionCarrera.NPMilesimasSalida)) {
                pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPMilesimasSalida);
                if (pe != null) {
                    errProv.SetError(txtMilesimasSalida, pe.Message);
                } else {
                    errProv.SetError(txtMilesimasSalida, "");
                    pe = obj.ErrorDePropiedad(EClasificacionCarrera.NPTiempoRealSalida);
                    if (pe != null) {
                        errProv.SetError(lblTiempoSalida, pe.Message);
                    } else {
                        errProv.SetError(lblTiempoSalida, "");
                        errProv.SetError(lblTiempoLLegada, "");
                    }
                }
            }
        }

        /// <summary>
        /// En este formulario no se aplica.
        /// </summary>
        public void CrearNuevo() {
            throw new Exception("El mtodo crear nuevo no se impelementa en este formulario.");
        }

        #endregion
    }
}