using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;
using Nibi.Negocio.Colecciones;

namespace Forms {
    public partial class FGrupo : Form, IFichaEntidad, IPestanha, IFichaEntidadCol {

        private ECampeonato obj = null;
        private EColeccion<ECampeonato> mColeccion = null;

        #region Constructores

        public FGrupo() {
            InitializeComponent();
        }

        public FGrupo(ECampeonato gr) : this() {
            Cargar(gr);
        }

        public FGrupo(IEColeccion col) : this() {
            CrearNuevo(col);
        }
        public FGrupo(IEColeccion col, int pos) : this() {
            Cargar(col, pos);
        }

        #endregion

        private void FGrupo_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FGrupo_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.Campeonato.EsNuevo) {
                    MessageBox.Show("No se puede guardar por que su campeonato asociado no est almacenado an.");
                } else {
                    if (obj.EstaSucio) {
                        DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Grupo", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            obj.Guardar();
                            MessageBox.Show("El grupo se ha guardado correctamente,\njunto con sus clases.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            if (btCancelar.Visible) {//el objeto es nuevo y se agregaran las pestaas oportunas
                                btCancelar.Visible = lblCancelar.Visible = false;
                            }
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            ECampeonato campeonato = null;
            if (obj.EsNuevo) {
                campeonato = obj.Campeonato;
            }
            obj.Deshacer();
            if (obj.EsNuevo) {
                obj.Campeonato = campeonato;
            }
            this.Cargar(obj);
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if (var == null) {
                throw new ArgumentNullException("Grupo");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ECampeonato;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonato(grupo).");
            }

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ECampeonato.NPNombre));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(ECampeonato.NPNombre)) {
                pe = obj.ErrorDePropiedad(ECampeonato.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new ECampeonato());
            obj.Nivel = 1;
        }

        public void SetCampeonato(ECampeonato campeonato){
            ECampeonato grupo = obj as ECampeonato;
            if (grupo == null) {
                throw new InvalidCastException("No se ha podido extraer la clase de este formulario.\nTal vez sea por no haber creado una clase.");
            }
            grupo.Campeonato = campeonato;
        }
        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Grupo");
            }
            mColeccion = col as EColeccion<ECampeonato>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonato(grupo).");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Grupo");
            }
            mColeccion = col as EColeccion<ECampeonato>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonato(clase).");
            }
            obj = (ECampeonato)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            Collection<Pestanha> colP = new Collection<Pestanha>();
            string titulo = "Clases " + this.obj.Nombre;
            FColeccion fClases = new FColeccion(TipoColeccion.Clases);
            fClases.TopLevel = false;
            fClases.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha(titulo, fClases));
            return colP;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion
    }
}