using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;

namespace Forms {
    public partial class FImagen : Form, IFichaEntidad, IFichaEntidadCol {

        private EImagen obj = null;
        private EColeccion<EImagen> mColeccion = null;

        public FImagen() {
            InitializeComponent();
        }

        private void FImagen_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
        }

        private void btExaminar_Click(object sender, EventArgs e) {
            OpenFileDialog ofdialog = new OpenFileDialog();
            ofdialog.Filter = "archivos de imagen (*.jpg)|*.jpg|Todos los archivos (*.*)|*.*";
            ofdialog.FilterIndex = 1;
            ofdialog.RestoreDirectory = true;
            if (ofdialog.ShowDialog() == DialogResult.OK) {
                try {
                    obj.Imagen = Image.FromFile(ofdialog.FileName);
                } catch (Exception ex) {
                    MessageBox.Show("No ha sido posible leer el archivo de disco. Error original:" + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Imagen", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("La imagen se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if (btCancelar.Visible) {
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            ECarrera carrera = null;
            if (obj.EsNuevo) {
                carrera = obj.Carrera;
            }
            obj.Deshacer();
            if (obj.EsNuevo) {
                obj.Carrera = carrera;
            }
            this.Cargar(obj);
        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("Imagen.");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = entidad as EImagen;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EImagen.");
            }
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            EPermiso permiso = EUsuario.Actual.GetMayorPermiso();
            btGuardar.Visible = lblAceptar.Visible = btDeshacer.Visible = lblDeshacer.Visible = btExaminar.Visible = permiso.Tipo != TipoPermiso.Publico;
            txtNombre.ReadOnly = txtDescripcion.ReadOnly = permiso.Tipo == TipoPermiso.Publico;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EImagen.NPNombre));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            if (e.PropertyName.Equals(EImagen.NPNombre)) {
                PropertyException pe = obj.ErrorDePropiedad(EImagen.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            }
        }

        public void CrearNuevo() {
            this.Cargar(new EImagen());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(Nibi.Negocio.Colecciones.IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Imagen");
            }
            mColeccion = col as EColeccion<EImagen>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EImagen.");
            }
            if (pos < 0 || pos >= mColeccion.Count) {
                throw new OverflowException("La posicin no se encuentra entre los valores mnimos y mximos de la coleccin");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Imagen");
            }
            mColeccion = col as EColeccion<EImagen>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EImagen.");
            }
            obj = mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        private void FImagen_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EsNuevo && mColeccion != null && mColeccion.Count > 0 && mColeccion.Contains(obj)) {
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
            } else if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btSalir_Click(object sender, EventArgs e) {
            this.Close();
        }
    }
}