using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using Nibi.Negocio.Colecciones;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FLicCopiloto : Form, IFichaEntidad, IFichaEntidadCol {
        private ELicenciaCopiloto obj = null;
        private EColeccion<ELicenciaCopiloto> mColeccion = null;

        #region Constructores

        public FLicCopiloto() {
            InitializeComponent();
        }

        public FLicCopiloto(ELicenciaCopiloto lcop) : this() {
            Cargar(lcop);
        }

        public FLicCopiloto(IEColeccion col) : this() {
            CrearNuevo(col);
        }
        public FLicCopiloto(IEColeccion col, int pos) : this() {
            Cargar(col, pos);
        }

        #endregion

        private void FLicOficial_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            }
            else {
                bind.ResetCurrentItem();
                if (obj.Licencia != null){
                    KeyValuePair<ELicencia, string> kvp = (KeyValuePair<ELicencia, string>)cbCodigo.SelectedItem;
                    lblNombreTipo.Text = kvp.Key.Nombre;
                }
            }
            if (this.Parent != null) {//poner transparente
                this.SetStyle(ControlStyles.SupportsTransparentBackColor , true);
                this.BackColor = Color.Transparent;
                //btSalir.Visible = false;
            }
        }

        private void FLicCopiloto_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void cbCodigo_SelectionChangeCommitted(object sender, EventArgs e) {
            KeyValuePair<ELicencia, string> kvp = (KeyValuePair<ELicencia, string>)cbCodigo.SelectedItem;
            lblNombreTipo.Text = kvp.Key.Nombre;
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.Copiloto.EsNuevo) {
                    MessageBox.Show("No se puede guardar.\nEl copiloto al que pertenece no est almacenado an.");
                } else {
                    if (obj.EstaSucio) {
                        DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Licencia Copiloto", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            obj.Guardar();
                            MessageBox.Show("La licencia se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                        if (dr == DialogResult.No) {
                            obj.Deshacer();
                        }
                    }
                }
            } catch(Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FColeccion fl = this.ParentForm as FColeccion;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            ECopiloto copiloto = null;
            if (obj.EsNuevo) {
                copiloto = obj.Copiloto;
            }
            obj.Deshacer();
            this.Cargar(obj);
            if (obj.EsNuevo) {
                obj.Copiloto = copiloto;
            }
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if (var == null) {
                throw new ArgumentNullException("Licencia Copiloto");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ELicenciaCopiloto;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicenciaCopiloto.");
            }
            //combobox
            if ((obj.Licencia != null && obj.Licencia.Activo) || obj.Licencia == null) {
                Collection<Collection<KeyValuePair<ELicencia, string>>> col = new Collection<Collection<KeyValuePair<ELicencia, string>>>();
                Collection<KeyValuePair<ELicencia, string>> cKvpLC1 = ELicencia.GetCollectionCombo(TipoDirigido.Copilotos);
                Collection<KeyValuePair<ELicencia, string>> cKvpLC2 = ELicencia.GetCollectionCombo(TipoDirigido.Participante);
                Collection<KeyValuePair<ELicencia, string>> cKvpLC3 = ELicencia.GetCollectionCombo(TipoDirigido.Otros);
                col.Add(cKvpLC1);
                col.Add(cKvpLC2);
                col.Add(cKvpLC3);
                Collection<KeyValuePair<ELicencia, string>> cKvpLC = new Collection<KeyValuePair<ELicencia, string>>();
                foreach (Collection<KeyValuePair<ELicencia, string>> colKey in col) {
                    foreach (KeyValuePair<ELicencia, string> KeyValuePair in colKey) {
                        cKvpLC.Add(KeyValuePair);
                    }
                }
                cbCodigo.DataSource = cKvpLC;
                cbCodigo.DisplayMember = "Value";
                cbCodigo.ValueMember = "Key";
            } else {
                cbCodigo.DataSource = null;
            }
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            //se comprueba que el objeto no estaba modificado anteriormente.
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            //cbCodigo.Enabled = obj.Licencia == null || obj.Licencia.Activo;
            if (cbCodigo.Items.Count == 0 && !obj.EsNuevo) {
                toolTip.SetToolTip(cbCodigo, "No se puede modificar por que la licencia est inactiva.");
            } else {
                toolTip.SetToolTip(cbCodigo, "");
            }
            try {
                KeyValuePair<ELicencia, string> kvp = (KeyValuePair<ELicencia, string>)cbCodigo.SelectedItem;
                lblNombreTipo.Text = kvp.Key.Nombre;
            } catch (Exception) { }
            if (obj.Licencia != null && !lblNombreTipo.Text.Equals(obj.Licencia.Nombre)) {//Esta vacio por lo que la licencia que va a cargar no est activa
                lblNombreTipo.Text = obj.Licencia.Nombre;
            }
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ELicenciaPiloto.NPLicencia));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ELicenciaPiloto.NPNumero));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            if (e.PropertyName.Equals(ELicenciaCopiloto.NPLicencia)) {
                PropertyException pe = obj.ErrorDePropiedad(ELicenciaCopiloto.NPLicencia);
                if (pe != null) {
                    errProv.SetError(cbCodigo, pe.Message);
                } else {
                    errProv.SetError(cbCodigo, "");
                }
            } else if (e.PropertyName.Equals(ELicenciaCopiloto.NPNumero)) {
                PropertyException pe = obj.ErrorDePropiedad(ELicenciaCopiloto.NPNumero);
                if (pe != null) {
                    errProv.SetError(txtNumero, pe.Message);
                } else {
                    errProv.SetError(txtNumero, "");
                }
            }
        }

        public void CrearNuevo() {
            this.Cargar(new ELicenciaCopiloto());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Licencia Copiloto");
            }
            mColeccion = col as EColeccion<ELicenciaCopiloto>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECopiloto.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Licencias Copiloto");
            }
            mColeccion = col as EColeccion<ELicenciaCopiloto>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicenciaCopiloto.");
            }
            obj = (ELicenciaCopiloto)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion
    }
}