using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;
using Nibi.Negocio.Colecciones;

namespace Forms {
    public partial class FLicEscuderia : Form, IFichaEntidad, IFichaEntidadCol {

        private ELicenciaEscuderia obj = null;
        private EColeccion<ELicenciaEscuderia> mColeccion = null;

        #region Constructores

        public FLicEscuderia() {
            InitializeComponent();
        }

        public FLicEscuderia(ELicenciaOficial lof) : this() {
            Cargar(lof);
        }

        public FLicEscuderia(IEColeccion col) : this() {
            CrearNuevo(col);
        }
        public FLicEscuderia(IEColeccion col, int pos) : this() {
            Cargar(col, pos);
        }

        #endregion

        private void FLicEscuderia_Load(object sender, EventArgs e) {
            if(obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
                if(obj.Licencia != null) {
                    KeyValuePair<ELicencia, string> kvp = (KeyValuePair<ELicencia, string>)cbCodigo.SelectedItem;
                    lblNombreTipo.Text = kvp.Key.Nombre;
                }
            }
            if(this.Parent != null) {//poner transparente
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FLicEscuderia_FormClosed(object sender, FormClosedEventArgs e) {
            if(obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void cbCodigo_SelectionChangeCommitted(object sender, EventArgs e) {
            KeyValuePair<ELicencia, string> kvp = (KeyValuePair<ELicencia, string>)cbCodigo.SelectedItem;
            lblNombreTipo.Text = kvp.Key.Nombre;
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.Escuderia.EsNuevo) {
                    MessageBox.Show("No se puede guardar.\nLa escudera a la que pertenece no est almacenada an.");
                } else {
                    if (obj.EstaSucio) {
                        DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Licencia Piloto", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            obj.Guardar();
                            MessageBox.Show("La licencia se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                        if (dr == DialogResult.No) {
                            obj.Deshacer();
                        }
                    }
                }
            } catch(Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FColeccion fl = this.ParentForm as FColeccion;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            EEscuderia escuderia = null;
            if (obj.EsNuevo) {
                escuderia = obj.Escuderia;
            }
            obj.Deshacer();
            if (obj.EsNuevo) {
                obj.Escuderia = escuderia;
            }
            this.Cargar(obj);
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if(var == null) {
                throw new ArgumentNullException("Licencia Escuderia");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ELicenciaEscuderia;
            if(obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicenciaEscuderia.");
            }
            //combobox
            if ((obj.Licencia != null && obj.Licencia.Activo) || obj.Licencia == null) {
                Collection<Collection<KeyValuePair<ELicencia, string>>> col = new Collection<Collection<KeyValuePair<ELicencia, string>>>();
                Collection<KeyValuePair<ELicencia, string>> cKvpLC1 = ELicencia.GetCollectionCombo(TipoDirigido.EMC);
                Collection<KeyValuePair<ELicencia, string>> cKvpLC3 = ELicencia.GetCollectionCombo(TipoDirigido.Otros);
                col.Add(cKvpLC1);
                col.Add(cKvpLC3);
                Collection<KeyValuePair<ELicencia, string>> cKvpLC = new Collection<KeyValuePair<ELicencia, string>>();
                foreach (Collection<KeyValuePair<ELicencia, string>> colKey in col) {
                    foreach (KeyValuePair<ELicencia, string> KeyValuePair in colKey) {
                        cKvpLC.Add(KeyValuePair);
                    }
                }
                cbCodigo.DataSource = cKvpLC;
                cbCodigo.DisplayMember = "Value";
                cbCodigo.ValueMember = "Key";
            } else {
                cbCodigo.DataSource = null;
            }
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            if (cbCodigo.Items.Count == 0 && !obj.EsNuevo) {
                toolTip.SetToolTip(cbCodigo, "No se puede modificar por que la licencia est inactiva.");
            } else {
                toolTip.SetToolTip(cbCodigo, "");
            }
            try {
                KeyValuePair<ELicencia, string> kvp = (KeyValuePair<ELicencia, string>)cbCodigo.SelectedItem;
                lblNombreTipo.Text = kvp.Key.Nombre;
            } catch (Exception) { }
            if (obj.Licencia != null && !lblNombreTipo.Text.Equals(obj.Licencia.Nombre)) {//Esta vacio por lo que la licencia que va a cargar no est activa
                lblNombreTipo.Text = obj.Licencia.Nombre;
            }
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ELicenciaEscuderia.NPLicencia));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ELicenciaEscuderia.NPNumero));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            if (e.PropertyName.Equals(ELicenciaEscuderia.NPLicencia)) {
                PropertyException pe = obj.ErrorDePropiedad(ELicenciaEscuderia.NPLicencia);
                if (pe != null) {
                    errProv.SetError(cbCodigo, pe.Message);
                } else {
                    errProv.SetError(cbCodigo, "");
                }
            } else if (e.PropertyName.Equals(ELicenciaEscuderia.NPNumero)) {
                PropertyException pe = obj.ErrorDePropiedad(ELicenciaEscuderia.NPNumero);
                if (pe != null) {
                    errProv.SetError(txtNumero, pe.Message);
                } else {
                    errProv.SetError(txtNumero, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new ELicenciaEscuderia());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Licencia Escuderia");
            }
            mColeccion = col as EColeccion<ELicenciaEscuderia>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicenciaEscuderia.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Licencia Escuderia");
            }
            mColeccion = col as EColeccion<ELicenciaEscuderia>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicenciaEscuderia.");
            }
            obj = (ELicenciaEscuderia)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion
    }
}