using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;
using Nibi.Negocio.Colecciones;


namespace Forms {
    public partial class FLicOficial : Form, IFichaEntidad, IFichaEntidadCol {

        private ELicenciaOficial obj = null;
        private EColeccion<ELicenciaOficial> mColeccion = null;

        #region Constructores

        public FLicOficial() {
            InitializeComponent();
        }

        public FLicOficial(ELicenciaOficial lof) : this() {
            Cargar(lof);
        }

        public FLicOficial(IEColeccion col) : this() {
            CrearNuevo(col);
        }
        public FLicOficial(IEColeccion col, int pos) : this() {
            Cargar(col, pos);
        }

        #endregion

        private void FLicOficial_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            }
            else {
                bind.ResetCurrentItem();
                if (obj.Licencia != null){
                    KeyValuePair<ELicencia, string> kvp = (KeyValuePair<ELicencia, string>)cbCodigo.SelectedItem;
                    lblNombreTipo.Text = kvp.Key.Nombre;
                }
            }
            if (this.Parent != null) {//poner transparente
                this.SetStyle(ControlStyles.SupportsTransparentBackColor , true);
                this.BackColor = Color.Transparent;
                //btSalir.Visible = false;
            }
        }

        private void FLicOficial_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void cbCodigo_SelectionChangeCommitted(object sender, EventArgs e) {
            KeyValuePair<ELicencia, string> kvp = (KeyValuePair<ELicencia, string>)cbCodigo.SelectedItem;
            lblNombreTipo.Text = kvp.Key.Nombre;
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.Oficial.EsNuevo) {
                    MessageBox.Show("No se puede guardar.\nEl oficial al que pertenece no est almacenado an.");
                } else {
                    //preguntar por el tipo cargo si coincide con el nombre de la licencia
                    //Avisar (warning)
                    if (obj.EstaSucio) {
                        DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Licencia Oficial", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            if (((obj.Licencia.TipoDirigido == TipoDirigido.Otros) && (obj.Cargo != TipoCargoOficial.Ninguno)) ||
                            ((obj.Licencia.TipoDirigido != TipoDirigido.Otros) && (obj.Cargo == TipoCargoOficial.Ninguno))) {
                                string s = null;
                                if (obj.Cargo == TipoCargoOficial.Ninguno) {
                                    s = "La licencia debe pertenecer a la regin de OTRAS LICENCIAS.";
                                } else {
                                    s = "El CARGO debe ser NINGUNO.";
                                }
                                dr = MessageBox.Show(s + "\nAun as, desea guardar los cambios?", "WARNING", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                            } else if (((obj.Licencia.TipoDirigido == TipoDirigido.OPracticas) && (obj.Cargo != TipoCargoOficial.CronoPracticas || obj.Cargo != TipoCargoOficial.CRutaPracticas)) ||
                            ((obj.Licencia.TipoDirigido != TipoDirigido.OPracticas) && (obj.Cargo == TipoCargoOficial.CronoPracticas || obj.Cargo == TipoCargoOficial.CRutaPracticas))) {
                                dr = MessageBox.Show("Si la LICENCIA es para oficiales en prcticas, el CARGO debe pertenecer a oficial en prcticas.\nAun as, desea guardar los cambios?");
                            } else if (!cbCargo.Text.ToUpper().Equals(lblNombreTipo.Text.ToUpper())) {
                                dr = MessageBox.Show("El nombre de la LICENCIA no coincide con el CARGO.\nAun as, desea guardar los cambios?", "WARNING", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                            }
                            if (dr == DialogResult.Yes) {
                                obj.Guardar();
                                MessageBox.Show("La licencia se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                btCancelar.Visible = lblCancelar.Visible = false;
                            }
                        }
                    }
                }
            } catch(Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FColeccion fl = this.ParentForm as FColeccion;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            EOficial oficial = null;
            if (obj.EsNuevo) {
                oficial = obj.Oficial;
            }
            obj.Deshacer();
            this.Cargar(obj);
            if (obj.EsNuevo) {
                obj.Oficial = oficial;
            }
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if (var == null) {
                throw new ArgumentNullException("Licencia Oficial");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ELicenciaOficial;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicenciaOficial.");
            }
            if ((obj.Licencia != null && obj.Licencia.Activo) || obj.Licencia == null) {
                Collection<Collection<KeyValuePair<ELicencia, string>>> col = new Collection<Collection<KeyValuePair<ELicencia, string>>>();
                Collection<KeyValuePair<ELicencia, string>> cKvpLC1 = ELicencia.GetCollectionCombo(TipoDirigido.Oficial);
                Collection<KeyValuePair<ELicencia, string>> cKvpLC2 = ELicencia.GetCollectionCombo(TipoDirigido.OPracticas);
                Collection<KeyValuePair<ELicencia, string>> cKvpLC3 = ELicencia.GetCollectionCombo(TipoDirigido.Otros);
                col.Add(cKvpLC1);
                col.Add(cKvpLC2);
                col.Add(cKvpLC3);
                Collection<KeyValuePair<ELicencia, string>> cKvpLC = new Collection<KeyValuePair<ELicencia, string>>();
                foreach (Collection<KeyValuePair<ELicencia, string>> colKey in col) {
                    foreach (KeyValuePair<ELicencia, string> KeyValuePair in colKey) {
                        cKvpLC.Add(KeyValuePair);
                    }
                }
                cbCodigo.DataSource = cKvpLC;
                cbCodigo.DisplayMember = "Value";
                cbCodigo.ValueMember = "Key";
            } else {
                cbCodigo.DataSource = null;
            }
            //combobox
            EnumConvert eCCargo = new EnumConvert();
            Collection<KeyValuePair<Enum, string>> cKvp = eCCargo.GetCollectionDescription(typeof(TipoCargoOficial));
            cbCargo.DataSource = cKvp;
            cbCargo.DisplayMember = "Value";
            cbCargo.ValueMember = "Key";

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            if (cbCodigo.Items.Count == 0 && !obj.EsNuevo) {
                toolTip.SetToolTip(cbCodigo, "No se puede modificar por que la licencia est inactiva.");
            } else {
                toolTip.SetToolTip(cbCodigo, "");
            }
            try {
                KeyValuePair<ELicencia, string> kvp = (KeyValuePair<ELicencia, string>)cbCodigo.SelectedItem;
                lblNombreTipo.Text = kvp.Key.Nombre;
            } catch (Exception) { }
            if (obj.Licencia != null && !lblNombreTipo.Text.Equals(obj.Licencia.Nombre)) {//Esta vacio por lo que la licencia que va a cargar no est activa
                lblNombreTipo.Text = obj.Licencia.Nombre;
            }
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ELicenciaOficial.NPLicencia));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ELicenciaOficial.NPNumero));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ELicenciaOficial.NPCargo));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            if (e.PropertyName.Equals(ELicenciaOficial.NPLicencia)) {
                PropertyException pe = obj.ErrorDePropiedad(ELicenciaOficial.NPLicencia);
                if (pe != null) {
                    errProv.SetError(cbCodigo, pe.Message);
                } else {
                    errProv.SetError(cbCodigo, "");
                }
            } else if (e.PropertyName.Equals(ELicenciaOficial.NPNumero)) {
                PropertyException pe = obj.ErrorDePropiedad(ELicenciaOficial.NPNumero);
                if (pe != null) {
                    errProv.SetError(txtNumero, pe.Message);
                } else {
                    errProv.SetError(txtNumero, "");
                }
            } else if (e.PropertyName.Equals(ELicenciaOficial.NPCargo)) {
                PropertyException pe = obj.ErrorDePropiedad(ELicenciaOficial.NPCargo);
                if (pe != null) {
                    errProv.SetError(cbCargo, pe.Message);
                } else {
                    errProv.SetError(cbCargo, "");
                }
            }
        }

        public void CrearNuevo() {
            this.Cargar(new ELicenciaOficial());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Licencia Oficial");
            }
            mColeccion = col as EColeccion<ELicenciaOficial>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicenciaOficial.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Licencia Oficial");
            }
            mColeccion = col as EColeccion<ELicenciaOficial>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicenciaOficial.");
            }
            obj = (ELicenciaOficial)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion
    }
}