using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;

namespace Forms {
    public partial class FOficialesCarrera : Form, IControlF {

        private EColeccion<ECargo> mColeccion = null;
        private ECarrera carrera = null;
        private ECargo cargoDC = null;
        private ECargo cargoPresCD = null;
        private ECargo cargoSC = null;
        private EColeccion<ECargo> cargoColCD = null;
        private bool estaCargado = false;

        #region Constructores
        public FOficialesCarrera() {
            InitializeComponent();
        }

        public FOficialesCarrera(ESinClaveBase esb) : this() {
            Cargar(esb);
        }
        #endregion

        #region IControlF

        public void Cargar(ESinClaveBase obj) {
            //el obj ser del tipo ECarrera.
            if (obj == null) {
                throw new ArgumentNullException("Oficiales Carrera");
            }
            carrera = obj as ECarrera;
            if (carrera == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECarrera.");
            }
            //col = carrera.OficialesCarrera;
            //Cargar(col);
            mColeccion =  carrera.OficialesCarrera;
            Cargar(mColeccion);
            estaCargado = true;
        }

        public bool EstaCargado() {
            return estaCargado;
        }

        public void Cargar() {
            if (carrera != null) {
                this.Cargar(carrera);
            }
        }
        #endregion

        private void FOficialesCarrera_Load(object sender, EventArgs e) {
        }

        public void Cargar(EColeccion<ECargo> c) {
            if(c == null) {
                throw new ArgumentNullException("Oficiales Carrera");
            }
            //recuperamos datos
            lblTitulo.Text = c.Titulo;
            cargoDC = ECargo.GetCargo(TipoCargoOficial.DCarrera, c, false);
            cargoSC = ECargo.GetCargo(TipoCargoOficial.SCarrera, c, false);
            cargoPresCD = ECargo.GetCargo(TipoCargoOficial.Ninguno, c, true);
            cargoColCD = ECargo.GetCollectionCargo(TipoCargoOficial.CDeportivo, c);
            //mostramos en pantalla
            if(cargoDC != null) {
                lblDirCarreraTxt.Text = cargoDC.Oficial.Nombre + " " + cargoDC.Oficial.Apellidos;
                lblLicDirCarreraTxt.Text = cargoDC.Licencia.Licencia.Codigo + "-" + cargoDC.Licencia.Numero;
            } else {
                lblDirCarreraTxt.Text = "..........";
                lblLicDirCarreraTxt.Text = "..........";
            }
            if(cargoSC != null) {
                lblSecCarreraTxt.Text = cargoSC.Oficial.Nombre + " " + cargoSC.Oficial.Apellidos;
                lblLicSecCarreraTxt.Text = cargoSC.Licencia.Licencia.Codigo + "-" + cargoSC.Licencia.Numero;
            } else {
                lblSecCarreraTxt.Text = "..........";
                lblLicSecCarreraTxt.Text = "..........";
            }
            if(cargoPresCD != null) {
                lblPresComDepTxt.Text = cargoPresCD.Oficial.Nombre + " " + cargoPresCD.Oficial.Apellidos;
                lblLicPresComDepTxt.Text = cargoPresCD.Licencia.Licencia.Codigo + "-" + cargoPresCD.Licencia.Numero;
            } else {
                lblPresComDepTxt.Text = "..........";
                lblLicPresComDepTxt.Text = "..........";
            }
            ECargo aux = null;
            foreach(ECargo ec in cargoColCD) {
                if(!ec.EsPresidenteCD) {
                    lbCD.Items.Add(ec.Oficial.Nombre + " " + ec.Oficial.Apellidos + " <-->  Licencia:" + ec.Licencia.Licencia.Codigo + "-" + ec.Licencia.Numero);
                } else {
                    aux = ec;
                }
            }
            if(aux != null) {
                cargoColCD.Remove(aux);
            }
        }

        private EColeccion<EOficial> FiltrarColeccion(TipoCargoOficial tco) {
            EColeccion<EOficial> aux = EOficial.GetCollection();
            EColeccion<EOficial> res = new EColeccion<EOficial>();
            EColeccion<EOficial> oficAsignados = carrera.ExtraerOficiales();
            foreach(EOficial ofic in aux) {
                if(!oficAsignados.Contains(ofic) && ofic.PuedeEjercer(tco)) {
                    res.Add(ofic);
                }
            }
            res.ColumnaMostrar("Nif");
            res.ColumnaMostrar("Nombre");
            res.ColumnaMostrar("Apellidos");
            return res;
        }

        private EOficial ExtraerOficial(EColeccion<EOficial> colOf) {
            FDialogo fd = new FDialogo(colOf);
            DialogResult dr = fd.ShowDialog();
            if(dr == DialogResult.OK) {
                return fd.ExtraerEleccion() as EOficial;
            }
            return null;
        }

        private void btCambDC_Click(object sender, EventArgs e) {
            EColeccion<EOficial> colOf = FiltrarColeccion(TipoCargoOficial.DCarrera);
            EOficial ofic = ExtraerOficial(colOf);
            if(ofic != null) {
                try {
                    if(cargoDC == null) {
                        cargoDC = mColeccion.AddNew();
                        cargoDC.Cargo = TipoCargoOficial.DCarrera;
                    }
                    cargoDC.Oficial = ofic;
                    cargoDC.Licencia = ofic.Licencia(cargoDC.Cargo);
                    lblDirCarreraTxt.Text = cargoDC.Oficial.Nombre + " " + cargoDC.Oficial.Apellidos;
                    lblLicDirCarreraTxt.Text = cargoDC.Licencia.Licencia.Codigo + "-" + cargoDC.Licencia.Numero;
                } catch(Exception ex) {
                    MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btQuitarDC_Click(object sender, EventArgs e) {
            if(cargoDC != null) {
                DialogResult dr = MessageBox.Show("Est seguro de quitar " + cargoDC.Oficial.Nombre + " " + cargoDC.Oficial.Apellidos + " del cargo Director de Carrera?", "INFO", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if(dr == DialogResult.Yes) {
                    try {
                        mColeccion.Remove(cargoDC);
                        cargoDC = null;
                        lblDirCarreraTxt.Text = "..........";
                        lblLicDirCarreraTxt.Text = "..........";
                    } catch(Exception ex) {
                        MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void btCambioSC_Click(object sender, EventArgs e) {
            EColeccion<EOficial> colOf = FiltrarColeccion(TipoCargoOficial.SCarrera);
            EOficial ofic = ExtraerOficial(colOf);
            if(ofic != null) {
                try {
                    if(cargoSC == null) {
                        cargoSC = mColeccion.AddNew();
                        cargoSC.Cargo = TipoCargoOficial.SCarrera;
                    }
                    cargoSC.Oficial = ofic;
                    cargoSC.Licencia = ofic.Licencia(cargoSC.Cargo);
                    lblSecCarreraTxt.Text = cargoSC.Oficial.Nombre + " " + cargoSC.Oficial.Apellidos;
                    lblLicSecCarreraTxt.Text = cargoSC.Licencia.Licencia.Codigo + "-" + cargoSC.Licencia.Numero;
                } catch(Exception ex) {
                    MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btQuitarSC_Click(object sender, EventArgs e) {
            if(cargoSC != null) {
                DialogResult dr = MessageBox.Show("Est seguro de quitar " + cargoSC.Oficial.Nombre + " " + cargoSC.Oficial.Apellidos + " del cargo Secretario de Carrera?", "INFO", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if(dr == DialogResult.Yes) {
                    try {
                        mColeccion.Remove(cargoSC);
                        cargoSC = null;
                        lblSecCarreraTxt.Text = "..........";
                        lblLicSecCarreraTxt.Text = "..........";
                    } catch(Exception ex) {
                        MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void btCambiarPCD_Click(object sender, EventArgs e) {
            EColeccion<EOficial> colOf = FiltrarColeccion(TipoCargoOficial.CDeportivo);
            EOficial ofic = ExtraerOficial(colOf);
            if(ofic != null) {
                try {
                    if(cargoPresCD == null) {
                        cargoPresCD = mColeccion.AddNew();
                        cargoPresCD.Cargo = TipoCargoOficial.CDeportivo;
                        cargoPresCD.EsPresidenteCD = true;
                    }
                    cargoPresCD.Oficial = ofic;
                    cargoPresCD.Licencia = ofic.Licencia(cargoPresCD.Cargo);
                    lblPresComDepTxt.Text = cargoPresCD.Oficial.Nombre + " " + cargoPresCD.Oficial.Apellidos;
                    lblLicPresComDepTxt.Text = cargoPresCD.Licencia.Licencia.Codigo + "-" + cargoPresCD.Licencia.Numero;
                } catch(Exception ex) {
                    MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btQuitarPCD_Click(object sender, EventArgs e) {
            if(cargoPresCD != null) {
                DialogResult dr = MessageBox.Show("Est seguro de quitar " + cargoPresCD.Oficial.Nombre + " " + cargoPresCD.Oficial.Apellidos + " del cargo Presidente de Comisarios Deportivos?", "INFO", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if(dr == DialogResult.Yes) {
                    try {
                        mColeccion.Remove(cargoPresCD);
                        cargoPresCD = null;
                        lblPresComDepTxt.Text = "..........";
                        lblLicPresComDepTxt.Text = "..........";
                    } catch(Exception ex) {
                        MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void btAgregarCD_Click(object sender, EventArgs e) {
            EColeccion<EOficial> colOf = FiltrarColeccion(TipoCargoOficial.CDeportivo);
            EOficial ofic = ExtraerOficial(colOf);
            if(ofic != null) {
                ECargo aux = mColeccion.AddNew();
                aux.Cargo = TipoCargoOficial.CDeportivo;
                aux.Oficial = ofic;
                aux.Licencia = ofic.Licencia(TipoCargoOficial.CDeportivo);
                try {
                    cargoColCD.Add(aux);
                    lbCD.Items.Add(aux.Oficial.Nombre + " " + aux.Oficial.Apellidos + " <-->  Licencia:" + aux.Licencia.Licencia.Codigo + "-" + aux.Licencia.Numero);
                } catch(Exception ex) {
                    MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btQuitarCD_Click(object sender, EventArgs e) {
            if(cargoColCD.Count > 0 && lbCD.SelectedIndex >= 0) {
                ECargo aux = cargoColCD[lbCD.SelectedIndex];
                DialogResult dr = MessageBox.Show("Est seguro de quitar " + aux.Oficial.Nombre + " " + aux.Oficial.Apellidos + " del cargo Comisario Deportivo?", "INFO", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                try {
                    cargoColCD.Remove(aux);
                    mColeccion.Remove(aux);
                    lbCD.Items.RemoveAt(lbCD.SelectedIndex);
                } catch(Exception ex) {
                    MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            } else {
                MessageBox.Show("Debe seleccionar un Comisario Deportivo para poder quitarlo.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (carrera.EsNuevo) {
                    MessageBox.Show("No se puede guardar por que la carrera no est almacenada an.");
                } else {
                    if (mColeccion.EstaSucio) {
                        DialogResult dr = MessageBox.Show("Est seguro de quiere guardar los cambios realizados?", "Guardar Cargos", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            mColeccion.Guardar();
                            MessageBox.Show("Los cambios se han guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }
    }
}