using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FPermiso : Form, IFichaEntidad, IFichaEntidadCol {

        private EPermiso obj = null;
        private EColeccion<EPermiso> mColeccion = null;

        public FPermiso() {
            InitializeComponent();
        }

        private void FPermiso_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void btAceptar_Click(object sender, EventArgs e) {
            try {
                if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Permiso", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("El Permiso se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if (btCancelar.Visible) {
                            btCancelar.Visible = false;
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            obj.Deshacer();
            Cargar(obj);
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            //El botn cancelar solo se ve cuando es nuevo.
            if (mColeccion != null) {
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
            } else {
                obj = null;
            }
            if (this.ParentForm != null) {
                FListados fl = this.ParentForm as FListados;
                if (fl != null) {
                    fl.dgv_SelectionChanged(null, null);
                }
            } else {
                this.Close();
            }
        }

        private void FPermiso_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        #region FichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("Usuario");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = entidad as EPermiso;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EUsuario.");
            }
            //combobox
            EnumConvert ec = new EnumConvert();
            Collection<KeyValuePair<Enum, string>> col = ec.GetCollectionDescription(typeof(TipoPermiso));
            cbPermiso.DataSource = col;
            cbPermiso.DisplayMember = "Value";
            cbPermiso.ValueMember = "Key";

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            if (e.PropertyName.Equals(EPermiso.NPTipo)) {
                PropertyException pe = obj.ErrorDePropiedad(EPermiso.NPTipo);
                if (pe != null) {
                    errProv.SetError(cbPermiso, pe.Message);
                } else {
                    errProv.SetError(cbPermiso, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new EPermiso());
            lblCancelar.Visible = true;
            btCancelar.Visible = true;
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(Nibi.Negocio.Colecciones.IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Permiso");
            }
            mColeccion = col as EColeccion<EPermiso>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EPermiso.");
            }
            if (pos < 0 || pos >= mColeccion.Count) {
                throw new OverflowException("La posicin no se encuentra entre los valores mnimos y mximos de la coleccin");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Permiso");
            }
            mColeccion = col as EColeccion<EPermiso>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EPermiso.");
            }
            obj = (EPermiso)mColeccion.AddNew();
            this.Cargar(obj);
            btCancelar.Visible = true;
            lblCancelar.Visible = true;
        }

        #endregion
    }
}