using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FSancion : Form, IFichaEntidad, IFichaEntidadCol {

        private ESancion obj = null;
        private EColeccion<ESancion> mColeccion = null;

        public FSancion() {
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }

        private void FSancion_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
                lblSalir.Visible = btSalir.Visible = false;
            }
            cbCronometrable.Enabled = this.Parent != null;
        }

        private void FSancion_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?\nSi lo hace no podr modificar esta sancin.", "Guardar Sancin", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("Se ha guardado correctamente.", "Guadar ok", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if (btCancelar.Visible) {
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            EPilotoCarrera inscrito = null;
            if (obj.EsNuevo) {
                inscrito = obj.PilotoCarrera;
            }
            obj.Deshacer();
            if (obj.EsNuevo) {
                obj.PilotoCarrera = inscrito;
            }
            this.Cargar(obj);
        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("Sancin");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = entidad as ESancion;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ESancion.");
            }
            //combobox
            EnumConvert convertidorEnumerados = new EnumConvert();
            Collection<KeyValuePair<Enum, string>> cKvp = convertidorEnumerados.GetCollectionDescription(typeof(TipoSancion));
            cbTipoSancion.DataSource = cKvp;
            cbTipoSancion.DisplayMember = "Value";
            cbTipoSancion.ValueMember = "Key";
            //combobox
            cbCronometrable.Items.Clear();
            if (obj.PilotoCarrera != null) {
                if (obj.PilotoCarrera.Carrera.Tipo == TipoCarrera.Rally) {
                    cbCronometrable.Items.Add("");
                }
                EColeccion<ECronometrable> coleccionCronometrables = obj.PilotoCarrera.Carrera.Cronometrables;
                foreach (ECronometrable cronometrable in coleccionCronometrables) {
                    cbCronometrable.Items.Add(cronometrable);
                    if (cronometrable.Equals(obj.Cronometrable)) {
                        cbCronometrable.SelectedIndex = cbCronometrable.Items.Count - 1;
                        //cbCronometrable.SelectedItem = cronometrable;
                    }
                }
            } else if (obj.Cronometrable != null) {
                cbCronometrable.Items.Add(obj.Cronometrable);
                cbCronometrable.SelectedIndex = cbCronometrable.Items.Count - 1;
            }

            
            //Tiempo
            txtHora.Enabled = txtMinutos.Enabled = txtSegundos.Enabled = txtMilesimas.Enabled = (obj.TipoSancion == TipoSancion.Temporal);
            //
            bind.Add(obj);
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            if (obj.PilotoCarrera.Carrera.EstaCerrada) {
                btCancelar.Enabled = btGuardar.Enabled = btDeshacer.Enabled = false;
            }
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESancion.NPCronometrable));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESancion.NPHora));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESancion.NPMinutos));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESancion.NPMilesimas));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESancion.NPSegundos));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESancion.NPImporte));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESancion.NPMotivo));
            //TOOLTIP
            toolTip.SetToolTip(txtHora, "Hora de la sancin");
            toolTip.SetToolTip(txtMinutos, "Minutos de la sancin");
            toolTip.SetToolTip(txtSegundos, "Segundos de la sancin");
            toolTip.SetToolTip(txtMilesimas, "Milsimas de la sancin");
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(ESancion.NPImporte)) {
                pe = obj.ErrorDePropiedad(ESancion.NPImporte);
                if (pe != null) {
                    errProv.SetError(txtImporte, pe.Message);
                } else {
                    errProv.SetError(txtImporte, "");
                }
            } else if (e.PropertyName.Equals(ESancion.NPMotivo)) {
                pe = obj.ErrorDePropiedad(ESancion.NPMotivo);
                if (pe != null) {
                    errProv.SetError(txtMotivo, pe.Message);
                } else {
                    errProv.SetError(txtMotivo, "");
                }
            } else if (e.PropertyName.Equals(ESancion.NPCronometrable)) {
                pe = obj.ErrorDePropiedad(ESancion.NPCronometrable);
                if (pe != null) {
                    errProv.SetError(cbCronometrable, pe.Message);
                } else {
                    errProv.SetError(cbCronometrable, "");
                }
            } else if (e.PropertyName.Equals(ESancion.NPHora)) {
                pe = obj.ErrorDePropiedad(ESancion.NPHora);
                if (pe != null) {
                    errProv.SetError(txtHora, pe.Message);
                } else {
                    errProv.SetError(txtHora, "");
                }
            } else if (e.PropertyName.Equals(ESancion.NPMinutos)) {
                pe = obj.ErrorDePropiedad(ESancion.NPMinutos);
                if (pe != null) {
                    errProv.SetError(txtMinutos, pe.Message);
                } else {
                    errProv.SetError(txtMinutos, "");
                }
            } else if (e.PropertyName.Equals(ESancion.NPSegundos)) {
                pe = obj.ErrorDePropiedad(ESancion.NPSegundos);
                if (pe != null) {
                    errProv.SetError(txtSegundos, pe.Message);
                } else {
                    errProv.SetError(txtSegundos, "");
                }
            } else if (e.PropertyName.Equals(ESancion.NPMilesimas)) {
                pe = obj.ErrorDePropiedad(ESancion.NPMilesimas);
                if (pe != null) {
                    errProv.SetError(txtMilesimas, pe.Message);
                } else {
                    errProv.SetError(txtMilesimas, "");
                }
            }
        }

        public void CrearNuevo() {
            this.Cargar(new ESancion());
            this.btCancelar.Visible = true;
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(Nibi.Negocio.Colecciones.IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Sanciones");
            }
            mColeccion = col as EColeccion<ESancion>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ESancion.");
            }
            if (pos < 0 || pos >= mColeccion.Count) {
                throw new OverflowException("La posicin no se encuentra entre los valores mnimos y mximos de la coleccin");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Sanciones");
            }
            mColeccion = col as EColeccion<ESancion>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ESancion.");
            }
            obj = mColeccion.AddNew();
            this.Cargar(obj);
            btCancelar.Visible = true;
        }

        #endregion

        private void cbCronometrable_SelectionChangeCommitted(object sender, EventArgs e) {
            if (cbCronometrable.SelectedItem != null && !cbCronometrable.SelectedItem.Equals(obj.Cronometrable)) {
                obj.Cronometrable = cbCronometrable.SelectedItem as ECronometrable;
            }
        }

        private void cbTipoSancion_SelectionChangeCommitted(object sender, EventArgs e) {
            txtHora.Enabled = txtMinutos.Enabled = txtSegundos.Enabled = txtMilesimas.Enabled = ((TipoSancion)cbTipoSancion.SelectedValue == TipoSancion.Temporal);
            if (!txtHora.Enabled) {
                txtHora.Text = txtMinutos.Text = txtSegundos.Text = "00";
                txtMilesimas.Text = "000";
            }
        }

        private void btSalir_Click(object sender, EventArgs e) {
            DialogResult dr = DialogResult.Yes;
            if (obj.EstaSucio) {
                dr = MessageBox.Show("Se perdern los cambios no guardados.\nDesea continuar?", "Salir", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
            }
            if (dr == DialogResult.Yes) {
                this.Close();
            }
        }
    }
}