using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using Nibi.Negocio.Colecciones;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FSector : Form, IFichaEntidad, IFichaEntidadCol {

        private ESector obj = null;
        private EColeccion<ESector> mColeccion = null;

        #region Constructores

        public FSector() {
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }

        public FSector(ESector sector) : this() {
            Cargar(sector);
        }

        public FSector(IEColeccion col) : this() {
            CrearNuevo(col);
        }

        public FSector(IEColeccion col, int pos) : this() {
            Cargar(col, pos);
        }

        #endregion

        private void FSector_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {//poner transparente
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FSector_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.Rally.EsNuevo) {
                    MessageBox.Show("No se puede guardar por que el rally no est almacenado an.");
                } else {
                    if (obj.EstaSucio) {
                        DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Sector", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            obj.Guardar();
                            MessageBox.Show("Sector guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            ERally rally = null;
            if (obj.EsNuevo) {
                rally = obj.Rally;
            }
            obj.Deshacer();
            if (obj.EsNuevo) {
                obj.Rally = rally;
            }
            this.Cargar(obj);
        }

        private void btVerTramo_Click(object sender, EventArgs e) {
            try {
                if (obj.EsNuevo) {
                    MessageBox.Show("No puede crear un tramo si el sector no est guardado.\nAlmacene primero este sector.");
                } else {
                    DialogResult dr = DialogResult.Yes;
                    ETramo tramo = null;
                    if (obj.Tramo == null) {
                        dr = MessageBox.Show("No ha creado ningn tramo.\nDesea crearlo ahora?", "Crear Tramo", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            tramo = new ETramo(obj);
                            obj.Tramo = tramo;
                        }
                    } else {
                        tramo = obj.Tramo;
                    }
                    if (dr == DialogResult.Yes) {
                        FTramo fTramo = new FTramo(tramo);
                        DialogResult drTramo = fTramo.ShowDialog();
                        if (drTramo == DialogResult.Cancel) {//fue nuevo y se le dio al botn cancelar.
                            obj.Tramo = null;
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if (var == null) {
                throw new ArgumentNullException("Sector");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ESector;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ESector.");
            }
            //rellenamos el combobox
            cbNumSeccion.DataSource = obj.SeccionesPosibles();
            cbNumSeccion.SelectedItem = obj.NumeroSeccion;
            //colocamos el nmero de sector
            lblNumSector.Text = "SECTOR N: " + obj.Numero.ToString();

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESector.NPNumero));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESector.NPNumeroSeccion));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESector.NPLongitud));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESector.NPCHIni));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESector.NPLocalizacionIni));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(ESector.NPNumero)) {
                pe = obj.ErrorDePropiedad(ESector.NPNumero);
                if (pe != null) {
                    errProv.SetError(lblNumSector, pe.Message);
                } else {
                    errProv.SetError(lblNumSector, "");
                }
            } else if (e.PropertyName.Equals(ESector.NPNumeroSeccion)){
                pe = obj.ErrorDePropiedad(ESector.NPNumeroSeccion);
                if (pe != null) {
                    errProv.SetError(cbNumSeccion, pe.Message);
                } else {
                    errProv.SetError(cbNumSeccion, "");
                }
            } else if (e.PropertyName.Equals(ESector.NPLongitud)) {
                pe = obj.ErrorDePropiedad(ESector.NPLongitud);
                if (pe != null) {
                    errProv.SetError(txtDistancia, pe.Message);
                } else {
                    errProv.SetError(txtDistancia, "");
                }
            } else if (e.PropertyName.Equals(ESector.NPCHIni)) {
                pe = obj.ErrorDePropiedad(ESector.NPCHIni);
                if (pe != null) {
                    errProv.SetError(txtCHIni, pe.Message);
                } else {
                    errProv.SetError(txtCHIni, "");
                }
            } else if (e.PropertyName.Equals(ESector.NPLocalizacionIni)) {
                pe = obj.ErrorDePropiedad(ESector.NPLocalizacionIni);
                if (pe != null) {
                    errProv.SetError(txtLocalizacionIni, pe.Message);
                } else {
                    errProv.SetError(txtLocalizacionIni, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new ESector());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Sector");
            }
            mColeccion = col as EColeccion<ESector>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ESector.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Sector");
            }
            mColeccion = col as EColeccion<ESector>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ESector.");
            }
            obj = (ESector)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        private void cbNumSeccion_SelectionChangeCommitted(object sender, EventArgs e) {
            if ((int)cbNumSeccion.SelectedItem != obj.NumeroSeccion) {
                obj.NumeroSeccion = (int)cbNumSeccion.SelectedItem;
            }
        }
    }
}