using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;

namespace Forms {
    public partial class FTramo : Form, IFichaEntidad, IFichaEntidadCol{

        private ETramo obj = null;
        private EColeccion<ETramo> mColeccion = null;

        #region Constructores

        public FTramo() {
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }

        public FTramo(ETramo tramo) : this() {
            Cargar(tramo);
        }

        public FTramo(IEColeccion col) : this() {
            CrearNuevo(col);
        }

        public FTramo(IEColeccion col, int pos) : this() {
            Cargar(col, pos);
        }

        public FTramo(ETramo tramo, FSector f) : this() {

        }

        #endregion

        private void FTramo_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {//poner transparente
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FTramo_FormClosed(object sender, FormClosedEventArgs e) {
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Tramo", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("Se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        btCancelar.Visible = lblCancelar.Visible = false;
                        btEliminar.Visible = lblEliminar.Visible = true;
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (mColeccion != null) {
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FColeccion fln = this.ParentForm as FColeccion;
                    if (fln != null) {
                        fln.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            ESector sector = null;
            if (obj.EsNuevo) {
                sector = obj.Sector;
            }
            obj.Deshacer();
            if (obj.EsNuevo) {
                obj.Sector = sector;
            }
            this.Cargar(obj);
        }

        private void btEliminar_Click(object sender, EventArgs e) {
            DialogResult dr = MessageBox.Show("Seguro que quiere eliminar el tramo definitivamente?", "ELIMINAR TRAMO", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr == DialogResult.Yes) {
                try {
                    //HACK
                    obj.Sector.Tramo = null;
                    obj.Eliminar();
                    if (AmbitoTransa.Actual != null) {
                        AmbitoTransa.Actual.Commit();
                    }
                    MessageBox.Show("Se ha eliminado correctamente.", "Eliminar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.DialogResult = DialogResult.Cancel;
                    this.Close();
                } catch (Exception ex) {
                    MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if (var == null) {
                throw new ArgumentNullException("Tramo");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ETramo;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ETramo.");
            }

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            //visibilidad de los botones.
            btEliminar.Visible = lblEliminar.Visible = !obj.EsNuevo;
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ETramo.NPNombre));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(ETramo.NPNombre)) {
                pe = obj.ErrorDePropiedad(ETramo.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new ETramo());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(Nibi.Negocio.Colecciones.IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Tramo");
            }
            mColeccion = col as EColeccion<ETramo>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ETramo.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Tramo");
            }
            mColeccion = col as EColeccion<ETramo>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ETramo.");
            }
            obj = (ETramo)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        private void btSalir_Click(object sender, EventArgs e) {
            try {
                if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Tramo", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("Se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
                if (obj.EsNuevo) {
                    DialogResult = DialogResult.Cancel;
                } else {
                    DialogResult = DialogResult.Yes;
                }
                this.Close();
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}