using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.IO;
using System.Collections.ObjectModel;
using Nibi.Negocio.Colecciones;

namespace Forms {
    public partial class FVehiculo : Form, IFichaEntidad, IFichaEntidadCol{

        private EVehiculo obj = null;
        private EColeccion<EVehiculo> mColeccion = null;

        #region Constructores

        public FVehiculo(){
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }
        public FVehiculo(EVehiculo ofic):this() {
            Cargar(ofic);
        }
        public FVehiculo(IEColeccion col) {
            CrearNuevo(col);
        }
        public FVehiculo(IEColeccion col , int pos) {
            Cargar(col , pos);
        }

        #endregion

        private void FVehiculo_Load(object sender, EventArgs e) {
            if(obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if(this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FVehiculo_FormClosed(object sender, FormClosedEventArgs e) {
            if(obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FColeccion fl = this.ParentForm as FColeccion;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.Piloto.EsNuevo) {
                    MessageBox.Show("No se puede guardar.\nEl piloto al que pertenece no est almacenado an.");
                } else {
                    if (obj.EstaSucio) {
                        DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Vehculo", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            obj.Guardar();
                            MessageBox.Show("El vehculo se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                    }
                }
            } catch(Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            EPiloto piloto = null;
            if (obj.EsNuevo) {
                piloto = obj.Piloto;
            }
            obj.Deshacer();
            this.Cargar(obj);
            if (obj.EsNuevo) {
                obj.Piloto = piloto;
            }
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if(var == null) {
                throw new ArgumentNullException("Vehculo");
            }
            if (obj != null) {
                bind.Clear();
                errProv.Clear();
            }
            obj = var as EVehiculo;
            if(obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EVehiculo.");
            }

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            //por si es nuevo el vehiculo y que se refleje los campos obligatorios.
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EVehiculo.NPMatricula));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EVehiculo.NPMarca));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EVehiculo.NPModelo));
            
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(EVehiculo.NPMatricula)) {
                pe = obj.ErrorDePropiedad(EVehiculo.NPMatricula);
                if (pe != null) {
                    errProv.SetError(txtMatricula, pe.Message);
                } else {
                    errProv.SetError(txtMatricula, "");
                }
            } else if (e.PropertyName.Equals(EVehiculo.NPMarca)) {
                pe = obj.ErrorDePropiedad(EVehiculo.NPMarca);
                if (pe != null) {
                    errProv.SetError(txtMarca, pe.Message);
                } else {
                    errProv.SetError(txtMarca, "");
                }
            } else if (e.PropertyName.Equals(EVehiculo.NPModelo)) {
                pe = obj.ErrorDePropiedad(EVehiculo.NPModelo);
                if (pe != null) {
                    errProv.SetError(txtModelo, pe.Message);
                } else {
                    errProv.SetError(txtModelo, "");
                }
            } else if (e.PropertyName.Equals(EVehiculo.NPCilindrada)) {
                pe = obj.ErrorDePropiedad(EVehiculo.NPCilindrada);
                if (pe != null) {
                    errProv.SetError(txtCilindrada, pe.Message);
                } else {
                    errProv.SetError(txtCilindrada, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new EVehiculo());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Vehiculo");
            }
            mColeccion = col as EColeccion<EVehiculo>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EVehiculo.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Vehiculo");
            }
            mColeccion = col as EColeccion<EVehiculo>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EVehiculo.");
            }
            obj = (EVehiculo)mColeccion.AddNew();
            this.Cargar(obj);
            btCancelar.Visible = true;
        }

        #endregion
    }
}