using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.IO;
using System.Collections.ObjectModel;
using Nibi.Negocio.Colecciones;

namespace Forms {
    public partial class FCampeonato : Form, IFichaEntidad, IPestanha, IFichaEntidadCol {

        private ECampeonato obj = null;
        private EColeccion<ECampeonato> mColeccion = null;

        #region Constructores

        public FCampeonato(){
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }
        public FCampeonato(EOficial ofic):this() {
            Cargar(ofic);
        }
        public FCampeonato(IEColeccion col) {
            CrearNuevo(col);
        }
        public FCampeonato(IEColeccion col , int pos) {
            Cargar(col , pos);
        }

        #endregion

        private void FCampeonato_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                //btGrupos.Visible = false;
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FCampeonato_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btBrowse_Click(object sender, EventArgs e) {
            OpenFileDialog ofdialog = new OpenFileDialog();
            ofdialog.Filter = "archivos de imagen (*.jpg)|*.jpg|Todos los archivos (*.*)|*.*";
            ofdialog.FilterIndex = 1;
            ofdialog.RestoreDirectory = true;
            if (ofdialog.ShowDialog() == DialogResult.OK) {
                try {
                    obj.Logo = Image.FromFile(ofdialog.FileName);
                } catch (Exception ex) {
                    MessageBox.Show("No ha sido posible leer el archivo de disco. Error original:" + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Campeonato", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("El campeonato se ha guardado correctamente,\njunto con sus grupos y clases correspondientes.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if (btCancelar.Visible) {//el objeto es nuevo y se agregaran las pestaas oportunas
                            btCancelar.Visible = lblCancelar.Visible = false;
                            if (this.ParentForm != null) {
                                FListados fl = this.ParentForm as FListados;
                                if (fl != null) {
                                    //fl.dgv_SelectionChanged(null, null);
                                    fl.AgregarPestanhas(this.ExtraerPestanhas());
                                }
                            }
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            bool activo = obj.Activo;
            obj.Deshacer();
            if (activo != obj.Activo) {
                obj.Activo = activo;
            }
            this.Cargar(obj);
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if (var == null) {
                throw new ArgumentNullException("Campeonato");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ECampeonato;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonato.");
            }
            //combobox
            EnumConvert ec = new EnumConvert();
            Collection<KeyValuePair<Enum, string>> cKvp = ec.GetCollectionDescription(typeof(TipoCarreraPuntuable));
            cbTipoCarrera.DataSource = cKvp;
            cbTipoCarrera.DisplayMember = "Value";
            cbTipoCarrera.ValueMember = "Key";

            cKvp = ec.GetCollectionDescription(typeof(TipoParticipanteAceptado));
            cbParticipante.DataSource = cKvp;
            cbParticipante.DisplayMember = "Value";
            cbParticipante.ValueMember = "Key";


            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ECampeonato.NPAlias));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ECampeonato.NPNombre));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(ECampeonato.NPAlias)) {
                pe = obj.ErrorDePropiedad(ECampeonato.NPAlias);
                if (pe != null) {
                    errProv.SetError(txtAlias, pe.Message);
                } else {
                    errProv.SetError(txtAlias, "");
                }
            } else if (e.PropertyName.Equals(ECampeonato.NPNombre)) {
                pe = obj.ErrorDePropiedad(ECampeonato.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new ECampeonato());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Campeonato");
            }
            mColeccion = col as EColeccion<ECampeonato>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonato.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Campeonato");
            }
            mColeccion = col as EColeccion<ECampeonato>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECampeonato.");
            }
            obj = (ECampeonato)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            Collection<Pestanha> colP = new Collection<Pestanha>();
            string titulo = "Grupos";
            FColeccion fGrupos = new FColeccion(TipoColeccion.Grupos);
            fGrupos.TopLevel = false;
            fGrupos.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha(titulo, fGrupos));
            return colP;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion
    }
}