using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;
using System.Collections;

namespace Forms {
    public partial class FClasificacion : Form, IControlF {

        private ECronometrable mCronometrable = null;
        private ERally mRally = null;
        private Collection<Panel> mColeccionPaneles = null;
        private Collection<Panel> mColeccionPanelesAbandonos = null;
        private ECarrera carrera = null; //para cargar los campeonatos puntuables.
        private Point puntoTituloAbandono, puntoPanelAbandono, puntoEncabezadoAbandono, puntoPanelOK;
        //Estas colecciones son para generar los documentos de impresin.
        private IEColeccion mColeccionGeneral = null, mColeccionCampeonato = null, mColeccionGeneralAbandonos = null, mColeccionCampeonatoAbandonos = null,
            mColeccionGeneralBase = null;
        private bool mEsGeneral = false;
        private ECampeonato mCampeonato = null; //Para clasificaciones GENERALES

        #region Constructores

        public FClasificacion() {
            InitializeComponent();
            puntoTituloAbandono = lblTituloAbandono.Location;
            puntoPanelAbandono = panelAbandono.Location;
            puntoEncabezadoAbandono = panelEncabezadoAbandono.Location;
            puntoPanelOK = panelClasificacion.Location;
            mColeccionPaneles = new Collection<Panel>();
            mColeccionPanelesAbandonos = new Collection<Panel>();
        }

        /// <summary>
        /// Se utiliza este constructor cuando se va a mostrar las clasificaciones generales.
        /// </summary>
        /// <param name="campeonato"></param>
        public FClasificacion(ECampeonato campeonato):this(){
            mCampeonato = campeonato;
            lblTipoClasificacion.Visible = rbPuntos.Visible = rbTiempo.Visible =
                lbCronometrables.Visible = lblCampeonato.Visible = lbCampeonato.Visible = false;
            panelAbandono.Visible = false;
            lblTP.Text = "PUNTOS";
            lblDorsal.Visible = false;
            lblVehiculo.Visible = false;
            lblEscuderia.Visible = false;
            lblTituloAbandono.Visible = false;
            panelEncabezadoAbandono.Visible = false;
            lblClasificaciones.Location = rbPuntos.Location;
            lblClasificaciones.Text = "Clasificaciones generales : " + campeonato.ToString().ToUpper();
            lblTitulo.Text = "";
        }

        #endregion

        private void FClasificacion_Load(object sender, EventArgs e) {
            panelClasificacion.Visible = false;
            if (mRally != null) {
                lblTitulo.Text = "CLASIFICACIONES DE " + mRally.ToString();
            } else if (mCronometrable != null) {
                lblTitulo.Text = "CLASIFICACIONES DE " + mCronometrable.ToString();
            }
            if (mCampeonato != null) {//Son clasificaciones generales.
                //cbGrupo.DataSource = mCampeonato.GetGrupos();
                cbGrupo.Items.Add("Ninguno");
                EColeccion<ECampeonato> colGrupos = mCampeonato.GetGrupos();
                foreach (ECampeonato grupo in colGrupos) {
                    cbGrupo.Items.Add(grupo);
                }
                this.cbGrupo_SelectionChangeCommitted(null, null);
            }
            if (EUsuario.Actual.GetMayorPermiso().Tipo == TipoPermiso.Publico) {
                btImprimir.Visible = lblImprimir.Visible = false;
            }
            if (this.ParentForm != null && this.ParentForm.GetType().Equals(typeof(FConsultas))) {
                btSalir.Visible = lblSalir.Visible = false;
            }
        }

        private void cbCampeonato_SelectionChangeCommitted(object sender, EventArgs e) {
            if (mCampeonato == null) {
                cbGrupo.Items.Clear();
                cbClase.Items.Clear();
                ECampeonatoPuntuable campPunt = lbCampeonato.SelectedItem as ECampeonatoPuntuable;
                if (campPunt != null) {
                    EColeccion<ECampeonato> col = campPunt.Campeonato.GetGrupos();
                    cbGrupo.Items.Add("Ninguno");
                    foreach (ECampeonato grupo in col) {
                        cbGrupo.Items.Add(grupo);
                    }
                }
            }
        }

        private void cbGrupo_SelectionChangeCommitted(object sender, EventArgs e) {
            cbClase.Items.Clear();
            ECampeonato grupo = cbGrupo.SelectedItem as ECampeonato;
            if (grupo != null) {
                //if (mCampeonato == null) {
                //    cbClase.Items.Add("Ninguno");
                //}
                cbClase.Items.Add("Ninguno");
                EColeccion<ECampeonato> col = grupo.GetClases();
                foreach (ECampeonato clase in col) {
                    cbClase.Items.Add(clase);
                }
            }
        }

        private void MostrarEncabezado() {
            if (mCampeonato == null) {
                if (rbPuntos.Checked) {
                    lblTP.Text = "PUNTOS";
                } else if (rbTiempo.Checked) {
                    lblTP.Text = "TIEMPO";
                } else {
                    throw new Exception("Tiene que elegir un criterio para la clasificacin.");
                }
            }
        }

        private void btOK_Click(object sender, EventArgs e) {
            try {
                MostrarEncabezado();
                //MIRAR SI ES DE CLASIFICACIONES GENERALES.
                if (mCampeonato != null) {//son clasificaciones generales, por tanto, hay que elegir una clase.
                    //ECampeonato clase = cbClase.SelectedItem as ECampeonato;
                    //if (clase == null) {
                    //    throw new Exception("Tiene que elegir una clase.");
                    //}
                    ECampeonato cgc = cbClase.SelectedItem as ECampeonato;
                    if (cgc == null) {
                        cgc = cbGrupo.SelectedItem as ECampeonato;
                    }
                    if (cgc == null) {
                        cgc = mCampeonato;
                    }
                    EColeccion<EClasificacionGeneralBase> colClasificaciones = cgc.ClasificacionesGenerales;
                    mEsGeneral = true;
                    lblTitulo.Text = "CLASIFICACIONES DE : " + cgc.ToString();
                    MostrarClasificacion(colClasificaciones);
                } else {
                    IEColeccion colClasifCarrera = null;
                    ECampeonato mClaseGrupoCampeonato = cbClase.SelectedItem as ECampeonato;//empezamos por el nivel ms bajo.
                    if (mClaseGrupoCampeonato == null && rbPuntos.Checked) {
                        throw new Exception("Si quiere ver los puntos tiene que elegir una clase.");
                    }
                    if (mClaseGrupoCampeonato == null) {//no es una clase, miramos si es un grupo.
                        mClaseGrupoCampeonato = cbGrupo.SelectedItem as ECampeonato;
                    }
                    if (mClaseGrupoCampeonato == null) {//no es un grupo, miramos si es un campeonato.
                        ECampeonatoPuntuable campPunt = lbCampeonato.SelectedItem as ECampeonatoPuntuable;
                        if (campPunt != null) {
                            mClaseGrupoCampeonato = campPunt.Campeonato;
                        }
                    }
                    //Recogemos el cronometrable (o la clasificacion general).
                    ECronometrable cronoElegido = lbCronometrables.SelectedItem as ECronometrable;
                    if (cronoElegido == null && mRally == null) {
                        throw new Exception("Tiene que elegir la clasificacin de alguna manga.");
                    }
                    mCronometrable = cronoElegido;
                    //Preguntamos ahora si es != null
                    if (mClaseGrupoCampeonato != null) {
                        if (mCronometrable != null) {
                            if (rbPuntos.Checked && mRally != null) {
                                throw new Exception("Si quiere ver los puntos del rally tiene que seleccionar la clasificacin general.");
                            }
                            if (rbPuntos.Checked) {
                                EManga mangaOficial = new EManga(cronoElegido.Id);
                                if (mangaOficial.TipoManga != TipoManga.MOfcial) {
                                    throw new Exception("Para ver los puntos tiene que ser de la manga oficial.");
                                }
                                colClasifCarrera = EClasificacionCarreraCampeonatoConPuntos.GetCollection(mClaseGrupoCampeonato, mangaOficial);
                            } else {
                                colClasifCarrera = EClasificacionCarreraCampeonato.GetCollection(mClaseGrupoCampeonato, mCronometrable);
                            }
                        } else if (mRally != null) {
                            if (rbPuntos.Checked) {
                                colClasifCarrera = EClasificacionCarreraCampeonatoConPuntos.GetCollection(mClaseGrupoCampeonato, mRally);
                            } else {
                                colClasifCarrera = EClasificacionCarreraCampeonato.GetCollection(mClaseGrupoCampeonato, mRally);
                            }
                        }
                        mEsGeneral = false;
                    } else if (rbPuntos.Checked) {
                        throw new Exception("Tiene que elegir una clase si quiere ver los puntos.");
                    } else {//hay que recoger la coleccion de clasificacion carrera (general).
                        if (mCronometrable != null) {
                            if (mCronometrable.Tipo == TipoCronometrable.Manga) {
                                colClasifCarrera = mCronometrable.ClasificacionesManga;
                            } else if (mCronometrable.Tipo == TipoCronometrable.Tramo) {
                                colClasifCarrera = mCronometrable.ClasificacionesTramo;
                            }
                            lblTitulo.Text = "CLASIFICACIONES DE " + mCronometrable.ToString();
                        } else if (mRally != null) {
                            colClasifCarrera = mRally.ClasificacionesRally;
                            lblTitulo.Text = "CLASIFICACIONES DE " + mRally.ToString();
                        }
                        mEsGeneral = true;
                    }
                    MostrarClasificacion(colClasifCarrera);
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Muestra la clasificacion que deben ser del tipo EClasificacionCarrera o EClasificacionCarreraCampeonato.
        /// </summary>
        /// <param name="col"></param>
        private void MostrarClasificacion(IEColeccion col) {
            LimpiarClasificacion();
            mColeccionGeneral = new EColeccion<EClasificacionCarrera>();
            mColeccionGeneralAbandonos = new EColeccion<EClasificacionCarrera>();
            mColeccionCampeonato = new EColeccion<EClasificacionCarreraCampeonato>();
            mColeccionCampeonatoAbandonos = new EColeccion<EClasificacionCarreraCampeonato>();
            mColeccionGeneralBase = new EColeccion<EClasificacionGeneralBase>();
            if (col != null && col.Count > 0) {
                col.ApplySort(EClasificacionCarrera.NPPuesto, ListSortDirection.Ascending);
                foreach (ESinClaveBase clasificacion in col) {
                    IClasificacion iClasif = clasificacion as IClasificacion;
                    if (iClasif == null) {
                        throw new InvalidCastException("La coleccin tiene que ser de clasificaciones.");
                    }
                    Panel p = new Panel();
                    p.BorderStyle = BorderStyle.FixedSingle;
                    if (iClasif.EsMostrable()) {
                        if (iClasif.HaAbandonado()) {
                            p.Size = panelAbandono.Size;
                            p.Anchor = panelClasificacion.Anchor;
                            CMuestraAbandono cAbandono = new CMuestraAbandono();
                            cAbandono.Cargar(clasificacion);
                            p.Controls.Add(cAbandono);
                            mColeccionPanelesAbandonos.Add(p);
                            if (mEsGeneral) {
                                mColeccionGeneralAbandonos.Add(clasificacion);
                            } else {
                                mColeccionCampeonatoAbandonos.Add(clasificacion);
                            }
                        } else {
                            p.Size = panelClasificacion.Size;
                            p.Anchor = panelAbandono.Anchor;
                            UserControl cClasificacion = null;
                            if (mCampeonato != null) {
                                CMuestraClasificacionGeneral cClasificacionGeneral = new CMuestraClasificacionGeneral();
                                cClasificacionGeneral.Cargar(clasificacion);
                                cClasificacion = cClasificacionGeneral;
                            } else {
                                CMuestraClasificacionCarrera cClasificacionCarrera = new CMuestraClasificacionCarrera();
                                cClasificacionCarrera.Cargar(clasificacion, rbPuntos.Checked);
                                cClasificacion = cClasificacionCarrera;
                            }
                            p.Controls.Add(cClasificacion);
                            mColeccionPaneles.Add(p);
                            if (mEsGeneral) {
                                if (mCampeonato != null) {
                                    mColeccionGeneralBase.Add(clasificacion);
                                } else {
                                    mColeccionGeneral.Add(clasificacion);
                                }
                            } else {
                                mColeccionCampeonato.Add(clasificacion);
                            }
                        }
                    }
                }
                MostrarClasificacionTiemposPuntos(mColeccionPaneles);
                MostarAbandonos(mColeccionPanelesAbandonos);
            }
        }

        private void MostarAbandonos(Collection<Panel> col) {
            foreach (Panel p in col) {
                if (col.IndexOf(p) == 0) {//es el primero.
                    p.Location = panelAbandono.Location;
                } else {
                    Point punto = new Point();
                    punto.X = col[col.IndexOf(p) - 1].Location.X;
                    punto.Y = col[col.IndexOf(p) - 1].Location.Y + p.Size.Height;
                    p.Location = punto;
                }
                this.Controls.Add(p);
            }
        }

        private void MostrarClasificacionTiemposPuntos(Collection<Panel> col) {
            foreach (Panel p in col) {
                if (col.IndexOf(p) == 0) {//es el primero.
                    p.Location = panelClasificacion.Location;
                } else {
                    Point punto = new Point();
                    punto.X = col[col.IndexOf(p) - 1].Location.X;
                    punto.Y = col[col.IndexOf(p) - 1].Location.Y + p.Size.Height;
                    p.Location = punto;
                    if (col.Count > 0) {//Establecemos la parte de abandonos.
                        punto.Y = lblTituloAbandono.Location.Y + p.Size.Height;
                        punto.X = lblTituloAbandono.Location.X;
                        lblTituloAbandono.Location = punto;

                        punto.Y = panelEncabezadoAbandono.Location.Y + p.Size.Height;
                        punto.X = panelEncabezadoAbandono.Location.X;
                        panelEncabezadoAbandono.Location = punto;

                        punto.Y = panelAbandono.Location.Y + p.Size.Height;
                        punto.X = panelEncabezadoAbandono.Location.X;
                        panelAbandono.Location = punto;
                    }
                }
                this.Controls.Add(p);
            }
        }

        private void LimpiarClasificacion() {
            if (mColeccionPaneles != null) {
                foreach (Panel panel in mColeccionPaneles) {
                    this.Controls.Remove(panel);
                }
                mColeccionPaneles.Clear();
            }
            if (mColeccionPanelesAbandonos != null) {
                foreach (Panel panel in mColeccionPanelesAbandonos) {
                    this.Controls.Remove(panel);
                }

                mColeccionPanelesAbandonos.Clear();
            }
            lblTituloAbandono.Location = puntoTituloAbandono;
            panelAbandono.Location = puntoPanelAbandono;
            panelEncabezadoAbandono.Location = puntoEncabezadoAbandono;
            panelClasificacion.Location = puntoPanelOK;
        }

        private void btSalir_Click(object sender, EventArgs e) {
            this.Close();
        }

        private void FClasificacion_Scroll(object sender, ScrollEventArgs e) {
            int diferenciaMovimiento = e.OldValue - e.NewValue;
            switch (e.ScrollOrientation) {
                case ScrollOrientation.HorizontalScroll:
                    puntoTituloAbandono.X = puntoTituloAbandono.X + diferenciaMovimiento;
                    puntoPanelAbandono.X = puntoPanelAbandono.X + diferenciaMovimiento;
                    puntoEncabezadoAbandono.X = puntoEncabezadoAbandono.X + diferenciaMovimiento;
                    puntoPanelOK.X = puntoPanelOK.X + diferenciaMovimiento;
                    break;
                case ScrollOrientation.VerticalScroll:
                    puntoTituloAbandono.Y = puntoTituloAbandono.Y + diferenciaMovimiento;
                    puntoPanelAbandono.Y = puntoPanelAbandono.Y + diferenciaMovimiento;
                    puntoEncabezadoAbandono.Y = puntoEncabezadoAbandono.Y + diferenciaMovimiento;
                    puntoPanelOK.Y = puntoPanelOK.Y + diferenciaMovimiento;
                    break;
            }
        }

        private void btImprimir_Click(object sender, EventArgs e) {
            FVistaInforme fvi = null;
            if (mCampeonato != null) {//son clasificaciones generales.
                fvi = new FVistaInforme("Forms.Informes.RClasificacionGeneral.rdlc", mColeccionGeneralBase);
            } else if (mEsGeneral) {//ClasificacionCarrera general.
                fvi = new FVistaInforme("Forms.Informes.RClasificacionCarrera.rdlc", mColeccionGeneral, mColeccionGeneralAbandonos);
            } else if (rbPuntos.Checked) {//Clasificacion Carrera Con Puntos.
                fvi = new FVistaInforme("Forms.Informes.RClasificacionCampeonatoConPuntos.rdlc", mColeccionCampeonato, mColeccionCampeonatoAbandonos);
            } else {
                fvi = new FVistaInforme("Forms.Informes.RClasificacionCampeonato.rdlc", mColeccionCampeonato, mColeccionCampeonatoAbandonos);
            }
            fvi.Show();
        }

        #region Cargar

        public void Cargar() {
            //Cargamos los campeonatos puntuables para la carrera.
            cbGrupo.Items.Clear();
            cbClase.Items.Clear();
            lbCampeonato.Items.Clear();
            lbCampeonato.Items.Add("Ninguno");
            if (carrera.Tipo == TipoCarrera.Rally) {
                lbCronometrables.Items.Add("General");
            }
            EColeccion<ECronometrable> colCronometrables = carrera.Cronometrables;
            foreach (ECronometrable crono in colCronometrables) {
                if (crono.Tipo != TipoCronometrable.Sector) {
                    lbCronometrables.Items.Add(crono);
                }   
            }
            EColeccion<ECampeonatoPuntuable> col = carrera.CampeonatosPuntuables;
            foreach (ECampeonatoPuntuable campPunt in col) {
                lbCampeonato.Items.Add(campPunt);
            }
            IEColeccion colClasifCarrera = null;
            if (mCronometrable != null) {
                if (mCronometrable.Tipo == TipoCronometrable.Manga) {
                    colClasifCarrera = mCronometrable.ClasificacionesManga;
                } else if (mCronometrable.Tipo == TipoCronometrable.Tramo) {
                    colClasifCarrera = mCronometrable.ClasificacionesTramo;
                }
            } else if (mRally != null) {
                colClasifCarrera = mRally.ClasificacionesRally;
            }
            MostrarEncabezado();
            //Hack:
            //colClasifCarrera.Cargar();
            //mEsGeneral = true;
            //MostrarClasificacion(colClasifCarrera);
            //mEstaCargado = true;
        }

        #endregion

        #region IControlF

        private bool mEstaCargado = false;
        public void Cargar(ESinClaveBase obj) {
            if (mCampeonato == null) {
                if (obj == null) {
                    throw new ArgumentNullException("Carrera");
                }
                mCronometrable = obj as ECronometrable;
                if (mCronometrable == null) {//probar si es una carrera (rally o subida)
                    //si es subida estamos en las consultas, as que le damos un cronometrable(manga).
                    carrera = obj as ECarrera;
                    mRally = obj as ERally;
                    if (mRally == null) {//es una subida.(zona de consultas).
                        ESubida subida = obj as ESubida;
                        if (subida == null) {
                            throw new InvalidCastException("La entidad no tiene los tipos adecuados.");
                        }
                        mCronometrable = subida.Mangas[0];
                    }
                } else if (mCronometrable.Tipo == TipoCronometrable.Sector) {
                    throw new InvalidCastException("La entidad debe ser de tipo ECronometrable y de tipo Manga o Tramo.");
                } else {
                    carrera = mCronometrable.Carrera;
                }
                Cargar();
            }
        }

        public bool EstaCargado() {
            return mEstaCargado;
        }

        #endregion
    }
}
