using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;

namespace Forms {
    public partial class FCronometrar : Form, IFichaEntidad {

        private ECronometrable obj = null;
        private int mIndiceInscrito = 0;
        private EColeccion<EPilotoCarrera> mColeccion = null;
        private bool mEstaCargado = false;
        private Form f = null;

        public FCronometrar(ECronometrable cronometrable){
            InitializeComponent();
            obj = cronometrable;
        }

        private void FCronometrar_Load(object sender, EventArgs e) {
            //if (this.Parent != null) {
            //    this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            //    this.BackColor = Color.Transparent;
            //}
            try {
                if (obj != null && obj.Carrera.EstaCerrada) {
                    btAplicar.Click -= btAplicar_Click;
                    this.toolTip.SetToolTip(btAplicar, "No se puede guardar. La carrera ya est cerrada.");
                }
                if (!mEstaCargado) {
                    Cargar(obj);
                }
                this.toolTip.SetToolTip(btSiguiente, "Siguiente");
                this.toolTip.SetToolTip(btPrimero, "Primero");
                this.toolTip.SetToolTip(btAnterior, "Anterior");
                this.toolTip.SetToolTip(btUltimo, "ltimo");
            } catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
        }

        private void btAplicar_Click(object sender, EventArgs e) {
            try {
                if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Est seguro de que quiere aplicar los cambios realizados?", "Guardar Cambios", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (obj != null && dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("Los cambios se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR GUARDANDO", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void MostrarCopiloto(bool necesitaCopiloto, ECopiloto copiloto) {
            lblApellidosCopiloto.Visible = necesitaCopiloto;
            lblDNICopiloto.Visible = necesitaCopiloto;
            lblNombreCopiloto.Visible = necesitaCopiloto;
            pbFotoCopiloto.Visible = necesitaCopiloto;
            if (necesitaCopiloto) {
                bindCopiloto.Add(copiloto);
            } else {
                //lblCopiloto.Text = "COPILOTO -----> NO SE NECESITA.";
                lblCopiloto.Visible = false;
                //datos del vehculo en el lugar del copiloto.
                int distanciaVC = lblVehiculo.Location.Y - lblCopiloto.Location.Y;
                lblVehiculo.Location = lblCopiloto.Location;
                lblMatricula.Location = new Point(lblMatricula.Location.X, lblMatricula.Location.Y - distanciaVC);
                lblMarca.Location = new Point(lblMarca.Location.X, lblMarca.Location.Y - distanciaVC);
                lblModelo.Location = new Point(lblModelo.Location.X, lblModelo.Location.Y - distanciaVC);

            }
        }

        private void ColocarEnPanel(Form f) {
            pCrono.Controls.Clear();
            f.FormBorderStyle = FormBorderStyle.None;
            f.Anchor = AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Top;
            f.TopLevel = false;
            pCrono.Controls.Add(f);
            f.Parent = pCrono;
            f.Dock = DockStyle.Fill;
        }

        private void btPrimero_Click(object sender, EventArgs e) {
            if (mIndiceInscrito == 0) {
                MessageBox.Show("Ya se ha alcanzado el primero.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            } else {
                int indiceAnterior = mIndiceInscrito;
                mIndiceInscrito = 0;
                while (mIndiceInscrito < mColeccion.Count && mColeccion[mIndiceInscrito].Estado != TipoEstado.Autorizado) {
                    mIndiceInscrito = mIndiceInscrito + 1;
                }
                if (mIndiceInscrito == indiceAnterior) {
                    MessageBox.Show("Ya se ha alcanzado el primero.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }else if (mIndiceInscrito < mColeccion.Count) {
                    this.Cargar();
                } else {
                    MessageBox.Show("No se encuentra el primero.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        private void btAnterior_Click(object sender, EventArgs e) {
            if (mIndiceInscrito > 0 && HayAutorizadosAtras()) {
                mIndiceInscrito = mIndiceInscrito - 1;
                while (mIndiceInscrito < mColeccion.Count && mColeccion[mIndiceInscrito].Estado != TipoEstado.Autorizado) {
                    mIndiceInscrito = mIndiceInscrito - 1;
                }
                if (mIndiceInscrito < mColeccion.Count) {
                    this.Cargar();
                }
            } else {
                MessageBox.Show("Ya se ha alcanzado el primero.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private bool HayAutorizadosAtras() {
            int n = mIndiceInscrito - 1;
            while (n >= 0) {
                if (mColeccion[n].Estado == TipoEstado.Autorizado) {
                    return true;
                } else {
                    n = n - 1;
                }
            }
            return false;
        }

        private void btSiguiente_Click(object sender, EventArgs e) {
            if (mColeccion != null) {
                if (mIndiceInscrito + 1 >= mColeccion.Count || !HayAutorizadosAdelante()) {
                    MessageBox.Show("Ya se ha alcanzado el ltimo.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                } else {
                    mIndiceInscrito = mIndiceInscrito + 1;
                    while (mIndiceInscrito < mColeccion.Count && mColeccion[mIndiceInscrito].Estado != TipoEstado.Autorizado) {
                        mIndiceInscrito = mIndiceInscrito + 1;
                    }
                    if (mIndiceInscrito < mColeccion.Count) {
                        this.Cargar();
                    }
                }
            } else {
                throw new InvalidOperationException("La coleccin est vaca.");
            }
        }

        private bool HayAutorizadosAdelante() {
            int n = mIndiceInscrito + 1;
            while (n < mColeccion.Count) {
                if (mColeccion[n].Estado == TipoEstado.Autorizado) {
                    return true;
                } else {
                    n = n + 1;
                }
            }
            return false;
        }

        private void btUltimo_Click(object sender, EventArgs e) {
            if (mIndiceInscrito == mColeccion.Count - 1) {
                MessageBox.Show("Ya se ha alcanzado el ltimo.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            } else {
                int indiceAnterior = mIndiceInscrito;
                mIndiceInscrito = mColeccion.Count - 1;
                while (mIndiceInscrito >= 0 && mColeccion.Count > mIndiceInscrito && mColeccion[mIndiceInscrito].Estado != TipoEstado.Autorizado) {
                    mIndiceInscrito = mIndiceInscrito - 1;
                }
                if (mIndiceInscrito == indiceAnterior) {
                    MessageBox.Show("Ya se ha alcanzado el ltimo.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }else if (mIndiceInscrito >= 0 && mIndiceInscrito < mColeccion.Count) {
                    this.Cargar();
                } else {
                    MessageBox.Show("No se encuentra el ltimo.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        #region IFichaEntidad

        private void Cargar() {
            //limpiamos los bind
            bindCopiloto.Clear();
            bindPiloto.Clear();
            bindVehiculo.Clear();
            bindPilotoCarrera.Clear();
            //
            if (mIndiceInscrito >= 0 && mIndiceInscrito < mColeccion.Count) {
                EPilotoCarrera mPilotoCarrera = mColeccion[mIndiceInscrito];
                if (mPilotoCarrera.Estado != TipoEstado.Autorizado) {
                    mIndiceInscrito = mIndiceInscrito + 1;
                    Cargar();
                } else {
                    bindPiloto.Add(mPilotoCarrera.Piloto);
                    bindVehiculo.Add(mPilotoCarrera.Vehiculo);
                    MostrarCopiloto(obj.Carrera.NecesitaCopiloto(), mPilotoCarrera.Copiloto);
                    lblTitulo.Text = obj.ToString();
                    bindPilotoCarrera.Add(mPilotoCarrera);
                    //falta cargar el tiempo que hizo, si es que lo hizo, si no, se crea.
                    ESinClaveBase esb = null;
                    if (obj.Tipo == TipoCronometrable.Sector) {
                        esb = obj.GetTiempoSector(mPilotoCarrera);
                        FCronoSector fcs = f as FCronoSector;
                        if (fcs == null) {
                            f = new FCronoSector();
                        }
                    } else if (obj.Tipo == TipoCronometrable.Manga || obj.Tipo == TipoCronometrable.Tramo) {
                        esb = obj.GetClasificacion(mPilotoCarrera);
                        FCronoClasificacion fcc = f as FCronoClasificacion;
                        if (fcc == null) {
                            f = new FCronoClasificacion();
                        }
                    }
                    IFichaEntidad ficha = f as IFichaEntidad;
                    if (ficha != null) {
                        ficha.Cargar(esb);
                        if (pCrono.Controls.Count == 0) {
                            ColocarEnPanel(f);
                            f.Show();
                        }
                    }
                    mEstaCargado = true;
                }
            } else {
                MessageBox.Show("No hay inscritos en esta carrera.", "WARNING", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("La entidad no puede estar vaca.");
            }
            obj = entidad as ECronometrable;
            if (obj == null) {
                throw new InvalidCastException("La entidad debe ser de tipo ECronometrable.");
            }
            try {
                mColeccion = obj.Carrera.PilotosCarrera;
                mColeccion.ApplySort(EPilotoCarrera.NPDorsal, ListSortDirection.Ascending);
                mColeccion.Cargar();
            } catch (Exception ex) {
                throw new Exception("Error en la coleccin de inscritos, no se ha podido cargar. Mensaje original: " + ex.Message);
            }
            if (mColeccion == null) {
                throw new Exception("Error en la coleccin de inscritos, la coleccin no se ha cargado.");
            }
            if (mIndiceInscrito < 0 || mIndiceInscrito >= mColeccion.Count) {
                if (mColeccion.Count == 0) {
                    MessageBox.Show("No hay inscritos en esta carrera.", "WARNING", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                } else {
                    //throw new OverflowException("El ndice debe ser como mximo el nmero de inscritos en la carrera.");
                    MessageBox.Show("No hay inscritos autorizados en esta carrera.", "WARNING", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            } else {
                Cargar();
            }
        }

        /// <summary>
        /// Aqu no se aplica el mtodo crear nuevo.
        /// </summary>
        public void CrearNuevo() {
            throw new Exception("El mtodo crear nuevo no se aplica a este formulario.");
        }

        #endregion

        private void btSalir_Click(object sender, EventArgs e) {
            this.Close();
        }

        private void FCronometrar_FormClosing(object sender, FormClosingEventArgs e) {
            try {
                if (obj.EstaSucio && !obj.Carrera.EstaCerrada ) {
                    DialogResult dr = MessageBox.Show("Est seguro de que quiere aplicar los cambios realizados?", "Guardar Cambios", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (obj != null && dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("Los cambios se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR GUARDANDO", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}