using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.IO;
using System.Collections.ObjectModel;
using Nibi.Negocio.Colecciones;

namespace Forms {
    public partial class FEscuderia : Form, IFichaEntidad, IPestanha, IFichaEntidadCol {

        private EEscuderia obj = null;
        private EColeccion<EEscuderia> mColeccion = null;

        #region Constructores

        public FEscuderia(){
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }
        public FEscuderia(EOficial ofic):this() {
            Cargar(ofic);
        }
        public FEscuderia(IEColeccion col) {
            CrearNuevo(col);
        }
        public FEscuderia(IEColeccion col , int pos) {
            Cargar(col , pos);
        }

        #endregion

        private void FEscuderia_Load(object sender, EventArgs e) {
            if(obj == null) {
                CrearNuevo();
            } else {
                bindDir.ResetCurrentItem();
                bindEsc.ResetCurrentItem();
            }
            if(this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FEscuderia_FormClosed(object sender, FormClosedEventArgs e) {
            if(obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btBrowse_Click(object sender, EventArgs e) {
            OpenFileDialog ofdialog = new OpenFileDialog();
            ofdialog.Filter = "archivos de imagen (*.jpg)|*.jpg|Todos los archivos (*.*)|*.*";
            ofdialog.FilterIndex = 1;
            ofdialog.RestoreDirectory = true;
            if (ofdialog.ShowDialog() == DialogResult.OK) {
                try {
                    obj.Logo = Image.FromFile(ofdialog.FileName);
                } catch (Exception ex) {
                    MessageBox.Show("No ha sido posible leer el archivo de disco. Error original:" + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (mColeccion != null) {
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
            } 
            if(this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if(obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Escudera", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if(dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("La escudera se ha guardado correctamente,\njunto con sus licencias, participantes y carreras organizadas.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if(btCancelar.Visible) {//el objeto es nuevo
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                    }
                }
            } catch(Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            bool activo = obj.Activo;
            obj.Deshacer();
            if (obj.Activo != activo) {
                obj.Activo = activo;
            }
            this.Cargar(obj);
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if(var == null) {
                throw new ArgumentNullException("Escuderia");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bindDir.Clear();
                bindEsc.Clear();
                errProv.Clear();
            }
            obj = var as EEscuderia;
            if(obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EEscuderia.");
            }
            //combobox
            EnumConvert eCTipo = new EnumConvert();
            Collection<KeyValuePair<Enum, string>> cKvp = eCTipo.GetCollectionDescription(typeof(TipoEscuderia));
            cbTipo.DataSource = cKvp;
            cbTipo.DisplayMember = "Value";
            cbTipo.ValueMember = "Key";

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bindEsc.Add(obj);
            if (obj.Direccion == null) {
                obj.Direccion = new EDireccion();
            }
            obj.Direccion.PropertyChanged += new PropertyChangedEventHandler(Direccion_PropertyChanged);
            bindDir.Add(obj.Direccion);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EEscuderia.NPNombre));
        }

        void Direccion_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(EDireccion.NPDomicilio)) {//TODO:
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPDomicilio);
                if (pe != null) {
                    errProv.SetError(txtDomicilio, pe.Message);
                } else {
                    errProv.SetError(txtDomicilio, "");
                }
            } else if (e.PropertyName.Equals(EDireccion.NPCodigoPostal)) {
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPCodigoPostal);
                if (pe != null) {
                    errProv.SetError(txtCodigoPostal, pe.Message);
                } else {
                    errProv.SetError(txtCodigoPostal, "");
                }
            } else if (e.PropertyName.Equals(EDireccion.NPTelefono)) {
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPTelefono);
                if (pe != null) {
                    errProv.SetError(txtTelefono, pe.Message);
                } else {
                    errProv.SetError(txtTelefono, "");
                }
            } else if (e.PropertyName.Equals(EDireccion.NPMovil)) {
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPMovil);
                if (pe != null) {
                    errProv.SetError(txtMovil, pe.Message);
                } else {
                    errProv.SetError(txtMovil, "");
                }
            } else if (e.PropertyName.Equals(EDireccion.NPFax)) {
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPFax);
                if (pe != null) {
                    errProv.SetError(txtFax, pe.Message);
                } else {
                    errProv.SetError(txtFax, "");
                }
            }
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            if (e.PropertyName.Equals(EEscuderia.NPNombre)) {
                PropertyException pe = obj.ErrorDePropiedad(EEscuderia.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new EEscuderia());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Escuderia");
            }
            mColeccion = col as EColeccion<EEscuderia>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EEscuderia.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Escuderia");
            }
            mColeccion = col as EColeccion<EEscuderia>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EEscuderia.");
            }
            obj = (EEscuderia)mColeccion.AddNew();
            this.Cargar(obj);
            btCancelar.Visible = true;
        }

        #endregion

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            Collection<Pestanha> colP = new Collection<Pestanha>();
            string titulo = "Licencias";
            FColeccion fLicNum = new FColeccion(TipoColeccion.Licencias);
            fLicNum.TopLevel = false;
            fLicNum.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha(titulo, fLicNum));
            //Participantes
            titulo = "Participantes";
            FColeccion fColParticipantes = new FColeccion(TipoColeccion.Participantes);
            fColParticipantes.TopLevel = false;
            fColParticipantes.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha(titulo, fColParticipantes));
            //Carreras
            titulo = "Carreras organizadas";
            FColeccion fCarreras = new FColeccion(TipoColeccion.Carreras);
            fCarreras.TopLevel = false;
            fCarreras.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha(titulo, fCarreras));
            return colP;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion
    }
}