using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FGaleria : Form, IControlF {

        private ECarrera objCarrera = null;
        private EColeccion<EImagen> mColeccionImagenes = null;
        private bool mEstaCargado = false;

        public FGaleria() {
            InitializeComponent();
        }

        private void FGaleria_Load(object sender, EventArgs e) {
            if (objCarrera != null) {
                lblTitulo.Text = "IMGENES DE " + objCarrera.Nombre;
                mColeccionImagenes = objCarrera.Imagenes;
            }
            if (EUsuario.Actual.GetMayorPermiso().Tipo == TipoPermiso.Publico) {
                pFlowCImagen.Dock = DockStyle.Fill;
                btSalir.Visible = lblSalir.Visible = lblTitulo.Visible = btCrearImagen.Visible = btEliminarImagenes.Visible = false;
            }
        }

        private void btCrearImagen_Click(object sender, EventArgs e) {
            int maxImagenes = mColeccionImagenes.Count;
            FImagen fimagen = new FImagen();
            fimagen.CrearNuevo(mColeccionImagenes);
            fimagen.ShowDialog();
            if (mColeccionImagenes.Count > 0 && mColeccionImagenes.Count > maxImagenes && !mColeccionImagenes[mColeccionImagenes.Count - 1].EsNuevo) {
                CImagen cimagen = new CImagen(mColeccionImagenes[mColeccionImagenes.Count - 1]);
                pFlowCImagen.Controls.Add(cimagen);
            }
        }

        private void btEliminarImagenes_Click(object sender, EventArgs e) {
            DialogResult dr = MessageBox.Show("Est seguro de eliminar todas las imgenes seleccionada?", "Eliminar Imgenes", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr == DialogResult.Yes) {
                try {
                    Collection<Control> col = new Collection<Control>();
                    foreach (Control control in pFlowCImagen.Controls) {
                        CImagen cimagen = control as CImagen;
                        if (cimagen != null && cimagen.EstaSeleccionado()) {
                            EImagen imagen = cimagen.Extraer();
                            if (imagen != null) {
                                imagen.Eliminar();
                                mColeccionImagenes.Remove(imagen);
                            }
                            col.Add(cimagen);
                            //pFlowCImagen.Controls.Remove(cimagen);
                        }
                    }
                    //mColeccionImagenes.Guardar();
                    foreach (Control control in col) {
                        pFlowCImagen.Controls.Remove(control);
                    }
                    MessageBox.Show("Se ha eliminado correctamente.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Information);
                } catch (Exception ex) {
                    MessageBox.Show("Error al eliminar las fotos. Mensaje original: " + ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    if (objCarrera != null) {
                        this.Cargar(objCarrera);
                    }
                }
            }
        }

        #region IControlF

        public void Cargar(ESinClaveBase obj) {
            if (obj == null) {
                throw new ArgumentNullException("Carrera");
            }
            objCarrera = obj as ECarrera;
            if (objCarrera == null) {
                throw new InvalidCastException("La entidad debe ser de tipo ECarrera.");
            }
            mColeccionImagenes = objCarrera.Imagenes;
            pFlowCImagen.Controls.Clear();
            if (mColeccionImagenes != null) {
                foreach (EImagen imagen in mColeccionImagenes) {
                    CImagen cimagen = new CImagen(imagen);
                    pFlowCImagen.Controls.Add(cimagen);
                }
            }
            mEstaCargado = true;
        }

        public bool EstaCargado() {
            return mEstaCargado;
        }

        #endregion

        private void btSalir_Click(object sender, EventArgs e) {
            this.Close();
        }
    }
}