using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FInscrito : Form, IFichaEntidad, IFichaEntidadCol {

        public EColeccion<EPilotoCarrera> mColeccion = null;
        private EPilotoCarrera obj;
        private bool estaCampeonatosCargados = false;

        #region Constructores

        public FInscrito() {
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }

        public FInscrito(EPilotoCarrera pilotoCarrera) : this() {
            obj = pilotoCarrera;
        }

        public FInscrito(EColeccion<EPilotoCarrera> col) : this() {
            mColeccion = col;
        }

        public FInscrito(IEColeccion col, int pos) : this() {
            Cargar(col, pos);
        }
        #endregion

        private void FInscrito_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FInscrito_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj != null && obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Inscripcin", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("Se ha guardado correctamente.", "Guadar ok", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        obj.Deshacer();
                        this.Cargar(obj);
                        btCancelar.Visible = lblCancelar.Visible = false;
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.btDeshacer_Click(null, null);
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (mColeccion.Contains(obj)) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FColeccion fc = this.ParentForm as FColeccion;
                    if (fc != null) {
                        fc.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            try {
                ECarrera carrera = null;
                if (obj.EsNuevo) {
                    carrera = obj.Carrera;
                }
                obj.Deshacer();
                if (obj.EsNuevo) {
                    obj.Carrera = carrera;
                }
                this.Cargar(obj);
                FColeccion fc = this.ParentForm as FColeccion;
                if (fc != null) {
                    fc.AplicarColores(obj, null);
                }
            } catch (Exception) {
            }
        }

        private void MostrarCopiloto(bool necesitaCop) {
            lblApellidosCopiloto.Visible = necesitaCop;
            lblDNICopiloto.Visible = necesitaCop;
            lblNombreCopiloto.Visible = necesitaCop;
            pbFotoCopiloto.Visible = necesitaCop;
            btCopiloto.Visible = necesitaCop;
            lblCambiarCopiloto.Visible = necesitaCop;
            pbCambioCopiloto.Visible = necesitaCop;
            if (necesitaCop) {
                bindCopiloto.Add(obj.Copiloto);
            }
        }

        private void btPiloto_Click(object sender, EventArgs e) {
            try {
                EColeccion<EPiloto> col = EPiloto.GetCollection(true);
                col.ColumnaMostrarRango(new string[] {EPiloto.NPNif, EPiloto.NPNombre, EPiloto.NPApellidos});
                FDialogo fd = new FDialogo(col);
                DialogResult dr = fd.ShowDialog();
                if (dr == DialogResult.OK) {
                    EPiloto piloto = fd.ExtraerEleccion() as EPiloto;
                    if (piloto != null) {
                        obj.Piloto = piloto;
                        obj.Vehiculo = null;
                        bindPiloto.Clear();
                        bindVehiculo.Clear();
                        bindPiloto.Add(obj.Piloto);
                        bindVehiculo.Add(obj.Vehiculo);
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show("Error al elegir piloto.\nError original:\n" + ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btVehiculo_Click(object sender, EventArgs e) {
            try {
                if (obj.Piloto == null) {
                    MessageBox.Show("No es posible seleccionar un vehiculo./nAntes tiene que elegir el piloto.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                } else {
                    FDialogo fd = new FDialogo(obj.Piloto.Vehiculos);
                    DialogResult dr = fd.ShowDialog();
                    if (dr == DialogResult.OK) {
                        EVehiculo vehiculo = fd.ExtraerEleccion() as EVehiculo;
                        if (vehiculo != null) {
                            obj.Vehiculo = vehiculo;
                            bindVehiculo.Clear();
                            bindVehiculo.Add(obj.Vehiculo);
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show("Error al elegir vehculo.\nError original:\n" + ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCopiloto_Click(object sender, EventArgs e) {
            try {
                EColeccion<ECopiloto> col = ECopiloto.GetCollection(true);
                col.ColumnaMostrarRango(new string[] { EPiloto.NPNif, EPiloto.NPNombre, EPiloto.NPApellidos });
                FDialogo fd = new FDialogo(col);
                DialogResult dr = fd.ShowDialog();
                if (dr == DialogResult.OK) {
                    ECopiloto copiloto = fd.ExtraerEleccion() as ECopiloto;
                    if (copiloto != null) {
                        obj.Copiloto = copiloto;
                        bindCopiloto.Clear();
                        bindCopiloto.Add(obj.Copiloto);
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show("Error al elegir copiloto.\nError original:\n" + ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void cbEstado_SelectionChangeCommitted(object sender, EventArgs e) {
            txtColorEstado.BackColor = obj.DevolverColor((TipoEstado)cbEstado.SelectedValue);
            FColeccion fc = this.ParentForm as FColeccion;
            if (fc != null) {
                fc.AplicarColores(obj, (TipoEstado)cbEstado.SelectedValue);
            }
        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("Inscripcin");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                bindPiloto.Clear();
                bindVehiculo.Clear();
                bindCopiloto.Clear();
                errProv.Clear();
            }
            obj = entidad as EPilotoCarrera;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EPilotoCarrera.");
            }
            //rellenamos el combobox
            EnumConvert ec = new EnumConvert();
            Collection<KeyValuePair<Enum, string>> ckvp = ec.GetCollectionDescription(typeof(TipoEstado));
            cbEstado.DataSource = ckvp;
            cbEstado.ValueMember = "Key";
            cbEstado.DisplayMember = "Value";

            //rellenamos los campos correspondientes al piloto, copiloto(si procede) y vehculo
            bindPiloto.Add(obj.Piloto);
            bindVehiculo.Add(obj.Vehiculo);
            if (obj.Carrera != null) {
                MostrarCopiloto(obj.Carrera.NecesitaCopiloto());
            } else {
                MostrarCopiloto(false);
            }

            IColor entidadColor = obj as IColor;
            this.txtColorEstado.BackColor = entidadColor.DevolverColor();
            //rellenamos el listBox con las campeonatos en los que ya se ha inscrito.
            lbCampeonatosInscritos.Items.Clear();
            EColeccion<ECampeonatoInscrito> colCampInscrito = obj.CampeonatosInscrito;
            foreach (ECampeonatoInscrito ci in colCampInscrito) {
                lbCampeonatosInscritos.Items.Add(ci.NPToString);
            }
            //rellenamos el combobox campeonato de los campeonatos puntuables de la carrera.
            if (!estaCampeonatosCargados) {//
                EColeccion<ECampeonatoPuntuable> colCampPunt = obj.Carrera.CampeonatosPuntuables;
                foreach (ECampeonatoPuntuable campPunt in colCampPunt) {
                    cbCampeonatoPuntuable.Items.Add(campPunt.Campeonato);
                }
                estaCampeonatosCargados = true;
            }

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            //se comprueba que el objeto no estaba modificado anteriormente.
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            if (obj.Carrera.EstaCerrada) {
                btGuardar.Click -= btGuardar_Click;
                btCopiloto.Click -= btCopiloto_Click;
                btPiloto.Click -= btPiloto_Click;
                btVehiculo.Click -= btVehiculo_Click;
                btQuitar.Click -= btQuitar_Click;
                btAgregar.Click -= btAgregar_Click;
                btCancelar.Click -= btCancelar_Click;
                this.toolTip.SetToolTip(btGuardar, "No se puede guardar. La carrera ya ha finalizado.");
                this.toolTip.SetToolTip(btPiloto, "No se puede modificar. La carrer ha finalizado.");
                this.toolTip.SetToolTip(btVehiculo, "No se puede modificar. La carrer ha finalizado.");
                this.toolTip.SetToolTip(btCopiloto, "No se puede modificar. La carrer ha finalizado.");
                this.toolTip.SetToolTip(btAgregar, "No se puede modificar. La carrer ha finalizado.");
                this.toolTip.SetToolTip(btQuitar, "No se puede modificar. La carrer ha finalizado.");
            } else {
                //Tooltip
                this.toolTip.SetToolTip(btAgregar, "Agrega un campeonato, con su grupo y clase elegido al inscrito.");
                this.toolTip.SetToolTip(btQuitar, "Quita de la lista el campeonato seleccionado.");
            }
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EPilotoCarrera.NPPiloto));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EPilotoCarrera.NPVehiculo));
            if (obj.Carrera.NecesitaCopiloto()) {
                this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EPilotoCarrera.NPCopiloto));
            }
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EPilotoCarrera.NPDorsal));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(EPilotoCarrera.NPPiloto)) {
                pe = obj.ErrorDePropiedad(EPilotoCarrera.NPPiloto);
                if (pe != null) {
                    errProv.SetError(btPiloto, pe.Message);
                } else {
                    errProv.SetError(btPiloto, "");
                }
            } else if (e.PropertyName.Equals(EPilotoCarrera.NPVehiculo)) {
                pe = obj.ErrorDePropiedad(EPilotoCarrera.NPVehiculo);
                if (pe != null) {
                    errProv.SetError(btVehiculo, pe.Message);
                } else {
                    errProv.SetError(btVehiculo, "");
                }
            } else if (e.PropertyName.Equals(EPilotoCarrera.NPCopiloto)) {
                pe = obj.ErrorDePropiedad(EPilotoCarrera.NPCopiloto);
                if (pe != null) {
                    errProv.SetError(btCopiloto, pe.Message);
                } else {
                    errProv.SetError(btCopiloto, "");
                }
            } else if (e.PropertyName.Equals(EPilotoCarrera.NPDorsal)) {
                pe = obj.ErrorDePropiedad(EPilotoCarrera.NPDorsal);
                if (pe != null) {
                    errProv.SetError(txtDorsal, pe.Message);
                } else {
                    errProv.SetError(txtDorsal, "");
                }
            } 
        }

        public void CrearNuevo() {
            Cargar(new EPilotoCarrera());
            btCancelar.Visible = true;
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(Nibi.Negocio.Colecciones.IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Inscritos");
            }
            mColeccion = col as EColeccion<EPilotoCarrera>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EPilotoCarrera.");
            }
            if (pos < 0 || pos >= mColeccion.Count) {
                throw new OverflowException("La posicin no se encuentra entre los valores mnimos y mximos de la coleccin");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Inscritos");
            }
            mColeccion = col as EColeccion<EPilotoCarrera>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EPilotoCarrera.");
            }
            obj = (EPilotoCarrera)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        private void btAgregar_Click(object sender, EventArgs e) {
            ECampeonato clase = cbClase.SelectedItem as ECampeonato;
            if (clase == null) {
                MessageBox.Show("Debe elegir un grupo para agregarlo.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            } else {
                ECampeonatoInscrito cInscrito = obj.CampeonatosInscrito.AddNew();
                cInscrito.Campeonato = clase;
                PropertyException pe = cInscrito.ErrorDePropiedad(ECampeonatoInscrito.NPCampeonato);
                if (pe != null) {
                    MessageBox.Show("Error al agregar. Error original: " + pe.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    obj.CampeonatosInscrito.CancelNew(obj.CampeonatosInscrito.IndexOf(cInscrito));
                } else {
                    DialogResult dr = DialogResult.Yes;
                    if (!cInscrito.CilindradaOK()) {
                        dr = MessageBox.Show("La cilindrada del vehculo no coincide con el intervalo del grupo seleccionado.\nAun as, desea continuar?", "WARNING", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                    }
                    if (dr == DialogResult.Yes) {
                        lbCampeonatosInscritos.Items.Add(cInscrito);
                    } else {
                        obj.CampeonatosInscrito.CancelNew(obj.CampeonatosInscrito.IndexOf(cInscrito));
                    }
                }
            }
        }

        private void btQuitar_Click(object sender, EventArgs e) {
            if (lbCampeonatosInscritos.SelectedIndex >= 0 && lbCampeonatosInscritos.SelectedIndex < lbCampeonatosInscritos.Items.Count) {
                DialogResult dr = MessageBox.Show("Est seguro de quiere eliminar el campeonato seleccionado?", "INFO", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.Yes) {
                    int indice = lbCampeonatosInscritos.SelectedIndex;
                    lbCampeonatosInscritos.Items.RemoveAt(indice);
                    obj.CampeonatosInscrito.RemoveAt(indice);
                }
            } else {
                MessageBox.Show("Debe seleccionar un elemento de la lista.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private ECampeonato campeonatoAnterior = null;
        private void cbCampeonatoPuntuable_SelectionChangeCommitted(object sender, EventArgs e) {
            ECampeonato campeonatoSeleccionado = cbCampeonatoPuntuable.SelectedItem as ECampeonato;
            if (campeonatoSeleccionado != null && !campeonatoSeleccionado.Equals(campeonatoAnterior)) {
                campeonatoAnterior = campeonatoSeleccionado;
                cbGrupo.Items.Clear();
                cbClase.Items.Clear();
                EColeccion<ECampeonato> col = campeonatoSeleccionado.GetGrupos();
                foreach (ECampeonato grupo in col) {
                    cbGrupo.Items.Add(grupo);
                }
            }
        }

        private ECampeonato grupoAnterior = null;
        private void cbGrupo_SelectionChangeCommitted(object sender, EventArgs e) {
            ECampeonato grupoSeleccionado = cbGrupo.SelectedItem as ECampeonato;
            if (grupoSeleccionado != null && !grupoSeleccionado.Equals(grupoAnterior)) {
                grupoAnterior = grupoSeleccionado;
                cbClase.Items.Clear();
                EColeccion<ECampeonato> col = grupoSeleccionado.GetClases();
                foreach (ECampeonato clase in col) {
                    cbClase.Items.Add(clase);
                }
            }
        }
    }
}