using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.IO;
using System.Collections.ObjectModel;
using Nibi.Negocio.Colecciones;

namespace Forms {
    public partial class FLicencia : Form, IFichaEntidad, IPestanha, IFichaEntidadCol {

        private ELicencia obj = null;
        private EColeccion<ELicencia> mColeccion = null;

        #region Constructores

        public FLicencia(){
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }
        public FLicencia(ELicencia lic):this() {
            Cargar(lic);
        }
        public FLicencia(IEColeccion col) {
            CrearNuevo(col);
        }
        public FLicencia(IEColeccion col , int pos) {
            Cargar(col , pos);
        }

        #endregion

        private void FLicencia_Load(object sender, EventArgs e) {
            if(obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if(this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
                //btSalir.Visible = false;
            }
        }

        private void FLicencia_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if(obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Licencia", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if(dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("La licencia se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if(btCancelar.Visible) {//el objeto es nuevo y se agregaran las pestaas oportunas
                            btCancelar.Visible = lblCancelar.Visible = false;
                            if(this.ParentForm != null) {
                                FListados fl = this.ParentForm as FListados;
                                if(fl != null) {
                                    //fl.dgv_SelectionChanged(null, null);
                                    fl.AgregarPestanhas(this.ExtraerPestanhas());
                                }
                            }
                        }
                    }
                }
            } catch(Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            bool activo = obj.Activo;
            obj.Deshacer();
            if (obj.Activo != activo) {
                obj.Activo = activo;
            }
            this.Cargar(obj);
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if (var == null) {
                throw new ArgumentNullException("Licencia");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ELicencia;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicencia.");
            }
            //Rellenar los combobox
            EnumConvert ec = new EnumConvert();
            Collection<KeyValuePair<Enum, string>> ckvp = ec.GetCollectionDescription(typeof(TipoDirigido));
            
            cbTipo.DataSource = ckvp;
            cbTipo.DisplayMember = "Value";
            cbTipo.ValueMember = "Key";

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ELicencia.NPCodigo));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ELicencia.NPNombre));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(ELicencia.NPImporte)) {
                pe = obj.ErrorDePropiedad(ELicencia.NPImporte);
                if (pe != null) {
                    errProv.SetError(txtImporte, pe.Message);
                } else {
                    errProv.SetError(txtImporte, "");
                }
            } else if (e.PropertyName.Equals(ELicencia.NPCodigo)) {
                pe = obj.ErrorDePropiedad(ELicencia.NPCodigo);
                if (pe != null) {
                    errProv.SetError(txtCodigo, pe.Message);
                } else {
                    errProv.SetError(txtCodigo, "");
                }
            } else if (e.PropertyName.Equals(ELicencia.NPNombre)) {
                pe = obj.ErrorDePropiedad(ELicencia.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new ELicencia());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Licencia");
            }
            mColeccion = col as EColeccion<ELicencia>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicencia.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Licencia");
            }
            mColeccion = col as EColeccion<ELicencia>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ELicencia.");
            }
            obj = (ELicencia)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            return null;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion
    }
}