using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.IO;
using System.Collections.ObjectModel;
using Nibi.Negocio.Colecciones;

namespace Forms
{
    public partial class FOficial : Form, IFichaEntidad, IPestanha, IFichaEntidadCol {
        private EOficial obj = null;
        private FColeccion fLicNum = null;
        private EColeccion<EOficial> mColeccion = null;

        #region Constructores

        public FOficial(){
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }
        public FOficial(EOficial ofic):this() {
            Cargar(ofic);
        }
        public FOficial(IEColeccion col) {
            CrearNuevo(col);
        }
        public FOficial(IEColeccion col , int pos) {
            Cargar(col , pos);
        }

        #endregion

        private void FOficial_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            }
            else {
                bindDir.ResetCurrentItem();
                bindOfic.ResetCurrentItem();
            }
            if (this.Parent != null) {
                //btLicOficial.Visible = false;
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FOficial_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btLicOficial_Click(object sender, EventArgs e) {
            fLicNum = new FColeccion(obj);           
            fLicNum.ShowDialog();
        }

        private void btBrowse_Click(object sender, EventArgs e) {
            OpenFileDialog ofdialog = new OpenFileDialog();
            ofdialog.Filter = "archivos de imagen (*.jpg)|*.jpg|Todos los archivos (*.*)|*.*";
            ofdialog.FilterIndex = 1;
            ofdialog.RestoreDirectory = true;
            if (ofdialog.ShowDialog() == DialogResult.OK) {
                try {
                    obj.Foto = Image.FromFile(ofdialog.FileName);
                } catch (Exception ex) {
                    MessageBox.Show("No ha sido posible leer el archivo de disco. Error original:" + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if(obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Oficial", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if(dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("El oficial se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if(btCancelar.Visible) {//el objeto es nuevo
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                    }
                }
            } catch(Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                obj_PropertyChanged(null, new PropertyChangedEventArgs(EOficial.NPNif));
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            obj.Deshacer();
            if (!obj.EsOficial) {
                obj.EsOficial = true;
            }
            this.Cargar(obj);
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if(var == null) {
                throw new ArgumentNullException("Oficial");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bindDir.Clear();
                bindOfic.Clear();
                errProv.Clear();
            }
            obj = var as EOficial;
            if(obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EOficial.");
            }
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bindOfic.Add(obj);
            if (obj.Direccion == null) {
                obj.Direccion = new EDireccion();
            }
            obj.Direccion.PropertyChanged += new PropertyChangedEventHandler(Direccion_PropertyChanged);
            bindDir.Add(obj.Direccion);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EPiloto.NPNif));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EPiloto.NPNombre));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EPiloto.NPApellidos));
        }

        void Direccion_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(EDireccion.NPDomicilio)) {
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPDomicilio);
                if (pe != null) {
                    errProv.SetError(txtDomicilio, pe.Message);
                } else {
                    errProv.SetError(txtDomicilio, "");
                }
            } else if (e.PropertyName.Equals(EDireccion.NPCodigoPostal)) {
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPCodigoPostal);
                if (pe != null) {
                    errProv.SetError(txtCodigoPostal, pe.Message);
                } else {
                    errProv.SetError(txtCodigoPostal, "");
                }
            } else if (e.PropertyName.Equals(EDireccion.NPTelefono)) {
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPTelefono);
                if (pe != null) {
                    errProv.SetError(txtTelefono, pe.Message);
                } else {
                    errProv.SetError(txtTelefono, "");
                }
            } else if (e.PropertyName.Equals(EDireccion.NPMovil)) {
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPMovil);
                if (pe != null) {
                    errProv.SetError(txtMovil, pe.Message);
                } else {
                    errProv.SetError(txtMovil, "");
                }
            } else if (e.PropertyName.Equals(EDireccion.NPFax)) {
                pe = obj.Direccion.ErrorDePropiedad(EDireccion.NPFax);
                if (pe != null) {
                    errProv.SetError(txtFax, pe.Message);
                } else {
                    errProv.SetError(txtFax, "");
                }
            }
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            if (e.PropertyName.Equals(EOficial.NPNif)) {
                EOficial oficialComprobar = new EOficial(txtNif.Text);
                if (oficialComprobar.Existe && !oficialComprobar.Equals(obj)) {
                    if (oficialComprobar.EsOficial) {
                        errProv.SetError(txtNif, "Ya existe un oficial con ese Nif.");
                    } else {
                        errProv.SetError(txtNif, "");
                        if (obj.EsNuevo) {
                            //borrar el objeto nuevo y asignar el pilotoComprobar a esta ficha
                            if (mColeccion != null) {
                                //comprobar antes del cancel new si el objeto tiene referencias
                                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                                mColeccion.Add(oficialComprobar);
                                oficialComprobar.EsOficial = true;
                                Cargar(oficialComprobar);
                            }
                        } else {
                            errProv.SetError(txtNif, "Existe otra persona con ese Nif."); 
                        }
                    }
                } else {
                    errProv.SetError(txtNif, "");
                }
            } else if (e.PropertyName.Equals(EPiloto.NPNombre)) {
                PropertyException pe = obj.ErrorDePropiedad(EPiloto.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            }
        }

        public void CrearNuevo() {
            Cargar(new EOficial());
            btCancelar.Visible = lblCancelar.Visible = true;
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col , int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Oficial");
            }
            mColeccion = col as EColeccion<EOficial>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EOficial.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Oficial");
            }
            mColeccion = col as EColeccion<EOficial>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EOficial.");
            }
            obj = (EOficial)mColeccion.AddNew();
            this.Cargar(obj);
            btCancelar.Visible = true;
        }

        #endregion

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            Collection<Pestanha> colP = new Collection<Pestanha>();
            string titulo = "Licencias";
            FColeccion fLicNum = new FColeccion(TipoColeccion.Licencias);
            fLicNum.TopLevel = false;
            fLicNum.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha(titulo, fLicNum));
            return colP;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion
    }
}