using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using Nibi.Negocio.Colecciones;
using System.Collections.ObjectModel;
using System.IO;

namespace Forms {

    public partial class FRally : Form, IFichaEntidad, IFichaEntidadCol, IPestanha{

        private ERally obj = null;
        private EColeccion<ERally> mColeccion = null;

        #region Constructores

        public FRally(){
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }
        public FRally(EOficial ofic):this() {
            Cargar(ofic);
        }
        public FRally(IEColeccion col) {
            CrearNuevo(col);
        }
        public FRally(IEColeccion col , int pos) {
            Cargar(col , pos);
        }

        #endregion

        private void FRally_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FRally_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (mColeccion != null) {
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
            } 
            if (this.ParentForm != null) {
                FListados fl = this.ParentForm as FListados;
                if (fl != null) {
                    fl.dgv_SelectionChanged(null, null);
                }
            } else {
                this.Close();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Rally", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("El rally ha sido guardado correctamente,\njunto con sus campeonatos a puntuar y sus oficiales.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if (btCancelar.Visible) {//el objeto es nuevo y se agregaran las pestaas oportunas
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            } 
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            bool activo = obj.Activo;
            obj.Deshacer();
            if (obj.Activo != activo) {
                obj.Activo = activo;
            }
            this.Cargar(obj);
        }

        private void btBrowse_Click(object sender, EventArgs e) {
            OpenFileDialog ofdialog = new OpenFileDialog();
            ofdialog.Filter = "archivos de imagen (*.jpg)|*.jpg|Todos los archivos (*.*)|*.*";
            ofdialog.FilterIndex = 1;
            ofdialog.RestoreDirectory = true;
            if (ofdialog.ShowDialog() == DialogResult.OK) {
                try {
                    obj.Logo = Image.FromFile(ofdialog.FileName);
                } catch (Exception ex) {
                    MessageBox.Show("No ha sido posible leer el archivo de disco. Error original:" + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btElegirEscuderia_Click(object sender, EventArgs e) {
            EColeccion<EEscuderia> col = EEscuderia.GetCollection();
            col.ColumnaMostrarRango(new string[] { EEscuderia.NPTipo, EEscuderia.NPNombre });
            FDialogo fd = new FDialogo(col);
            DialogResult dr = fd.ShowDialog();
            if (dr == DialogResult.OK) {
                EEscuderia esc = fd.ExtraerEleccion() as EEscuderia;
                if (esc != null) {
                    obj.EscuderiaOrganiza = esc;
                    txtOrganizador.Text = esc.Nombre;
                    dr = MessageBox.Show("Desea que el logo del evento sea el mismo que el de la escudera elegida?", "", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Logo = esc.Logo;
                    }
                }
            }
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if (var == null) {
                throw new ArgumentNullException("Rally");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ERally;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ERally.");
            }

            //Organizador
            if (obj.EscuderiaOrganiza != null) {
                txtOrganizador.Text = obj.EscuderiaOrganiza.Nombre;
            } else {
                txtOrganizador.Text = obj.Organizador;
            }
            
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ERally.NPAlias));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ERally.NPNombre));
            //BOTN GUARDAR (si est cerrada no debe funcionar)
            if (!obj.EstaCerrada) {
                btGuardar.Click -= btGuardar_Click;
                btGuardar.Click += new EventHandler(btGuardar_Click);
                toolTip.SetToolTip(btGuardar, "");
            } else {
                btGuardar.Click -= btGuardar_Click;
                toolTip.SetToolTip(btGuardar, "No se puede usar por que la carrera ya ha finalizado.");
            }
            //ToolTip
            this.toolTip.SetToolTip(btElegirEscuderia, "Elige una escudera para que sea la organizadora del evento.");
            this.toolTip.SetToolTip(txtOrganizador, "Si escribe, la escudera elegida (si tuviese) se borrar como organizadora.");

        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(ERally.NPNombre)) {
                pe = obj.ErrorDePropiedad(ERally.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            } else if (e.PropertyName.Equals(ERally.NPAlias)) {
                pe = obj.ErrorDePropiedad(ERally.NPAlias);
                if (pe != null) {
                    errProv.SetError(txtAlias, pe.Message);
                } else {
                    errProv.SetError(txtAlias, "");
                }
            }
        }

        public void CrearNuevo() {
            this.Cargar(new ERally());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Rally");
            }
            mColeccion = col as EColeccion<ERally>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ERally.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Rally");
            }
            mColeccion = col as EColeccion<ERally>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ERally.");
            }
            obj = (ERally)mColeccion.AddNew();
            this.Cargar(obj);
            btCancelar.Visible = true;
        }

        #endregion

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            Collection<Pestanha> colP = new Collection<Pestanha>();
            FOficialesCarrera foc = new FOficialesCarrera();
            foc.TopLevel = false;
            foc.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha("Oficiales", foc));
            FColeccion fc = new FColeccion(TipoColeccion.CampeonatosPuntuables);
            fc.TopLevel = false;
            fc.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha("Campeonatos", fc));
            FColeccion fcSec = new FColeccion(TipoColeccion.Sectores);
            fcSec.TopLevel = false;
            fcSec.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha("Sectores", fcSec));
            return colP;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion

        private void txtOrganizador_TextChanged(object sender, EventArgs e) {
            if (obj.EscuderiaOrganiza == null || !obj.EscuderiaOrganiza.Nombre.Equals(txtOrganizador.Text)) {
                obj.Organizador = txtOrganizador.Text;
            }
        }
    }
}