using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using Nibi.Negocio.Colecciones;
using System.Collections.ObjectModel;
using System.IO;

namespace Forms {
    public partial class FSubida : Form, IFichaEntidad, IFichaEntidadCol, IPestanha {

        private ESubida obj = null;
        private EColeccion<ESubida> mColeccion = null;

        #region Constructores

        public FSubida(){
            InitializeComponent();
            errProv.BlinkRate = 200;
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
        }
        public FSubida(EOficial ofic):this() {
            Cargar(ofic);
        }
        public FSubida(IEColeccion col) {
            CrearNuevo(col);
        }
        public FSubida(IEColeccion col , int pos) {
            Cargar(col , pos);
        }

        #endregion

        private void FSubida_Load(object sender, EventArgs e) {
            if(obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if(this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        private void FSubida_FormClosed(object sender, FormClosedEventArgs e) {
            if(obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if(obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Subida", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if(dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("La subida ha sido guardada correctamente,\njunto con sus campeonatos a puntuar y sus oficiales.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if(btCancelar.Visible) {//el objeto es nuevo y se agregaran las pestaas oportunas
                            btCancelar.Visible = lblCancelar.Visible = false;
                        }
                    }
                }
            } catch(Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            } 
        }

        private void btBrowse_Click(object sender, EventArgs e) {
            OpenFileDialog ofdialog = new OpenFileDialog();
            ofdialog.Filter = "archivos de imagen (*.jpg)|*.jpg|Todos los archivos (*.*)|*.*";
            ofdialog.FilterIndex = 1;
            ofdialog.RestoreDirectory = true;
            if (ofdialog.ShowDialog() == DialogResult.OK) {
                try {
                    obj.Logo = Image.FromFile(ofdialog.FileName);
                } catch (Exception ex) {
                    MessageBox.Show("No ha sido posible leer el archivo de disco. Error original:" + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        
        private void btDeshacer_Click(object sender, EventArgs e) {
            bool activo = obj.Activo;
            obj.Deshacer();
            if (obj.Activo != activo) {
                obj.Activo = activo;
            }
            this.Cargar(obj);
        }

        private void btElegirEscuderia_Click(object sender, EventArgs e) {
            EColeccion<EEscuderia> col = EEscuderia.GetCollection();
            col.ColumnaMostrarRango(new string[] {EEscuderia.NPTipo, EEscuderia.NPNombre });
            FDialogo fd = new FDialogo(col);
            DialogResult dr = fd.ShowDialog();
            if(dr == DialogResult.OK) {
                EEscuderia esc = fd.ExtraerEleccion() as EEscuderia;
                if(esc != null) {
                    obj.EscuderiaOrganiza = esc;
                    txtOrganizador.Text = esc.Nombre;
                    dr = MessageBox.Show("Desea que el logo del evento sea el mismo que el de la escudera elegida?", "", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Logo = esc.Logo;
                    }
                }
            }
        }

        private void txtOrganizador_TextChanged(object sender, EventArgs e) {
            if (obj.EscuderiaOrganiza == null || !obj.EscuderiaOrganiza.Nombre.Equals(txtOrganizador.Text)) {
                obj.Organizador = txtOrganizador.Text;
            }
        }

        private void cbFases_SelectionChangeCommitted(object sender, EventArgs e) {
            obj.NumeroFasesDeseado = cbFases.SelectedIndex + 1;
        }

        #region IFichaEntidad

        public void Cargar(object var) {
            if(var == null) {
                throw new ArgumentNullException("Subida");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = var as ESubida;
            if(obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ESubida.");
            }

            //Organizador
            if(obj.EscuderiaOrganiza != null) {
                txtOrganizador.Text = obj.EscuderiaOrganiza.Nombre;
            } else {
                txtOrganizador.Text = obj.Organizador;
            }

            //numero de fases
            cbFases.SelectedIndex = obj.NumeroFasesActual - 1;

            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESubida.NPAlias));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(ESubida.NPNombre));
            //BOTN GUARDAR (si est cerrada no debe funcionar)
            if (!obj.EstaCerrada) {
                btGuardar.Click -= btGuardar_Click;
                btGuardar.Click += new EventHandler(btGuardar_Click);
                toolTip.SetToolTip(btGuardar, "");
            } else {
                btGuardar.Click -= btGuardar_Click;
                toolTip.SetToolTip(btGuardar, "No se puede usar por que la carrera ya ha finalizado.");
            }
            //ToolTip
            this.toolTip.SetToolTip(btElegirEscuderia, "Elige una escudera para que sea la organizadora del evento.");
            this.toolTip.SetToolTip(txtOrganizador, "Si escribe, la escudera elegida (si tuviese) se borrar como organizadora.");
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(ESubida.NPNombre)) {
                pe = obj.ErrorDePropiedad(ESubida.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            } else if (e.PropertyName.Equals(ESubida.NPAlias)) {
                pe = obj.ErrorDePropiedad(ESubida.NPAlias);
                if (pe != null) {
                    errProv.SetError(txtAlias, pe.Message);
                } else {
                    errProv.SetError(txtAlias, "");
                }
            }
        }

        public void CrearNuevo() {
            this.Cargar(new ESubida());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(IColeccionVista col, int pos) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Subida");
            }
            mColeccion = col as EColeccion<ESubida>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ESubida.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if(col == null) {
                throw new ArgumentNullException("Coleccin Subida");
            }
            mColeccion = col as EColeccion<ESubida>;
            if(mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ESubida.");
            }
            obj = (ESubida)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            Collection<Pestanha> colP = new Collection<Pestanha>();
            FOficialesCarrera foc = new FOficialesCarrera();
            foc.TopLevel = false;
            foc.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha("Oficiales", foc));
            FColeccion fc = new FColeccion(TipoColeccion.CampeonatosPuntuables);
            fc.TopLevel = false;
            fc.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha("Campeonatos", fc));
            return colP;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion
    }
}