using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FUsuario : Form, IFichaEntidad, IFichaEntidadCol, IPestanha {

        private EUsuario obj = null;
        private EColeccion<EUsuario> mColeccion = null;

        public FUsuario() {
            InitializeComponent();
            errProv.BlinkStyle = ErrorBlinkStyle.AlwaysBlink;
            errProv.BlinkRate = 200;
        }

        private void FUsuario_Load(object sender, EventArgs e) {
            if (obj == null) {
                CrearNuevo();
            } else {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
                lblSalir.Visible = false;
                btSalir.Visible = false;
                lblNota.Visible = false;
            }
            if (EUsuario.Actual != null) {
                lblNota.Visible = false;
            }
        }

        private void btSalir_Click(object sender, EventArgs e) {
            if (obj.EstaSucio) {
                DialogResult dr = MessageBox.Show("Quiere salir sin guardar?", "Salir", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.Yes) {
                    if (mColeccion != null && mColeccion.Count > 0) {
                        if (mColeccion.Count == 1 && mColeccion[0].EsNuevo) {
                            Application.Exit();
                        }
                        dr = DialogResult.Cancel;
                        this.Close();
                    } else {
                        this.Close();
                    }
                }
            } else {
                dr = DialogResult.Cancel;
                this.Close();
            }
        }

        private void btAceptar_Click(object sender, EventArgs e) {
            try {
                if (String.IsNullOrEmpty(txtPass.Text)) {
                    MessageBox.Show("Tiene que escribir un password.", "WARNING", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                } else if (String.IsNullOrEmpty(txtRepitePass.Text) || !txtRepitePass.Text.Equals(txtPass.Text)) {
                    MessageBox.Show("El password no lo ha repetido correctamente.", "WARNING", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                } else if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Usuario", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("El usuario se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if (btCancelar.Visible) {
                            btCancelar.Visible = false;
                            if (this.ParentForm != null) {
                                FListados fl = this.ParentForm as FListados;
                                if (fl != null) {
                                    fl.AgregarPestanhas(this.ExtraerPestanhas());
                                }
                            } else {
                                //estamos en la ventana de nuevo usuario as que cerramos.
                                dr = DialogResult.OK;
                                this.Close();
                            }
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            obj.Deshacer();
            Cargar(obj);
        }

        private void btCancelar_Click(object sender, EventArgs e) {
            //El botn cancelar solo se ve cuando es nuevo.
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("Usuario");
            }
            if (obj != null) {
                //Limpiamos el/los bindingsource por si se cambia de objeto.
                bind.Clear();
                errProv.Clear();
            }
            obj = entidad as EUsuario;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EUsuario.");
            }
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EUsuario.NPNick));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EUsuario.NPNif));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EUsuario.NPNombre));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EUsuario.NPApellidos));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EUsuario.NPEmail));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EUsuario.NPContrasenha));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            PropertyException pe = null;
            if (e.PropertyName.Equals(EUsuario.NPNick)) {
                pe = obj.ErrorDePropiedad(EUsuario.NPNick);
                if (pe != null) {
                    errProv2.SetError(txtNick, pe.Message);
                } else {
                    errProv2.SetError(txtNick, "");
                }
            } else if (e.PropertyName.Equals(EUsuario.NPNif)) {
                EUsuario userComprobar = new EUsuario(txtNif.Text);
                if (userComprobar.Existe && !userComprobar.Equals(obj)) {
                    if (userComprobar.EsUsuario) {
                        errProv.SetError(txtNif, "Ya existe un usuario con ese Nif.");
                    } else if (this.ParentForm != null){
                        errProv.SetError(txtNif, "");
                        if (obj.EsNuevo) {
                            //borrar el objeto nuevo y asignar el pilotoComprobar a esta ficha
                            if (mColeccion != null) {
                                //comprobar antes del cancel new si el objeto tiene referencias
                                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                                mColeccion.Add(userComprobar);
                                userComprobar.EsUsuario = true;
                                Cargar(userComprobar);
                            }
                        } else {
                            errProv.SetError(txtNif, "Ya existe un usuario con ese Nif.");
                            //TODO: fusionar
                        }
                    } else {
                        errProv.SetError(txtNif, "Ya hay una persona con este NIF. Pruebe otro distinto.");
                    }
                } else {
                    errProv.SetError(txtNif, "");
                }
            } else if (e.PropertyName.Equals(EUsuario.NPNombre)) {
                pe = obj.ErrorDePropiedad(EUsuario.NPNombre);
                if (pe != null) {
                    errProv.SetError(txtNombre, pe.Message);
                } else {
                    errProv.SetError(txtNombre, "");
                }
            } else if (e.PropertyName.Equals(EUsuario.NPEmail)) {
                pe = obj.ErrorDePropiedad(EUsuario.NPEmail);
                if (pe != null) {
                    errProv.SetError(txtEmail, pe.Message);
                } else {
                    errProv.SetError(txtEmail, "");
                }
            } else if (e.PropertyName.Equals(EUsuario.NPContrasenha)) {
                pe = obj.ErrorDePropiedad(EUsuario.NPContrasenha);
                if (pe != null) {
                    errProv.SetError(txtPass, pe.Message);
                } else {
                    errProv.SetError(txtPass, "");
                }
            } else if (e.PropertyName.Equals(EUsuario.NPApellidos)) {
                pe = obj.ErrorDePropiedad(EUsuario.NPApellidos);
                if (pe != null) {
                    errProv2.SetError(txtApellidos, pe.Message);
                } else {
                    errProv2.SetError(txtApellidos, "");
                }
            } 
        }

        public void CrearNuevo() {
            Cargar(new EUsuario());
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(Nibi.Negocio.Colecciones.IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Usuario");
            }
            mColeccion = col as EColeccion<EUsuario>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EUsuario.");
            }
            if (pos < 0 || pos >= mColeccion.Count) {
                throw new OverflowException("La posicin no se encuentra entre los valores mnimos y mximos de la coleccin");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Usuario");
            }
            mColeccion = col as EColeccion<EUsuario>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EUsuario.");
            }
            obj = (EUsuario)mColeccion.AddNew();
            this.Cargar(obj);
        }

        #endregion

        #region IPestanha

        public System.Collections.ObjectModel.Collection<Pestanha> ExtraerPestanhas() {
            Collection<Pestanha> colP = new Collection<Pestanha>();
            string titulo = "Permisos";
            FColeccion fLicNum = new FColeccion(TipoColeccion.Permisos);
            fLicNum.TopLevel = false;
            fLicNum.FormBorderStyle = FormBorderStyle.None;
            colP.Add(new Pestanha(titulo, fLicNum));
            return colP;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion

        private void txtPass_Leave(object sender, EventArgs e) {
            obj.Contrasenha = txtPass.Text;
        }

        private DialogResult dr;
        public DialogResult GetDialogResult() {
            return dr;
        }

        private void FUsuario_FormClosed(object sender, FormClosedEventArgs e) {
            if (obj != null && obj.EstaSucio) {
                obj.Deshacer();
            }
        }
    }
}