using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FMuestraCampeonato : Form, IFichaEntidad, IPestanha {

        private ECampeonato obj = null;
        private Collection<KeyValuePair<ECampeonato, EColeccion<ECampeonato>>> mKvpGrupoClases = null;

        public FMuestraCampeonato() {
            InitializeComponent();
            mKvpGrupoClases = new Collection<KeyValuePair<ECampeonato, EColeccion<ECampeonato>>>();
            lbClases.SelectedValueChanged += new EventHandler(lbClases_SelectedValueChanged);
        }

        private void FMuestraCampeonato_Load(object sender, EventArgs e) {
            if (obj != null) {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            Collection<Pestanha> col = new Collection<Pestanha>();
            FClasificacion fclasificacion = new FClasificacion(this.obj);
            fclasificacion.FormBorderStyle = FormBorderStyle.None;
            fclasificacion.TopLevel = false;
            col.Add(new Pestanha("Clasificacion general", fclasificacion));
            return col;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("Campeonato.");
            }
            if (obj != null) {//Este form ya estaba en ejecucin
                bind.Clear();
                bindClase.Clear();
            }
            obj = entidad as ECampeonato;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo EParticipante.");
            }
            bind.Add(obj);
            lbClases.DataSource = null;
            EColeccion<ECampeonato> mColeccionGrupos = obj.GetGrupos();
            lbGrupos.SelectedValueChanged += new EventHandler(lbGrupos_SelectedValueChanged);
            lbGrupos.DataSource = mColeccionGrupos;
            this.lbGrupos_SelectedValueChanged(null, null);
        }
        
        /// <summary>
        /// Este mtodo no se implementa.
        /// </summary>
        public void CrearNuevo() {
            throw new Exception("Este mtodo no se implementa.");
        }

        #endregion

        void lbGrupos_SelectedValueChanged(object sender, EventArgs e) {
            bindClase.Clear();
            ECampeonato grupo = lbGrupos.SelectedValue as ECampeonato;
            if (grupo != null) {
                bool encontrado = false;
                int indice = 0;
                while (!encontrado && indice < mKvpGrupoClases.Count) {
                    KeyValuePair<ECampeonato, EColeccion<ECampeonato>> kvp = mKvpGrupoClases[indice];
                    if (grupo.Equals(kvp.Key)) {
                        encontrado = true;
                        lbClases.DataSource = kvp.Value;
                        lbClases.SelectedIndex = -1;
                    } else {
                        indice = indice + 1;
                    }
                }
                if (!encontrado) {
                    EColeccion<ECampeonato> mClases = grupo.GetClases();
                    lbClases.DataSource = mClases;
                    lbClases.SelectedIndex = -1;
                    mKvpGrupoClases.Add(new KeyValuePair<ECampeonato,EColeccion<ECampeonato>>(grupo, mClases));
                }
            }
        }

        void lbClases_SelectedValueChanged(object sender, EventArgs e) {
            bindClase.Clear();
            ECampeonato clase = lbClases.SelectedValue as ECampeonato;
            if (clase != null) {
                bindClase.Add(clase);
            }
        }
    }
}