using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FMuestraCarrera : Form, IFichaEntidad, IPestanha, IFichaMultiEntidad {

        private ECarrera obj = null;

        public FMuestraCarrera() {
            InitializeComponent();
        }

        private void FMuestraCarrera_Load(object sender, EventArgs e) {
            if (obj != null) {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("Carrera.");
            }
            if (obj != null) {//Este form ya estaba en ejecucin
                bind.Clear();
            }
            obj = entidad as ECarrera;
            if (obj == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo ECarrera.");
            }
            bind.Add(obj);
        }

        /// <summary>
        /// Este mtodo no se implementa para este form.
        /// </summary>
        public void CrearNuevo() {
            throw new Exception("No se implementa para este form.");
        }

        #endregion

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            Collection<Pestanha> col = new Collection<Pestanha>();
            FColeccion fcInscritos = new FColeccion(TipoColeccion.Inscripciones);
            fcInscritos.FormBorderStyle = FormBorderStyle.None;
            fcInscritos.TopLevel = false;
            col.Add(new Pestanha("Inscritos", fcInscritos));
            FClasificacion fClasificacion = new FClasificacion();
            fClasificacion.TopLevel = false;
            fClasificacion.FormBorderStyle = FormBorderStyle.None;
            col.Add(new Pestanha("Clasificacin", fClasificacion));
            FGaleria fgaleria = new FGaleria();
            fgaleria.TopLevel = false;
            fgaleria.FormBorderStyle = FormBorderStyle.None;
            col.Add(new Pestanha("Imgenes", fgaleria));
            return col;
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #region IFichaMultiEntidad

        /// <summary>
        /// Este mtodo no se implementa en este form.
        /// </summary>
        /// <param name="tipoEntidad"></param>
        public void CrearNuevo(Type tipoEntidad) {
            throw new Exception("Este mtodo no se implementa");
        }

        #endregion

        #endregion
        
        #region IFichaMultiEntidad

        /// <summary>
        /// Este mtodo no se implementa en este form.
        /// </summary>
        /// <param name="tipoEntidad"></param>
        void IFichaMultiEntidad.CrearNuevo(Type tipoEntidad) {
            throw new Exception("Este mtodo no se implementa en este form.");
        }

        #endregion
    }
}