using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;

namespace Forms {
    public partial class FMuestraInscrito : Form, IFichaEntidad {

        EPilotoCarrera obj = null;

        public FMuestraInscrito() {
            InitializeComponent();
        }

        private void FMuestraInscrito_Load(object sender, EventArgs e) {
            if (obj != null) {
                bind.ResetCurrentItem();
            }
            if (this.Parent != null) {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
                this.BackColor = Color.Transparent;
            }
        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                obj = null;
                bind.Clear();
                arbolClasificacion.Nodes.Clear();
                arbolSanciones.Nodes.Clear();
            } else {
                if (obj != null) {//el form ya estaba creado.
                    bind.Clear();
                }
                obj = entidad as EPilotoCarrera;
                if (obj == null) {
                    throw new InvalidCastException("La entidad debe ser del tipo EPilotoCarrera");
                }
                bind.Add(obj);
                lblTituloCopiloto.Visible = obj.Copiloto != null;
                arbolClasificacion.Nodes.Clear();
                arbolSanciones.Nodes.Clear();
                if (obj.Estado == TipoEstado.Autorizado) {
                    ColocarClasificaciones();
                    ColocarSanciones();
                }
            }
        }

        private void ColocarSanciones() {
            EColeccion<ESancion> colSanciones = obj.Sanciones;
            foreach (ESancion sancion in colSanciones) {
                TreeNode nRaiz = new TreeNode();
                string titulo;
                if (sancion.Cronometrable != null) {
                    titulo = sancion.Cronometrable.ToString() + ". Motivo: " + sancion.Motivo;
                } else {//si cronometrable == null es por que la sancin es para el rally completo.
                    titulo = "Carrera completa. Motivo: " + sancion.Motivo;
                }
                nRaiz.Text = titulo;
                TreeNode nodo = new TreeNode();
                if (sancion.Importe > 0) {
                    nodo.Text = "Importe: " + sancion.Importe.ToString() + "; ";
                }
                switch (sancion.TipoSancion) {
                    case TipoSancion.Descalificacion:
                        nodo.Text = nodo.Text + "Descalificado.";
                        break;
                    case TipoSancion.Temporal:
                        nodo.Text = nodo.Text + " Tiempo: " + CToString.ToString(sancion.Tiempo.Value);
                        break;
                    default:
                        break;
                }
                nRaiz.Nodes.Add(nodo);
                arbolSanciones.Nodes.Add(nRaiz);
            }
        }

        private void ColocarClasificaciones() {
            EColeccion<ECampeonatoInscrito> colCI = obj.CampeonatosInscrito;
            ECarrera carrera = obj.Carrera;
            EColeccion<EClasificacionCarrera> colClasificaciones = obj.Clasificaciones;
            if (carrera.Tipo == TipoCarrera.Subida) {
                foreach (EClasificacionCarrera clasificacion in colClasificaciones) {
                    //Nodo Raiz
                    string titulo;
                    TreeNode nRaiz = new TreeNode();
                    if (clasificacion.Abandono) {
                        titulo = clasificacion.NombreClasificacion + " - ABANDONO. Motivo: " + clasificacion.MotivoAbandono;
                    } else {
                        titulo = clasificacion.NombreClasificacion + " - Puesto: " + clasificacion.Puesto.ToString() + ". Tiempo: " + clasificacion.TiempoEfectivoToString;
                        //ClasificacionesCampeonato
                        foreach (ECampeonatoInscrito ci in colCI) {
                            EClasificacionCarreraCampeonato clasificacionCarreraCampeonato = clasificacion.GetClasificacionCarreraCampeonato(ci.Campeonato);
                            string cadenaNodo = ci.ToString() + ". Puesto: " + clasificacionCarreraCampeonato.Puesto.ToString();
                            nRaiz.Nodes.Add(cadenaNodo);
                        }
                    }
                    nRaiz.Text = titulo;
                    arbolClasificacion.Nodes.Add(nRaiz);
                }
            } else if (carrera.Tipo == TipoCarrera.Rally){
                if (colClasificaciones.Count > 0) {//primero la clasificacion general.
                    TreeNode nTotal = new TreeNode();
                    string titulo = null;
                    if (colClasificaciones[0].Abandono) {
                        titulo = colClasificaciones[0].NombreClasificacion + " - ABANDONO. Motivo: " + colClasificaciones[0].MotivoAbandono;
                    } else {
                        titulo = colClasificaciones[0].NombreClasificacion + " - Puesto: " + colClasificaciones[0].Puesto.ToString() + ". Tiempo: " + colClasificaciones[0].TiempoEfectivoToString;
                    }
                    nTotal.Text = titulo;
                    //campeonatos inscrito de la general.
                    foreach (ECampeonatoInscrito ci in colCI) {
                        EClasificacionCarreraCampeonato clasificacionCarreraCampeonato = colClasificaciones[0].GetClasificacionCarreraCampeonato(ci.Campeonato);
                        string cadenaNodo = ci.ToString() + ". Puesto: " + clasificacionCarreraCampeonato.Puesto.ToString();
                        nTotal.Nodes.Add(cadenaNodo);
                    }
                    arbolClasificacion.Nodes.Add(nTotal);
                }
                EColeccion<ETiempoPilotoSector> colTiemposSector = obj.TiemposPilotoSector;
                foreach (ETiempoPilotoSector tiemposector in colTiemposSector) {
                    ESector sector = new ESector(tiemposector.Cronometrable.Id);
                    TreeNode nSector = new TreeNode(sector.ToString() + " - Tiempo: " + CToString.ToString(tiemposector.Tiempo));
                    ETramo tramo = sector.Tramo;
                    if (tramo != null) {
                        EClasificacionCarrera clasifTramo = tramo.GetClasificacion(obj);
                        TreeNode nTramo = new TreeNode();
                        string texto = null;
                        if (clasifTramo.Abandono) {
                            texto = clasifTramo.NombreClasificacion + " - ABANDONO. Motivo: " + clasifTramo.MotivoAbandono;
                        } else {
                            texto = clasifTramo.NombreClasificacion + " - Puesto: " + clasifTramo.Puesto.ToString() + ". Tiempo: " + clasifTramo.TiempoEfectivoToString;
                            //Clasificaciones Campeonato
                            foreach (ECampeonatoInscrito ci in colCI) {
                                EClasificacionCarreraCampeonato clasificacionCarreraCampeonato = clasifTramo.GetClasificacionCarreraCampeonato(ci.Campeonato);
                                string cadenaNodo = ci.ToString() + ". Puesto: " + clasificacionCarreraCampeonato.Puesto.ToString();
                                nTramo.Nodes.Add(cadenaNodo);
                            }
                        }
                        nTramo.Text = texto;
                        nSector.Nodes.Add(nTramo);
                    }
                    arbolClasificacion.Nodes.Add(nSector);
                }
            }
        }

        /// <summary>
        /// No se implementa.
        /// </summary>
        public void CrearNuevo() {
            throw new Exception("Este mtodo no se implementa");
        }

        #endregion
    }
}