using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;

namespace Forms {
    public partial class FAsistenciaBriefing : Form, IFichaEntidad {

        private EDocListaBriefing obj;
        private EColeccion<EPilotoCarrera> mColeccionNoPresentados = null;

        public FAsistenciaBriefing() {
            InitializeComponent();
        }

        private void btAgregar_Click(object sender, EventArgs e) {
            if (lvNoPresentados.SelectedIndices[0] >= 0 && lvNoPresentados.SelectedIndices[0] < mColeccionNoPresentados.Count) {
                EAsistenciaBriefing nuevo = obj.AsistenciasBriefing.AddNew();
                nuevo.PilotoCarrera = mColeccionNoPresentados[lvNoPresentados.SelectedIndices[0]];
                mColeccionNoPresentados.Remove(nuevo.PilotoCarrera);
                lvNoPresentados.Items.RemoveAt(lvNoPresentados.SelectedIndices[0]);
                MostrarEnLista(lvPresentados, nuevo.PilotoCarrera);
            } else {
                MessageBox.Show("Debe elegir un inscrito de la lista de no presentados.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void btQuitar_Click(object sender, EventArgs e) {
            if (lvPresentados.SelectedIndices[0] >= 0 && lvPresentados.SelectedIndices[0] < obj.AsistenciasBriefing.Count) {
                DialogResult dr = MessageBox.Show("Est seguro de quitar el inscrito de la lista de presentados?", "Quitar Inscrito", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.Yes) {
                    EAsistenciaBriefing viejo = obj.AsistenciasBriefing[lvPresentados.SelectedIndices[0]];
                    lvPresentados.Items.RemoveAt(lvPresentados.SelectedIndices[0]);
                    MostrarEnLista(lvNoPresentados, viejo.PilotoCarrera);
                    mColeccionNoPresentados.Add(viejo.PilotoCarrera);
                    obj.AsistenciasBriefing.Remove(viejo);
                }
            } else {
                MessageBox.Show("Debe elegir un inscrito de la lista de presentados.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void MostrarEnLista(ListView lista, EPilotoCarrera inscrito) {
            string cadenaNumero = inscrito.Dorsal.ToString();
            string cadenaConcursante = "";
            string cadenaInscrito = "";
            if (inscrito.Piloto.Escuderia != null) {
                cadenaConcursante = inscrito.Piloto.Escuderia.ToString();
            }
            cadenaInscrito = inscrito.Piloto.ToString();
            if (inscrito.Copiloto != null) {
                cadenaInscrito = cadenaInscrito + " ; " + inscrito.Copiloto.ToString();
            }
            lista.Items.Add(new ListViewItem(new string[] { cadenaNumero, cadenaConcursante, cadenaInscrito}));
        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("Documento Lista Briefing.");
            }
            obj = entidad as EDocListaBriefing;
            if (obj == null) {
                throw new InvalidCastException("La entidad debe ser de tipo Documento Lista Briefing.");
            }
            //Rellenar los listbox
            lvNoPresentados.Items.Clear();
            lvPresentados.Items.Clear();
            this.mColeccionNoPresentados = obj.GetNoPresentadosAlBriefing();
            foreach (EPilotoCarrera inscrito in mColeccionNoPresentados) {
                MostrarEnLista(lvNoPresentados, inscrito);
            }
            EColeccion<EAsistenciaBriefing> colPresentados = obj.AsistenciasBriefing;
            foreach (EAsistenciaBriefing asistencia in colPresentados) {
                if (asistencia.PilotoCarrera != null) {
                    MostrarEnLista(lvPresentados, asistencia.PilotoCarrera);
                }
            }
        }

        /// <summary>
        /// No se implementa.
        /// </summary>
        public void CrearNuevo() {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}