using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FDocumento : Form, IFichaEntidad, IFichaEntidadCol, IFichaMultiEntidad {

        private EDocumento obj = null;
        private EColeccion<EDocumento> mColeccion = null;
        private Form fDetalleDocumento = null;

        public FDocumento() {
            InitializeComponent();
        }

        private void FDocumentos_Load(object sender, EventArgs e) {

        }

        #region IFichaEntidad

        public void Cargar(object entidad) {
            if (entidad == null) {
                throw new ArgumentNullException("Documento.");
            }
            if (obj != null) {
                bind.Clear();
                errProv.Clear();
            }
            obj = entidad as EDocumento;
            if (obj == null) {
                throw new InvalidCastException("La entidad debe ser de tipo Documento.");
            }
            obj.PropertyChanged += new PropertyChangedEventHandler(obj_PropertyChanged);
            EnumConvert ec = new EnumConvert();
            //Emisor
            Collection<KeyValuePair<Enum, string>> colkvpEmisor = ec.GetCollectionDescription(typeof(TipoCargoOficial));
            cbEmisor.DataSource = colkvpEmisor;
            cbEmisor.ValueMember = "Key";
            cbEmisor.DisplayMember = "Value";
            //Receptores
            Collection<KeyValuePair<Enum, string>> colkvpReceptor = ec.GetCollectionDescription(typeof(TipoCargoOficial));
            cbReceptor.DataSource = colkvpReceptor;
            cbReceptor.ValueMember = "Key";
            cbReceptor.DisplayMember = "Value";
            //agregar los ya asignados.
            lbReceptores.Items.Clear();
            foreach (EReceptor receptor in obj.Receptores) {
                lbReceptores.Items.Add(receptor);
            }
            //
            bind.Add(obj);
            btCancelar.Visible = lblCancelar.Visible = obj.EsNuevo;
            //Establecer panel.
            Form fdoc = null;
            switch (obj.TipoDocumento) {
                //case TipoDocumento.VerificacionesTecnicas:
                //    break;
                //case TipoDocumento.ListaAutorizados:
                //    break;
                case TipoDocumento.AsistenciaBriefing:
                    fdoc = new FAsistenciaBriefing();
                    break;
                case TipoDocumento.InformeFinal:
                    fdoc = new FDocInformeFinal();
                    break;
                default:
                    throw new InvalidCastException("El tipo de documento no es correcto.");
            }
            if (fDetalleDocumento == null || !fDetalleDocumento.Name.Equals(fdoc)) {
                pDocumentoDetalle.Controls.Clear();
                fDetalleDocumento = fdoc;
                fDetalleDocumento.FormBorderStyle = FormBorderStyle.None;
                fDetalleDocumento.TopLevel = false;
                //fDetalleDocumento.Location = pDocumentoDetalle.Location;
                fDetalleDocumento.AutoScroll = true;
                pDocumentoDetalle.Controls.Add(fDetalleDocumento);
                fDetalleDocumento.Show();
            }
            IFichaEntidad ificha = fDetalleDocumento as IFichaEntidad;
            ificha.Cargar(obj);
            //Tipo documento (label)
            lblTipoDocumento.Text = ec.GetDescription(obj.TipoDocumento);
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EDocumento.NPCodigo));
            this.obj_PropertyChanged(null, new PropertyChangedEventArgs(EDocumento.NPFecha));
        }

        void obj_PropertyChanged(object sender, PropertyChangedEventArgs e) {
            if (e.PropertyName.Equals(EDocumento.NPCodigo)) {
                PropertyException pe = obj.ErrorDePropiedad(EDocumento.NPCodigo);
                if (pe != null) {
                    errProv.SetError(txtCodigo, pe.Message);
                } else {
                    errProv.SetError(txtCodigo, "");
                }
            } else if (e.PropertyName.Equals(EDocumento.NPFecha)) {
                PropertyException pe = obj.ErrorDePropiedad(EDocumento.NPFecha);
                if (pe != null) {
                    errProv.SetError(dtpFecha, pe.Message);
                } else {
                    errProv.SetError(dtpFecha, "");
                }
            }
        }

        public void CrearNuevo() {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        #region IFichaEntidadCol

        public void Cargar(Nibi.Negocio.Colecciones.IColeccionVista col, int pos) {
            if (col == null) {
                throw new ArgumentNullException("Coleccin Documentos");
            }
            mColeccion = col as EColeccion<EDocumento>;
            if (mColeccion == null) {
                throw new InvalidCastException("La entidad de este formulario debe ser de tipo Documento.");
            }
            if (pos < 0 || pos >= mColeccion.Count) {
                throw new OverflowException("La posicin no se encuentra entre los valores mnimos y mximos de la coleccin.");
            }
            this.Cargar(mColeccion[pos]);
        }

        public void CrearNuevo(IEColeccion col) {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        private void btCancelar_Click(object sender, EventArgs e) {
            if (!obj.EsNuevo) {
                if (mColeccion != null) {
                    mColeccion.Remove(obj);
                }
                obj = null;
            } else if (mColeccion != null) {
                int anterior = mColeccion.Count;
                mColeccion.CancelNew(mColeccion.IndexOf(obj));
                if (anterior == mColeccion.Count) {
                    mColeccion.Remove(obj);
                }
            }
            if (this.ParentForm != null) {
                if (mColeccion != null && mColeccion.Count == 0) {
                    this.Close();
                } else {
                    FListados fl = this.ParentForm as FListados;
                    if (fl != null) {
                        fl.dgv_SelectionChanged(null, null);
                    }
                }
            } else {
                this.Close();
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (obj.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar los datos modificados?", "Guardar Documento", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Guardar();
                        MessageBox.Show("El documento se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        btCancelar.Visible = lblCancelar.Visible = false;
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btDeshacer_Click(object sender, EventArgs e) {
            if (obj != null) {
                ECarrera carrera = null;
                if (obj.EsNuevo) {
                    carrera = obj.Carrera;
                }
                obj.Deshacer();
                if (obj.EsNuevo) {
                    obj.Carrera = carrera;
                }
                obj.Cargar(obj);
            }
        }

        private void btImprimir_Click(object sender, EventArgs e) {
            FVistaInforme fv = null;
            switch (obj.TipoDocumento) {
                //case TipoDocumento.VerificacionesTecnicas:
                //    break;
                //case TipoDocumento.ListaAutorizados:
                //    break;
                case TipoDocumento.AsistenciaBriefing:
                    EDocListaBriefing docAsistenciaBriefing = obj as EDocListaBriefing;
                    fv = new FVistaInforme("Forms.Informes.RAsistenciaBriefing.rdlc", docAsistenciaBriefing.AsistenciasBriefing);
                    break;
                case TipoDocumento.InformeFinal:
                    EDocInformeFinal docInformefinal = obj as EDocInformeFinal;
                    fv = new FVistaInforme("Forms.Informes.RInformeFinal.rdlc", docInformefinal);
                    break;
            }
            fv.Show();
        }

        private void btAgregar_Click(object sender, EventArgs e) {
            try {
                if (!obj.TieneReceptor((TipoCargoOficial)cbReceptor.SelectedValue)) {
                    EReceptor receptor = obj.Receptores.AddNew();
                    receptor.Receptor = (TipoCargoOficial)cbReceptor.SelectedValue;
                    lbReceptores.Items.Add(receptor);                    
                } else {
                    MessageBox.Show("Ya tiene el receptor " + cbReceptor.SelectedText + ".\nPara agregar, seleccione otro cargo.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btQuitar_Click(object sender, EventArgs e) {
            try {
                if (lbReceptores.SelectedIndex >= 0 && lbReceptores.SelectedIndex < lbReceptores.Items.Count) {
                    DialogResult dr = MessageBox.Show("Esta seguro de que quiere quitar ese cargo de oficial?", "Quitar", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        obj.Receptores.RemoveAt(lbReceptores.SelectedIndex);
                        lbReceptores.Items.RemoveAt(lbReceptores.SelectedIndex);
                    }
                } else {
                    MessageBox.Show("Debe seleccionar una fila para quitar.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #region IFichaMultiEntidad

        /// <summary>
        /// Este mtodo no se implementa.
        /// </summary>
        /// <param name="tipoEntidad"></param>
        public void CrearNuevo(Type tipoEntidad) {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}