using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;

namespace Forms {
    public partial class FAcceso : Form {

        private EColeccion<EUsuario> mColeccion = null;
        private FPrincipal fPrincipal = null;

        public FAcceso() {
            InitializeComponent();
        }

        public FAcceso(EColeccion<EUsuario> col) : this() {
            mColeccion = col;
        }

        private void FAcceso_Load(object sender, EventArgs e) {
            //Preguntar si es la primera vez que se accede al sistema.
            if (mColeccion == null) {
                DialogResult dr = MessageBox.Show("Error al capturar datos.", "ERROR", MessageBoxButtons.RetryCancel, MessageBoxIcon.Warning);
                while (dr == DialogResult.Retry) {
                    try {
                        mColeccion = EUsuario.GetCollection();
                    } catch (Exception ex) {
                        MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        Application.Exit();
                    }
                    if (mColeccion != null) {
                        dr = DialogResult.OK;
                    } else {
                        dr = MessageBox.Show("Error al capturar datos.", "ERROR", MessageBoxButtons.RetryCancel, MessageBoxIcon.Warning);
                    }
                }
                if (dr == DialogResult.Cancel) {
                    Application.Exit();
                }
            }else if (mColeccion.Count == 0) {
                FUsuario fusuario = FabrFAutomovilismo.Actual.ObtenerFNuevo(mColeccion) as FUsuario;
                if (fusuario != null) {
                    MessageBox.Show("No hay ningn usuario en el sistema, se crear el primero con privilegios de administrador.", "MENSAJE DE BIENVENIDA", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    DialogResult dr = fusuario.ShowDialog();
                    dr = fusuario.GetDialogResult();
                    if (dr == DialogResult.Cancel) {
                        Application.Exit();
                    } else if (mColeccion.Count > 0){
                        EUsuario user = mColeccion[0];
                        txtNick.Text = user.Nick;
                    } else {
                        MessageBox.Show("Error, el software se cerrar.", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        Application.Exit();
                    }
                }
            }
        }

        private void btAceptar_Click(object sender, EventArgs e) {
            if (String.IsNullOrEmpty(txtNick.Text)) {
                MessageBox.Show("Tiene que escribir un nick.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            } else if (String.IsNullOrEmpty(txtPassword.Text)) {
                MessageBox.Show("Tiene que escribir un Password.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            } else {
                //TODO: buscar de la coleccion.
                EUsuario usuarioActual = new EUsuario();
                usuarioActual.Cargar(EUsuario.NPNick, txtNick.Text);
                if (usuarioActual.Existe && usuarioActual.Activo) {
                    string passEncriptado = usuarioActual.ObtenerPass(txtPassword.Text);
                    if (usuarioActual.Contrasenha.Equals(passEncriptado)) {
                        //El usuario est bien, se agrega y se inicia la pantalla principal.
                        MessageBox.Show("Usuario correcto.\n\nBienvenido : " + usuarioActual.Nick + ".\nPermiso : " + usuarioActual.GetMayorPermiso().ToString() +".", "Login OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        EUsuario.Actual = usuarioActual;
                        fPrincipal = new FPrincipal();
                        this.Visible = false;
                        txtPassword.Text = "";
                        fPrincipal.FormClosed += new FormClosedEventHandler(fPrincipal_FormClosed);
                        fPrincipal.Show();

                    } else {
                        MessageBox.Show("El password no es correcto.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                } else {
                    MessageBox.Show("El nick no es correcto.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
            }
        }

        void fPrincipal_FormClosed(object sender, FormClosedEventArgs e) {
            this.Visible = true;
        }

        private void btRecuperar_Click(object sender, EventArgs e) {
            FEnviarCorreo fec = new FEnviarCorreo();
            fec.ShowDialog();
        }

        private void btNuevoUsuario_Click(object sender, EventArgs e) {
            try {
                FUsuario fusuario = FabrFAutomovilismo.Actual.ObtenerFNuevo(mColeccion) as FUsuario;
                if (fusuario != null) {
                    fusuario.ShowDialog();
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btSalir_Click(object sender, EventArgs e) {
            this.Close();
        }
    }
}