using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.Collections.ObjectModel;

namespace Forms {

    public enum TipoColeccion {
        Ninguno,
        Vehiculos,
        Licencias,
        Grupos,
        Clases,
        Sectores,
        Inscripciones,
        CampeonatosPuntuables,
        Participantes,
        Carreras,
        Sanciones,
        Permisos,
        CarrerasParticipadas,
        Documentos
    }

    public partial class FColeccion : Form, IControlF, IFiltroBuscar, IPestanha{

        private IEColeccion mColeccion = null;
        private IEColeccion mColeccionContieneEntidadVerdadera = null;//solo para algunos casos especiales.
        private ESinClaveBase objPadre = null, obj = null;
        private TipoColeccion tipoColeccion = TipoColeccion.Ninguno;
        private bool SeColorea = false, mostrarForm = true;
        private Form formHijo = null;
        
        #region Constructores

        private FColeccion() {
            InitializeComponent();
        }

        public FColeccion(TipoColeccion tcol) : this(){
            tipoColeccion = tcol;
        }

        public FColeccion(IEColeccion c):this() {
            mColeccion = c;
            if (c != null) {
                lblTitulo.Text = c.Titulo;
            } else {
                lblTitulo.Text = "";
            }
            tipoColeccion = TipoColeccion.Ninguno;
        }

        public FColeccion(ESinClaveBase esb) : this(){
            objPadre = esb;
        }

        #endregion

        private void FColeccion_Load(object sender, EventArgs e) {
            dgv.SelectionChanged += new EventHandler(dgv_SelectionChanged);
            if (tipoColeccion == TipoColeccion.Ninguno) {
                dgv.DataSource = mColeccion;
                controlBusqueda.Coleccion = mColeccion;
            }
            if (mColeccion as EColeccion<EPilotoCarrera> != null && dgv.Columns["HoraVerificacion"] != null) {
                //damos formato al datagrid.
                dgv.Columns["HoraVerificacion"].DefaultCellStyle.Format = "HH:mm";
            }
            if (this.ParentForm != null && this.ParentForm.GetType().Equals(typeof(FConsultas))) {
                btAgregar.Visible = lblAgregar.Visible = false;
                btEliminar.Visible = lblEliminar.Visible = false;
                btNuevo.Visible = lblNuevo.Visible = false;
                btSalir.Visible = lblSalir.Visible = false;
                dgv.Height = dgv.Height + 150;
                controlBusqueda.Location = new Point(dgv.Location.X + dgv.Width + 5, dgv.Location.Y);
                controlBusqueda.Anchor = AnchorStyles.Left | AnchorStyles.Top;
            } else {
                lblSalir.Visible = btSalir.Visible = this.ParentForm == null || this.ParentForm.GetType().Equals(typeof(FCompeticion));
                if (tipoColeccion == TipoColeccion.Participantes || tipoColeccion == TipoColeccion.Carreras) {
                    btAgregar.Visible = true;
                    btEliminar.Visible = true;
                    btNuevo.Visible = lblNuevo.Visible = false;
                    btEliminar.Location = btAgregar.Location;
                    lblEliminar.Location = lblAgregar.Location;
                    btAgregar.Location = btNuevo.Location;
                    lblAgregar.Location = lblNuevo.Location;
                } else if (tipoColeccion == TipoColeccion.Vehiculos || tipoColeccion == TipoColeccion.Licencias || tipoColeccion == TipoColeccion.Sectores || tipoColeccion == TipoColeccion.Inscripciones ||
                    tipoColeccion == TipoColeccion.Clases || tipoColeccion == TipoColeccion.Grupos || tipoColeccion == TipoColeccion.Sanciones || tipoColeccion == TipoColeccion.Permisos || tipoColeccion == TipoColeccion.CampeonatosPuntuables
                    || tipoColeccion == TipoColeccion.Documentos) {

                    btAgregar.Visible = lblAgregar.Visible = false;
                    btEliminar.Visible = true;
                    btNuevo.Visible = true;
                    btEliminar.Location = btAgregar.Location;
                    lblEliminar.Location = lblAgregar.Location;
                }
                if (!btSalir.Visible) {
                    lblTitulo.Location = lblSalir.Location;
                }
            }
            //HACK:
            if (mColeccion != null && mColeccion.Count > 0 && (tipoColeccion == TipoColeccion.Sanciones || tipoColeccion == TipoColeccion.Documentos)) {
                dgv_SelectionChanged(null, null);
            }
            //quitar el componente de bsqueda.
            if (tipoColeccion != TipoColeccion.Inscripciones) {
                controlBusqueda.Visible = false;
            }
            if (this.ParentForm != null && this.ParentForm.GetType().Equals(typeof(FConsultas))) {
                controlBusqueda.Visible = true;
            }
            //Centrar el grid y los botones para participantes de escuderia y carreras organizadas.
            if (tipoColeccion == TipoColeccion.Participantes || tipoColeccion == TipoColeccion.Carreras) {
                dgv.Location = new Point(dgv.Location.X + this.Width/2 - dgv.Width/2, dgv.Location.Y);
                pFicha.Visible = false;
                btAgregar.Location = new Point(btAgregar.Location.X + this.Width / 2 - btAgregar.Width / 2, btAgregar.Location.Y);
                btEliminar.Location = new Point(btEliminar.Location.X + this.Width /2 - btEliminar.Width / 2, btEliminar.Location.Y);
                lblAgregar.Location = new Point(lblAgregar.Location.X + this.Width / 2 - lblAgregar.Width / 2, lblAgregar.Location.Y);
                lblEliminar.Location = new Point(lblEliminar.Location.X + this.Width / 2 - lblEliminar.Width / 2, lblEliminar.Location.Y);
            }
            if (!controlBusqueda.Visible && tipoColeccion != TipoColeccion.Sanciones && tipoColeccion != TipoColeccion.Documentos && tipoColeccion != TipoColeccion.Permisos) {
                dgv.Size = new Size(dgv.Width, dgv.Height * 2);
                btAgregar.Location = new Point(btAgregar.Location.X, btAgregar.Location.Y + dgv.Height/2);
                btEliminar.Location = new Point(btEliminar.Location.X, btEliminar.Location.Y + dgv.Height / 2);
                lblAgregar.Location = new Point(lblAgregar.Location.X, lblAgregar.Location.Y + dgv.Height / 2);
                lblEliminar.Location = new Point(lblEliminar.Location.X, lblEliminar.Location.Y + dgv.Height / 2);
                btNuevo.Location = new Point(btNuevo.Location.X, btNuevo.Location.Y + dgv.Height / 2);
                lblNuevo.Location = new Point(lblNuevo.Location.X, lblNuevo.Location.Y + dgv.Height / 2);
            }
        }

        private void FColeccion_FormClosed(object sender, FormClosedEventArgs e) {
        }

        public void dgv_SelectionChanged(object sender, EventArgs e) {
            try {
                if (dgv.CurrentRow != null && mostrarForm) {
                    obj = (ESinClaveBase)dgv.CurrentRow.DataBoundItem;
                    if (mColeccionContieneEntidadVerdadera != null) {//para casos especiales
                        int indice = 0;
                        bool terminado = false;
                        while (!terminado && indice < mColeccionContieneEntidadVerdadera.Count) {
                            EPilotoCarrera inscrito = mColeccionContieneEntidadVerdadera[indice] as EPilotoCarrera;
                            ECarrera carrera = obj as ECarrera;
                            if (inscrito != null && carrera != null && inscrito.Carrera.Equals(carrera)) {
                                terminado = true;
                                obj = inscrito;
                            } else {
                                indice = indice + 1;
                            }
                        }
                        if (!terminado) {
                            obj = null;
                        }
                    }
                    if (formHijo != null) {//quiere decir que ya tiene un form cargado.
                        IFichaEntidad cambiarObjeto = formHijo as IFichaEntidad;
                        if (cambiarObjeto != null) {
                            cambiarObjeto.Cargar(obj);
                        }
                    } else if (!bFormDeNuevo){//se obtiene uno nuevo.
                        if (mColeccion.Titulo != null && mColeccion.Titulo.Equals("Grupos")) {
                            FabrFGrupo fabr = new FabrFGrupo();
                            formHijo = fabr.ObtenerF(mColeccion, mColeccion.IndexOf(obj)) as Form;
                        } else if (mColeccion.Titulo != null && mColeccion.Titulo.Equals("Clases")) {
                            FabrFClase fabr = new FabrFClase();
                            formHijo = fabr.ObtenerF(mColeccion, mColeccion.IndexOf(obj)) as Form;
                        } else if (!obj.NombreDeLaEntidad.Equals(ESinClaveBase.ObtenerNombreEntidad(typeof(ECarrera)))) {
                            if (this.ParentForm != null && this.ParentForm.GetType().Equals(typeof(FConsultas))) {
                                FabrFAutomovilismoDetalles fabr = new FabrFAutomovilismoDetalles();
                                formHijo = fabr.ObtenerF(obj) as Form;
                            } else {
                                formHijo = FabrFBase.Actual.ObtenerF(mColeccion, mColeccion.IndexOf(obj)) as Form;
                            }
                        }
                        if (formHijo != null) {
                            ColocarEnPanel(formHijo, pFicha);
                            //Preguntamos si estamos en el formConsultas, para colocar el form al lado del control busqueda.
                            if (this.ParentForm != null && this.ParentForm.GetType().Equals(typeof(FConsultas)) && controlBusqueda.Visible) {
                                formHijo.Location = new Point(formHijo.Location.X + controlBusqueda.Width + 5, formHijo.Location.Y);
                                formHijo.Anchor = AnchorStyles.Bottom | AnchorStyles.Top | AnchorStyles.Left;
                            }
                        }
                    }
                    if (formHijo != null) {
                        IPestanha fpestanhas = formHijo as IPestanha;
                        FListados flistados = this.ParentForm as FListados;
                        FConsultas fconsultas = this.ParentForm as FConsultas;
                        if (fpestanhas != null) {
                            if (flistados != null) {
                                flistados.AgregarPestanhas(fpestanhas.ExtraerPestanhas(), obj);
                            } else if (fconsultas != null) {
                                fconsultas.AgregarPestanhas(fpestanhas.ExtraerPestanhas(), obj);
                            }
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private bool bFormDeNuevo = false;
        private void btNuevo_Click(object sender, EventArgs e) {
            if (formHijo != null) {
                this.Controls.Remove(formHijo);
                //formHijo.Close();
            }
            formHijo = null;
            bFormDeNuevo = true;
            if (tipoColeccion == TipoColeccion.Grupos) {
                FabrFGrupo fabr = new FabrFGrupo();
                formHijo = fabr.ObtenerFNuevo(mColeccion) as Form;
            } else if (tipoColeccion == TipoColeccion.Clases) {
                FabrFClase fabr = new FabrFClase();
                formHijo = fabr.ObtenerFNuevo(mColeccion) as Form;
            } else if (tipoColeccion == TipoColeccion.Documentos) {//PREGUNTAR QU TIPO DE DOCUMENTO QUIERE CREAR.
                EnumConvert ec = new EnumConvert();
                FElegir fe = new FElegir(ec.GetCollectionDescription(typeof(TipoDocumento)));
                DialogResult dr = fe.ShowDialog();
                if (dr == DialogResult.OK) {
                    try {
                        KeyValuePair<Enum, string> kvp = (KeyValuePair<Enum, string>)fe.Devolver();
                        TipoDocumento t = (TipoDocumento)kvp.Key;
                        EDocumento nuevoDoc = null;
                        switch (t) {
                            //case TipoDocumento.VerificacionesTecnicas:
                            //    nuevoDoc = new EDocVerificacionTecnica();
                            //    //HACK:
                            //    throw new Exception();
                            //    break;
                            //case TipoDocumento.ListaAutorizados:
                            //    nuevoDoc = new EDocListaAutorizados();
                            //    //HACK:
                            //    throw new Exception();
                            //    break;
                            case TipoDocumento.AsistenciaBriefing:
                                nuevoDoc = new EDocListaBriefing(objPadre as ECarrera);
                                break;
                            case TipoDocumento.InformeFinal:
                                nuevoDoc = new EDocInformeFinal(objPadre as ECarrera);
                                break;
                            default:
                                throw new Exception("Error creando el documento.");
                        }
                        if (nuevoDoc != null) {
                            mColeccion.Add(nuevoDoc);
                            FDocumento fdoc = new FDocumento();
                            fdoc.Cargar(mColeccion, mColeccion.IndexOf(nuevoDoc));
                            formHijo = fdoc;
                        } else {
                            throw new Exception("Error creando el documento.");
                        }
                    } catch (Exception) { }
                }
            }else {
                formHijo = FabrFBase.Actual.ObtenerFNuevo(mColeccion) as Form;
            }
            bFormDeNuevo = false;
            try {
                ColocarEnPanel(formHijo, pFicha);
            } catch (Exception) {}
        }

        private void btEliminar_Click(object sender, EventArgs e) {
            ESinClaveBase objElim = obj;
            int pos = mColeccion.IndexOf(obj);
            try {
                if(dgv.CurrentRow != null) {
                    DialogResult dr = MessageBox.Show("Est seguro de que quiere eliminar el registro seleccionado?", "CONFIRMAR", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if(dr == DialogResult.Yes) {
                        objElim = (ESinClaveBase)dgv.CurrentRow.DataBoundItem;
                        IQuitar objquitar = objElim as IQuitar;
                        if (objquitar != null) {
                            objquitar.Quitar();
                            objElim.Guardar();
                            mColeccion.Remove(objElim);
                            //mColeccion.Cargar();
                        } else {
                            objElim.Eliminar();
                            mColeccion.Remove(objElim);
                            //mColeccion.Guardar();
                        }
                        if (AmbitoTransa.Actual != null) {
                            AmbitoTransa.Actual.Commit();
                        }
                        MessageBox.Show("Se ha eliminado correctamente.", "Eliminar ok", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if (mColeccion.Count == 0 && formHijo != null) {
                            this.Controls.Remove(formHijo);
                            formHijo.Close();
                        }
                    }
                } else {
                    MessageBox.Show("Seleccione una fila", "WARNING", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            } catch(Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                if (!mColeccion.Contains(objElim)) {
                    mColeccion.Insert(pos, objElim);
                }
            }
        }
        
        private void btAgregar_Click(object sender, EventArgs e) {
            try {
                IEColeccion col = null;
                if (tipoColeccion == TipoColeccion.Participantes) {
                    col = EParticipante.GetCollection(mColeccion as EColeccion<EParticipante>) as EColeccion<EParticipante>;
                } else if (tipoColeccion == TipoColeccion.Carreras) {
                    col = ECarrera.GetCollection(false, mColeccion as EColeccion<ECarrera>) as EColeccion<ECarrera>;
                    col.ColumnaMostrarRango(new string[] {ECarrera.NPTipo, ECarrera.NPNombre, ECarrera.NPAlias});
                }

                FDialogo fd = new FDialogo(col);
                DialogResult dr = fd.ShowDialog();
                if (dr == DialogResult.OK) {
                    ESinClaveBase esb = fd.ExtraerEleccion();
                    EParticipante participante = esb as EParticipante;
                    if (participante != null) {
                        participante.Escuderia = objPadre as EEscuderia;
                        participante.Guardar();
                        mColeccion.Add(participante);
                    }
                    ECarrera carrera = esb as ECarrera;
                    if (carrera != null) {
                        mColeccion.Add(carrera);
                        carrera.EscuderiaOrganiza = objPadre as EEscuderia;
                        carrera.Guardar();
                    }
                    MessageBox.Show("Se ha agregado correctamente.", "Agregar ok", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ColocarEnPanel(Form f, Panel p) {
            if (!this.Controls.Contains(f)) {
                p.Visible = false;
                f.TopLevel = false;
                f.FormBorderStyle = FormBorderStyle.None;
                this.Controls.Add(f);
                f.Location = p.Location;
                f.AutoScroll = true;
                f.Show();
            }
        }

        private void AplicarColores() {//da los colores a todos los elementos del datagrid.
            foreach (DataGridViewRow fila in dgv.Rows) {
                IColor objColor = fila.DataBoundItem as IColor;
                if (objColor != null) {
                    Color color = objColor.DevolverColor();
                    fila.DefaultCellStyle.BackColor = color;
                }
            }
        }

        public void AplicarColores(ESinClaveBase entidad, int indice) {//aplica el color de la entidad al objeto correspondiente del datagrid.
            IColor entidadColor = entidad as IColor;
            if (entidadColor != null && indice >= 0 && indice < dgv.Rows.Count) {
                dgv.Rows[indice].DefaultCellStyle.BackColor = entidadColor.DevolverColor(); 
            }
        }

        public void AplicarColores(ESinClaveBase entidad, object o) {
            IColor eColor = entidad as IColor;
            if (eColor != null){
                bool encontrado = false;
                int indice = 0;
                while (!encontrado && indice < dgv.Rows.Count) {
                    if (entidad.Equals(dgv.Rows[indice].DataBoundItem as ESinClaveBase)) {
                        if (o == null) {
                            dgv.Rows[indice].DefaultCellStyle.BackColor = eColor.DevolverColor();
                        } else {
                            dgv.Rows[indice].DefaultCellStyle.BackColor = eColor.DevolverColor(o);
                        }
                        encontrado = true;
                    } else {
                        indice = indice + 1;
                    }
                }
	        }
        }

        private void dgv_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e) {
            if (SeColorea) {
                AplicarColores();
            }
        }

        private void dgv_ColumnHeaderMouseClick(object sender, DataGridViewCellMouseEventArgs e) {
            if (SeColorea) {
                AplicarColores();
            }
        }

        #region IControlF

        public void Cargar(ESinClaveBase obj) {
            if(obj == null) {
                throw new ArgumentNullException("Vacio");
            }

            EOficial oficial = obj as EOficial;
            if(oficial != null) {
                mColeccion = oficial.LicenciasOficial;
                lblTitulo.Text = "Licencias de: " + oficial.Apellidos + ", " + oficial.Nombre;
                dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
            }
            EParticipante participante = obj as EParticipante;
            if (participante != null) {
                if (tipoColeccion == TipoColeccion.CarrerasParticipadas) {
                    mColeccionContieneEntidadVerdadera = participante.CarrerasParticipadas;
                    mColeccion = new EColeccion<ECarrera>();
                    mColeccion.ColumnaMostrarRango(new string[] { ECarrera.NPTemporada, ECarrera.NPNombre });
                    foreach (EPilotoCarrera entidadPilotoCarrera in mColeccionContieneEntidadVerdadera) {
                        mColeccion.Add(entidadPilotoCarrera.Carrera);
                    }
                    lblTitulo.Text = "Carreras participadas.";
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                    controlBusqueda.Visible = false;
                }
            }
            EPiloto piloto = obj as EPiloto;
            if(piloto != null) {//comprobar si se trata de licencias o de vehculos
                if(this.tipoColeccion == TipoColeccion.Licencias) {
                    mColeccion = piloto.LicenciasPiloto;
                    lblTitulo.Text = "Licencias de: " + piloto.Apellidos + ", " + piloto.Nombre;
                } else if (tipoColeccion == TipoColeccion.Vehiculos) {
                    mColeccion = piloto.Vehiculos;
                    lblTitulo.Text = "Vehculos de: " + piloto.Apellidos + ", " + piloto.Nombre;
                }
                dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
            }
            ECopiloto copiloto = obj as ECopiloto;
            if(copiloto != null) {
                if (tipoColeccion == TipoColeccion.Licencias) {
                    mColeccion = copiloto.LicenciasCopiloto;
                    lblTitulo.Text = "Licencias de: " + copiloto.Apellidos + ", " + copiloto.Nombre;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                }
            }
            EEscuderia escuderia = obj as EEscuderia;
            if(escuderia != null) {
                if (tipoColeccion == TipoColeccion.Licencias) {
                    mColeccion = escuderia.LicenciaEscuderia;
                    lblTitulo.Text = "Licencias de: " + escuderia.Nombre;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                } else if (tipoColeccion == TipoColeccion.Participantes) {
                    mColeccion = escuderia.Participantes;
                    lblTitulo.Text = "Participantes de: " + escuderia.Nombre;
                    mostrarForm = false;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                } else if (tipoColeccion == TipoColeccion.Carreras) {
                    mColeccion = escuderia.Carreras;
                    lblTitulo.Text = "Carreras organizadas de: " + escuderia.Nombre;
                    mostrarForm = false;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                }
            }
            ECampeonato campeonato = obj as ECampeonato;
            if (campeonato != null) {//puede ser un campeonato, grupo o clase
                if (tipoColeccion == TipoColeccion.Grupos) {//es un campeonato
                    mColeccion = campeonato.GetGrupos();
                    lblTitulo.Text = "Grupos de: " + campeonato.Nombre;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                    this.toolTip.SetToolTip(btNuevo, "Si no aparece la pestaa de clases, haga doble click sobre un grupo.");
                } else if (tipoColeccion == TipoColeccion.Clases) {//es un grupo
                    mColeccion = campeonato.GetClases();
                    lblTitulo.Text = "Clases de :" + campeonato.Nombre;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                }
            }
            ERally rally = obj as ERally;
            ECarrera carrera = obj as ECarrera;
            if (rally != null && tipoColeccion == TipoColeccion.Sectores) {//
                mColeccion = rally.Sectores;
                lblTitulo.Text = "Sectores de: " + rally.Nombre;
                dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
            }else if (carrera != null) {
                if (tipoColeccion == TipoColeccion.Inscripciones) {
                    EColeccion<EPilotoCarrera> col = carrera.PilotosCarrera;
                    col.ApplySort(EPilotoCarrera.NPDorsal, ListSortDirection.Ascending);
                    mColeccion = col;
                    //lblTitulo.Visible = false;
                    lblTitulo.Text = "INSCRITOS";
                    SeColorea = true;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.DisplayedCells;
                } else if (tipoColeccion == TipoColeccion.CampeonatosPuntuables) {
                    mColeccion = carrera.CampeonatosPuntuables;
                    lblTitulo.Text = "Campeonatos Puntuables de: " + carrera.Nombre;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                } else if (tipoColeccion == TipoColeccion.Documentos) {
                    mColeccion = carrera.Documentos;
                    lblTitulo.Text = "Documentos de: " + carrera.Nombre;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                }
                //si est cerrada se inhabilitan los botones.
                if (carrera.EstaCerrada && tipoColeccion != TipoColeccion.Documentos) {
                    //ToolTip.
                    btAgregar.Click -= btAgregar_Click;
                    btEliminar.Click -= btEliminar_Click;
                    btNuevo.Click -= btNuevo_Click;
                    toolTip.SetToolTip(btAgregar, "La carrera est cerrada. No se puede hacer ninguna modificacin.");
                    toolTip.SetToolTip(btEliminar, "La carrera est cerrada. No se puede hacer ninguna modificacin.");
                    toolTip.SetToolTip(btNuevo, "La carrera est cerrada. No se puede hacer ninguna modificacin.");
                }
            }
            EPilotoCarrera inscrito = obj as EPilotoCarrera;
            if (inscrito != null) {
                if (tipoColeccion == TipoColeccion.Sanciones) {
                    mColeccion = inscrito.Sanciones;
                    lblTitulo.Text = "SANCIONES DEl DORSAL N " + inscrito.ToString();
                    SeColorea = true;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.DisplayedCells;
                }
                //si la carrera del inscrito est cerrada se inhabilitan los botones.
                if (inscrito.Carrera.EstaCerrada) {
                    btAgregar.Enabled = btEliminar.Enabled = btNuevo.Enabled = false;
                }
            }
            EUsuario user = obj as EUsuario;
            if (user != null) {
                if (tipoColeccion == TipoColeccion.Permisos) {
                    mColeccion = user.Permisos;
                    lblTitulo.Text = "PERMISOS DEL USUARIO: " + user.Nick;
                    dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                }
            }
            dgv.DataSource = mColeccion;
            controlBusqueda.Coleccion = mColeccion;
            objPadre = obj;
        }

        public bool EstaCargado() {
            return mColeccion != null;
        }

        #endregion

        #region IFiltroBuscar

        public void Aplicar(IEColeccion col) {
            dgv.DataSource = col;
        }

        #endregion

        private void btSalir_Click(object sender, EventArgs e) {
            this.Close();
        }

        #region IPestanha

        public Collection<Pestanha> ExtraerPestanhas() {
            IPestanha fpestanha = formHijo as IPestanha;
            if (fpestanha != null) {
                return fpestanha.ExtraerPestanhas();
            }
            return new Collection<Pestanha>();
        }

        public ESinClaveBase ObjFicha() {
            return obj;
        }

        #endregion

        private void dgv_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e) {
            this.dgv_SelectionChanged(null, null);
        }
    }
}