using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;

namespace Forms {
    public partial class FCompeticion : Form {

        private Form formularioAnterior = null;
        private ECarrera carrera = null;

        public FCompeticion(Size s, Point p, ECarrera c, FormWindowState ws) {
            InitializeComponent();
            if (ws == FormWindowState.Maximized) {
                this.WindowState = FormWindowState.Maximized;
            } else {
                this.Size = s;
                this.Location = p;
            }
            carrera = c;
        }

        private void FCompeticion_Load(object sender, EventArgs e) {
            if (carrera.EstaCerrada) {
                btCerrarCarrera.Click -= btCerrarCarrera_Click;
                this.toolTip.SetToolTip(btCerrarCarrera, "La carrera ya est finalizada. Este botn no funciona.");
            }
            this.Text += " : " + carrera.Nombre;
            if (carrera.EstaCerrada) {
                this.Text += " (Finalizada)";
            }
        }

        private void btSalir_Click(object sender, EventArgs e) {
            this.Close();
        }

        private void ColocarEnPanel(Form f) {
            if (formularioAnterior != null) {
                formularioAnterior.Close();
            }
            formularioAnterior = f;
            f.FormBorderStyle = FormBorderStyle.None;
            f.TopLevel = false;
            f.Anchor = AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Top;
            pForm.Controls.Clear();
            pForm.Controls.Add(f);
            f.Parent = pForm;
            this.BackColor = f.BackColor;
            f.Dock = DockStyle.Fill;
        }

        private void btInscripciones_Click(object sender, EventArgs e) {
            FColeccion fc = new FColeccion(TipoColeccion.Inscripciones);
            ColocarEnPanel(fc);
            fc.Cargar(carrera);
            fc.Show();
        }

        private void btTomaTiempos_Click(object sender, EventArgs e) {
            //abrir un dilogo con las fases o sectores/tramos que vamos a cronometrar.
            if (carrera.PilotosCarrera.Count > 0) {
                FElegir fEligeCrono = new FElegir(carrera);
                DialogResult dr = fEligeCrono.ShowDialog();
                if (dr == DialogResult.OK) {
                    ECronometrable cronometrable = fEligeCrono.Devolver() as ECronometrable;
                    if (cronometrable != null) {
                        FCronometrar fcrono = new FCronometrar(cronometrable);
                        ColocarEnPanel(fcrono);
                        fcrono.Cargar(cronometrable);
                        fcrono.Show();
                    }
                }
            } else {
                MessageBox.Show("No hay inscritos en esta carrera", "WARNING", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void btClasificacion_Click(object sender, EventArgs e) {
            FClasificacion fClasificacion = new FClasificacion();
            ColocarEnPanel(fClasificacion);
            if (carrera.Tipo == TipoCarrera.Rally) {
                ERally rally = new ERally(carrera.Id);
                fClasificacion.Cargar(rally);
            } else {
                ESubida subida = new ESubida(carrera.Id);
                fClasificacion.Cargar(subida);
            }
            fClasificacion.Show();
        }

        private void btSanciones_Click(object sender, EventArgs e) {
            FDialogo fd = new FDialogo(carrera.PilotosCarrera);
            DialogResult dr = fd.ShowDialog();
            if (dr == DialogResult.OK) {
                EPilotoCarrera inscrito = fd.ExtraerEleccion() as EPilotoCarrera;
                if (inscrito != null) {
                    FColeccion fc = new FColeccion(TipoColeccion.Sanciones);
                    fc.Cargar(inscrito);
                    ColocarEnPanel(fc);
                    fc.Show();
                }
            }
        }

        private void btCerrarCarrera_Click(object sender, EventArgs e) {
            DialogResult dr = MessageBox.Show("Est seguro de quiere dar por finalizada la carrera?\nNo se podr modificar inscripciones, tiempos ni sanciones.", "INFO", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk);
            if (dr == DialogResult.Yes) {
                try {
                    carrera.Cerrar();
                    MessageBox.Show("La carrera se ha cerrado correctamente.", "CERRAR CARRERA OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.btCerrarCarrera.Click -= btCerrarCarrera_Click;
                    this.toolTip.SetToolTip(btCerrarCarrera, "La carrera ya est finalizada. Este botn no funciona.");
                } catch (Exception ex) {
                    MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btDocumentos_Click(object sender, EventArgs e) {
            FColeccion fc = new FColeccion(TipoColeccion.Documentos);
            fc.Cargar(carrera);
            ColocarEnPanel(fc);
            fc.Show();
        }

        private void btImagenes_Click(object sender, EventArgs e) {
            FGaleria fgaleria = new FGaleria();
            fgaleria.Cargar(carrera);
            ColocarEnPanel(fgaleria);
            fgaleria.Show();
        }
    }
}