using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Negocio;
using Nibi.Negocio;
using System.Collections.ObjectModel;

namespace Forms {
    public partial class FConsultas : Form {

        private ESinClaveBase obj = null;
        private Collection<bool> colPestanhasCargadas;

        public FConsultas() {
            InitializeComponent();
            colPestanhasCargadas = new Collection<bool>();
        }

        public FConsultas(Size s, Point p, FormWindowState ws) : this() {
            if (ws == FormWindowState.Maximized) {
                this.WindowState = FormWindowState.Maximized;
            } else {
                this.Size = s;
                this.Location = p;
            }
        }

        private void FConsultas_Load(object sender, EventArgs e) {
            lblTemporada.Text = "TEMPORADA:\n" + ETemporada.Actual.Anho.ToString();
        }

        private void btAtras_Click(object sender, EventArgs e) {
            panelSplit.Panel1.Controls.Clear();
            panelSplit.Panel2.Controls.Clear();
            this.Close();
        }

        private void MostrarForm(Form f) {
            this.panelSplit.BackColor = Color.LightGray;
            panelSplit.Panel1.Controls.Clear();
            f.Dock = DockStyle.Fill;
            f.TopLevel = false;
            f.FormBorderStyle = FormBorderStyle.None;
            f.Parent = panelSplit.Panel1;
            panelSplit.Panel1.Controls.Add(f);
            f.FormClosed += new FormClosedEventHandler(f_FormClosed);
            f.Show();
            tabControlFicha.Visible = true;
        }

        void f_FormClosed(object sender, FormClosedEventArgs e) {
            //tabControlFicha.TabPages.Clear();
            //tabControlFicha.Controls.Clear();
            //tabControlFicha.Dispose();
            //tabControlFicha.Visible = false;
        }

        private void btPilotos_Click(object sender, EventArgs e) {
            EColeccion<EParticipante> col = EParticipante.GetCollectionPilotos();
            col.ColumnaMostrarRango(new string[] { EParticipante.NPNif, EParticipante.NPNombre, EParticipante.NPApellidos });
            MostrarForm(new FColeccion(col));
        }

        private void btCopilotos_Click(object sender, EventArgs e) {
            EColeccion<EParticipante> col = EParticipante.GetCollectionCopilotos();
            col.ColumnaMostrarRango(new string[] {EParticipante.NPNif, EParticipante.NPNombre, EParticipante.NPApellidos });
            MostrarForm(new FColeccion(col));
        }

        private void btCarreras_Click(object sender, EventArgs e) {
            EColeccion<ECarrera> col = ECarrera.GetCollection(true);
            col.ColumnaMostrarRango(new string[] {ECarrera.NPTemporada, ECarrera.NPNombre, ECarrera.NPAlias });
            MostrarForm(new FColeccion(col));
        }

        private void btCampeonatos_Click(object sender, EventArgs e) {
            EColeccion<ECampeonato> col = ECampeonato.GetCampeonatos(true);
            col.ColumnaMostrarRango(new string[] { ECampeonato.NPTemporada, ECampeonato.NPNombre, ECampeonato.NPAlias});
            MostrarForm(new FColeccion(col));
        }

        public void AgregarPestanhas(Collection<Pestanha> coleccion, ESinClaveBase esb) {
            obj = esb;
            tabControlFicha.TabPages.Clear();
            if (coleccion != null) {
                tabControlFicha.TabPages.Add(">>>>");
                colPestanhasCargadas.Add(true);
                foreach (Pestanha p in coleccion) {
                    TabPage tp = ObtenerFicha(p.Nombre, tabControlFicha.TabPages);
                    if (tp == null) {
                        tp = new TabPage(p.Nombre);
                        tabControlFicha.TabPages.Add(tp);
                        tp.Enter += new EventHandler(tp_Enter);
                        tp.AutoScroll = true;
                        colPestanhasCargadas.Add(false);
                        tp.BackColor = Color.LightSteelBlue;
                    } else {
                        tp.Controls.Clear();
                    }
                    tp.Controls.Add(p.Control);
                    if (tabControlFicha.TabPages.Count == 1) {//es la primera, la mostramos.
                        tp_Enter(tp, null);
                    }
                }
            }
        }

        private TabPage ObtenerFicha(string nombre, TabControl.TabPageCollection tabPageCollection) {
            foreach (TabPage pagina in tabPageCollection) {
                if (pagina.Text.Equals(nombre)) {
                    return pagina;
                }
            }
            return null;
        }

        void tp_Enter(object sender, EventArgs e) {
            TabPage pagina = sender as TabPage;
            if (pagina != null && pagina.Controls.Count > 0) {
                Control c = pagina.Controls[0];
                if (!c.Visible) {
                    c.Dock = DockStyle.Fill;
                    c.BackColor = pagina.BackColor;
                    c.Show();
                }
                IControlF cf = c as IControlF;
                if (cf != null && (!cf.EstaCargado() || !colPestanhasCargadas[tabControlFicha.TabPages.IndexOf(pagina)])) {
                    cf.Cargar(obj);
                    colPestanhasCargadas[tabControlFicha.TabPages.IndexOf(pagina)] = true;
                }
            }
        }

        //public bool EstanPestanhasYaAgregadas() {
        //    return tabControlFicha.Controls.Count > 0;
        //}

        //public void CambiarObjetosPestanha(ESinClaveBase esb) {
        //    if (esb != null && obj != null) {
        //        if ((esb.GetType() == typeof(EPiloto) || esb.GetType() == typeof(ECopiloto) || esb.GetType() == typeof(EParticipante)) &&
        //            (obj.GetType() == typeof(EPiloto) || obj.GetType() == typeof(ECopiloto) || obj.GetType() == typeof(EParticipante))) {
        //            obj = esb;
        //            for (int indice = 0; indice < colPestanhasCargadas.Count; indice++) {
        //                colPestanhasCargadas[indice] = false;
        //            }
        //            if (tabControlFicha.TabPages.Count > 0) {
        //                tp_Enter(tabControlFicha.SelectedTab, null);
        //            }
        //        } else {
        //            colPestanhasCargadas.Clear();
        //            IPestanha ipestanha = esb as IPestanha;
        //            if (ipestanha != null) {
        //                AgregarPestanhas(ipestanha.ExtraerPestanhas(), esb);
        //            }
        //        }
        //    } else if (esb != null) {
        //        colPestanhasCargadas.Clear();
        //        IPestanha ipestanha = esb as IPestanha;
        //        if (ipestanha != null) {
        //            AgregarPestanhas(ipestanha.ExtraerPestanhas(), esb);
        //        }
        //    }
            
        //}
    }
}