using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.Collections.ObjectModel;

namespace Forms
{
    public partial class FListados : Form, IFiltroBuscar
    {
        private IEColeccion mColeccion = null;
        private ESinClaveBase obj = null;
        private ESinClaveBase objSecundario = null; //para el caso de los grupos, para luego cargarlos.
        private string nombreEntidadAnterior = null;
        bool activos_inactivos = true;

        #region Constructores

        public FListados(IEColeccion col) {
            InitializeComponent();
            mColeccion = col;
            lblTitulo.Text = col.Titulo;
        }

        #endregion

        private void FListados_Load(object sender, EventArgs e){
            cBusqueda1.Coleccion = mColeccion;
            dgv.DataSource = mColeccion;
            dgv.SelectionChanged += new EventHandler(dgv_SelectionChanged);
            lblDarAlta.Visible = btAlta.Visible = false;
        }

        private void btEliminar_Click(object sender, EventArgs e){
            int pos = mColeccion.IndexOf(obj);
            ESinClaveBase objElim = obj;
            try{
                if (dgv.CurrentRow != null) {
                    DialogResult dr = MessageBox.Show("Est seguro de que quiere eliminar el registro seleccionado?", "CONFIRMAR", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if(dr == DialogResult.Yes) {
                        LimpiarTab();
                        mColeccion.Remove(objElim);
                        objElim.Eliminar();
                        MessageBox.Show("El registro ha sido eliminado correctamente.", "Eliminar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if (mColeccion.Count == 0) {
                            LimpiarTab();
                            pFicha.Controls.Clear();
                            pFicha.Text = "";
                        }
                     }
                }
                else {
                    MessageBox.Show("Seleccione una fila", "WARNING", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (Exception ex){
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                if (!mColeccion.Contains(objElim)) {
                    mColeccion.Insert(pos, objElim);
                }
            }
        }

        private void btNuevo_Click(object sender, EventArgs e) {
            LimpiarTab();
            Form ficha = (Form)FabrFBase.Actual.ObtenerFNuevo(mColeccion);
            ColocarEnPanel(ficha , pFicha);
            IPestanha p = ficha as IPestanha;
            if (p != null) {
                obj = p.ObjFicha();
                pFicha.Text = obj.NombreDeLaEntidad;
                this.AgregarPestanhas(p.ExtraerPestanhas());
            }
        }

        private void ColocarEnPanel(Form f, Panel p) {
            f.TopLevel = false;
            //f.Parent = p;
            f.FormBorderStyle = FormBorderStyle.None;
            //f.Dock = DockStyle.Fill;
            if (pFicha.Controls.Count > 0) {
                pFicha.Controls.Clear();
            }
            p.Controls.Add(f);
            f.Location = new Point(1, 1);
            f.Show();
        }

        public void dgv_SelectionChanged(object sender, EventArgs e) {
            LimpiarTab();
            if (dgv.CurrentRow != null) {
                obj = (ESinClaveBase)dgv.CurrentRow.DataBoundItem;
                if (nombreEntidadAnterior == null) {
                    nombreEntidadAnterior = obj.NombreDeLaEntidad;
                } else if (!nombreEntidadAnterior.Equals(obj.NombreDeLaEntidad)) {
                    pFicha.Controls.Clear();
                }
                //Preguntar si ya se ha cargado un form.
                Form ficha = null;
                if (pFicha.Controls.Count == 0) {// no se carg el formFicha todava
                    ficha = FabrFBase.Actual.ObtenerF(obj) as Form;
                    ColocarEnPanel(ficha, pFicha);
                    pFicha.Text = obj.NombreDeLaEntidad;
                } else {//ya se ha cargado, solo hay que cambiar el obj. Primero se pregunta si el objeto es el adecuado.
                    try {
                        ficha = pFicha.Controls[0] as Form;
                        IFichaEntidad cambiarObjeto = ficha as IFichaEntidad;
                        if (cambiarObjeto != null) {
                            cambiarObjeto.Cargar(obj);
                            objSecundario = null;
                        }
                    } catch (Exception) {//El objeto no es el que queriamos. Se carga un form nuevo.
                        LimpiarTab();
                        pFicha.Controls.Clear();
                        pFicha.Text = "";
                        ficha = FabrFBase.Actual.ObtenerF(obj) as Form;
                        if (ficha != null) {
                            ColocarEnPanel(ficha, pFicha);
                            pFicha.Text = obj.NombreDeLaEntidad;
                        }
                    }
                }
                IPestanha p = ficha as IPestanha;
                if (p != null) {
                    AgregarPestanhas(p.ExtraerPestanhas());
                }
            }
        }

        private void LimpiarTab() {
            foreach (TabPage tp in tabControlFicha.TabPages) {
                //tp.Controls.Clear();
                if(!tp.Equals(pFicha)) {
                    tp.Dispose();
                }
            }
            tabControlFicha.SelectTab(pFicha);
        }

        public void AgregarPestanhas(Collection<Pestanha> colp) {
            if(colp != null) {
                foreach(Pestanha p in colp) {
                    TabPage tp = ObtenerFicha(p.Nombre, tabControlFicha.TabPages);
                    if(tp == null) {
                        tp = new TabPage(p.Nombre);
                        tabControlFicha.TabPages.Add(tp);
                        tp.Enter += new EventHandler(tp_Enter);
                        tp.BackColor = pFicha.BackColor;
                    }
                    tp.AutoScroll = true;
                    tp.Controls.Add(p.Control);
                }
            }
        }

        public void AgregarPestanhas(Collection<Pestanha> colp, ESinClaveBase esb) {
            objSecundario = esb;
            AgregarPestanhas(colp);
        }

        private void tp_Enter(object sender , EventArgs e) {
            TabPage tp = sender as TabPage;
            if (tp != null && tp.Controls.Count > 0) {
                Control c = tp.Controls[0];
                if (!c.Visible) {
                    c.Dock = DockStyle.Fill;
                    IControlF cf = c as IControlF;
                    if (cf != null && !cf.EstaCargado()) {
                        if (objSecundario != null) {
                            cf.Cargar(objSecundario);
                            objSecundario = null;
                        } else {
                            cf.Cargar(obj);
                        }
                    }
                    c.BackColor = tp.BackColor;
                    c.Show();
                }
            }
        }

        /// <summary>
        /// Devuelve un tabpage de la coleccion pasada como parmetro cuyo texto mostrado como cabecera coincide
        /// con el string p pasado como parmaetro
        /// </summary>
        /// <param name="p"></param>
        /// <param name="tabPageCollection"></param>
        /// <returns></returns>
        private TabPage ObtenerFicha(string p , TabControl.TabPageCollection tabPageCollection) {
            foreach (TabPage tp in tabPageCollection) {
                if (tp.Text.Equals(p)) {
                    return tp;
                } else if ((p.Contains("Clases") && tp.Text.Contains("Clases")) || 
                    (p.Contains("Sectores") && tp.Text.Contains("Sectores"))){

                    tp.Text = p;
                    tp.Controls.Clear();
                    return tp;
                }
            }
            return null;
        }

        private void FListados_FormClosing(object sender, FormClosingEventArgs e) {
            LimpiarTab();
        }

        private void btBaja_Click(object sender, EventArgs e) {
            if (obj != null) {
                DialogResult dr = MessageBox.Show("Est seguro de que quiere dar de baja?", "INFO", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.Yes) {
                    IAltaBaja objBaja = obj as IAltaBaja;
                    if (objBaja != null) {
                        objBaja.DarAltaBaja(false);
                        try {
                            obj.Guardar();
                            mColeccion.Cargar();
                            MessageBox.Show("Se ha dado de baja correctamente", "Dar Baja OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        } catch (Exception ex) {
                            MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    } else {
                        MessageBox.Show("Esta entidad no tiene modo baja/alta", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                }
            } else {
                MessageBox.Show("Tiene que seleccionar una fila.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void btGuardar_Click(object sender, EventArgs e) {
            try {
                if (mColeccion.EstaSucio) {
                    DialogResult dr = MessageBox.Show("Desea guardar todos los cambios realizados de todas las filas?", "Guardar", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        mColeccion.Guardar();
                        MessageBox.Show("Se ha guardado correctamente.", "Guardar OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        foreach (TabPage tp in tabControlFicha.TabPages) {
                            ReestablecerNombrePanel(tp);
                        }
                    }
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void ReestablecerNombrePanel(TabPage tpag) {
            if (tpag.Text.Contains("*")) {
                tpag.Text = tpag.Text.Remove(tpag.Text.Length - 2);
            }
        }

        #region IFiltroBuscar

        public void Aplicar(IEColeccion col) {
            dgv.DataSource = col;
        }

        #endregion

        public void CambiarColeccion(IEColeccion eColeccion, bool activos) {
            mColeccion = eColeccion;
            dgv.DataSource = mColeccion;
            if (mColeccion.Count == 0) {
                LimpiarTab();
                pFicha.Controls.Clear();
                pFicha.Text = "";
            }
            lblTitulo.Text = mColeccion.Titulo;
            cBusqueda1.Coleccion = eColeccion;
            if (!activos) {
                lblMostrarAI.Text = "Mostrar\nActivos";
            } else {
                lblMostrarAI.Text = "Mostrar\nInactivos";
            }
            activos_inactivos = activos;
            btAlta.Visible = lblDarAlta.Visible = !activos_inactivos;
            btNuevo.Visible = lblNuevo.Visible = btEliminar.Visible = lblEliminar.Visible =
                btGuardar.Visible = lblGuardar.Visible = btBaja.Visible = lblDarBaja.Visible = activos_inactivos;

        }

        private void btSalir_Click(object sender, EventArgs e) {
            this.Close();
        }

        private void btInactivos_Click(object sender, EventArgs e) {
            activos_inactivos = !activos_inactivos;
            switch (mColeccion.Titulo.ToUpper()) {
                case "PILOTOS":
                    this.CambiarColeccion(EPiloto.GetCollection(activos_inactivos), activos_inactivos);
                    break;
                case "COPILOTOS" :
                    this.CambiarColeccion(ECopiloto.GetCollection(activos_inactivos), activos_inactivos);
                    break;
                case "OFICIALES":
                    this.CambiarColeccion(EOficial.GetCollection(activos_inactivos), activos_inactivos);
                    break;
                case "ESCUDERAS":
                    this.CambiarColeccion(EEscuderia.GetCollection(activos_inactivos), activos_inactivos);
                    break;
                case "LICENCIAS":
                    this.CambiarColeccion(ELicencia.GetCollection(activos_inactivos), activos_inactivos);
                    break;
                case "CAMPEONATOS":
                    this.CambiarColeccion(ECampeonato.GetCollection(activos_inactivos, 0, false), activos_inactivos);
                    break;
                case "SUBIDAS":
                    this.CambiarColeccion(ESubida.GetCollection(activos_inactivos, false), activos_inactivos);
                    break;
                case "RALLYES":
                    this.CambiarColeccion(ERally.GetCollection(activos_inactivos, false), activos_inactivos);
                    break;
                case "USUARIOS":
                    this.CambiarColeccion(EUsuario.GetCollection(activos_inactivos), activos_inactivos);
                    break;
                default:
                    break;
            }
        }

        private void btAlta_Click(object sender, EventArgs e) {
            if (obj != null) {
                DialogResult dr = MessageBox.Show("Est seguro de que quiere dar de alta?", "INFO", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.Yes) {
                    IAltaBaja objBaja = obj as IAltaBaja;
                    if (objBaja != null) {
                        objBaja.DarAltaBaja(true);
                        try {
                            obj.Guardar();
                            mColeccion.Cargar();
                            MessageBox.Show("Se ha dado de alta correctamente", "Dar Alta OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        } catch (Exception ex) {
                            MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    } else {
                        MessageBox.Show("Esta entidad no tiene modo baja/alta", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                }
            } else {
                MessageBox.Show("Tiene que seleccionar una fila.", "INFO", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }
    }
    public struct Pestanha {
        public string Nombre;
        //IControlPestaa
        public Control Control;

        public Pestanha(string nom , Control c) {
            Nombre = nom;
            Control = c;
        }
    }
}