using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using System.Diagnostics;
using Forms.Properties;

namespace Forms {
    public partial class FPrincipal : Form {

        private EPermiso permisoActual = null;

        public FPrincipal() {
            InitializeComponent();
            permisoActual = EUsuario.Actual.GetMayorPermiso();
        }

        private void FPrincipal_Load(object sender, EventArgs e) {
            //bgwPrincipal.RunWorkerAsync();
            lblTemporada.Text = "TEMPORADA:\n" + ETemporada.Actual.Anho.ToString();
            if (EUsuario.Actual.GetMayorPermiso().Tipo != TipoPermiso.Publico) {
                EColeccion<ECarrera> col = ECarrera.GetCollection(true);
                foreach (ECarrera carrera in col) {
                    mcCalendario.AddBoldedDate(carrera.FechaComienzo);
                }
                this.toolTip.SetToolTip(mcCalendario, "Pulse las fechas en negrita para acceder a la gestin de la carrera seleccionada.");
            }
            //Visibilidad de los botones
            btMantenimiento.Visible = lblDatosBasicos.Visible = permisoActual.PermisoIgualOSuperior(TipoPermiso.Secretario);
            btUtilidades.Visible = lblUtilidades.Visible = permisoActual.PermisoIgualOSuperior(TipoPermiso.Administrador);
            btCompeticion.Visible = lblGestionCarreras.Visible = permisoActual.PermisoIgualOSuperior(TipoPermiso.Cronometrador) || permisoActual.PermisoIgualOSuperior(TipoPermiso.Secretario);
        }

        private void temporizador_Tick(object sender, EventArgs e) {
            lblFechaHora.Text = DateTime.Now.ToString();
        }

        private void btSalir_Click(object sender, EventArgs e) {
            DialogResult dr = MessageBox.Show("Est seguro de que quiere salir de la aplicacin?:", "Salir", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr == DialogResult.Yes) {
                Application.Exit();
            }
        }

        private void btMantenimiento_Click(object sender, EventArgs e) {
            FMantenimiento fm = new FMantenimiento(this.Size, this.Location, WindowState);
            MostrarForm(fm);
        }

        private void btCompeticion_Click(object sender, EventArgs e) {
            EColeccion<ECarrera> colCarreras = ECarrera.GetCollection(false);
            colCarreras.ColumnaMostrarRango(new string[] {ECarrera.NPTipo, ECarrera.NPNombre, ECarrera.NPAlias });
            FDialogo fd = new FDialogo(colCarreras);
            DialogResult dr = fd.ShowDialog();
            if (dr == DialogResult.OK) {
                ECarrera carrera = fd.ExtraerEleccion() as ECarrera;
                if (carrera != null) {
                    FCompeticion fc = new FCompeticion(this.Size, this.Location, carrera, WindowState);
                    MostrarForm(fc);
                }
            }
        }

        private void MostrarForm(Form f) {
            this.Visible = false;
            f.Show();
            f.FormClosed += new FormClosedEventHandler(f_FormClosed);
        }

        private void f_FormClosed(object sender, FormClosedEventArgs e) {
            Form f = sender as Form;
            if (f != null) {
                this.Size = f.Size;
                this.Location = f.Location;
            }
            lblTemporada.Text = "TEMPORADA:\n" + ETemporada.Actual.Anho.ToString();
            this.Visible = true;
        }

        private void btUtilidades_Click(object sender, EventArgs e) {
            FUtilidades fu = new FUtilidades(this.Size, this.Location, WindowState);
            MostrarForm(fu);
        }

        private void btCerrarSesion_Click(object sender, EventArgs e) {
            DialogResult dr = MessageBox.Show("Est seguro que quiere cerrar la sesin actual?", "Cerrar Sesin", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr == DialogResult.Yes) {
                this.Close();
            }
        }

        private void mcCalendario_DateSelected(object sender, DateRangeEventArgs e) {
            if (EUsuario.Actual.GetMayorPermiso().Tipo != TipoPermiso.Publico) {
                EColeccion<ECarrera> col = ECarrera.GetCarreras(e.Start, e.End);
                if (col.Count == 1) {
                    DialogResult dr = MessageBox.Show("La carrera seleccionada es: " + col[0].Nombre + ".\nDesea entrar?:", "Gestin Carreras", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (dr == DialogResult.Yes) {
                        FCompeticion fc = new FCompeticion(this.Size, this.Location, col[0], this.WindowState);
                        MostrarForm(fc);
                    }
                } else if (col.Count > 1) {//TODO: seleccionar mas de una carrera.
                    FDialogo fd = new FDialogo(col);
                    DialogResult dr = fd.ShowDialog();
                    if (dr == DialogResult.OK) {
                        ECarrera carrera = fd.ExtraerEleccion() as ECarrera;
                        if (carrera != null) {
                            FCompeticion fc = new FCompeticion(this.Size, this.Location, carrera, this.WindowState);
                            MostrarForm(fc);
                        }
                    }
                }
            }
        }

        private void mcCalendario_MouseHover(object sender, EventArgs e) {
            //TODO: mostrar nombre de la carrera como en formato ayuda.
        }

        private void btInfo_Click(object sender, EventArgs e) {
            FConsultas fc = new FConsultas(this.Size, this.Location, WindowState);
            MostrarForm(fc);
        }

        private void btAcercaDe_Click(object sender, EventArgs e) {
            FAcercaDe facercade = new FAcercaDe();
            facercade.ShowDialog();
        }

        private void btManual_Click(object sender, EventArgs e) {
            Process pr = new Process();
            pr.StartInfo.WindowStyle = ProcessWindowStyle.Maximized;
            pr.StartInfo.FileName = Application.StartupPath + Settings.Default.RutaManual;
            pr.Start();
        }

        private void btUsuario_Click(object sender, EventArgs e) {
            FUsuario fusuario = new FUsuario();
            fusuario.Cargar(EUsuario.Actual);
            fusuario.ShowDialog();
        }

        //private void bgwPrincipal_DoWork(object sender, DoWorkEventArgs e) {
        //    for (int indice = 0; indice < 1000; indice++) {
        //        bgwPrincipal.ReportProgress(indice / 10);
        //        System.Threading.Thread.Sleep(500);
        //    }
        //    lblProgreso.Text = "En Proceso...";
        //}

        //private void bgwPrincipal_ProgressChanged(object sender, ProgressChangedEventArgs e) {
        //    pbPrincipal.Value = e.ProgressPercentage;
        //}

        //private void bgwPrincipal_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
        //    lblProgreso.Text = "Terminado.";
        //}
    }
}