using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nibi.Negocio;
using Negocio;
using Nibi.Dal;
using Forms.Properties;
using System.IO;

namespace Forms {
    public partial class FUtilidades : Form {

        private FListados fListado = null;

        public FUtilidades() {
            InitializeComponent();
        }

        public FUtilidades(Size s, Point p, FormWindowState ws) : this() {
            if (ws == FormWindowState.Maximized) {
                this.WindowState = FormWindowState.Maximized;
            } else {
                this.Size = s;
                this.Location = p;
            }
        }

        private void btSalir_Click(object sender, EventArgs e) {
            this.Close();
        }

        private void MostrarForm(IEColeccion col){
            if (fListado != null) {
                fListado.Close();
            }
            fListado = new FListados(col);
            fListado.TopLevel = false;
            fListado.Parent = pVentana;
            fListado.FormBorderStyle = FormBorderStyle.None;
            pVentana.Controls.Add(fListado);
            pVentana.Visible = true;
            fListado.Dock = DockStyle.Fill;
            fListado.Show();
        }

        private void btUsuarios_Click(object sender, EventArgs e) {
            MostrarForm(EUsuario.GetCollection(true));
        }

        private void btCambioBD_Click(object sender, EventArgs e) {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.InitialDirectory = @"C:\Users\DaNy\Desktop";
            fileDialog.Filter = "Archivo de Microsoft Access (*.mdb)|*.mdb|All files (*.*)|*.*" ;
            fileDialog.RestoreDirectory = true;
            fileDialog.FilterIndex = 2;
            if (fileDialog.ShowDialog() == DialogResult.OK) {
                string cadenaSettings = Settings.Default.RutaConexion;
                int indice = cadenaSettings.IndexOf("Source=");
                string nuevaConexion = cadenaSettings.Replace(cadenaSettings.Substring(indice + 7), fileDialog.FileName);
                FabrDBase.Actual = FabrDBase.CrearFabrD(nuevaConexion);
                Settings.Default.RutaConexion = nuevaConexion;
                Settings.Default.Save();
                MessageBox.Show("Se ha cambiado la ruta correctamente.", "OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void btRutaImagenes_Click(object sender, EventArgs e) {
            FolderBrowserDialog folderDialog = new FolderBrowserDialog();
            if (folderDialog.ShowDialog() == DialogResult.OK) {
                Settings.Default.RutaImagenes = folderDialog.SelectedPath;
                Settings.Default.Save();
                MessageBox.Show("Se ha cambiado la ruta correctamente.", "OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void btCopiaSeguridad_Click(object sender, EventArgs e) {
            try {
                string archivoACopiar = Settings.Default.RutaConexion.Substring(Settings.Default.RutaConexion.IndexOf("Source=") + 7);
                FolderBrowserDialog folderDialog = new FolderBrowserDialog();
                if (folderDialog.ShowDialog() == DialogResult.OK) {
                    string destino = folderDialog.SelectedPath + @"\copia.mdb";
                    if (File.Exists(destino)) {
                        DialogResult dr = MessageBox.Show("Ya existe un fichero con ese nombre.\nDesea reemplazarlo?:", "Copiar", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.Yes) {
                            File.Copy(archivoACopiar, destino, true);
                        }
                    } else {
                        File.Copy(archivoACopiar, destino);
                    }
                    MessageBox.Show("La copia se ha realizado correctamente.", "Copia OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btCerrarTemporada_Click(object sender, EventArgs e) {
            try {
                DialogResult dr = MessageBox.Show("Est seguro de que quiere terminar con la temporada actual\ny empezar una nueva?\n\nTemporada actual: " + ETemporada.Actual.Anho + "\nNueva temporada: " + (ETemporada.Actual.Anho + 1)
                    + "\n\nSi acepta, se cerrarn finalizarn y se cerrarn todas las carreras de la tempordad actual.", "Nueva temporada", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.Yes) {
                    ETemporada.NuevaTemporada();
                    MessageBox.Show("La temporada " + (ETemporada.Actual.Anho - 1) + " ha terminado.\nComienza la temporada " + ETemporada.Actual.Anho, "INFO", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void FUtilidades_Load(object sender, EventArgs e) {

        }
    }
}