using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    #region ENUM

    public enum TipoCarreraPuntuable {
        [Description("Montaa")]
        Montanha = 1,
        Rally,
        Todos
    }

    public enum TipoParticipanteAceptado {
        Piloto = 1,
        Copiloto,
        Pareja
    }

    #endregion

    [Tabla("Campeonatos")]
    public class ECampeonato : EIdBase, IAltaBaja {

        #region Constructores
        public ECampeonato() : base() { }

        public ECampeonato(int vId) : base(vId) { }

        public ECampeonato(string alias) : this() {
            Cargar(NPAlias, alias);
        }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            Nombre = null;
            Alias = null;
            Nivel = 0;
            Campeonato = null;
            Activo = true;
            Temporada = ETemporada.Actual;
            mParticipanteAceptado = TipoParticipanteAceptado.Piloto;
            mCarreraPuntuable = TipoCarreraPuntuable.Todos;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Campeonato")) {
                if (this.EsNuevo) {
                    if (Nivel == 0) {//es un campeonato, por tanto tiene que tener al menos un grupo.
                        if (mColeccionGrupos == null || mColeccionGrupos.Count == 0) {
                            throw new Exception("Para guardar hace falta crear como mnimo un grupo.");
                        }
                    } else if (Nivel == 1) {//es un grupo, por tanto tiene que tener al menos una clase
                        if (mColeccionClases == null || mColeccionClases.Count == 0) {
                            throw new Exception("Para guardar hace falta crear como mnimo una clase.");
                        }
                    }
                }
                base.GuardarCore();
                if (mLogoCargada)
                    CentroImagenesE.Actual.AddImagen(this, Logo);
                if (mColeccionClases != null) {
                    mColeccionClases.Guardar();
                }
                if (mColeccionGrupos != null) {
                    mColeccionGrupos.Guardar();
                }
                if (mClasificacionesGenerales != null) {
                    mClasificacionesGenerales.Guardar();
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            //mClasificaciones = null;
            mLogoCargada = false;
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return true; } 
        }

        public override string NombreDeLaEntidad {
            get { return "Campeonato"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPNombre; } 
        }
        #endregion

        #region Propiedades

        public const string NPNombre = "Nombre";
        private string mNombre;

        [Campo(Tamanyo = 100), Requerido]
        public string Nombre {
            get {
                PrePropertyRead(NPNombre);
                return mNombre;
            }
            set {
                PrePropertyChanged(NPNombre);
                mNombre = value;
                if (String.IsNullOrEmpty(mNombre)) {
                    AddError(new PropertyException(NPNombre, this, "El nombre no puede ser vaco.")); ;
                } else {
                    RemoveError(NPNombre);
                }
                OnPropertyChanged(NPNombre);
            }
        }

        public const string NPAlias = "Alias";
        private string mAlias;

        [Campo(Tamanyo = 10), Unico]
        public string Alias {
            get {
                PrePropertyRead(NPAlias);
                return mAlias;
            }
            set {
                PrePropertyChanged(NPAlias);
                mAlias = value;
                if (String.IsNullOrEmpty(mAlias)) {
                    AddError(new PropertyException(NPAlias, this, "El Alias no puede estar vaco."));
                } else {
                    RemoveError(NPAlias);
                }
                OnPropertyChanged(NPAlias);
            }
        }

        public const string NPCarreraPuntuable = "CarreraPuntuable";
        private TipoCarreraPuntuable mCarreraPuntuable;
        private bool mCarreraPuntuableCargada = false;
        private TipoCarreraPuntuable mAnterioPuntuable;

        [Campo("CarreraPuntuable")]
        public TipoCarreraPuntuable CarreraPuntuable {
            get {
                PrePropertyRead(NPCarreraPuntuable);
                return mCarreraPuntuable;
            }
            set {
                PrePropertyChanged(NPCarreraPuntuable);
                mCarreraPuntuable = value;
                if (!mCarreraPuntuableCargada) {
                    mCarreraPuntuableCargada = true;
                    mAnterioPuntuable = mCarreraPuntuable;
                }
                if (mAnterioPuntuable != mCarreraPuntuable && CampeonatosPuntuables.Count > 0) {
                    AddError(new PropertyException(NPCarreraPuntuable, this, "No se puede cambiar las carreras a las que punta por que este campeonato ya tiene asignada alguna carrera."));
                } else {
                    RemoveError(NPCarreraPuntuable);
                }
                OnPropertyChanged(NPCarreraPuntuable);
            }
        }

        public const string NPParticipanteAceptado = "ParticipanteAceptado";
        private TipoParticipanteAceptado mParticipanteAceptado;
        private bool mParticipanteCargado = false;
        private TipoParticipanteAceptado mAnteriorParticipante;

        [Campo("ParticipanteAceptado")]
        public TipoParticipanteAceptado ParticipanteAceptado {
            get {
                PrePropertyRead(NPParticipanteAceptado);
                return mParticipanteAceptado;
            }
            set {
                PrePropertyChanged(NPParticipanteAceptado);
                mParticipanteAceptado = value;
                if (!mParticipanteCargado) {
                    mParticipanteCargado = true;
                    mAnteriorParticipante = mParticipanteAceptado;
                }
                if (mAnteriorParticipante != mParticipanteAceptado && ClasificacionesGenerales.Count > 0) {
                    AddError(new PropertyException(NPParticipanteAceptado, this, "No se puede modificar el tipo de participante que puntuar para este campeonato por que ya tiene creada clasificaciones a tal efecto."));
                } else {
                    RemoveError(NPParticipanteAceptado);
                }
                OnPropertyChanged(NPParticipanteAceptado);
            }
        }

        public const string NPCilindradaMinima = "CilindradaMinima";
        private double mCilindradaMinima;

        [Campo]
        public double CilindradaMinima {
            get {
                PrePropertyRead(NPCilindradaMinima);
                return mCilindradaMinima;
            }
            set {
                PrePropertyChanged(NPCilindradaMinima);
                mCilindradaMinima = value;
                if (CilindradaMaxima < CilindradaMinima) {
                    AddError(new PropertyException(NPCilindradaMinima, this, "La cilindrada mxima no puede ser menor que la cilindrada mnima."));
                } else {
                    RemoveError(NPCilindradaMinima);
                    RemoveError(NPCilindradaMaxima);
                }
                OnPropertyChanged(NPCilindradaMinima);
            }
        }

        public const string NPCilindradaMaxima = "CilindradaMaxima";
        private double mCilindradaMaxima;

        [Campo]
        public double CilindradaMaxima {
            get {
                PrePropertyRead(NPCilindradaMaxima);
                return mCilindradaMaxima;
            }
            set {
                PrePropertyChanged(NPCilindradaMaxima);
                mCilindradaMaxima = value;
                if (CilindradaMaxima < CilindradaMinima) {
                    AddError(new PropertyException(NPCilindradaMaxima, this, "La cilindrada mxima no puede ser menor que la cilindrada mnima."));
                } else {
                    RemoveError(NPCilindradaMinima);
                    RemoveError(NPCilindradaMaxima);
                }
                OnPropertyChanged(NPCilindradaMaxima);
            }
        }

        public const string NPNivel = "Nivel";
        private int mNivel;

        [Campo]
        public int Nivel {
            get {
                PrePropertyRead(NPNivel);
                return mNivel;
            }
            set {
                PrePropertyChanged(NPNivel);
                mNivel = value;
                if (mNivel > 0) {
                    RemoveError(NPAlias);
                }
                OnPropertyChanged(NPNivel);
            }
        }

        public const string NPCampeonato = "Campeonato";
        private ECampeonato mCampeonato;

        [Campo("CampeonatoId")]
        public ECampeonato Campeonato {
            get {
                PrePropertyRead(NPCampeonato);
                return mCampeonato;
            }
            set {
                PrePropertyChanged(NPCampeonato);
                mCampeonato = value;
                OnPropertyChanged(NPCampeonato);
            }
        }

        public const string NPActivo = "Activo";
        private bool mActivo;

        [Campo]
        public bool Activo {
            get {
                PrePropertyRead(NPActivo);
                return mActivo;
            }
            set {
                PrePropertyChanged(NPActivo);
                mActivo = value;
                OnPropertyChanged(NPActivo);
            }
        }


        public const string NPLogo = "Logo";
        private System.Drawing.Image mLogo;
        private bool mLogoCargada = false;

        public System.Drawing.Image Logo {
            get {
                if (!mLogoCargada) {
                    mLogo = CentroImagenesE.Actual.GetImagen(this);
                    mLogoCargada = true;
                }
                return mLogo;
            }
            set {
                mLogo = value;
                mLogoCargada = true;
                OnPropertyChanged(NPLogo);
            }
        }

        public const string NPTemporada = "Temporada";
        private ETemporada mTemporada;

        [Campo("TemporadaId", Tamanyo = 50)]
        public ETemporada Temporada {
            get {
                PrePropertyRead(NPTemporada);
                return mTemporada;
            }
            set {
                PrePropertyChanged(NPTemporada);
                mTemporada = value;
                OnPropertyChanged(NPTemporada);
            }
        }

        #endregion

        #region ClasificacionesGenerales
        public const string NPClasificacionesGenerales = "ClasificacionesGenerales";
        private EColeccion<EClasificacionGeneralBase> mClasificacionesGenerales = null;

        /// <summary>Devuelve una coleccin con todos las ClasificacionesGenerales. El campeonato 
        /// tiene que ser una clase (Nivel 2)</summary>
        public EColeccion<EClasificacionGeneralBase> ClasificacionesGenerales {
            get {
                //if (this.Nivel != 2) {//TODO: revisar si hay que mandar clasificaciones a niveles superiores.
                //    return null;
                //}
                if(mClasificacionesGenerales == null) {
                    mClasificacionesGenerales = new EColeccion<EClasificacionGeneralBase>();
                    if (this.GetParticipanteAceptado() == TipoParticipanteAceptado.Pareja) {
                        EColeccion<EClasificacionGeneralPareja> col = new EColeccion<EClasificacionGeneralPareja>();
                        col.Filtros.Add(EClasificacionGeneralBase.NPCampeonato, this);
                        col.Filtros.Add(EClasificacionGeneralBase.NPTipoClasificaciongeneral, TipoClasificacionGeneral.Pareja);
                        col.Cargar();
                        foreach (EClasificacionGeneralPareja obj in col) {
                            mClasificacionesGenerales.Add(obj);
                        }
                    } else if (this.GetParticipanteAceptado() == TipoParticipanteAceptado.Piloto) {
                        EColeccion<EClasificacionGeneralPiloto> col = new EColeccion<EClasificacionGeneralPiloto>();
                        col.Filtros.Add(EClasificacionGeneralBase.NPCampeonato, this);
                        col.Filtros.Add(EClasificacionGeneralBase.NPTipoClasificaciongeneral, TipoClasificacionGeneral.Piloto);
                        col.Cargar();
                        foreach (EClasificacionGeneralPiloto obj in col) {
                            mClasificacionesGenerales.Add(obj);
                        }
                    } else if (this.GetParticipanteAceptado() == TipoParticipanteAceptado.Copiloto) {
                        EColeccion<EClasificacionGeneralCopiloto> col = new EColeccion<EClasificacionGeneralCopiloto>();
                        col.Filtros.Add(EClasificacionGeneralBase.NPTipoClasificaciongeneral, TipoClasificacionGeneral.Copiloto);
                        col.Filtros.Add(EClasificacionGeneralBase.NPCampeonato, this);
                        col.Cargar();
                        foreach (EClasificacionGeneralCopiloto obj in col) {
                            mClasificacionesGenerales.Add(obj);
                        }
                    }
                }
                return mClasificacionesGenerales;
            }
        }

        #endregion

        #region CampeonatosPuntuables
        private const string NPCampeonatosPuntuables = "CampeonatosPuntuables";
        private EColeccion<ECampeonatoPuntuable> mCampeonatosPuntuables = null;

        /// <summary>Devuelve una coleccin con todos los CampeonatosPuntuables.</summary>
        private EColeccion<ECampeonatoPuntuable> CampeonatosPuntuables {
            get {
                if(mCampeonatosPuntuables == null) {
                    mCampeonatosPuntuables = new EColeccion<ECampeonatoPuntuable>();
                    if(!this.EsNuevo) {
                        mCampeonatosPuntuables.Filtros.Add(ECampeonatoPuntuable.NPCampeonato, this);
                        mCampeonatosPuntuables.Cargar();
                    }
                }
                return mCampeonatosPuntuables;
            }
        }

        #endregion

        #region CampeonatosInscrito
        private const string NPCampeonatosInscrito = "CampeonatosInscrito";
        private EColeccion<ECampeonatoInscrito> mCampeonatosInscrito = null;

        /// <summary>Devuelve una coleccin con todos los CampeonatosInscrito.</summary>
        private EColeccion<ECampeonatoInscrito> CampeonatosInscrito {
            get {
                if(mCampeonatosInscrito == null) {
                    mCampeonatosInscrito = new EColeccion<ECampeonatoInscrito>();
                    if(!this.EsNuevo) {
                        mCampeonatosInscrito.Filtros.Add(ECampeonatoInscrito.NPCampeonato, this);
                        mCampeonatosInscrito.Cargar();
                    }
                }
                return mCampeonatosInscrito;
            }
        }

        #endregion

        #region Colecciones

        private static EColeccion<ECampeonato> GetCollection(int nivel, bool todasLasTemporadas) {
            return GetCollection(false, false, nivel, todasLasTemporadas);
        }

        private static EColeccion<ECampeonato> GetCollection(bool vacio, bool asincronamente, int nivel, bool todasLasTemporadas) {
            EColeccion<ECampeonato> col = new EColeccion<ECampeonato>();
            col.Filtros.Add(NPNivel, nivel);
            col.Filtros.Add(NPActivo, true);
            if (nivel == 0 && !todasLasTemporadas) {
                col.Filtros.Add(NPTemporada, ETemporada.Actual);
            }
            col.ColumnaMostrarRango(new string[] { NPNombre, NPAlias, NPCarreraPuntuable, NPParticipanteAceptado });
            if (!vacio)
                col.Cargar(asincronamente);
            col.Titulo = "CAMPEONATOS";
            return col;
        }

        /// <summary>
        /// Coleccin de campeonatos
        /// </summary>
        /// <param name="activo"></param>
        /// <param name="nivel"></param>
        /// <param name="todasLasTemporadas"></param>
        /// <returns></returns>
        public static EColeccion<ECampeonato> GetCollection(bool activo, int nivel, bool todasLasTemporadas) {
            EColeccion<ECampeonato> col = new EColeccion<ECampeonato>();
            col.Filtros.Add(NPNivel, nivel);
            col.Filtros.Add(NPActivo, activo);
            if (nivel == 0 && todasLasTemporadas) {
                col.Filtros.Add(NPTemporada, ETemporada.Actual);
            }
            col.ColumnaMostrarRango(new string[] { NPNombre, NPAlias, NPCarreraPuntuable, NPParticipanteAceptado });
            col.Cargar();
            col.Titulo = "CAMPEONATOS";
            return col;
        }

        #endregion

        /// <summary>
        /// Devuelve una coleccion de campeonatos.
        /// </summary>
        /// <param name="tco"></param>
        /// <returns></returns>
        public static EColeccion<ECampeonato> GetCampeonatos(bool todasLasTemporadas) {
            return GetCollection(0, todasLasTemporadas);
        }
        
        /// <summary>
        /// Devuelve una coleccion de campeonatos sin los campeonatos dados en la coleccin de campeonatos puntuables.
        /// </summary>
        /// <param name="col"></param>
        /// <returns></returns>
        public static EColeccion<ECampeonato> GetCampeonatos(EColeccion<ECampeonatoPuntuable> col) {
            EColeccion<ECampeonato> res = GetCollection(0, false);
            if (col != null) {
                foreach (ECampeonatoPuntuable cp in col) {
                    if (res.Contains(cp.Campeonato)) {
                        res.Remove(cp.Campeonato);
                    }
                }
            }
            return res;
        }

        private EColeccion<ECampeonato> mColeccionGrupos = null;
        /// <summary>
        /// Devuelve una coleccion de grupos.
        /// </summary>
        /// <param name="tco"></param>
        /// <returns></returns>
        public EColeccion<ECampeonato> GetGrupos() {
            //if (mColeccionGrupos != null) {
            //    return mColeccionGrupos;
            //}
            if (Nivel != 0) {
                AddError(new PropertyException("La entidad debe ser un Campeonato"));
            }
            EColeccion<ECampeonato> col = new EColeccion<ECampeonato>();
            col.Filtros.Add(NPCampeonato, this);
            col.ColumnaMostrarRango(new string[] { NPNombre });
            col.Cargar();
            col.Titulo = "Grupos";
            mColeccionGrupos = col;
            mColeccionGrupos.AddingNew += new AddingNewEventHandler(mColeccionGrupos_AddingNew);
            return col;
        }

        void mColeccionGrupos_AddingNew(object sender, AddingNewEventArgs e) {
            ECampeonato entidad = new ECampeonato();
            entidad.Nivel = 1;
            entidad.Campeonato = this;
            e.NewObject = entidad;
        }

        private EColeccion<ECampeonato> mColeccionClases = null;
        /// <summary>
        /// Devuelve una coleccion de clases.
        /// </summary>
        /// <param name="tco"></param>
        /// <returns></returns>
        public EColeccion<ECampeonato> GetClases() {
            //if (mColeccionClases != null) {
            //    return mColeccionClases;
            //}
            if (Nivel != 1) {
                AddError(new PropertyException("La entidad debe ser un Grupo"));
            }
            EColeccion<ECampeonato> col = new EColeccion<ECampeonato>();
            col.Filtros.Add(NPCampeonato, this);
            col.ColumnaMostrarRango(new string[] { NPNombre, NPCilindradaMinima, NPCilindradaMaxima });
            col.Cargar();
            col.Titulo = "Clases";
            mColeccionClases = col;
            mColeccionClases.AddingNew += new AddingNewEventHandler(mColeccionClases_AddingNew);
            return col;
        }

        void mColeccionClases_AddingNew(object sender, AddingNewEventArgs e) {
            ECampeonato entidad = new ECampeonato();
            entidad.Nivel = 2;
            entidad.Campeonato = this;
            e.NewObject = entidad;
        }

        #region IBaja

        public void DarAltaBaja(bool alta) {
            Activo = alta;
        }

        #endregion

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar Campeonato")) {
                if (CampeonatosInscrito.Count > 0) {
                    throw new Exception("No se puede eliminar. Hay inscritos que puntan en una o varias carreras.");
                }
                if (Nivel == 1) {//Es un grupo. Eliminamos sus clases si es posible.
                    EColeccion<ECampeonato> clases = this.GetClases();
                    clases.Clear();
                    clases.Guardar();
                } 
                if (Nivel == 0){
                    EColeccion<ECampeonato> grupos = this.GetGrupos();
                    grupos.Clear();
                    grupos.Guardar();
                    int indice = 0;
                    EColeccion<ECampeonatoPuntuable> colCampeonatosPuntuables = CampeonatosPuntuables;
                    while (indice < colCampeonatosPuntuables.Count) {
                        colCampeonatosPuntuables[indice].Eliminar();
                        indice = indice + 1;
                    }
                }
                base.Eliminar();
                transa.Commit();
            }
        }

        public override bool EstaSucio {
            get {
                return base.EstaSucio || (mColeccionGrupos != null && mColeccionGrupos.EstaSucio) || (mColeccionClases != null && mColeccionClases.EstaSucio);
            }
        }

        /// <summary>
        /// Devuelve los puntos que tiene el inscrito con el puesto pasado como parmetro. Si no se encuentra
        /// devuelve null.
        /// </summary>
        /// <param name="puesto"></param>Puesto de la clasificacin, debe de ser mayor que cero y como mucho el
        /// el nmero de inscritos en el campeonato.
        /// <returns></returns>
        internal int? GetPuntos(int puesto) {
            //if (this.Nivel != 2) {
            //    throw new Exception("Los puntos solo se obtienen si este campeonato es una clase.");
            //}
            EColeccion<EClasificacionGeneralBase> col = ClasificacionesGenerales;
            foreach (EClasificacionGeneralBase obj in col) {
                if (obj.Puesto == puesto) {
                    return obj.Puntos;
                }
            }
            return null;
        }

        /// <summary>
        /// Actualiza la clasificacin. La entidad que llama a este mtodo tiene que ser una clase (no grupo ni campeonato).
        /// </summary>
        internal void ActualizarClasificacion(){
            //if (this.Nivel != 2) {//no es una clase.
            //    throw new Exception("La clasificacin solo se actualiza si este campeonato es una clase.");
            //}
            EColeccion<EClasificacionGeneralBase> col = ClasificacionesGenerales;
            col.ApplySort(EClasificacionGeneralBase.NPPuntos, ListSortDirection.Descending);
            if (col.Count > 0) {
                EClasificacionGeneralBase primero = col[0];
                primero.PuntosPrimero = primero.Puntos;
                primero.Puesto = 1;
                primero.PuntosAnterior = null;
                int indice = 1;
                while (indice < col.Count) {
                    EClasificacionGeneralBase obj = col[indice];
                    indice = indice + 1;
                    obj.Puesto = indice;
                    obj.PuntosAnterior = col[indice - 1].Puntos;
                    obj.PuntosPrimero = primero.Puntos;
                }
            }
        }

        /// <summary>
        /// Para llamar a este mtodo esta clase tiene que ser una clase (Nivel = 2).
        /// </summary>
        /// <param name="inscrito"></param>
        /// <param name="colClasificacionesconpuntos"></param>
        internal void TraspasarPuntos(EPilotoCarrera inscrito, EColeccion<EClasificacionCarreraCampeonatoConPuntos> colClasificacionesconpuntos) {
            //if (this.Nivel != 2) {//SUBIMOS NIVELES.
            //    throw new Exception("Tiene que ser una clase para que funcione este mtodo.");
            //}
            EColeccion<EClasificacionGeneralBase> colgenerales = this.ClasificacionesGenerales;
            EClasificacionGeneralBase clasificacionGeneralATraspasar = null;
            bool terminado = false;
            int indice = 0;
            while (!terminado && indice < colgenerales.Count) {
                EClasificacionGeneralBase clasifgeneral = colgenerales[indice];
                if (this.GetParticipanteAceptado() == TipoParticipanteAceptado.Pareja) {
                    EPareja pareja = new EPareja(inscrito.Piloto, inscrito.Copiloto);
                    EClasificacionGeneralPareja clasificacionPareja = clasifgeneral as EClasificacionGeneralPareja;
                    if (clasificacionPareja == null) {
                        throw new Exception("FATAL ERROR");
                    }
                    if (clasificacionPareja.Pareja.Equals(pareja)) {
                        clasificacionGeneralATraspasar = clasificacionPareja;
                        terminado = true;
                    }
                } else {
                    EClasificacionGeneralParticipante clasificacionParticipante = clasifgeneral as EClasificacionGeneralParticipante;
                    if (clasificacionParticipante == null) {
                        throw new Exception("FATAL ERROR");
                    }
                    if ((this.GetParticipanteAceptado() == TipoParticipanteAceptado.Piloto && clasificacionParticipante.Participante.Equals(inscrito.Piloto))
                        || (this.GetParticipanteAceptado() == TipoParticipanteAceptado.Copiloto && clasificacionParticipante.Participante.Equals(inscrito.Copiloto))) {
                        terminado = true;
                        clasificacionGeneralATraspasar = clasificacionParticipante;
                    } 
                }
                indice = indice + 1;
            }
            if (!terminado) {
                //crear en la variable clasificacionGeneralATraspasar Y  agregarla a la coleccion.
                //tener en cuenta el participante aceptado.
                if (this.GetParticipanteAceptado() == TipoParticipanteAceptado.Pareja) {
                    EPareja pareja = new EPareja();
                    pareja.Piloto = inscrito.Piloto;
                    pareja.Copiloto = inscrito.Copiloto;
                    pareja.Guardar();
                    clasificacionGeneralATraspasar = new EClasificacionGeneralPareja(pareja);
                } else if (this.GetParticipanteAceptado() == TipoParticipanteAceptado.Piloto) {
                    clasificacionGeneralATraspasar = new EClasificacionGeneralPiloto(inscrito.Piloto);
                } else if (this.GetParticipanteAceptado() == TipoParticipanteAceptado.Copiloto) {
                    clasificacionGeneralATraspasar = new EClasificacionGeneralCopiloto(inscrito.Copiloto);
                }
                clasificacionGeneralATraspasar.Campeonato = this;
                this.ClasificacionesGenerales.Add(clasificacionGeneralATraspasar);
            }
            //TRASPASAR PUNTOS
            foreach (EClasificacionCarreraCampeonatoConPuntos objconpuntos in colClasificacionesconpuntos) {
                clasificacionGeneralATraspasar.Puntos = clasificacionGeneralATraspasar.Puntos + objconpuntos.Puntos;
            }
        }

        /// <summary>
        /// Devuelve el participante aceptado del campeonato.
        /// </summary>
        /// <returns></returns>
        private TipoParticipanteAceptado GetParticipanteAceptado() {
            if (this.Nivel == 0) {
                return this.ParticipanteAceptado;
            } else {
                return this.Campeonato.GetParticipanteAceptado();
            }
        }
    }
}
