using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    public enum TipoClasificacionGeneral {
        Piloto,
        Copiloto,
        Pareja
    }

    [Tabla("ClasificacionesGenerales")]
    public abstract class EClasificacionGeneralBase : EIdBase, IClasificacion {

        #region Constructores

        public EClasificacionGeneralBase() : base() { }

        public EClasificacionGeneralBase(int vId) : base(vId) { }

        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            mPuesto = 0;
            mPuntos = 0;
            mCampeonato = null;
        }

        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "ClasificaionGeneralBase"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPPuesto; }
        }
        #endregion

        #region Propiedades

        public const string NPTipoClasificaciongeneral = "TipoClasificaciongeneral";
        private TipoClasificacionGeneral mTipoClasificaciongeneral;

        [Campo("TipoClasificacionGeneral", Tamanyo = 50), Requerido]
        public TipoClasificacionGeneral TipoClasificaciongeneral {
            get {
                PrePropertyRead(NPTipoClasificaciongeneral);
                return mTipoClasificaciongeneral;
            }
            set {
                PrePropertyChanged(NPTipoClasificaciongeneral);
                mTipoClasificaciongeneral = value;
                OnPropertyChanged(NPTipoClasificaciongeneral);
            }
        }

        public const string NPPuesto = "Puesto";
        private int mPuesto;

        [Campo]
        public int Puesto {
            get {
                PrePropertyRead(NPPuesto);
                return mPuesto;
            }
            set {
                PrePropertyChanged(NPPuesto);
                mPuesto = value;
                OnPropertyChanged(NPPuesto);
            }
        }

        public const string NPPuntos = "Puntos";
        private int mPuntos;

        [Campo]
        public int Puntos {
            get {
                PrePropertyRead(NPPuntos);
                return mPuntos;
            }
            set {
                PrePropertyChanged(NPPuntos);
                mPuntos = value;
                OnPropertyChanged(NPPuntos);
            }
        }

        public const string NPCampeonato = "Campeonato";
        private ECampeonato mCampeonato;

        [Campo("CampeonatoId"), Requerido]
        public ECampeonato Campeonato {
            get {
                PrePropertyRead(NPCampeonato);
                return mCampeonato;
            }
            set {
                PrePropertyChanged(NPCampeonato);
                mCampeonato = value;
                OnPropertyChanged(NPCampeonato);
            }
        }

        public const string NPPuntosAnterior = "PuntosAnterior";
        private int? mPuntosAnterior;
        private bool mPuntosAnteriorCargado = false;

        public int? PuntosAnterior {
            get {
                if (!mPuntosAnteriorCargado) {
                    mPuntosAnterior = GetPuntos(this.Puesto - 1);
                    mPuntosAnteriorCargado = true;
                }
                return mPuntosAnterior;
            }
            set {
                mPuntosAnterior = value;
                mPuntosAnteriorCargado = true;
            }
        }

        public const string NPPuntosPrimero = "PuntosPrimero";
        private int? mPuntosPrimero;
        private bool mPuntosPrimeroCargado = false;

        public int? PuntosPrimero {
            get {
                if (!mPuntosPrimeroCargado) {
                    mPuntosPrimero = GetPuntos(1);
                    mPuntosPrimeroCargado = true;
                }
                return mPuntosPrimero;
            }
            set {
                mPuntosPrimero = value;
                mPuntosPrimeroCargado = true;
            }
        }

        #region Impresion

        public string DifPuntosPrimero {
            get {
                if (PuntosPrimero.HasValue) {
                    int diferencia = PuntosPrimero.Value - Puntos;
                    return diferencia.ToString();
                }
                return "..........";
            }
        }

        public string DifPuntosAnterior {
            get {
                if (PuntosAnterior.HasValue) {
                    int diferencia = PuntosAnterior.Value - Puntos;
                    return diferencia.ToString();
                }
                return "..........";
            }
        }

        public string NombreCampeonato {
            get {
                if (Campeonato != null && Campeonato.Nivel == 2) {
                    return Campeonato.Campeonato.Campeonato.Nombre + " - " + Campeonato.Campeonato.Nombre + " - " + Campeonato.Nombre;
                }
                return "..........";
            }
        }

        public abstract string NombreConcursante {get;}

        #endregion

        #endregion

        private int? GetPuntos(int puesto) {
            if (puesto > 0 && Campeonato != null && Campeonato.ClasificacionesGenerales.Count > puesto) {
                return Campeonato.GetPuntos(puesto);
            }
            return null;
        }

        #region IClasificacion

        public bool HaAbandonado() {
            return false;
        }

        public bool EsMostrable() {
            return true;
        }

        #endregion
    }
}

