using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    [Tabla("Parejas")]
    public class EPareja : EIdBase {

        #region Constructores

        public EPareja() : base() { }

        public EPareja(int vId) : base(vId) { }

        /// <summary>
        /// Carga una pareja con los participantes pasados como parmetro, si no se encuentra la instancia es nueva.
        /// </summary>
        /// <param name="piloto"></param>
        /// <param name="copiloto"></param>
        public EPareja(EPiloto piloto, ECopiloto copiloto) : this() {
            this.Cargar(NPPiloto, piloto, NPCopiloto, copiloto);    
        }

        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
        }

        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "Pareja"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return this.ToString(); }
        }

        #endregion

        #region Propiedades

        public const string NPPiloto = "Piloto";
        private EPiloto mPiloto;

        [Campo("PilotoId"), Requerido]
        public EPiloto Piloto {
            get {
                PrePropertyRead(NPPiloto);
                return mPiloto;
            }
            set {
                PrePropertyChanged(NPPiloto);
                mPiloto = value;
                OnPropertyChanged(NPPiloto);
            }
        }

        public const string NPCopiloto = "Copiloto";
        private ECopiloto mCopiloto;

        [Campo("CopilotoId"), Requerido]
        public ECopiloto Copiloto {
            get {
                PrePropertyRead(NPCopiloto);
                return mCopiloto;
            }
            set {
                PrePropertyChanged(NPCopiloto);
                mCopiloto = value;
                OnPropertyChanged(NPCopiloto);
            }
        }

        #endregion

        #region Colecciones

        public static EColeccion<EPareja> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EPareja> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EPareja> col = new EColeccion<EPareja>();
            col.ColumnaMostrarRango(new string[] { NPPiloto, NPCopiloto });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }
        #endregion

        public override string ToString() {
            return Piloto.ToString() + " - " + Copiloto.ToString();
        }

        public override bool Equals(object obj) {
            EPareja pareja = obj as EPareja;
            if (pareja != null) {
                return this.Piloto != null && this.Piloto.Equals(pareja.Piloto) && this.Copiloto != null && this.Copiloto.Equals(pareja.Copiloto);
            }
            return base.Equals(obj);
        }
    }
}
