using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;
using System.Collections.ObjectModel;

namespace Negocio {

    [Tabla("CampeonatosInscritos")]
    public class ECampeonatoInscrito : EIdBase {

        #region Constructores

        public ECampeonatoInscrito() : base() { }

        public ECampeonatoInscrito(int vId) : base(vId) { }

        public ECampeonatoInscrito(ECampeonato clase, EPilotoCarrera inscrito) : base() {
            PilotoCarrera = inscrito;
            Campeonato = clase;
        }

        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            mCampeonato = null;
            mPilotoCarrera = null;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Campeonato Inscrito")) {
                base.GuardarCore();
                //Creamos y guardamos las clasificaciones Carrera Campeonatos Oportunos.
                Collection<IEColeccion> mColeccionClasificacionesCampeonato = CrearClasificacionesCampeonatos();
                //comprobamos si hay que guardar las clasificaciones.
                if (mColeccionClasificacionesCampeonato != null) {
                    foreach (IEColeccion colClasificacion in mColeccionClasificacionesCampeonato) {
                        colClasificacion.Guardar();
                    }
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "CampeonatoInscrito"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get {
                if (Campeonato != null) {
                    return this.ToString();
                }
                return ""; 
            } 
        }
        #endregion

        #region Propiedades

        public const string NPCampeonato = "Campeonato";
        private ECampeonato mCampeonato;
        /// <summary>
        /// Representa una clase dentro del campeonato y ste est contenido en un campeonato puntuable en la carrera
        /// del inscrito.
        /// </summary>
        [Campo("ClaseId"), Requerido]
        public ECampeonato Campeonato {
            get {
                PrePropertyRead(NPCampeonato);
                return mCampeonato;
            }
            set {
                PrePropertyChanged(NPCampeonato);
                mCampeonato = value;
                //se comprueba que esta clase pertenece a un campeonato de los campeonatos puntuables de la carrera
                // de pilotoCarrera(inscrito).
                if (EstaOK())  {
                    RemoveError(NPCampeonato);
                } else {
                    RemoveError(NPCampeonato);
                    if (String.IsNullOrEmpty(mMotivoFallo)) {
                        mMotivoFallo = "Error en el campeonato. Propiedad: Campeonato.";
                    }
                    AddError(new PropertyException(NPCampeonato, this, mMotivoFallo));
                    mMotivoFallo = null;
                }
                OnPropertyChanged(NPCampeonato);
            }
        }

        public const string NPPilotoCarrera = "PilotoCarrera";
        private EPilotoCarrera mPilotoCarrera;

        [Campo("PilotoCarreraId"), Requerido]
        public EPilotoCarrera PilotoCarrera {
            get {
                PrePropertyRead(NPPilotoCarrera);
                return mPilotoCarrera;
            }
            set {
                PrePropertyChanged(NPPilotoCarrera);
                mPilotoCarrera = value;
                //comprobar la propiedad campeonato
                if (EstaOK()) {
                    RemoveError(NPCampeonato);
                } else {
                    RemoveError(NPCampeonato);
                    AddError(new PropertyException(NPCampeonato, this, "Error en el campeonato. Propiedad: Inscrito."));
                }
                OnPropertyChanged(NPPilotoCarrera);
            }
        }

        #endregion

        #region Colecciones
        public static EColeccion<ECampeonatoInscrito> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<ECampeonatoInscrito> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<ECampeonatoInscrito> col = new EColeccion<ECampeonatoInscrito>();
            col.ColumnaMostrarRango(new string[] {  });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }
        #endregion

        private string mMotivoFallo = null;
        private bool EstaOK() {
            ECampeonato campeonatoTHIS = null;
            if (mCampeonato != null && mCampeonato.Campeonato != null) {
                campeonatoTHIS = mCampeonato.Campeonato.Campeonato;
            }
            //primero nos aseguramos que el campeonato(que es un grupo) no se encuentra ya entre los campeonatosInscrito
            if (PilotoCarrera != null) {
                EColeccion<ECampeonatoInscrito> col = PilotoCarrera.CampeonatosInscrito;
                foreach (ECampeonatoInscrito ci in col) {
                    ECampeonato campeonatoCI = null;
                    //Recogemos los campeonatos generales de la entidad y del campeonatoInscrito.
                    if (ci.Campeonato != null && ci.Campeonato.Campeonato != null) {
                        campeonatoCI = ci.Campeonato.Campeonato.Campeonato;
                    }
                    //Comprobamos que no son iguales, si es as, ocurre un error.
                    if (!ci.Equals(this) && campeonatoCI != null && campeonatoTHIS != null && campeonatoTHIS.Equals(campeonatoCI)) {
                        mMotivoFallo = "El participante ya tiene asignado este campeonato.";
                        return false;
                    }
                }
            }//ahora comprobamos que el grupo tiene un campeonato principal entre los campeonatos puntuables de
            //la carrera de pilotoCarrera.
            
            if (campeonatoTHIS != null && PilotoCarrera != null && PilotoCarrera.Carrera != null) {
                EColeccion<ECampeonatoPuntuable> col = PilotoCarrera.Carrera.CampeonatosPuntuables;
                foreach (ECampeonatoPuntuable campPunt in col) {//comprobamos que el campeonato principal est entre los
                    if (campeonatoTHIS.Equals(campPunt.Campeonato)) {//campeonatos puntuables de la carrera.
                        return true;
                    }
                }
                mMotivoFallo = "El campeonato no se encuentra entre los campeonatos puntuables de la carrera.";
            }
            return false;
        }

        /// <summary>
        /// Devuelve true si la cilindrada del vehculo del inscrito est entre el mximo y el mnimo de la clase.
        /// Devuelve false en caso contrario.
        /// </summary>
        /// <returns></returns>
        public bool CilindradaOK() {
            if (mCampeonato != null && mPilotoCarrera != null) {
                EVehiculo vehiculo = mPilotoCarrera.Vehiculo;
                if (vehiculo != null) {
                    return vehiculo.Cilindrada >= mCampeonato.CilindradaMinima && vehiculo.Cilindrada <= mCampeonato.CilindradaMaxima;
                }
            }
            return false;
        }

        public override string ToString() {
            return mCampeonato.Campeonato.Campeonato.ToString() + " - " + mCampeonato.Campeonato.ToString() + " - " + mCampeonato.ToString();
        }

        internal bool ClasePertenece(ECampeonato c) {
            return Campeonato != null && Campeonato.Campeonato != null && c != null &&
                (c.Equals(Campeonato.Campeonato) || c.Equals(Campeonato.Campeonato.Campeonato));
        }

        private Collection<IEColeccion> CrearClasificacionesCampeonatos() {
            Collection<IEColeccion> mColeccionClasificacionesCampeonato = new Collection<IEColeccion>();
            EColeccion<ECronometrable> mColeccionCronometrable = this.PilotoCarrera.Carrera.Cronometrables;
            foreach (ECronometrable cronometrable in mColeccionCronometrable) {
                if (cronometrable.Tipo == TipoCronometrable.Manga) {
                    EColeccion<EClasificacionManga> col = cronometrable.ClasificacionesManga;
                    foreach (EClasificacionManga clasificacion in col) {
                        if (clasificacion.PilotoCarrera.Equals(this.PilotoCarrera)) {//primero con puntos
                            //comprobar si la manga es la oficial.
                            EManga mangaOficial = new EManga(clasificacion.Cronometrable.Id);
                            if (mangaOficial.TipoManga == TipoManga.MOfcial) {
                                mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonatoConPuntos(Campeonato, mangaOficial, clasificacion));
                            } else {
                                mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonato(Campeonato, clasificacion));
                            }
                            mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonato(Campeonato.Campeonato, clasificacion));
                            mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonato(Campeonato.Campeonato.Campeonato, clasificacion));
                        }
                    }
                } else if (cronometrable.Tipo == TipoCronometrable.Tramo) {
                    EColeccion<EClasificacionTramo> col = cronometrable.ClasificacionesTramo;
                    foreach (EClasificacionTramo clasificacion in col) {
                        if (clasificacion.PilotoCarrera.Equals(this.PilotoCarrera)) {
                            mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonato(Campeonato, clasificacion));
                            mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonato(Campeonato.Campeonato, clasificacion));
                            mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonato(Campeonato.Campeonato.Campeonato, clasificacion));
                        }
                    }
                }
            }
            if (this.PilotoCarrera.Carrera.Tipo == TipoCarrera.Rally) {
                ERally rally = new ERally(PilotoCarrera.Carrera.Id);
                if (rally.Existe) {
                    EColeccion<EClasificacionTotalRally> col = rally.ClasificacionesRally;
                    bool encontrado = false;
                    int indice = 0;
                    while (!encontrado && indice < col.Count) {
                        EClasificacionTotalRally clasificacion = col[indice];
                        if (clasificacion.PilotoCarrera.Equals(this.PilotoCarrera)) {//primero con puntos
                            mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonatoConPuntos(Campeonato, clasificacion));
                            mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonato(Campeonato.Campeonato, clasificacion));
                            mColeccionClasificacionesCampeonato.Add(CrearClasificacionCarreraCampeonato(Campeonato.Campeonato.Campeonato, clasificacion));
                            encontrado = true;
                        } else {
                            indice = indice + 1;
                        }
                    }
                }
            }
            return mColeccionClasificacionesCampeonato;
        }

        private EColeccion<EClasificacionCarreraCampeonatoConPuntos> CrearClasificacionCarreraCampeonatoConPuntos(ECampeonato campeonato, EManga mangaOficial, EClasificacionManga clasificacion) {
            EClasificacionCarreraCampeonatoConPuntos nuevo = new EClasificacionCarreraCampeonatoConPuntos();
            nuevo.Campeonato = campeonato;
            clasificacion.ClasificacionesCarreraCampeonatos.Add(nuevo);
            nuevo.ClasificacionCarrera = clasificacion;
            nuevo.Guardar();
            EColeccion<EClasificacionCarreraCampeonatoConPuntos> col = EClasificacionCarreraCampeonatoConPuntos.GetCollection(campeonato, mangaOficial);
            EClasificacionCarreraCampeonatoConPuntos.ActualizarClasificacion(col);
            return col;
        }

        private EColeccion<EClasificacionCarreraCampeonatoConPuntos> CrearClasificacionCarreraCampeonatoConPuntos(ECampeonato campeonato, EClasificacionTotalRally clasificacion) {
            EClasificacionCarreraCampeonatoConPuntos nuevo = new EClasificacionCarreraCampeonatoConPuntos();
            nuevo.Campeonato = campeonato;
            clasificacion.ClasificacionesCarreraCampeonatos.Add(nuevo);
            nuevo.ClasificacionCarrera = clasificacion;
            nuevo.Guardar();
            EColeccion<EClasificacionCarreraCampeonatoConPuntos> col = EClasificacionCarreraCampeonatoConPuntos.GetCollection(campeonato, clasificacion.Rally);
            EClasificacionCarreraCampeonatoConPuntos.ActualizarClasificacion(col);
            return col;
        }

        private EColeccion<EClasificacionCarreraCampeonato> CrearClasificacionCarreraCampeonato(ECampeonato campeonato, EClasificacionManga clasManga) {
            EClasificacionCarreraCampeonato nuevo = new EClasificacionCarreraCampeonato();
            nuevo.ClasificacionCarrera = clasManga;
            clasManga.ClasificacionesCarreraCampeonatos.Add(nuevo);
            nuevo.Campeonato = campeonato;
            nuevo.Guardar();
            EColeccion<EClasificacionCarreraCampeonato> col = EClasificacionCarreraCampeonato.GetCollection(campeonato, clasManga.Cronometrable);
            EClasificacionCarreraCampeonato.ActualizarClasificacion(col);
            return col;
        }

        private EColeccion<EClasificacionCarreraCampeonato> CrearClasificacionCarreraCampeonato(ECampeonato campeonato, EClasificacionTramo clasTramo) {
            EClasificacionCarreraCampeonato nuevo = new EClasificacionCarreraCampeonato();
            nuevo.ClasificacionCarrera = clasTramo;
            clasTramo.ClasificacionesCarreraCampeonatos.Add(nuevo);
            nuevo.Campeonato = campeonato;
            nuevo.Guardar();
            EColeccion<EClasificacionCarreraCampeonato> col = EClasificacionCarreraCampeonato.GetCollection(campeonato, clasTramo.Cronometrable);
            EClasificacionCarreraCampeonato.ActualizarClasificacion(col);
            return col;
        }

        private EColeccion<EClasificacionCarreraCampeonato> CrearClasificacionCarreraCampeonato(ECampeonato campeonato, EClasificacionTotalRally clasRally) {
            EClasificacionCarreraCampeonato nuevo = new EClasificacionCarreraCampeonato();
            nuevo.ClasificacionCarrera = clasRally;
            clasRally.ClasificacionesCarreraCampeonatos.Add(nuevo);
            nuevo.Campeonato = campeonato;
            EColeccion<EClasificacionCarreraCampeonato> col = EClasificacionCarreraCampeonato.GetCollection(campeonato, clasRally.Rally);
            EClasificacionCarreraCampeonato.ActualizarClasificacion(col);
            return col;
        }

        public override void Eliminar() {
            if (this.PilotoCarrera.Carrera.Tipo == TipoCarrera.Rally) {
                ERally rally = new ERally(this.PilotoCarrera.Carrera.Id);
                if (rally.Existe && rally.Tipo == TipoCarrera.Rally) {
                    ECampeonato campeonato = this.Campeonato;
                    while (campeonato != null) {
                        if (campeonato.Nivel == 2) {//con puntos
                            EColeccion<EClasificacionCarreraCampeonatoConPuntos> col = EClasificacionCarreraCampeonatoConPuntos.GetCollection(campeonato, rally);
                            EClasificacionCarreraCampeonatoConPuntos clasificacion = GetClasificacionConPuntos(PilotoCarrera, col);
                            col.Remove(clasificacion);
                            clasificacion.Eliminar();
                            EClasificacionCarreraCampeonatoConPuntos.ActualizarClasificacion(col);
                            col.Guardar();
                        } else {
                            EColeccion<EClasificacionCarreraCampeonato> col = EClasificacionCarreraCampeonato.GetCollection(campeonato, rally);
                            EClasificacionCarreraCampeonato clasificacion = GetClasificacion(PilotoCarrera, col);
                            col.Remove(clasificacion);
                            clasificacion.Eliminar();
                            EClasificacionCarreraCampeonato.ActualizarClasificacion(col);
                            col.Guardar();
                        }
                        campeonato = campeonato.Campeonato;
                    }
                }
            }
            EColeccion<ECronometrable> colCronometrable = this.PilotoCarrera.Carrera.Cronometrables;
            foreach (ECronometrable cronometrable in colCronometrable) {
                if (cronometrable.Tipo != TipoCronometrable.Sector) {
                    ECampeonato campeonato = this.Campeonato;
                    while (campeonato != null) {
                        bool eliminadoconpuntos = false;
                        if (cronometrable.Tipo == TipoCronometrable.Manga && campeonato.Nivel == 2) {
                            EManga mangaOficial = new EManga(cronometrable.Id);
                            if (mangaOficial.TipoManga == TipoManga.MOfcial) {
                                EColeccion<EClasificacionCarreraCampeonatoConPuntos> col = EClasificacionCarreraCampeonatoConPuntos.GetCollection(campeonato, mangaOficial);
                                EClasificacionCarreraCampeonatoConPuntos clasificacion = GetClasificacionConPuntos(PilotoCarrera, col);
                                col.Remove(clasificacion);
                                clasificacion.Eliminar();
                                EClasificacionCarreraCampeonatoConPuntos.ActualizarClasificacion(col);
                                col.Guardar();
                                eliminadoconpuntos = true;
                            }
                        }
                        if (!eliminadoconpuntos) {
                            EColeccion<EClasificacionCarreraCampeonato> col = EClasificacionCarreraCampeonato.GetCollection(campeonato, cronometrable);
                            EClasificacionCarreraCampeonato clasificacion = GetClasificacion(PilotoCarrera, col);
                            if (clasificacion != null) {
                                col.Remove(clasificacion);
                                clasificacion.Eliminar();
                            }
                            EClasificacionCarreraCampeonato.ActualizarClasificacion(col);
                            col.Guardar();
                        }
                        campeonato = campeonato.Campeonato;
                    }
                }
            }
            base.Eliminar();
        }

        /// <summary>
        /// Devuelve la primera aparicin de la clasificacion de campeonato de la coleccion dados el campeonato y el inscrito.
        /// </summary>
        /// <param name="campeonato"></param>
        /// <param name="PilotoCarrera"></param>
        /// <returns></returns>
        private EClasificacionCarreraCampeonato GetClasificacion(EPilotoCarrera inscrito, EColeccion<EClasificacionCarreraCampeonato> col) {
            foreach (EClasificacionCarreraCampeonato clasificacion in col) {
                if (clasificacion.ClasificacionCarrera.PilotoCarrera.Equals(inscrito)) {
                    return clasificacion;
                }
            }
            return null;
        }

        private EClasificacionCarreraCampeonatoConPuntos GetClasificacionConPuntos(EPilotoCarrera inscrito, EColeccion<EClasificacionCarreraCampeonatoConPuntos> col) {
            foreach (EClasificacionCarreraCampeonatoConPuntos clasificacion in col) {
                if (clasificacion.ClasificacionCarrera.PilotoCarrera.Equals(inscrito)) {
                    return clasificacion;
                }
            }
            return null;
        }
    }
}

