using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    public enum TipoCalculoPuntos {
        Coeficiente = 1,
        [Description("Tabla Fija")]
        TFija
    }

    [Tabla("CampeonatosPuntuables")]
    public class ECampeonatoPuntuable : EIdBase {

        #region Constructores
        public ECampeonatoPuntuable() : base() { }

        public ECampeonatoPuntuable(int vId) : base(vId) { }

        public ECampeonatoPuntuable(ECampeonato campeonato, ECarrera carrera) : base() {
            this.Cargar(NPCarrera, carrera, NPCampeonato, campeonato);
        }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            Carrera = null;
            Campeonato = null;
            TipoCalculoPuntos = TipoCalculoPuntos.Coeficiente;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar CampeonatoPuntuable")) {
                base.GuardarCore();
                if (mTipoCalculoPuntos == TipoCalculoPuntos.TFija) {
                    CalculadorTablaFija calcTabla = mCalculadorPuntos as CalculadorTablaFija;
                    if (calcTabla == null || calcTabla.Tabla.Count == 0) {
                        throw new Exception("Tiene que rellenar la tabla con puestos y puntos.");
                    }
                    if (calcTabla.TablaConPuestosDuplicados()) {
                        throw new Exception("La tabla tiene puestos duplicados.\nElimine los que sean innecesarios.");
                    }
                    calcTabla.Tabla.Guardar();
                } else {
                    CalculadorTablaFija calcTabla = new CalculadorTablaFija(this);
                    foreach (ETablaFija tfija in calcTabla.Tabla) {
                        tfija.Eliminar();
                    }
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            CalculadorTablaFija calcTabla = mCalculadorPuntos as CalculadorTablaFija;
            if (calcTabla != null) {
                calcTabla.Deshacer();
            }
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "CampeonatoPuntuable"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPCampeonato; } 
        }
        #endregion

        #region Propiedades

        public const string NPTipoCalculoPuntos = "TipoCalculoPuntos";
        private TipoCalculoPuntos mTipoCalculoPuntos;

        [Campo]
        public TipoCalculoPuntos TipoCalculoPuntos {
            get {
                PrePropertyRead(NPTipoCalculoPuntos);
                return mTipoCalculoPuntos;
            }
            set {
                PrePropertyChanged(NPTipoCalculoPuntos);
                mTipoCalculoPuntos = value;
                OnPropertyChanged(NPTipoCalculoPuntos);
            }
        }

        public const string NPCarrera = "Carrera";
        private ECarrera mCarrera;

        [Campo("CarreraId"), Requerido]
        public ECarrera Carrera {
            get {
                PrePropertyRead(NPCarrera);
                return mCarrera;
            }
            set {
                PrePropertyChanged(NPCarrera);
                mCarrera = value;
                OnPropertyChanged(NPCarrera);
            }
        }

        public const string NPCampeonato = "Campeonato";
        private ECampeonato mCampeonato;

        [Campo("CampeonatoId"), Requerido]
        public ECampeonato Campeonato {
            get {
                PrePropertyRead(NPCampeonato);
                return mCampeonato;
            }
            set {
                PrePropertyChanged(NPCampeonato);
                mCampeonato = value;
                OnPropertyChanged(NPCampeonato);
            }
        }

        public const string NPCalculadorPuntos = "CalculadorPuntos";
        private CalculadorPuntos mCalculadorPuntos = null;

        public CalculadorPuntos CalculadorPuntos {
            get {
                if (mCalculadorPuntos != null) {
                    if (mCalculadorPuntos.TipoCalculador != mTipoCalculoPuntos) {
                        if (mTipoCalculoPuntos == TipoCalculoPuntos.Coeficiente) {
                            mCalculadorPuntos = new CalculadorCoeficiente(this);
                        } else {
                            mCalculadorPuntos = new CalculadorTablaFija(this);
                        }
                    }
                } else {
                    if (mTipoCalculoPuntos == TipoCalculoPuntos.Coeficiente) {
                        mCalculadorPuntos = new CalculadorCoeficiente(this);
                    } else {
                        mCalculadorPuntos = new CalculadorTablaFija(this);
                    }
                }
                return mCalculadorPuntos;
            }
        }

        #endregion

        public override bool EstaSucio {
            get {
                bool mEstaTablaSucia = false;
                CalculadorTablaFija calcTabla = mCalculadorPuntos as CalculadorTablaFija;
                if (calcTabla != null) {
                    mEstaTablaSucia = calcTabla.Tabla.EstaSucio;
                }
                return base.EstaSucio || mEstaTablaSucia;
            }
        }

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar CampeonatoPuntuable")) {
                //si hay inscritos con este campeonato puntuable dar error.
                EColeccion<EPilotoCarrera> colPC = Carrera.PilotosCarrera;
                foreach (EPilotoCarrera pc in colPC) {
                    EColeccion<ECampeonatoInscrito> colCI = pc.CampeonatosInscrito;
                    foreach (ECampeonatoInscrito ci in colCI) {
                        if (ci.Campeonato.Campeonato.Campeonato.Equals(this.Campeonato)) {
                            throw new Exception("No se puede eliminar el campeonato puntuable.\nTiene inscripciones de otros participantes a este campeonato.\n\n(Elmine las inscripciones a este campeonato antes).");
                        }
                    }
                }
                //Eliminar tablas
                CalculadorTablaFija calcTabla = new CalculadorTablaFija(this);
                foreach (ETablaFija tfija in calcTabla.Tabla) {
                    tfija.Eliminar();
                }
                base.Eliminar();
                transa.Commit();
            }
        }
    }
}

