using System.ComponentModel;
using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    [Tabla("CargosOficialCarrera")]
    public class ECargo : EIdBase {
        #region Constructores
        public ECargo() : base() { }

        public ECargo(int vId) : base(vId) { }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            mCargo = 0;
            mOficial = null;
            mCarrera = null;
        }

        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "Cargo"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPCargo; }
        }
        #endregion

        #region Propiedades

        public const string NPCargo = "Cargo";
        private TipoCargoOficial mCargo;

        [Campo("Cargo")]
        public TipoCargoOficial Cargo {
            get {
                PrePropertyRead(NPCargo);
                return mCargo;
            }
            set {
                PrePropertyChanged(NPCargo);
                mCargo = value;
                OnPropertyChanged(NPCargo);
            }
        }

        public const string NPOficial = "Oficial";
        private EOficial mOficial;

        [Campo("PersonaId")]
        public EOficial Oficial {
            get {
                PrePropertyRead(NPOficial);
                return mOficial;
            }
            set {
                PrePropertyChanged(NPOficial);
                mOficial = value;
                OnPropertyChanged(NPOficial);
            }
        }

        public const string NPCarrera = "Carrera";
        private ECarrera mCarrera;

        [Campo("CarreraId")]
        public ECarrera Carrera {
            get {
                PrePropertyRead(NPCarrera);
                return mCarrera;
            }
            set {
                PrePropertyChanged(NPCarrera);
                mCarrera = value;
                OnPropertyChanged(NPCarrera);
            }
        }

        public const string NPEsPresidenteCD = "EsPresidenteCD";
        private bool mEsPresidenteCD;

        [Campo]
        public bool EsPresidenteCD {
            get {
                PrePropertyRead(NPEsPresidenteCD);
                return mEsPresidenteCD;
            }
            set {
                PrePropertyChanged(NPEsPresidenteCD);
                mEsPresidenteCD = value;
                if(this.EsPresidenteCD && this.Cargo != TipoCargoOficial.CDeportivo) {
                    AddError( new PropertyException(NPEsPresidenteCD, this, "El cargo debe ser de tipo comisario deportivo."));
                } else {
                    RemoveError(NPEsPresidenteCD);
                }
                OnPropertyChanged(NPEsPresidenteCD);
            }
        }

        public const string NPLicencia = "Licencia";
        private ELicenciaOficial mLicencia;

        [Campo("LicenciaNumeradaId")]
        public ELicenciaOficial Licencia {
            get {
                PrePropertyRead(NPLicencia);
                return mLicencia;
            }
            set {
                PrePropertyChanged(NPLicencia);
                mLicencia = value;
                //TODO: Validacin del dato: AddError()
                if(!Licencia.Cargo.Equals(Cargo) && !Licencia.Oficial.Equals(Oficial) && !Oficial.LicenciasOficial.Contains(Licencia)) {
                    AddError(new PropertyException(NPLicencia, this, "Esta Licencia debe pertenecer al oficial correspondiente."));
                } else {
                    RemoveError(NPLicencia);
                }
                OnPropertyChanged(NPLicencia);
            }
        }

        #endregion

        /// <summary>
        /// busqueda de ECargo dada una coleccin, si est devuelve la primera aparicin.
        /// </summary>
        /// <param name="tco"></param>
        /// <param name="col"></param>
        /// <param name="pres"></param>
        /// <returns></returns>
        public static ECargo GetCargo(TipoCargoOficial tco, EColeccion<ECargo> col, bool pres) {
            foreach(ECargo c in col) {
                if((pres && c.EsPresidenteCD) || (c.Cargo.Equals(tco))) {
                    return c;
                }
            }
            return null;
        }
        
        /// <summary>
        /// devuelve un subconjunto del conjunto dado en base a un Tipo Cargo Oficial
        /// </summary>
        /// <param name="tco"></param>
        /// <param name="col"></param>
        /// <returns></returns>
        public static EColeccion<ECargo> GetCollectionCargo(TipoCargoOficial tco, EColeccion<ECargo> col) {
            EColeccion<ECargo> res = new EColeccion<ECargo>();
            foreach(ECargo c in col) {
                if(c.Cargo.Equals(tco)) {
                    res.Add(c);
                }
            }
            return res;
        }
    }
}
