using System.ComponentModel;
using Nibi.Dal;
using Nibi.Negocio;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System;

namespace Negocio {

    public enum TipoCarrera {
        Ninguno = 0,
        Subida = 1,
        Rally
    }

    [Tabla("Carreras")]
    public class ECarrera : EIdBase, IQuitar, IAltaBaja {

        #region Constructores

        public ECarrera() : base() { }

        public ECarrera(int vId) : base(vId) { }

        public ECarrera(string alias)
            : this() {
            this.Cargar(NPAlias, alias);
        }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            mTipo = 0;
            Nombre = null;
            Alias = null;
            mOrganizador = null;
            mLocalidad = null;
            mProvincia = null;
            mCoeficiente = 0;
            mEstaCerrada = false;
            mEscuderiaOrganiza = null;
            mTemporada = ETemporada.Actual;
            Activo = true;
        }

        protected override void GuardarCore() {
            using(AmbitoTransa transa = this.CrearAmbitoTransa("Guardar diferentes colecciones (Carrera)")) {
                base.GuardarCore();
                //if(mCronometrables != null)      
                //    mCronometrables.Guardar();   
                if(mOficialesCarrera != null)
                    mOficialesCarrera.Guardar();
                if (mCampeonatosPuntuables != null)
                    mCampeonatosPuntuables.Guardar();
                if (mLogoCargada)
                    CentroImagenesE.Actual.AddImagen(this, Logo);
                if (mPilotosCarrera != null) {
                    mPilotosCarrera.Guardar();
                }
                if (mDocumentos != null) {
                    mDocumentos.Guardar();
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mCronometrables = null;
            mOficialesCarrera = null;
            mLogoCargada = false;
            mDocumentos = null;
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "Carrera"; }
        }

        public override bool Modificable(string nombreProp) {
            if(!EstaCargado) return true;

            switch(nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPNombre; }
        }
        #endregion

        #region Propiedades

        public const string NPTipo = "Tipo";
        private TipoCarrera mTipo;

        [Campo("Tipo")]
        public TipoCarrera Tipo {
            get {
                PrePropertyRead(NPTipo);
                return mTipo;
            }
            set {
                PrePropertyChanged(NPTipo);
                mTipo = value;
                OnPropertyChanged(NPTipo);
            }
        }

        public const string NPNombre = "Nombre";
        private string mNombre;

        [Campo(Tamanyo = 70), Requerido]
        public string Nombre {
            get {
                PrePropertyRead(NPNombre);
                return mNombre;
            }
            set {
                PrePropertyChanged(NPNombre);
                mNombre = value;
                OnPropertyChanged(NPNombre);
            }
        }

        public const string NPAlias = "Alias";
        private string mAlias;

        [Campo(Tamanyo = 10), Unico, Requerido]
        public string Alias {
            get {
                PrePropertyRead(NPAlias);
                return mAlias;
            }
            set {
                PrePropertyChanged(NPAlias);
                mAlias = value;
                OnPropertyChanged(NPAlias);
            }
        }

        public const string NPOrganizador = "Organizador";
        private string mOrganizador;

        [Campo(Tamanyo = 70)]
        public string Organizador {
            get {
                PrePropertyRead(NPOrganizador);
                return mOrganizador;
            }
            set {
                PrePropertyChanged(NPOrganizador);
                mOrganizador = value;
                //Si escuderiaOrganiza != null se pone a null
                if(mOrganizador != null) {
                    EscuderiaOrganiza = null;
                }
                OnPropertyChanged(NPOrganizador);
            }
        }

        //Escuderia organizadora
        public const string NPEscuderiaOrganiza = "EscuderiaOrganiza";
        private EEscuderia mEscuderiaOrganiza;

        [Campo("EscuderiaId", Tamanyo = 50)]
        public EEscuderia EscuderiaOrganiza {
            get {
                PrePropertyRead(NPEscuderiaOrganiza);
                return mEscuderiaOrganiza;
            }
            set {
                PrePropertyChanged(NPEscuderiaOrganiza);
                mEscuderiaOrganiza = value;
                //Si Organizador != null se pone a null
                if(mEscuderiaOrganiza != null) {
                    Organizador = null;
                }
                OnPropertyChanged(NPEscuderiaOrganiza);
            }
        }


        public const string NPLocalidad = "Localidad";
        private string mLocalidad;

        [Campo(Tamanyo = 50)]
        public string Localidad {
            get {
                PrePropertyRead(NPLocalidad);
                return mLocalidad;
            }
            set {
                PrePropertyChanged(NPLocalidad);
                mLocalidad = value;
                OnPropertyChanged(NPLocalidad);
            }
        }

        public const string NPProvincia = "Provincia";
        private string mProvincia;

        [Campo(Tamanyo = 50)]
        public string Provincia {
            get {
                PrePropertyRead(NPProvincia);
                return mProvincia;
            }
            set {
                PrePropertyChanged(NPProvincia);
                mProvincia = value;
                OnPropertyChanged(NPProvincia);
            }
        }

        public const string NPFechaComienzo = "FechaComienzo";
        private DateTime mFechaComienzo;

        [Campo, Requerido]
        public DateTime FechaComienzo {
            get {
                PrePropertyRead(NPFechaComienzo);
                return mFechaComienzo;
            }
            set {
                PrePropertyChanged(NPFechaComienzo);
                mFechaComienzo = value;
                OnPropertyChanged(NPFechaComienzo);
            }
        }

        public const string NPFechaFin = "FechaFin";
        private DateTime mFechaFin;

        [Campo, Requerido]
        public DateTime FechaFin {
            get {
                PrePropertyRead(NPFechaFin);
                return mFechaFin;
            }
            set {
                PrePropertyChanged(NPFechaFin);
                mFechaFin = value;
                OnPropertyChanged(NPFechaFin);
            }
        }

        public const string NPCoeficiente = "Coeficiente";
        private int mCoeficiente;

        [Campo]
        public int Coeficiente {
            get {
                PrePropertyRead(NPCoeficiente);
                return mCoeficiente;
            }
            set {
                PrePropertyChanged(NPCoeficiente);
                mCoeficiente = value;
                OnPropertyChanged(NPCoeficiente);
            }
        }

        public const string NPEstaCerrada = "EstaCerrada";
        private bool mEstaCerrada;

        [Campo]
        public bool EstaCerrada {
            get {
                PrePropertyRead(NPEstaCerrada);
                return mEstaCerrada;
            }
            set {
                PrePropertyChanged(NPEstaCerrada);
                mEstaCerrada = value;
                OnPropertyChanged(NPEstaCerrada);
            }
        }

        public const string NPLogo = "Logo";
        private System.Drawing.Image mLogo;
        private bool mLogoCargada = false;

        public System.Drawing.Image Logo {
            get {
                if (!mLogoCargada) {
                    mLogo = CentroImagenesE.Actual.GetImagen(this);
                    mLogoCargada = true;
                }
                return mLogo;
            }
            set {
                mLogo = value;
                mLogoCargada = true;
                OnPropertyChanged(NPLogo);
            }
        }

        public const string NPActivo = "Activo";
        private bool mActivo;

        [Campo]
        public bool Activo {
            get {
                PrePropertyRead(NPActivo);
                return mActivo;
            }
            set {
                PrePropertyChanged(NPActivo);
                mActivo = value;
                OnPropertyChanged(NPActivo);
            }
        }

        public const string NPTemporada = "Temporada";
        private ETemporada mTemporada;

        [Campo("TemporadaId")]
        public ETemporada Temporada {
            get {
                PrePropertyRead(NPTemporada);
                return mTemporada;
            }
            set {
                PrePropertyChanged(NPTemporada);
                mTemporada = value;
                OnPropertyChanged(NPTemporada);
            }
        }

        #endregion

        #region PilotosCarrera
        public const string NPPilotosCarrera = "PilotosCarrera";
        private EColeccion<EPilotoCarrera> mPilotosCarrera = null;

        /// <summary>Devuelve una coleccin con todos los PilotosCarrera.</summary>
        public EColeccion<EPilotoCarrera> PilotosCarrera {
            get {
                if(mPilotosCarrera == null) {
                    mPilotosCarrera = new EColeccion<EPilotoCarrera>();
                    if(!this.EsNuevo) {
                        mPilotosCarrera.Filtros.Add(EPilotoCarrera.NPCarrera, this);
                        mPilotosCarrera.Cargar();
                    }
                    mPilotosCarrera.AddingNew += new AddingNewEventHandler(mPilotosCarrera_AddingNew);
                    mPilotosCarrera.ColumnaMostrarRango(new string[] {EPilotoCarrera.NPDorsal, EPilotoCarrera.NPEstado, EPilotoCarrera.NPPiloto, EPilotoCarrera.NPVehiculo });
                    if (NecesitaCopiloto()) {
                        mPilotosCarrera.ColumnaMostrar(EPilotoCarrera.NPCopiloto);
                    }
                    mPilotosCarrera.Titulo = "INSCRITOS";
                }
                return mPilotosCarrera;
            }
        }

        private void mPilotosCarrera_AddingNew(object sender, AddingNewEventArgs e) {
            EPilotoCarrera entidad = new EPilotoCarrera();
            entidad.Carrera = this;
            e.NewObject = entidad;
        }

        #endregion

        #region OficialesCarrera
        public const string NPOficialesCarrera = "OficialesCarrera";
        private EColeccion<ECargo> mOficialesCarrera = null;

        /// <summary>Devuelve una coleccin con todos los OficialesCarrera.</summary>
        public EColeccion<ECargo> OficialesCarrera {
            get {
                if(mOficialesCarrera == null) {
                    mOficialesCarrera = new EColeccion<ECargo>("OFICIALES DE " + this.Nombre);
                    if(!this.EsNuevo) {
                        mOficialesCarrera.Filtros.Add(ECargo.NPCarrera, this);
                        mOficialesCarrera.Cargar();
                    }
                    mOficialesCarrera.AddingNew += new AddingNewEventHandler(mOficialesCarrera_AddingNew);
                }
                return mOficialesCarrera;
            }
        }

        private void mOficialesCarrera_AddingNew(object sender, AddingNewEventArgs e) {
            ECargo entidad = new ECargo();
            entidad.Carrera = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Cronometrables

        public const string NPCronometrables = "Cronometrables";
        private EColeccion<ECronometrable> mCronometrables = null;

        /// <summary>Devuelve una coleccin con todos los Cronometrables.</summary>
        //public abstract EColeccion<ECronometrable> Cronometrables { get;} (IEColeccion)
        public EColeccion<ECronometrable> Cronometrables {
            get {
                if (mCronometrables == null) {
                    mCronometrables = new EColeccion<ECronometrable>();
                    //switch (this.Tipo) {
                    //    case TipoCarrera.Ninguno:
                    //        return null;
                    //    case TipoCarrera.Subida:
                    //        ESubida subida = new ESubida(this.Id);
                    //        if (subida.Existe) {
                    //            EColeccion<EManga> col = subida.Mangas;
                    //            foreach (EManga m in col) {
                    //                mCronometrables.Add(m);
                    //            }
                    //        }
                    //        break;
                    //    case TipoCarrera.Rally:
                    //        ERally rally = new ERally(this.Id);
                    //        if (rally.Existe) {
                    //            EColeccion<ESector> col = rally.Sectores;
                    //            foreach (ESector sector in col) {
                    //                mCronometrables.Add(sector);
                    //                if (sector.Tramo != null) {
                    //                    mCronometrables.Add(sector.Tramo);
                    //                }
                    //            }
                    //        }
                    //        break;
                    //    default:
                    //        return null;
                    //}
                    if (!this.EsNuevo) {
                        mCronometrables.Filtros.Add(ECronometrable.NPCarrera, this);
                        mCronometrables.Cargar();
                    }
                }
                return mCronometrables;
            }
        }

        #endregion

        #region CampeonatosPuntuables
        public const string NPCampeonatosPuntuables = "CampeonatosPuntuables";
        private EColeccion<ECampeonatoPuntuable> mCampeonatosPuntuables = null;

        /// <summary>Devuelve una coleccin con todos los CampeonatosPuntuables.</summary>
        public EColeccion<ECampeonatoPuntuable> CampeonatosPuntuables {
            get {
                if(mCampeonatosPuntuables == null) {
                    mCampeonatosPuntuables = new EColeccion<ECampeonatoPuntuable>();
                    if(!this.EsNuevo) {
                        mCampeonatosPuntuables.Filtros.Add(ECampeonatoPuntuable.NPCarrera, this);
                        mCampeonatosPuntuables.Titulo = "Campeonatos Puntuables";
                        mCampeonatosPuntuables.Cargar();
                    }
                    mCampeonatosPuntuables.AddingNew += new AddingNewEventHandler(mCampeonatosPuntuables_AddingNew);
                    mCampeonatosPuntuables.ColumnaMostrarRango(new string[] {ECampeonatoPuntuable.NPCampeonato, ECampeonatoPuntuable.NPTipoCalculoPuntos });
                }
                return mCampeonatosPuntuables;
            }
        }

        private void mCampeonatosPuntuables_AddingNew(object sender, AddingNewEventArgs e) {
            ECampeonatoPuntuable entidad = new ECampeonatoPuntuable();
            entidad.Carrera = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Documentos
        public const string NPDocumentos = "Documentos";
        private EColeccion<EDocumento> mDocumentos = null;

        /// <summary>Devuelve una coleccin con todos los Documentos.</summary>
        public EColeccion<EDocumento> Documentos {
            get {
                if(mDocumentos == null) {
                    mDocumentos = new EColeccion<EDocumento>();
                    //Primero creamos los tipos de documentos, los cargamos y luego los unimos todos en una coleccin
                    //de EDocumento.
                    //DOCUMENTOS BRIEFING
                    EColeccion<EDocListaBriefing> mcolbriefing = new EColeccion<EDocListaBriefing>();
                    mcolbriefing.Filtros.Add(EDocumento.NPTipoDocumento, TipoDocumento.AsistenciaBriefing);
                    mcolbriefing.Cargar();
                    foreach (EDocListaBriefing documento in mcolbriefing) {
                        mDocumentos.Add(documento);
                    }
                    //DOCUMENTOS DE VERIFICACION TECNICA
                    //EColeccion<EDocVerificacionTecnica> mcolVerifTecnica = new EColeccion<EDocVerificacionTecnica>();
                    //mcolVerifTecnica.Filtros.Add(EDocumento.NPTipoDocumento, TipoDocumento.VerificacionesTecnicas);
                    //mcolVerifTecnica.Cargar();
                    //foreach (EDocVerificacionTecnica documento in mcolVerifTecnica) {
                    //    mDocumentos.Add(documento);
                    //}
                    //DOCUMENTOS DE LISTA AUTORIZADOS
                    //EColeccion<EDocListaAutorizados> mcolAutorizados = new EColeccion<EDocListaAutorizados>();
                    //mcolAutorizados.Filtros.Add(EDocumento.NPTipoDocumento, TipoDocumento.ListaAutorizados);
                    //mcolAutorizados.Cargar();
                    //foreach (EDocListaAutorizados documento in mcolAutorizados) {
                    //    mDocumentos.Add(documento);
                    //}
                    //DOCUMENTOS INFORME FINAL
                    EColeccion<EDocInformeFinal> mcolInformeFinal = new EColeccion<EDocInformeFinal>();
                    mcolInformeFinal.Filtros.Add(EDocumento.NPTipoDocumento, TipoDocumento.InformeFinal);
                    mcolInformeFinal.Cargar();
                    foreach (EDocInformeFinal documento in mcolInformeFinal) {
                        mDocumentos.Add(documento);
                    }
                }
                mDocumentos.ColumnaMostrarRango(new string[] { EDocumento.NPTipoDocumento, EDocumento.NPCodigo, EDocumento.NPFecha });
                return mDocumentos;
            }
        }

        #endregion

        #region Imagenes
        public const string NPImagenes = "Imagenes";
        private EColeccion<EImagen> mImagenes = null;

        /// <summary>Devuelve una coleccin con todos los Imagenes.</summary>
        public EColeccion<EImagen> Imagenes {
            get {
                if(mImagenes == null) {
                    mImagenes = new EColeccion<EImagen>();
                    if(!this.EsNuevo) {
                        mImagenes.Filtros.Add(EImagen.NPCarrera, this);
                        mImagenes.Cargar();
                    }
                    mImagenes.AddingNew += new AddingNewEventHandler(mImagenes_AddingNew);
                }
                return mImagenes;
            }
        }

        private void mImagenes_AddingNew(object sender, AddingNewEventArgs e) {
            EImagen entidad = new EImagen();
            entidad.Carrera = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Colecciones

        public static EColeccion<ECarrera> GetCollection(bool todasLasTemporadas) {
            return GetCollection(false, false, todasLasTemporadas);
        }

        public static EColeccion<ECarrera> GetCollection(bool vacio, bool asincronamente, bool todasLasTemporadas) {
            EColeccion<ECarrera> col = new EColeccion<ECarrera>();
            col.Titulo = "CARRERAS";
            //col.Filtros.Add(NPTemporada, ETemporada.Actual);
            col.ColumnaMostrarRango(new string[] { NPTipo, NPNombre, NPAlias, NPFechaComienzo, NPFechaFin, NPLocalidad, NPProvincia });
            if (!vacio) {
                EColeccion<ESubida> colSubida = ESubida.GetCollection(todasLasTemporadas);
                EColeccion<ERally> colRally = ERally.GetCollection(todasLasTemporadas);
                foreach (ESubida s in colSubida) {
                    col.Add(s);
                }
                foreach (ERally r in colRally) {
                    col.Add(r);
                }
                //col.Cargar(asincronamente);
            }
            return col;
        }

        public static EColeccion<ECarrera> GetCollection(bool activo, bool todasLasTemporadas) {
            EColeccion<ECarrera> col = new EColeccion<ECarrera>();
            col.Titulo = "CARRERAS";
            //col.Filtros.Add(NPTemporada, ETemporada.Actual);
            col.ColumnaMostrarRango(new string[] { NPTipo, NPNombre, NPAlias, NPFechaComienzo, NPFechaFin, NPLocalidad, NPProvincia });
            EColeccion<ESubida> colSubida = ESubida.GetCollection(activo, todasLasTemporadas);
            EColeccion<ERally> colRally = ERally.GetCollection(activo, todasLasTemporadas);
            foreach (ESubida s in colSubida) {
                col.Add(s);
            }
            foreach (ERally r in colRally) {
                col.Add(r);
            }
            return col;
        }

        /// <summary>
        /// Devuelve todas las carreras que no se encuentran en la coleccin pasada como parmetro.
        /// </summary>
        /// <param name="p"></param>
        /// <param name="eColeccion"></param>
        /// <returns></returns>
        public static EColeccion<ECarrera> GetCollection(bool p, EColeccion<ECarrera> eColeccion) {
            EColeccion<ECarrera> col = ECarrera.GetCollection(p);
            col.Titulo = "CARRERAS";
            if (eColeccion != null) {
                foreach (ECarrera carrera in eColeccion) {
                    bool terminado = false;
                    int indice = 0;
                    ECarrera carreraAEliminar = null;
                    while (!terminado && indice < col.Count) {
                        carreraAEliminar = col[indice];
                        terminado = carreraAEliminar.Id == carrera.Id;
                        indice = indice + 1;
                    }
                    col.Remove(carreraAEliminar);
                }
            }
            return col;
        }

        #endregion

        public virtual bool NecesitaCopiloto(){
            return Tipo == TipoCarrera.Rally;
        }

        public EColeccion<EOficial> ExtraerOficiales() {
            EColeccion<ECargo> col = this.OficialesCarrera;
            EColeccion<EOficial> res = new EColeccion<EOficial>();
            foreach(ECargo cargo in col) {
                if(cargo.Oficial != null) {
                    res.Add(cargo.Oficial);
                }
            }
            return res;
        }

        public bool EstaInscrito(ESinClaveBase o) {
            EColeccion<EPilotoCarrera> col = this.PilotosCarrera;
            int contador = 0;//numero de veces de la inscripcin
            foreach (EPilotoCarrera pilotoCarrera in col) {
                EPiloto piloto = o as EPiloto;
                ECopiloto copiloto = o as ECopiloto;
                if ((piloto != null && piloto.Equals(pilotoCarrera.Piloto)) || (copiloto != null && copiloto.Equals(pilotoCarrera.Copiloto))) {
                    if (contador == 1) {//hemos encontrado que la persona est inscrita ms de una vez.
                        return true;
                    } else {
                        contador = contador + 1;
                    }
                }
            }
            return false;
        }

        #region IQuitar

        public void Quitar() {
            EscuderiaOrganiza = null;
        }

        #endregion

        #region IBaja

        public void DarAltaBaja(bool alta) {
            Activo = alta;
        }

        #endregion

        public override bool EstaSucio {
            get {
                return base.EstaSucio || (mCampeonatosPuntuables != null && mCampeonatosPuntuables.EstaSucio) || (mCronometrables != null && mCronometrables.EstaSucio) ||
                    (mOficialesCarrera != null && mOficialesCarrera.EstaSucio) || (mPilotosCarrera != null && mPilotosCarrera.EstaSucio);
            }
        }

        /// <summary>
        /// Devuelve una coleccin de carreras cuya fechas de comienzo estn entre el intervalo pasado
        /// como parmetro.
        /// </summary>
        /// <param name="inicio"></param>
        /// <param name="fin"></param>
        /// <returns></returns>
        public static EColeccion<ECarrera> GetCarreras(DateTime inicio, DateTime fin) {
            EColeccion<ECarrera> col = new EColeccion<ECarrera>();
            EColeccion<ECarrera> todas = ECarrera.GetCollection(true);
            foreach (ECarrera carrera in todas) {
                if (carrera.FechaComienzo.CompareTo(inicio) >= 0 && carrera.FechaComienzo.CompareTo(fin) <= 0) {
                    col.Add(carrera);
                }
            }
            return col;
        }

        //public abstract void Cerrar();
        public void Cerrar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("CERRAR CARRERA")) {
                if (EstaCerrada) {
                    throw new Exception("La carrera ya est cerrada, no se puede realizar esta accin.");
                }
                Collection<KeyValuePair<ECampeonato, EColeccion<EPilotoCarrera>>> colkvpCI = new Collection<KeyValuePair<ECampeonato, EColeccion<EPilotoCarrera>>>();
                EColeccion<EPilotoCarrera> colinscritos = PilotosCarrera;
                foreach (EPilotoCarrera objInscrito in colinscritos) {
                    if (objInscrito.Estado == TipoEstado.Autorizado) {
                        EColeccion<ECampeonatoInscrito> coleccionci = objInscrito.CampeonatosInscrito;
                        foreach (ECampeonatoInscrito objci in coleccionci) {
                            KeyValuePair<ECampeonato, EColeccion<EPilotoCarrera>> kvp = new KeyValuePair<ECampeonato, EColeccion<EPilotoCarrera>>();
                            int indice = 0;
                            bool encontrado = false;
                            while (!encontrado && indice < colkvpCI.Count) {
                                kvp = colkvpCI[indice];
                                encontrado = kvp.Key.Equals(objci.Campeonato);
                                indice = indice + 1;
                            }
                            if (!encontrado) {
                                kvp = new KeyValuePair<ECampeonato, EColeccion<EPilotoCarrera>>(objci.Campeonato, new EColeccion<EPilotoCarrera>());
                                colkvpCI.Add(kvp);
                            }
                            kvp.Value.Add(objInscrito);
                        }
                    }
                }
                //en colkvpCI estan todos los campeonatos de inscrito no repetidos.
                //por cada CI hay que coger sus clasificaciones y sumar las de CCcampeonatoConPuntos.
                foreach (KeyValuePair<ECampeonato, EColeccion<EPilotoCarrera>> objkvp in colkvpCI) {
                    foreach (EPilotoCarrera inscrito in objkvp.Value) {
                        EColeccion<EClasificacionCarreraCampeonatoConPuntos> colClasificacionesconpuntos = new EColeccion<EClasificacionCarreraCampeonatoConPuntos>();//esta coleccion es por si tiene ms de una fase la subida y sumar todos los puntos.
                        if (this.Tipo == TipoCarrera.Rally) {
                            EClasificacionCarreraCampeonatoConPuntos objConpuntos = inscrito.Clasificaciones[0].GetClasificacionCarreraCampeonatoConPuntos(objkvp.Key);
                            if (objConpuntos == null) {
                                throw new Exception("FATAL ERROR");
                            }
                            if (!objConpuntos.HaAbandonado()) {
                                colClasificacionesconpuntos.Add(objConpuntos);
                            }
                        } else if (this.Tipo == TipoCarrera.Subida) {
                            ESubida subida = new ESubida(this.Id);
                            EColeccion<EManga> mangas = subida.Mangas;
                            foreach (EManga manga in mangas) {
                                if (manga.TipoManga == TipoManga.MOfcial) {
                                    EClasificacionCarrera clasificacionCarrera = manga.GetClasificacion(inscrito);
                                    if (clasificacionCarrera == null) {
                                        throw new Exception("FATAL ERROR");
                                    }
                                    EClasificacionCarreraCampeonatoConPuntos objConpuntos = clasificacionCarrera.GetClasificacionCarreraCampeonatoConPuntos(objkvp.Key);
                                    if (objConpuntos == null) {
                                        throw new Exception("FATAL ERROR");
                                    }
                                    if (!objConpuntos.HaAbandonado()) {
                                        colClasificacionesconpuntos.Add(objConpuntos);
                                    }
                                }
                            }
                        } else {
                            throw new Exception("FATAL ERROR");
                        }
                        //traspaso de puntos
                        if (colClasificacionesconpuntos.Count > 0) {
                            //objkvp.Key es una clase
                            objkvp.Key.TraspasarPuntos(inscrito, colClasificacionesconpuntos);
                            objkvp.Key.Campeonato.TraspasarPuntos(inscrito, colClasificacionesconpuntos);
                            objkvp.Key.Campeonato.Campeonato.TraspasarPuntos(inscrito, colClasificacionesconpuntos);
                        }
                    }
                    objkvp.Key.ActualizarClasificacion();
                    objkvp.Key.Campeonato.ActualizarClasificacion();
                    objkvp.Key.Campeonato.Campeonato.ActualizarClasificacion();
                    objkvp.Key.Guardar();
                    objkvp.Key.Campeonato.Guardar();
                    objkvp.Key.Campeonato.Campeonato.Guardar();
                }
                EstaCerrada = true;
                this.Guardar();
                transa.Commit();
            }
        }

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar carrera")) {
                if (EstaCerrada) {
                    throw new Exception("No se puede eliminar. La carrera ya est cerrada");
                }
                int indice = 0;
                EColeccion<ECargo> colCargos = OficialesCarrera;
                while (indice < colCargos.Count) {
                    colCargos[indice].Eliminar();
                    indice = indice + 1;
                }
                indice = 0;
                EColeccion<ECampeonatoPuntuable> colCampPunt = CampeonatosPuntuables;
                while (indice < colCampPunt.Count) {
                    colCampPunt[indice].Eliminar();
                    indice = indice + 1;
                }
                transa.Commit();
            }
            base.Eliminar();
        }

        /// <summary>
        /// Devuelve todos los campeonatosInscrito de los diferentes pilotosCarrera
        /// </summary>
        /// <returns></returns>
        public EColeccion<ECampeonatoInscrito> GetCampeonatosInscrito() {
            EColeccion<ECampeonatoInscrito> resultado = new EColeccion<ECampeonatoInscrito>();
            EColeccion<ECampeonato> colCampeonatos = new EColeccion<ECampeonato>();
            EColeccion<EPilotoCarrera> colInscritos = this.PilotosCarrera;
            foreach (EPilotoCarrera inscrito in colInscritos) {
                EColeccion<ECampeonatoInscrito> colCampInscritos = inscrito.CampeonatosInscrito;
                foreach (ECampeonatoInscrito ci in colCampInscritos) {
                    if (!colCampeonatos.Contains(ci.Campeonato)) {
                        resultado.Add(ci);
                        colCampeonatos.Add(ci.Campeonato);
                    }
                }
            }
            return resultado;
        }
    }
}