using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    public enum TipoCronometrable {
        Ninguno = 0,
        Manga = 1,
        Sector = 2,
        Tramo = 3
    }

    [Tabla("Cronometrables")]
    public class ECronometrable : EIdBase {

        #region Constructores

        public ECronometrable() : base() { }

        public ECronometrable(int vId) : base(vId) { }

        public ECronometrable(double longitud) : base() {
            Longitud = longitud;
        }

        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            Longitud = 0.0;
            EstaCerrado = false;
            EsAcumulable = false;
            Tipo = TipoCronometrable.Ninguno;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar cronometrable.")) {
                base.GuardarCore();
                if (mClasificacionesManga != null) {
                    ActualizarClasificacion();
                    mClasificacionesManga.Guardar();
                    //this.ActualizarYGuardarClasficacionesCampeonato();
                }
                if (mClasificacionesTramo != null) {
                    ActualizarClasificacion();
                    mClasificacionesTramo.Guardar();
                    //this.ActualizarYGuardarClasficacionesCampeonato();
                    ERally rally = new ERally(this.Carrera.Id);
                    if (rally.Existe && rally.Tipo == TipoCarrera.Rally) {
                        rally.ActualizarClasificacion();
                        //foreach (EClasificacionTotalRally clasificacionRally in rally.ClasificacionesRally) {
                        //    clasificacionRally.ActualizarClasificacionesCampeonato();
                        //}
                        rally.ClasificacionesRally.Guardar();
                        rally.ActualizarYGuardarClasificacionCarreraCampeonato();
                    }
                }
                if (mTiemposPilotoSector != null) {
                    mTiemposPilotoSector.Guardar();
                }
                transa.Commit();
            }
            if (mClasificacionesManga != null || mClasificacionesTramo != null) {
                this.ActualizarYGuardarClasficacionesCampeonato();
            }
        }

        public override void Deshacer() {
            mTiemposPilotoSector = null;
            mClasificacionesTramo = null;
            mClasificacionesManga = null;
            base.Deshacer();
        }


        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "Cronometrable"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPNombreCronometrable; } 
        }
        #endregion

        #region Propiedades

        public const string NPNombreCronometrable = "NombreCronometrable";
        private string mNombreCronometrable;
        private bool mNombreCargado = false;

        public string NombreCronometrable {
            get {
                if (!mNombreCargado) {
                    mNombreCronometrable = this.ToString();
                    mNombreCargado = true;
                }
                return mNombreCronometrable;
            }
        }


        public const string NPLongitud = "Longitud";
        private double mLongitud;

        [Campo]
        public double Longitud {
            get {
                PrePropertyRead(NPLongitud);
                return mLongitud;
            }
            set {
                PrePropertyChanged(NPLongitud);
                mLongitud = value;
                OnPropertyChanged(NPLongitud);
            }
        }

        public const string NPEstaCerrado = "EstaCerrado";
        private bool mEstaCerrado;

        [Campo]
        public bool EstaCerrado {
            get {
                PrePropertyRead(NPEstaCerrado);
                return mEstaCerrado;
            }
            set {
                PrePropertyChanged(NPEstaCerrado);
                mEstaCerrado = value;
                OnPropertyChanged(NPEstaCerrado);
            }
        }

        public const string NPEsAcumulable = "EsAcumulable";
        private bool mEsAcumulable;

        [Campo]
        public bool EsAcumulable {
            get {
                PrePropertyRead(NPEsAcumulable);
                return mEsAcumulable;
            }
            set {
                PrePropertyChanged(NPEsAcumulable);
                mEsAcumulable = value;
                OnPropertyChanged(NPEsAcumulable);
            }
        }

        public const string NPCarrera = "Carrera";
        private ECarrera mCarrera;

        [Campo("CarreraId")]
        public ECarrera Carrera {
            get{
                PrePropertyRead(NPCarrera);
                return mCarrera;
            }
            set{
            PrePropertyChanged(NPCarrera);
            mCarrera = value;
            OnPropertyChanged(NPCarrera);
            }
        }
        //public abstract ECarrera Carrera { get;}

        public const string NPTipo = "Tipo";
        private TipoCronometrable mTipo;

        [Campo]
        public TipoCronometrable Tipo {
            get {
                PrePropertyRead(NPTipo);
                return mTipo;
            }
            set {
                PrePropertyChanged(NPTipo);
                mTipo = value;
                OnPropertyChanged(NPTipo);
            }
        }

        private bool TieneClasificacion {
            get {
                return Tipo == TipoCronometrable.Manga || Tipo == TipoCronometrable.Tramo;
            }
        }

        #endregion

        #region ClasificacionesManga
        public const string NPClasificacionesManga = "ClasificacionesManga";
        protected EColeccion<EClasificacionManga> mClasificacionesManga = null;

        /// <summary>Devuelve una coleccin con todos los ClasificacionesManga.</summary>
        public EColeccion<EClasificacionManga> ClasificacionesManga {
            get {
                if (Tipo != TipoCronometrable.Manga) {
                    return null;
                }
                if(mClasificacionesManga == null) {
                    mClasificacionesManga = new EColeccion<EClasificacionManga>();
                    if(!this.EsNuevo) {
                        mClasificacionesManga.Filtros.Add(EClasificacionManga.NPCronometrable, this);
                        mClasificacionesManga.Cargar();
                    }
                    mClasificacionesManga.AddingNew += new AddingNewEventHandler(mClasificacionesManga_AddingNew);
                }
                //mClasificacionesManga.ApplySort(EClasificacionManga.NPPuesto, ListSortDirection.Ascending);
                return mClasificacionesManga;
            }
        }

        private void mClasificacionesManga_AddingNew(object sender, AddingNewEventArgs e) {
            EClasificacionManga entidad = new EClasificacionManga();
            entidad.Cronometrable = this;
            e.NewObject = entidad;
            //establecemos el puesto.
            entidad.Puesto = mClasificacionesManga.Count + 1;
        }

        #endregion

        #region ClasificacionesTramo
        public const string NPClasificacionesTramo = "ClasificacionesTramo";
        protected EColeccion<EClasificacionTramo> mClasificacionesTramo = null;

        /// <summary>Devuelve una coleccin con todos los ClasificacionesTramo.</summary>
        public EColeccion<EClasificacionTramo> ClasificacionesTramo {
            get {
                if (Tipo != TipoCronometrable.Tramo) {
                    return null;
                }
                if(mClasificacionesTramo == null) {
                    mClasificacionesTramo = new EColeccion<EClasificacionTramo>();
                    if(!this.EsNuevo) {
                        mClasificacionesTramo.Filtros.Add(EClasificacionTramo.NPCronometrable, this);
                        mClasificacionesTramo.Cargar();
                    }
                    mClasificacionesTramo.AddingNew += new AddingNewEventHandler(mClasificacionesTramo_AddingNew);
                } 
                return mClasificacionesTramo;
            }
        }

        private void mClasificacionesTramo_AddingNew(object sender, AddingNewEventArgs e) {
            EClasificacionTramo entidad = new EClasificacionTramo();
            entidad.Cronometrable = this;
            e.NewObject = entidad;
            //establecemos el puesto
            entidad.Puesto = mClasificacionesTramo.Count + 1;
        }

        #endregion

        #region TiemposPilotoSector
        public const string NPTiemposPilotoSector = "TiemposPilotoSector";
        protected EColeccion<ETiempoPilotoSector> mTiemposPilotoSector = null;

        /// <summary>Devuelve una coleccin con todos los TiemposPilotoSector.</summary>
        public EColeccion<ETiempoPilotoSector> TiemposPilotoSector {
            get {
                if (Tipo != TipoCronometrable.Sector) {
                    return null;
                }
                if(mTiemposPilotoSector == null) {
                    mTiemposPilotoSector = new EColeccion<ETiempoPilotoSector>();
                    if(!this.EsNuevo) {
                        mTiemposPilotoSector.Filtros.Add(ETiempoPilotoSector.NPCronometrable, this);
                        mTiemposPilotoSector.Cargar();
                    }
                    mTiemposPilotoSector.AddingNew += new AddingNewEventHandler(mTiemposPilotoSector_AddingNew);
                }
                mTiemposPilotoSector.ApplySort(ETiempoPilotoSector.NPTiempo, ListSortDirection.Ascending);
                return mTiemposPilotoSector;
            }
        }

        private void mTiemposPilotoSector_AddingNew(object sender, AddingNewEventArgs e) {
            ETiempoPilotoSector entidad = new ETiempoPilotoSector();
            entidad.Cronometrable = this;
            e.NewObject = entidad;
        }
       
        #endregion

        #region Sanciones
        public const string NPSanciones = "Sanciones";
        private EColeccion<ESancion> mSanciones = null;

        /// <summary>Devuelve una coleccin con todos los Sanciones.</summary>
        public EColeccion<ESancion> Sanciones {
            get {
                if(mSanciones == null) {
                    mSanciones = new EColeccion<ESancion>();
                    if(!this.EsNuevo) {
                        mSanciones.Filtros.Add(ESancion.NPCronometrable, this);
                        mSanciones.Cargar();
                    }
                    mSanciones.AddingNew += new AddingNewEventHandler(mSanciones_AddingNew);
                }
                return mSanciones;
            }
        }

        private void mSanciones_AddingNew(object sender, AddingNewEventArgs e) {
            ESancion entidad = new ESancion();
            entidad.Cronometrable = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Colecciones
        public static EColeccion<ECronometrable> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<ECronometrable> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<ECronometrable> col = new EColeccion<ECronometrable>();
            col.ColumnaMostrarRango(new string[] { NPLongitud, NPEstaCerrado, NPEsAcumulable });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }
        #endregion

        public override string ToString() {
            switch (Tipo) {
                case TipoCronometrable.Ninguno:
                    return null;
                case TipoCronometrable.Manga:
                    EManga m = new EManga(this.Id);
                    return m.ToString();
                case TipoCronometrable.Sector:
                    ESector s = new ESector(this.Id);
                    return s.ToString();
                case TipoCronometrable.Tramo:
                    ETramo t = new ETramo(this.Id);
                    return t.ToString();
                default:
                    return null;
            }
            
        }

        //internal abstract void ActualizarClasificacion();
        internal void ActualizarClasificacion() {
            IEColeccion col = null;
            switch (Tipo) {
                case TipoCronometrable.Manga:
                    col = this.ClasificacionesManga;
                    break;
                case TipoCronometrable.Tramo:
                    col = this.ClasificacionesTramo;
                    break;
                default:
                    break;
            }
            if (col != null) {
                col.ApplySort(EClasificacionCarrera.NPTiempoEfectivo, ListSortDirection.Ascending);
                EClasificacionCarrera primero = null;
                int puestoActual = 1;
                for (int indice = 0; indice < col.Count; indice++) {
                    EClasificacionCarrera clasificacionCarrera = col[indice] as EClasificacionCarrera;
                    if (clasificacionCarrera.HaAbandonado() || !clasificacionCarrera.EsMostrable()) {
                        clasificacionCarrera.Puesto = 0;
                    } else {
                        if (puestoActual == 1) {
                            primero = clasificacionCarrera;
                            primero.TiempoAnterior = null;
                            //primero.ClasificacionAnterior = null;
                        } else {
                            clasificacionCarrera.TiempoAnterior = (col[indice - 1] as EClasificacionCarrera).TiempoEfectivo;
                            //cc.ClasificacionAnterior = col[indice - 1] as EClasificacionCarrera;
                        }
                        clasificacionCarrera.Puesto = puestoActual;
                        clasificacionCarrera.TiempoPrimero = primero.TiempoEfectivo;
                        //cc.ClasificacionPrimero = primero;
                        puestoActual = puestoActual + 1;
                    }
                }
            }
        }

        /// <summary>
        /// Devuelve la clasificacion de un participante, si no se encuentra devuelve null.
        /// </summary>
        /// <param name="mPilotoCarrera"></param>
        /// <returns></returns>
        //public abstract EClasificacionCarrera GetClasificacion(EPilotoCarrera inscrito);
        public EClasificacionCarrera GetClasificacion(EPilotoCarrera mPilotoCarrera) {
            //HACK: optimizar mtodo.
            IEColeccion col = null;
            EClasificacionCarrera res = null;
            switch (Tipo) {
                case TipoCronometrable.Ninguno:
                    return null;
                case TipoCronometrable.Manga:
                    col = this.ClasificacionesManga;
                    break;
                case TipoCronometrable.Sector:
                    return null;
                case TipoCronometrable.Tramo:
                    col = this.ClasificacionesTramo;
                    break;
                default:
                    return null;
            }
            return BuscarClasificacion(col, mPilotoCarrera);

            //if (res != null) {
            //    return res;
            //}
            ////Aqu no ha encontrado la clasificacin, por tanto, creamos una nueva.
            //if (Tipo == TipoCronometrable.Manga) {
            //    EClasificacionManga nuevo = this.ClasificacionesManga.AddNew();
            //    res = nuevo;
            //} else if (Tipo == TipoCronometrable.Tramo) {
            //    EClasificacionTramo nuevo = this.ClasificacionesTramo.AddNew();
            //    //Faltaria decirle a qu clasificacion rally va.
            //    ERally rally = new ERally(this.Carrera.Id);
            //    if (rally.EsNuevo) {
            //        throw new System.Exception("Error en la captura de datos.");
            //    }
            //    EColeccion<EClasificacionTotalRally> colClasifRally = rally.ClasificacionesRally;
            //    int indice = 0;
            //    bool encontrado = false;
            //    while (!encontrado && indice < colClasifRally.Count) {
            //        EClasificacionTotalRally clasRally = colClasifRally[indice];
            //        if (clasRally.PilotoCarrera.Equals(mPilotoCarrera)) {
            //            encontrado = true;
            //            nuevo.ClasificacionRally = clasRally;
            //        } else {
            //            indice = indice + 1;
            //        }
            //    }
            //    if (!encontrado) {
            //        EClasificacionTotalRally nuevaClasRally = rally.ClasificacionesRally.AddNew();
            //        nuevo.ClasificacionRally = nuevaClasRally;
            //    }
            //    res = nuevo;
            //}
            //if (res != null) {
            //    res.PilotoCarrera = mPilotoCarrera;
            //    ActualizarClasificacion();
            //    return res;
            //}
            //throw new System.Exception("Error en la captura de datos.");
        }

        protected static EClasificacionCarrera BuscarClasificacion(IEColeccion col, EPilotoCarrera mPilotoCarrera) {
            if (col != null) {
                foreach (EClasificacionCarrera clasif in col) {
                    if (clasif.PilotoCarrera.Equals(mPilotoCarrera)) {
                        return clasif;
                    }
                }
            }
            return null;
        }

        public ETiempoPilotoSector GetTiempoSector(EPilotoCarrera mPilotoCarrera) {
            if (Tipo != TipoCronometrable.Sector) {
                new System.Exception("Error al capturar los datos, este cronometrable no es un sector.");
            }
            EColeccion<ETiempoPilotoSector> col = this.TiemposPilotoSector;
            if (col != null) {
                foreach (ETiempoPilotoSector tiempoSector in col) {
                    if (tiempoSector.PilotoCarrera.Equals(mPilotoCarrera)) {
                        return tiempoSector;
                    }
                }
                //si no lo ha encontrado se crea uno nuevo.
                ETiempoPilotoSector nuevo = col.AddNew();
                nuevo.PilotoCarrera = mPilotoCarrera;
                return nuevo;
            }
            return null;
        }

        internal TimeSpan? GetTiempo(int puesto) {
            if (puesto > 0 && Tipo == TipoCronometrable.Manga) {
                EColeccion<EClasificacionManga> col = this.ClasificacionesManga;
                foreach (EClasificacionManga clasificacionManga in col) {
                    if (clasificacionManga.Puesto == puesto) {
                        return clasificacionManga.TiempoEfectivo;
                    }
                }
            } else if (puesto > 0 && Tipo == TipoCronometrable.Tramo) {
                EColeccion<EClasificacionTramo> col = this.ClasificacionesTramo;
                foreach (EClasificacionTramo clasificacionTramo in col) {
                    if (clasificacionTramo.Puesto == puesto) {
                        return clasificacionTramo.TiempoEfectivo;
                    }
                }
            }
            return null;
        }

        /// <summary>
        /// Actualiza todas las clasificaciones de los campeonatos (tambien por puntos) y las guarda en la base de datos.
        /// </summary>
        private void ActualizarYGuardarClasficacionesCampeonato() {
            EColeccion<ECampeonatoInscrito> colci = this.Carrera.GetCampeonatosInscrito();
            foreach (ECampeonatoInscrito ci in colci) {
                bool hayconpuntos = false;
                if (this.Tipo == TipoCronometrable.Manga) {//mirar si hay que poner por puntos.
                    EManga manga = new EManga(this.Id);
                    if (manga.TipoManga == TipoManga.MOfcial) {
                        EColeccion<EClasificacionCarreraCampeonatoConPuntos> colconpuntos = EClasificacionCarreraCampeonatoConPuntos.GetCollection(ci.Campeonato, this);
                        EClasificacionCarreraCampeonatoConPuntos.ActualizarClasificacion(colconpuntos);
                        colconpuntos.Guardar();
                        hayconpuntos = true;
                    }
                }
                if (!hayconpuntos) {
                    EColeccion<EClasificacionCarreraCampeonato> colccclase = EClasificacionCarreraCampeonato.GetCollection(ci.Campeonato, this);
                    EClasificacionCarreraCampeonato.ActualizarClasificacion(colccclase);
                    colccclase.Guardar();
                }
                EColeccion<EClasificacionCarreraCampeonato> colccgrupo = EClasificacionCarreraCampeonato.GetCollection(ci.Campeonato.Campeonato, this);
                EClasificacionCarreraCampeonato.ActualizarClasificacion(colccgrupo);
                colccgrupo.Guardar();
                EColeccion<EClasificacionCarreraCampeonato> colccCampeonato = EClasificacionCarreraCampeonato.GetCollection(ci.Campeonato.Campeonato.Campeonato, this);
                EClasificacionCarreraCampeonato.ActualizarClasificacion(colccCampeonato);
                colccCampeonato.Guardar();
            }
        }

        public override bool EstaSucio {
            get {
                return base.EstaSucio || (mClasificacionesManga != null && mClasificacionesManga.EstaSucio) || (mClasificacionesTramo != null && mClasificacionesTramo.EstaSucio) || (mTiemposPilotoSector != null && mTiemposPilotoSector.EstaSucio);
            }
        }
    }
}
