using System.ComponentModel;
using System;
using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    [Tabla("DocumentosInformesFinales")]
    public class EDocInformeFinal : EDocumento {

        #region Constructores

        public EDocInformeFinal() : base() { }

        public EDocInformeFinal(int vId) : base(vId) { }

        public EDocInformeFinal(ECarrera carrera) : base(carrera) {
            NumeroInscritos = carrera.PilotosCarrera.Count;
        }

        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            TipoDocumento = TipoDocumento.InformeFinal;
        }

        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "DocInformeFinal"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return ""; }
        }
        #endregion

        #region Propiedades

        private int mNumeroInscritos;
        public int NumeroInscritos {
            get {
                if (Carrera != null && Carrera.PilotosCarrera != null) {
                    mNumeroInscritos = Carrera.PilotosCarrera.Count;
                }
                return mNumeroInscritos;
            }
            private set {
                mNumeroInscritos = value;
            }
        }

        public const string NPNumeroEquiposAutorizados = "NumeroEquiposAutorizados";
        private int mNumeroEquiposAutorizados;

        [Campo]
        public int NumeroEquiposAutorizados {
            get {
                PrePropertyRead(NPNumeroEquiposAutorizados);
                return mNumeroEquiposAutorizados;
            }
            set {
                PrePropertyChanged(NPNumeroEquiposAutorizados);
                mNumeroEquiposAutorizados = value;
                OnPropertyChanged(NPNumeroEquiposAutorizados);
            }
        }

        public const string NPNumeroEquiposSalidos = "NumeroEquiposSalidos";
        private int mNumeroEquiposSalidos;

        [Campo]
        public int NumeroEquiposSalidos {
            get {
                PrePropertyRead(NPNumeroEquiposSalidos);
                return mNumeroEquiposSalidos;
            }
            set {
                PrePropertyChanged(NPNumeroEquiposSalidos);
                mNumeroEquiposSalidos = value;
                OnPropertyChanged(NPNumeroEquiposSalidos);
            }
        }

        public const string NPHoraSalidaVehiculo1 = "HoraSalidaVehiculo1";
        private DateTime mHoraSalidaVehiculo1;

        [Campo]
        public DateTime HoraSalidaVehiculo1 {
            get {
                PrePropertyRead(NPHoraSalidaVehiculo1);
                return mHoraSalidaVehiculo1;
            }
            set {
                PrePropertyChanged(NPHoraSalidaVehiculo1);
                mHoraSalidaVehiculo1 = value;
                OnPropertyChanged(NPHoraSalidaVehiculo1);
            }
        }

        public const string NPHoraEntradaParqueParticipante1 = "HoraEntradaParqueParticipante1";
        private DateTime mHoraEntradaParqueParticipante1;

        [Campo]
        public DateTime HoraEntradaParqueParticipante1 {
            get {
                PrePropertyRead(NPHoraEntradaParqueParticipante1);
                return mHoraEntradaParqueParticipante1;
            }
            set {
                PrePropertyChanged(NPHoraEntradaParqueParticipante1);
                mHoraEntradaParqueParticipante1 = value;
                OnPropertyChanged(NPHoraEntradaParqueParticipante1);
            }
        }

        public const string NPNumParticipante1EntraParque = "NumParticipante1EntraParque";
        private int mNumParticipante1EntraParque;

        [Campo]
        public int NumParticipante1EntraParque {
            get {
                PrePropertyRead(NPNumParticipante1EntraParque);
                return mNumParticipante1EntraParque;
            }
            set {
                PrePropertyChanged(NPNumParticipante1EntraParque);
                mNumParticipante1EntraParque = value;
                OnPropertyChanged(NPNumParticipante1EntraParque);
            }
        }

        public const string NPHoraEntradaParticipanteUltimo = "HoraEntradaParticipanteUltimo";
        private DateTime mHoraEntradaParticipanteUltimo;

        [Campo]
        public DateTime HoraEntradaParticipanteUltimo {
            get {
                PrePropertyRead(NPHoraEntradaParticipanteUltimo);
                return mHoraEntradaParticipanteUltimo;
            }
            set {
                PrePropertyChanged(NPHoraEntradaParticipanteUltimo);
                mHoraEntradaParticipanteUltimo = value;
                OnPropertyChanged(NPHoraEntradaParticipanteUltimo);
            }
        }

        public const string NPNumParticipanteUltimoEntraParque = "NumParticipanteUltimoEntraParque";
        private int mNumParticipanteUltimoEntraParque;

        [Campo]
        public int NumParticipanteUltimoEntraParque {
            get {
                PrePropertyRead(NPNumParticipanteUltimoEntraParque);
                return mNumParticipanteUltimoEntraParque;
            }
            set {
                PrePropertyChanged(NPNumParticipanteUltimoEntraParque);
                mNumParticipanteUltimoEntraParque = value;
                OnPropertyChanged(NPNumParticipanteUltimoEntraParque);
            }
        }

        public const string NPNumEquiposParqueCerrado = "NumEquiposParqueCerrado";
        private int mNumEquiposParqueCerrado;

        [Campo]
        public int NumEquiposParqueCerrado {
            get {
                PrePropertyRead(NPNumEquiposParqueCerrado);
                return mNumEquiposParqueCerrado;
            }
            set {
                PrePropertyChanged(NPNumEquiposParqueCerrado);
                mNumEquiposParqueCerrado = value;
                OnPropertyChanged(NPNumEquiposParqueCerrado);
            }
        }

        public const string NPNumEquiposClasificados = "NumEquiposClasificados";
        private int mNumEquiposClasificados;

        [Campo]
        public int NumEquiposClasificados {
            get {
                PrePropertyRead(NPNumEquiposClasificados);
                return mNumEquiposClasificados;
            }
            set {
                PrePropertyChanged(NPNumEquiposClasificados);
                mNumEquiposClasificados = value;
                OnPropertyChanged(NPNumEquiposClasificados);
            }
        }

        public const string NPHoraPublicacionClasificacion = "HoraPublicacionClasificacion";
        private DateTime mHoraPublicacionClasificacion;

        [Campo]
        public DateTime HoraPublicacionClasificacion {
            get {
                PrePropertyRead(NPHoraPublicacionClasificacion);
                return mHoraPublicacionClasificacion;
            }
            set {
                PrePropertyChanged(NPHoraPublicacionClasificacion);
                mHoraPublicacionClasificacion = value;
                OnPropertyChanged(NPHoraPublicacionClasificacion);
            }
        }

        public const string NPObservaciones = "Observaciones";
        private string mObservaciones;

        [Campo]
        public string Observaciones {
            get {
                PrePropertyRead(NPObservaciones);
                return mObservaciones;
            }
            set {
                PrePropertyChanged(NPObservaciones);
                mObservaciones = value;
                OnPropertyChanged(NPObservaciones);
            }
        }

        #region Informe
        
        public string NPHoraSalidaVehiculo1ToString {
            get {
                return CToString.ToString(HoraSalidaVehiculo1.Hour, HoraSalidaVehiculo1.Minute);
            }
        }

        public string NPHoraEntradaParqueParticipante1ToString {
            get {
                return CToString.ToString(HoraEntradaParqueParticipante1.Hour, HoraEntradaParqueParticipante1.Minute);
            }
        }

        public string NPHoraEntradaParticipanteUltimoToString {
            get {
                return CToString.ToString(HoraEntradaParticipanteUltimo.Hour, HoraEntradaParticipanteUltimo.Minute);
            }
        }

        public string NPHoraPublicacionClasificacionToString {
            get {
                return CToString.ToString(HoraPublicacionClasificacion.Hour, HoraPublicacionClasificacion.Minute);
            }
        }

        #endregion

        #endregion
    }
}

