using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    [Tabla("Documentos")]
    public class EDocListaBriefing : EDocumento {

        #region Constructores

        public EDocListaBriefing() : base() { }

        public EDocListaBriefing(int vId) : base(vId) { }

        public EDocListaBriefing(ECarrera carrera) : base(carrera) { }

        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            TipoDocumento = TipoDocumento.AsistenciaBriefing;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Documento lista briefing")) {
                base.GuardarCore();
                if (mAsistenciasBriefing != null) {
                    mAsistenciasBriefing.Guardar();
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mAsistenciasBriefing = null;
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "DocListaBriefing"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return ""; }
        }
        #endregion

        #region AsistenciasBriefing

        public const string NPAsistenciasBriefing = "AsistenciasBriefing";
        private EColeccion<EAsistenciaBriefing> mAsistenciasBriefing = null;

        /// <summary>Devuelve una coleccin con todos los AsistenciasBriefing.</summary>
        public EColeccion<EAsistenciaBriefing> AsistenciasBriefing {
            get {
                if(mAsistenciasBriefing == null) {
                    mAsistenciasBriefing = new EColeccion<EAsistenciaBriefing>();
                    if(!this.EsNuevo) {
                        mAsistenciasBriefing.Filtros.Add(EAsistenciaBriefing.NPDocumentoAsistenciaBriefing, this);
                        mAsistenciasBriefing.Cargar();
                    }
                    mAsistenciasBriefing.AddingNew += new AddingNewEventHandler(mAsistenciasBriefing_AddingNew);
                }
                return mAsistenciasBriefing;
            }
        }

        private void mAsistenciasBriefing_AddingNew(object sender, AddingNewEventArgs e) {
            EAsistenciaBriefing entidad = new EAsistenciaBriefing();
            entidad.DocumentoAsistenciaBriefing = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Propiedades

        #endregion

        public override bool EstaSucio {
            get {
                return base.EstaSucio || (mAsistenciasBriefing != null && mAsistenciasBriefing.EstaSucio);
            }
        }

        public EColeccion<EPilotoCarrera> GetNoPresentadosAlBriefing() {
            EColeccion<EPilotoCarrera> colNoPresentados = new EColeccion<EPilotoCarrera>();
            if (this.Carrera != null) {
                EColeccion<EPilotoCarrera> colInscritos = Carrera.PilotosCarrera;
                foreach (EPilotoCarrera inscrito in colInscritos) {
                    if (!Presentado(inscrito)) {
                        colNoPresentados.Add(inscrito);
                    }
                }
            }
            return colNoPresentados;
        }

        private bool Presentado(EPilotoCarrera inscrito) {
            EColeccion<EAsistenciaBriefing> colAsistencia = this.AsistenciasBriefing;
            foreach (EAsistenciaBriefing asistencia in colAsistencia) {
                if (inscrito.Equals(asistencia.PilotoCarrera)) {
                    return true;
                }
            }
            return false;
        }
    }
}

