using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    public enum TipoDocumento {
        [Description("Asistencia al Briefing")]
        AsistenciaBriefing = 1,
        [Description("Informe final del desarrollo de la prueba")]
        InformeFinal
        //[Description("Verificaciones Tcnicas")]
        //VerificacionesTecnicas,
        //[Description("Lista de Autorizados")]
        //ListaAutorizados,
    }

    [Tabla("Documentos")]
    public abstract class EDocumento : EIdBase {

        #region Constructores

        public EDocumento() : base() { }

        public EDocumento(int vId) : base(vId) { }

        public EDocumento(ECarrera carrera) : base() {
            this.Carrera = carrera;
        }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            mTipoDocumento = TipoDocumento.AsistenciaBriefing;
            Codigo = null;
            mCarrera = null;
            mFecha = DateTime.Now;
            mEmisor = TipoCargoOficial.Ninguno;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Documento")) {
                base.GuardarCore();
                if (mReceptores != null) {
                    mReceptores.Guardar();
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mReceptores = null;
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "Documento"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return ""; }
        }
        #endregion

        #region Receptores
        public const string NPReceptores = "Receptores";
        private EColeccion<EReceptor> mReceptores = null;

        /// <summary>Devuelve una coleccin con todos los Receptores.</summary>
        public EColeccion<EReceptor> Receptores {
            get {
                if(mReceptores == null) {
                    mReceptores = new EColeccion<EReceptor>();
                    if(!this.EsNuevo) {
                        mReceptores.Filtros.Add(EReceptor.NPDocumento, this);
                        mReceptores.Cargar();
                    }
                    mReceptores.AddingNew += new AddingNewEventHandler(mReceptores_AddingNew);
                }
                return mReceptores;
            }
        }

        private void mReceptores_AddingNew(object sender, AddingNewEventArgs e) {
            EReceptor entidad = new EReceptor();
            entidad.Documento = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Propiedades

        public const string NPTipoDocumento = "TipoDocumento";
        private TipoDocumento mTipoDocumento;

        [Campo]
        public TipoDocumento TipoDocumento {
            get {
                PrePropertyRead(NPTipoDocumento);
                return mTipoDocumento;
            }
            set {
                PrePropertyChanged(NPTipoDocumento);
                mTipoDocumento = value;
                OnPropertyChanged(NPTipoDocumento);
            }
        }

        public const string NPCodigo = "Codigo";
        private string mCodigo;

        [Campo(Tamanyo = 50), Requerido, Unico]
        public string Codigo {
            get {
                PrePropertyRead(NPCodigo);
                return mCodigo;
            }
            set {
                PrePropertyChanged(NPCodigo);
                mCodigo = value;
                OnPropertyChanged(NPCodigo);
            }
        }

        public const string NPFecha = "Fecha";
        private DateTime mFecha;

        [Campo]
        public DateTime Fecha {
            get {
                PrePropertyRead(NPFecha);
                return mFecha;
            }
            set {
                PrePropertyChanged(NPFecha);
                mFecha = value;
                OnPropertyChanged(NPFecha);
            }
        }

        public const string NPCarrera = "Carrera";
        private ECarrera mCarrera;

        [Campo("CarreraId"), Requerido]
        public virtual ECarrera Carrera {
            get {
                PrePropertyRead(NPCarrera);
                return mCarrera;
            }
            set {
                PrePropertyChanged(NPCarrera);
                mCarrera = value;
                OnPropertyChanged(NPCarrera);
            }
        }

        public const string NPEmisor = "Emisor";
        private TipoCargoOficial mEmisor;

        [Campo]
        public TipoCargoOficial Emisor {
            get {
                PrePropertyRead(NPEmisor);
                return mEmisor;
            }
            set {
                PrePropertyChanged(NPEmisor);
                mEmisor = value;
                OnPropertyChanged(NPEmisor);
            }
        }

        /// <summary>
        /// Para los informes.
        /// </summary>
        public string CarreraToString {
            get {
                if (Carrera != null) {
                    return Carrera.Nombre;
                }
                return "";
            }
        }

        /// <summary>
        /// Para los informes.
        /// </summary>
        public string FechaToString {
            get {
                return CToString.ToString(Fecha);
            }
        }

        /// <summary>
        /// Para los informes.
        /// </summary>
        public string EmisorToString {
            get {
                EnumConvert ec = new EnumConvert();
                return ec.GetDescription(Emisor);
            }
        }

        /// <summary>
        /// Para los informes.
        /// </summary>
        public string ReceptoresToString {
            get {
                string cadena = "";
                foreach (EReceptor receptor in Receptores) {
                    cadena += receptor + "\n";
                }
                return cadena;
            }
        }

        #endregion

        public override bool EstaSucio {
            get {
                return base.EstaSucio || (mReceptores != null && mReceptores.EstaSucio);
            }
        }

        public bool TieneReceptor(TipoCargoOficial cargoOficial) {
            EColeccion<EReceptor> colReceptores = Receptores;
            foreach (EReceptor receptor in colReceptores) {
                if (receptor.Receptor.Equals(cargoOficial)) {
                    return true;
                }
            }
            return false;
        }

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar entidad")) {
                //Eliminamos los receptores.
                int indice = 0;
                while (indice < Receptores.Count) {
                    EReceptor r = Receptores[indice];
                    r.Eliminar();
                    indice = indice + 1;
                }
                base.Eliminar();
                transa.Commit();
            }
            
        }
    }
}

