using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    [Tabla("Imagenes")]
    public class EImagen : EIdBase {
        #region Constructores

        public EImagen() : base() { }

        public EImagen(int vId) : base(vId) { }

        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            Nombre = null;
            Descripcion = null;
            Carrera = null;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Imagen")) {
                base.GuardarCore();
                if (mImagenCargada) {
                    CentroImagenesE.Actual.AddImagen(this, Imagen);
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mImagenCargada = false;
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "Imagen"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return ToString(); }
        }
        #endregion

        #region Propiedades

        public const string NPImagen = "Imagen";
        private System.Drawing.Image mImagen;
        private bool mImagenCargada = false;

        public System.Drawing.Image Imagen {
            get {
                if (!mImagenCargada) {
                    mImagen = CentroImagenesE.Actual.GetImagen(this);
                    mImagenCargada = true;
                }
                return mImagen;
            }
            set {
                mImagen = value;
                mImagenCargada = true;
                OnPropertyChanged(NPImagen);
            }
        }

        public const string NPNombre = "Nombre";
        private string mNombre;

        [Campo(Tamanyo = 80), Requerido]
        public string Nombre {
            get {
                PrePropertyRead(NPNombre);
                return mNombre;
            }
            set {
                PrePropertyChanged(NPNombre);
                mNombre = value;
                OnPropertyChanged(NPNombre);
            }
        }

        public const string NPDescripcion = "Descripcion";
        private string mDescripcion;

        [Campo]
        public string Descripcion {
            get {
                PrePropertyRead(NPDescripcion);
                return mDescripcion;
            }
            set {
                PrePropertyChanged(NPDescripcion);
                mDescripcion = value;
                OnPropertyChanged(NPDescripcion);
            }
        }

        public const string NPCarrera = "Carrera";
        private ECarrera mCarrera;

        [Campo("CarreraId"), Requerido]
        public ECarrera Carrera {
            get {
                PrePropertyRead(NPCarrera);
                return mCarrera;
            }
            set {
                PrePropertyChanged(NPCarrera);
                mCarrera = value;
                OnPropertyChanged(NPCarrera);
            }
        }

        #endregion

        public override string ToString() {
            return this.Nombre;
        }

        public override void Eliminar() {
            //TODO: borrar la imagen en el pc.
            base.Eliminar();
        }
    }
}

