using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    public enum TipoManga {
        Ninguno = 0,
        [Description("Entrenamientos libres")]
        ELibres = 1,
        [Description("Entrenamientos oficiales")]
        EOficiales,
        [Description("Manga oficial")]
        MOfcial
    }

    public enum TipoEstadoManga {
        Ninguno = 0,
        NoComenzado = 1,
        EnCurso,
        Finalizado
    }

    [Tabla("Mangas")]
    public class EManga : ECronometrable {
        #region Constructores
        public EManga() : base() { }

        public EManga(int vId) : base(vId) { }

        public EManga(TipoManga tm, string fase, ESubida subida) : base(subida.Longitud) {
            mSubida = subida;
            mNombreFase = fase;
            mTipoManga = tm;
            mEsPuntuable = mTipoManga == TipoManga.MOfcial;
        }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            Estado = TipoEstadoManga.NoComenzado;
            base.Tipo = TipoCronometrable.Manga;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Manga")) {
                CrearClasificaciones();
                base.Carrera = this.Subida;
                base.GuardarCore();
                //if (mClasificacionesManga != null) {
                //    ActualizarClasificacion();
                //    mClasificacionesManga.Guardar();
                //}
                if (colClasificacionesCreadas != null) {
                    colClasificacionesCreadas.Guardar();
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "Manga"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return this.ToString(); } 
        }
        #endregion

        #region Propiedades

        public const string NPTipoManga = "TipoManga";
        private TipoManga mTipoManga;

        [Campo]
        public TipoManga TipoManga {
            get {
                PrePropertyRead(NPTipoManga);
                return mTipoManga;
            }
            set {
                PrePropertyChanged(NPTipoManga);
                mTipoManga = value;
                mEsPuntuable = mTipoManga != TipoManga.ELibres;
                OnPropertyChanged(NPTipoManga);
            }
        }

        public const string NPEsPuntuable = "EsPuntuable";
        private bool mEsPuntuable;

        public bool EsPuntuable {
            get {
                return mEsPuntuable;
            }
        }

        public const string NPEstado = "Estado";
        private TipoEstadoManga mEstado;

        [Campo]
        public TipoEstadoManga Estado {
            get {
                PrePropertyRead(NPEstado);
                return mEstado;
            }
            set {
                PrePropertyChanged(NPEstado);
                mEstado = value;
                OnPropertyChanged(NPEstado);
            }
        }

        public const string NPHoraComienzo = "HoraComienzo";
        private DateTime mHoraComienzo;

        [Campo]
        public DateTime HoraComienzo {
            get {
                PrePropertyRead(NPHoraComienzo);
                return mHoraComienzo;
            }
            set {
                PrePropertyChanged(NPHoraComienzo);
                mHoraComienzo = value;
                OnPropertyChanged(NPHoraComienzo);
            }
        }

        public const string NPNombreFase = "NombreFase";
        private string mNombreFase;

        [Campo(Tamanyo = 50)]
        public string NombreFase {
            get {
                PrePropertyRead(NPNombreFase);
                return mNombreFase;
            }
            set {
                PrePropertyChanged(NPNombreFase);
                mNombreFase = value;
                OnPropertyChanged(NPNombreFase);
            }
        }

        public const string NPSubida = "Subida";
        private ESubida mSubida;

        [Campo("CarreraId"), Requerido]
        public ESubida Subida {
            get {
                PrePropertyRead(NPSubida);
                return mSubida;
            }
            set {
                PrePropertyChanged(NPSubida);
                mSubida = value;
                OnPropertyChanged(NPSubida);
            }
        }

        //public override ECarrera Carrera {
        //    get { return Subida; }
        //}
        #endregion

        public override string ToString() {
            EnumConvert ec = new EnumConvert();
            return "Fase " + NombreFase + ". " + ec.GetDescription(this.TipoManga);
        }

        private EColeccion<EClasificacionCarreraCampeonato> colClasificacionesCreadas = null;
        private void CrearClasificaciones() {
            if (EsNuevo) {
                colClasificacionesCreadas = new EColeccion<EClasificacionCarreraCampeonato>();
                EColeccion<EPilotoCarrera> colInscritos = this.Subida.PilotosCarrera;
                foreach (EPilotoCarrera inscrito in colInscritos) {
                    EClasificacionManga clasificacion = this.ClasificacionesManga.AddNew();
                    clasificacion.PilotoCarrera = inscrito;
                    //Creamos las clasificaciones carrera campeonato
                    foreach (ECampeonatoInscrito ci in inscrito.CampeonatosInscrito) {
                        EClasificacionCarreraCampeonato clasifCampeonato = new EClasificacionCarreraCampeonato();
                        clasifCampeonato.ClasificacionCarrera = clasificacion;
                        clasifCampeonato.Campeonato = ci.Campeonato.Campeonato.Campeonato;
                        colClasificacionesCreadas.Add(clasifCampeonato);
                        EClasificacionCarreraCampeonato clasifGrupo = new EClasificacionCarreraCampeonato();
                        clasifGrupo.ClasificacionCarrera = clasificacion;
                        clasifGrupo.Campeonato = ci.Campeonato.Campeonato;
                        colClasificacionesCreadas.Add(clasifGrupo);
                        EClasificacionCarreraCampeonato clasifClase = null;
                        if (this.TipoManga == TipoManga.MOfcial) {
                            clasifClase = new EClasificacionCarreraCampeonatoConPuntos();
                        } else {
                            clasifClase = new EClasificacionCarreraCampeonato();
                        }
                        clasifClase.Campeonato = ci.Campeonato;
                        clasifClase.ClasificacionCarrera = clasificacion;
                        colClasificacionesCreadas.Add(clasifClase);
                    }
                }
            }
        }

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar Manga")) {
                using (AmbitoTransa transa2 = this.CrearAmbitoTransa("Eliminar cronometrable.")) {
                    int indice2 = 0;
                    while (this.Sanciones.Count > indice2) {
                        Sanciones[indice2].Eliminar();
                        indice2 = indice2 + 1;
                    }
                    transa2.Commit();
                }
                int indice = 0;
                while (ClasificacionesManga.Count > indice) {
                    EClasificacionManga obj = ClasificacionesManga[indice];
                    //if (!obj.EstaVacio) {
                    //    throw new Exception("No se puede eliminar la Manga.\nTiene clasificaciones.");
                    //}
                    obj.Eliminar();
                    indice = indice + 1;
                }
                base.Eliminar();
                transa.Commit();
            }
        }
    }
}
