using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("Carreras")]
    public class ERally : ECarrera {

        #region Constructores
        public ERally() : base() { }

        public ERally(int vId) : base(vId) { }

        public ERally(string alias) : base(alias) { }

        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            Tipo = TipoCarrera.Rally;
        }


        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Rally")) {	
                base.GuardarCore();
                if(mSectores != null)      //Si se ha cargado
                    mSectores.Guardar();   //guardar si hemos cambiado algo
                if (mClasificacionesRally != null) {
                    mClasificacionesRally.Guardar();
                    this.ActualizarYGuardarClasificacionCarreraCampeonato();
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mSectores = null;
            base.Deshacer();
        }


        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "Rally"; }
        }

        public override bool Modificable(string nombreProp) {
            if(!EstaCargado) return true;

            switch(nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPNombre; }
        }
        #endregion

        #region Propiedades


        #endregion

        #region Sectores
        private int mNumeroSectores = 0;
        public const string NPSectores = "Sectores";
        private EColeccion<ESector> mSectores = null;

        /// <summary>Devuelve una coleccin con todos los Sectores.</summary>
        public EColeccion<ESector> Sectores {
            get {
                if(mSectores == null) {
                    mSectores = new EColeccion<ESector>();
                    if(!this.EsNuevo) {
                        mSectores.Filtros.Add(ESector.NPRally, this);
                        mSectores.ColumnaMostrar("NumeroSeccion");
                        mSectores.ColumnaMostrar("Numero");
                        mSectores.ColumnaMostrar("Longitud");
                        mSectores.Cargar();
                    }
                    mSectores.AddingNew += new AddingNewEventHandler(mSectores_AddingNew);
                    mSectores.ListChanged += new ListChangedEventHandler(mSectores_ListChanged);
                }
                if (mNumeroSectores <= mSectores.Count) {
                    mNumeroSectores = mSectores.Count;
                }
                return mSectores;
            }
        }

        private void mSectores_AddingNew(object sender, AddingNewEventArgs e) {
            ESector entidad = new ESector();
            int ulitmoNumeroSector = 0;
            int ulitmoNumeroSeccion = 0;
            if (mSectores.Count > 0) {
                ESector ultimoSector = mSectores[mSectores.Count - 1];
                ulitmoNumeroSector = ultimoSector.Numero;
                ulitmoNumeroSeccion = ultimoSector.NumeroSeccion;
            }
            entidad.Numero = ulitmoNumeroSector + 1;
            entidad.NumeroSeccion = ulitmoNumeroSeccion;
            entidad.Rally = this;
            e.NewObject = entidad;
            //actualizamos el nmero de sectores actual para que cuando solo se elimine se actualice.
            mNumeroSectores++;
        }

        private void mSectores_ListChanged(object sender, ListChangedEventArgs e) {
            if(!Sectores.EsValido)
                AddError(new PropertyException(NPSectores, this, Sectores.AlgunError.Message, Sectores.AlgunError));
            else
                RemoveError(NPSectores);
            //comprobar si se ha eliminado para actualizar los sectores.
            if (mSectores.Count > 0 && mNumeroSectores > mSectores.Count) {
                mNumeroSectores = mSectores.Count;
                ActualizarSectores();
            }
            OnPropertyCollectionChanged(NPSectores);
        }
        #endregion

        //public override EColeccion<ECronometrable> Cronometrables {
        //    get {
        //        EColeccion<ECronometrable> col = new EColeccion<ECronometrable>();
        //        EColeccion<ESector> colSectores = Sectores;
        //        foreach (ESector sector in colSectores) {
        //            col.Add(sector);
        //            if (sector.Tramo != null) {
        //                col.Add(sector.Tramo);
        //            }
        //        }
        //        return col;
        //    }
        //}

        #region ClasificacionesRally
        public const string NPClasificacionesRally = "ClasificacionesRally";
        private EColeccion<EClasificacionTotalRally> mClasificacionesRally = null;

        /// <summary>Devuelve una coleccin con todos los ClasificacionesRally.</summary>
        public EColeccion<EClasificacionTotalRally> ClasificacionesRally {
            get {
                if(mClasificacionesRally == null) {
                    mClasificacionesRally = new EColeccion<EClasificacionTotalRally>();
                    if(!this.EsNuevo) {
                        mClasificacionesRally.Filtros.Add(EClasificacionTotalRally.NPRally, this);
                        mClasificacionesRally.Cargar();
                    }
                    mClasificacionesRally.AddingNew += new AddingNewEventHandler(mClasificacionesRally_AddingNew);
                }
                return mClasificacionesRally;
            }
        }

        private void mClasificacionesRally_AddingNew(object sender, AddingNewEventArgs e) {
            EClasificacionTotalRally entidad = new EClasificacionTotalRally();
            entidad.Rally = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Colecciones
        public static EColeccion<ERally> GetCollection(bool todasLasTemporadas) {
            return GetCollection(false, false, todasLasTemporadas);
        }

        public static EColeccion<ERally> GetCollection(bool vacio, bool asincronamente, bool todasLasTemporadas) {
            EColeccion<ERally> col = new EColeccion<ERally>("RALLYES");
            col.Filtros.Add(NPTipo, FiltroSimple.TipoOperador.Igual, TipoCarrera.Rally);
            col.Filtros.Add(NPActivo, true);
            if (!todasLasTemporadas) {
                col.Filtros.Add(NPTemporada, ETemporada.Actual);
            }
            col.ColumnaMostrarRango(new string[] { NPNombre, NPAlias, NPFechaComienzo, NPFechaFin, NPCoeficiente, NPLocalidad, NPProvincia });
            if(!vacio)
                col.Cargar(asincronamente);
            return col;
        }

        public static EColeccion<ERally> GetCollection(bool activo, bool todasLasTemporadas) {
            EColeccion<ERally> col = new EColeccion<ERally>("RALLYES");
            col.Filtros.Add(NPTipo, FiltroSimple.TipoOperador.Igual, TipoCarrera.Rally);
            col.Filtros.Add(NPActivo, activo);
            if (!todasLasTemporadas) {
                col.Filtros.Add(NPTemporada, ETemporada.Actual);
            }
            col.ColumnaMostrarRango(new string[] { NPNombre, NPAlias, NPFechaComienzo, NPFechaFin, NPCoeficiente, NPLocalidad, NPProvincia });
            col.Cargar();
            return col;
        }
        #endregion

        public override bool NecesitaCopiloto() {
            return true;
        }

        private void ActualizarSectores() {
            //primero comprobamos que hay sectores en la seccin 1.
            if (mSectores[0].NumeroSeccion != 1) {//hay que empezar desde el primer sector
                mSectores[0].NumeroSeccion--;
            }
            if (mSectores[0].Numero != 1) {
                mSectores[0].Numero--;
            }
            ActualizarSector(1);
        }

        private void ActualizarSector(int indice) {
            if (indice < mSectores.Count) {
                if (indice > 0) {
                    ESector sector = mSectores[indice];
                    ESector sectorAnt = mSectores[indice - 1];
                    if (sector.Numero-1 > sectorAnt.Numero ) {//hay una diferencia de ms de uno
                        sector.Numero--;
                    }
                    if (sector.NumeroSeccion-1 > sectorAnt.NumeroSeccion) {
                        sector.NumeroSeccion--;
                    }
                }
                indice = indice + 1;
                ActualizarSector(indice);
            }
        }

        internal void ActualizarClasificacion() {
            EColeccion<EClasificacionTotalRally> col = ClasificacionesRally;
            if (col != null) {
                col.ApplySort(EClasificacionCarrera.NPTiempoEfectivo, ListSortDirection.Ascending);
                EClasificacionCarrera primero = null;
                int puestoActual = 1;
                for (int indice = 0; indice < col.Count; indice++) {
                    EClasificacionCarrera clasificacionCarrera = col[indice] as EClasificacionCarrera;
                    if (clasificacionCarrera.Abandono || !clasificacionCarrera.EsMostrable()) {
                        clasificacionCarrera.Puesto = 0;
                    } else {
                        if (puestoActual == 1) {
                            primero = clasificacionCarrera;
                            primero.TiempoAnterior = null;
                            //primero.ClasificacionAnterior = null;
                        } else {
                            clasificacionCarrera.TiempoAnterior = (col[indice - 1] as EClasificacionCarrera).TiempoEfectivo;
                            //cc.ClasificacionAnterior = col[indice - 1] as EClasificacionCarrera;
                        }
                        clasificacionCarrera.Puesto = puestoActual;
                        clasificacionCarrera.TiempoPrimero = primero.TiempoEfectivo;
                        //cc.ClasificacionPrimero = primero;
                        puestoActual = puestoActual + 1;
                    }
                }
            }
        }

        internal TimeSpan? GetTiempo(int puesto) {
            if (puesto > 0) {
                EColeccion<EClasificacionTotalRally> col = this.ClasificacionesRally;
                foreach (EClasificacionTotalRally clasifRally in col) {
                    if (clasifRally.Puesto == puesto) {
                        return clasifRally.TiempoEfectivo;
                    }
                }
            }
            return null;
        }

        internal EClasificacionTotalRally GetClasificacion(EPilotoCarrera inscrito) {
            foreach (EClasificacionTotalRally clasificacion in ClasificacionesRally) {
                if (clasificacion.PilotoCarrera.Equals(inscrito)) {
                    return clasificacion;
                }
            }
            return null;
        }

        //public override void Cerrar() {
        //    //TODO: aplicar puntos.
        //    this.EstaCerrada = true;
        //    this.Guardar();
        //}

        public override bool EstaSucio {
            get {
                return base.EstaSucio || (mSectores != null && mSectores.EstaSucio);
            }
        }

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar Rally")) {
                int indice = 0;
                EColeccion<EPilotoCarrera> colInscrito = PilotosCarrera;
                while (indice < colInscrito.Count) {
                    colInscrito[indice].Eliminar();
                    indice = indice + 1;
                }
                indice = 0;
                EColeccion<ESector> colSectores = Sectores;
                while (indice < colSectores.Count) {
                    colSectores[indice].Eliminar();
                    indice = indice + 1;
                }
                base.Eliminar();
                transa.Commit();
            }
        }

        internal void ActualizarYGuardarClasificacionCarreraCampeonato() {
            EColeccion<ECampeonatoInscrito> colci = this.GetCampeonatosInscrito();
            foreach (ECampeonatoInscrito ci in colci) {
                EColeccion<EClasificacionCarreraCampeonatoConPuntos> colccClase = EClasificacionCarreraCampeonatoConPuntos.GetCollection(ci.Campeonato, this);
                EClasificacionCarreraCampeonatoConPuntos.ActualizarClasificacion(colccClase);
                colccClase.Guardar();
                EColeccion<EClasificacionCarreraCampeonato> colccgrupo = EClasificacionCarreraCampeonato.GetCollection(ci.Campeonato.Campeonato, this);
                EClasificacionCarreraCampeonato.ActualizarClasificacion(colccgrupo);
                colccgrupo.Guardar();
                EColeccion<EClasificacionCarreraCampeonato> colccCampeonato = EClasificacionCarreraCampeonato.GetCollection(ci.Campeonato.Campeonato.Campeonato, this);
                EClasificacionCarreraCampeonato.ActualizarClasificacion(colccCampeonato);
                colccCampeonato.Guardar();
            }
        }
    }
}
