using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Collections;

namespace Negocio {

    [Tabla("Sectores")]
    public class ESector : ECronometrable {

        #region Constructores
        public ESector() : base() { }

        public ESector(int vId) : base(vId) { }

        public ESector(ERally rally, int numSeccion, int numeroSector)
            : base() {
            Rally = rally;
            NumeroSeccion = numSeccion;
            Numero = numeroSector;
        }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo(){
            base.InicializarObjetoNuevo();
            Numero = 0;
            NumeroSeccion = 0;
            mTramo = null;
            CHIni = null;
            mCHFin = null;
            mHoraFin = DateTime.Now;
            HoraIni = DateTime.Now;
            LocalizacionIni = null;
            mLocalizacionFin = null;
            base.Tipo = TipoCronometrable.Sector;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Sector")) {
                CrearTiemposSector();
                base.Carrera = mRally;
                base.GuardarCore();
                //if (mTiemposPilotoSector != null) {
                //    mTiemposPilotoSector.Guardar();
                //}
                transa.Commit();
            }
        }

        public override void Deshacer() {
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return true; }  
        }

        public override string NombreDeLaEntidad {
            get { return "Sector"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                //case NPCHIni:
                //    return Tramo == null;
                //case NPLocalizacionIni:
                //    return Tramo == null;
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return this.ToString(); }
        }
        #endregion

        #region Propiedades

        public const string NPNumero = "Numero";
        private int mNumero;

        [Campo, Requerido]
        public int Numero {
            get {
                PrePropertyRead(NPNumero);
                return mNumero;
            }
            set {
                PrePropertyChanged(NPNumero);
                mNumero = value;
                if (mNumero == 0) {
                    AddError(new PropertyException(NPNumero, this, "El nmero de sector debe ser mayor que cero"));
                } else {
                    RemoveError(NPNumero);
                }
                OnPropertyChanged(NPNumero);
            }
        }

        public const string NPNumeroSeccion = "NumeroSeccion";
        private int mNumeroSeccion;

        [Campo, Requerido]
        public int NumeroSeccion {
            get {
                PrePropertyRead(NPNumeroSeccion);
                return mNumeroSeccion;
            }
            set {
                PrePropertyChanged(NPNumeroSeccion);
                mNumeroSeccion = value;
                if (mNumeroSeccion == 0) {
                    AddError(new PropertyException(NPNumeroSeccion, this, "El nmero de seccin debe ser mayor que cero"));
                } else {
                    RemoveError(NPNumeroSeccion);
                }
                OnPropertyChanged(NPNumeroSeccion);
            }
        }

        public const string NPCHIni = "CHIni";
        private string mCHIni;

        [Campo(Tamanyo = 30), Requerido]
        public string CHIni {
            get {
                PrePropertyRead(NPCHIni);
                return mCHIni;
            }
            set {
                PrePropertyChanged(NPCHIni);
                mCHIni = value;
                if (mTramo != null && mCHIni != mTramo.Nombre) {
                    mCHIni = mTramo.Nombre;
                }
                OnPropertyChanged(NPCHIni);
            }
        }

        public const string NPHoraIni = "HoraIni";
        private DateTime mHoraIni;

        [Campo, Requerido]
        public DateTime HoraIni {
            get {
                PrePropertyRead(NPHoraIni);
                return mHoraIni;
            }
            set {
                PrePropertyChanged(NPHoraIni);
                mHoraIni = value;
                OnPropertyChanged(NPHoraIni);
            }
        }

        public const string NPLocalizacionIni = "LocalizacionIni";
        private string mLocalizacionIni;

        [Campo(Tamanyo = 100)]
        public string LocalizacionIni {
            get {
                PrePropertyRead(NPLocalizacionIni);
                return mLocalizacionIni;
            }
            set {
                PrePropertyChanged(NPLocalizacionIni);
                mLocalizacionIni = value;
                if (mTramo != null && mLocalizacionIni != mTramo.Localizacion) {
                    mLocalizacionIni = mTramo.Localizacion;
                }
                OnPropertyChanged(NPLocalizacionIni);
            }
        }

        public const string NPCHFin = "CHFin";
        private string mCHFin;

        [Campo(Tamanyo = 30)]
        public string CHFin {
            get {
                PrePropertyRead(NPCHFin);
                return mCHFin;
            }
            set {
                PrePropertyChanged(NPCHFin);
                mCHFin = value;
                OnPropertyChanged(NPCHFin);
            }
        }

        public const string NPHoraFin = "HoraFin";
        private DateTime mHoraFin;

        [Campo, Requerido]
        public DateTime HoraFin {
            get {
                PrePropertyRead(NPHoraFin);
                return mHoraFin;
            }
            set {
                PrePropertyChanged(NPHoraFin);
                mHoraFin = value;
                OnPropertyChanged(NPHoraFin);
            }
        }

        public const string NPLocalizacionFin = "LocalizacionFin";
        private string mLocalizacionFin;

        [Campo(Tamanyo = 100)]
        public string LocalizacionFin {
            get {
                PrePropertyRead(NPLocalizacionFin);
                return mLocalizacionFin;
            }
            set {
                PrePropertyChanged(NPLocalizacionFin);
                mLocalizacionFin = value;
                OnPropertyChanged(NPLocalizacionFin);
            }
        }

        public const string NPRally = "Rally";
        private ERally mRally;

        [Campo("CarreraId"), Requerido]
        public ERally Rally {
            get {
                PrePropertyRead(NPRally);
                return mRally;
            }
            set {
                PrePropertyChanged(NPRally);
                mRally = value;
                //a cronometrable hay que decirle que la carrera es este rally.
                //base.Carrera = mRally;
                //se calcula la mxima seccin disponible.
                //NumeroSeccion = this.ObtenerSeccionMax();
                OnPropertyChanged(NPRally);
            }
        }

        public const string NPTramo = "Tramo";
        private ETramo mTramo;

        public ETramo Tramo {
            get {
                //PrePropertyRead(NPTramo);
                //return mTramo;

                //HACK
                mTramo = ETramo.DevolverTramo(this);
                return mTramo;
            }
            set {
                //PrePropertyChanged(NPTramo);
                mTramo = value;
                OnPropertyChanged(NPTramo);
            }
        }

        public TimeSpan? Intervalo {
            get {
                return this.GetTiempoPredeterminado();
            }
        }

        //public override ECarrera Carrera {
        //    get { return Rally; }
        //}
        #endregion

        #region Colecciones
        public static EColeccion<ESector> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<ESector> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<ESector> col = new EColeccion<ESector>();
            //TODO: Ponemos las propiedades que queremos mostrar en el listado.
            col.ColumnaMostrarRango(new string[] {  });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }
        #endregion
        
        public TimeSpan GetTiempoPredeterminado() {
            return HoraFin.Subtract(HoraIni);
        }

        public Collection<int> SeccionesPosibles() {
            Collection<int> col = new Collection<int>();
            int maxSeccion = ObtenerSeccionMax();
            if (this.EsNuevo) {
                if (maxSeccion > 0) {
                    col.Add(maxSeccion);
                }
                col.Add(maxSeccion+1);
            } else {
                int posSector = PosicionSector();
                if (mNumero == 1 || posSector ==  0) {
                    col.Add(mNumeroSeccion);
                } else {
                    if (posSector == 3) {//es el ltimo y hay ms de un sector en esa seccin.
                        col.Add(mNumeroSeccion);
                        col.Add(mNumeroSeccion + 1);
                    } else if (posSector == 2) {//Hay ms de un sector en esa seccin y este sector es el primero.
                        col.Add(mNumeroSeccion - 1);
                        col.Add(mNumeroSeccion);
                    } else {//Es nico o hay ms de dos y no es ni el primero ni el ltimo en su seccin
                        if (posSector == 1 && mNumeroSeccion != 1) {
                            col.Add(mNumeroSeccion - 1);
                        }
                        col.Add(mNumeroSeccion);
                        if (posSector == 1 && mNumeroSeccion != maxSeccion) {
                            col.Add(mNumeroSeccion + 1);
                        }
                    }
                }
            }
            return col;
        }

        //devuelve la posicin del sector dentro de su seccin
        private int PosicionSector() {
            EColeccion<ESector> col = SectoresDeUnaSeccion(this.Rally.Sectores, this.mNumeroSeccion);
            if (col.Count == 1) {//es nico
                return 1;
            }
            int pos = col.IndexOf(this);
            if (pos == 0) {//es el primero y hay ms de uno
                return 2;
            } else if (pos == col.Count - 1) {//es el ltimo y hay ms de uno
                return 3;
            } else if (pos > 0) {//est en medio.
                return 0;
            }
            return -1;
        }

        //devuelve una coleccin de sectores de forma ordenada dada una seccion
        public static EColeccion<ESector> SectoresDeUnaSeccion(EColeccion<ESector> col, int numeroSeccion) {
            EColeccion<ESector> res = new EColeccion<ESector>();
            foreach (ESector sector in col) {
                if (sector.mNumeroSeccion == numeroSeccion) {
                    res.Add(sector);
                }
            }
            return res;
        }

        private int ObtenerSeccionMax() {
            if (this.Rally != null) {
                EColeccion<ESector> col = this.Rally.Sectores;
                if (col != null && col.Count > 0) {
                    return col[col.Count - 1].mNumeroSeccion;
                }
            }
            return 1;
        }

        public override string ToString() {
            return "Sector: " + Numero + ". " + LocalizacionIni + "(" + CHIni + ") a" + LocalizacionFin + "(" + CHFin + ")";
        }

        private void CrearTiemposSector() {
            if (EsNuevo) {
                EColeccion<EPilotoCarrera> colInscritos = this.Rally.PilotosCarrera;
                foreach (EPilotoCarrera inscrito in colInscritos) {
                    ETiempoPilotoSector nuevo = this.TiemposPilotoSector.AddNew();
                    nuevo.PilotoCarrera = inscrito;
                }
            }
        }

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Sector")) {
                using (AmbitoTransa transa2 = this.CrearAmbitoTransa("Eliminar cronometrable.")) {
                    int indice2 = 0;
                    while (this.Sanciones.Count > indice2) {
                        Sanciones[indice2].Eliminar();
                        indice2 = indice2 + 1;
                    }
                    transa2.Commit();
                }
                ETramo t = Tramo;
                //if (t != null && !t.EstaVacio) {
                //    throw new Exception("No se puede eliminar el sector. Su tramo no est vaco.");
                //} else if (t != null) {
                //    t.Eliminar();
                //}
                if (t != null) {
                    t.Eliminar();
                }
                int indice = 0;
                while (indice < TiemposPilotoSector.Count) {
                    ETiempoPilotoSector obj = TiemposPilotoSector[indice];
                    //if (!obj.EstaVacio) {
                    //    throw new Exception("No se puede eliminar el sector. Contiene tiempos de inscritos.");
                    //}
                    obj.Eliminar();
                    indice = indice + 1;
                }
                base.Eliminar();
                transa.Commit();
            }
        }

        //internal override void ActualizarClasificacion() {
        //    throw new Exception("No existe clasificacion en un sector.");
        //}

        //public override EClasificacionCarrera GetClasificacion(EPilotoCarrera inscrito) {
        //    throw new Exception("No existe clasficicacin en los sectores, por tanto, no se puede buscar.");
        //}
    }
}
