using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;
using System.Text;

namespace Negocio {

    [Tabla("Carreras")]
    public class ESubida : ECarrera {
        #region Constructores

        public ESubida() : base() { }

        public ESubida(int vId) : base(vId) { }

        public ESubida(string alias) : base(alias) { }

        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            Tipo = TipoCarrera.Subida;
            mHayEntrenosLibres = false;
            Longitud = 0.0;
            
        }

        protected override void GuardarCore() {//actualizar mangas y fases.
            this.ActualizarFases();
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Subida")) {
                base.GuardarCore();
                if (mMangas != null)      //Si se ha cargado
                    mMangas.Guardar();   //guardar si hemos cambiado algo
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mMangas = null;
            base.Deshacer();
        }


        public override bool EsMasculino {
            get { return false; }  
        }

        public override string NombreDeLaEntidad {
            get { return "Subida"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPNombre; } 
        }
        #endregion

        #region Propiedades

        public const string NPHayEntrenosLibres = "HayEntrenosLibres";
        private bool mHayEntrenosLibres;

        [Campo]
        public bool HayEntrenosLibres {
            get {
                PrePropertyRead(NPHayEntrenosLibres);
                return mHayEntrenosLibres;
            }
            set {
                PrePropertyChanged(NPHayEntrenosLibres);
                mHayEntrenosLibres = value;
                OnPropertyChanged(NPHayEntrenosLibres);
            }
        }

        public const string NPLongitud = "Longitud";
        private double mLongitud;

        [Campo]
        public double Longitud {
            get {
                PrePropertyRead(NPLongitud);
                return mLongitud;
            }
            set {
                PrePropertyChanged(NPLongitud);
                mLongitud = value;
                OnPropertyChanged(NPLongitud);
            }
        }

        public const string NPNumeroFasesDeseado = "NumeroFasesDeseado";
        private int mNumeroFasesDeseado = 0;

        public int NumeroFasesDeseado {
            get {
                return mNumeroFasesDeseado;
            }
            set {
                mNumeroFasesDeseado = value;
            }
        }

        public const string NPNumeroFasesActual = "NumeroFasesActual";
        private int mNumeroFasesActual = 0;

        public int NumeroFasesActual {
            get {
                if (mNumeroFasesActual == 0) {
                    if (Mangas.Count > 0) {
                        string s = "";
                        foreach (EManga manga in Mangas) {
                            if (!s.Contains(manga.NombreFase)) {
                                mNumeroFasesActual++;
                                s += manga.NombreFase;
                            }
                        }
                    }
                }
                return mNumeroFasesActual;
            }
        }

        #endregion

        #region Mangas
        public const string NPMangas = "Mangas";
        private EColeccion<EManga> mMangas = null;

        /// <summary>Devuelve una coleccin con todos los Mangas.</summary>
        public EColeccion<EManga> Mangas {
            get {
                if(mMangas == null) {
                    mMangas = new EColeccion<EManga>();
                    if(!this.EsNuevo) {
                        mMangas.Filtros.Add(EManga.NPSubida, this);
                        mMangas.Cargar();
                    } 
                    mMangas.AddingNew += new AddingNewEventHandler(mMangas_AddingNew);
                    mMangas.ListChanged += new ListChangedEventHandler(mMangas_ListChanged);
                }
                return mMangas;
            }
        }

        private void mMangas_AddingNew(object sender, AddingNewEventArgs e) {
            EManga entidad = new EManga();
            entidad.Subida = this;
            e.NewObject = entidad;
        }

        private void mMangas_ListChanged(object sender, ListChangedEventArgs e) {
            if(!Mangas.EsValido)
                AddError(new PropertyException(NPMangas, this, Mangas.AlgunError.Message, Mangas.AlgunError));
            else
                RemoveError(NPMangas);
            OnPropertyCollectionChanged(NPMangas);
        }
        #endregion

        #region Colecciones

        public static EColeccion<ESubida> GetCollection(bool todasLasTemporadas) {
            return GetCollection(false, false, todasLasTemporadas);
        }

        public static EColeccion<ESubida> GetCollection(bool vacio, bool asincronamente, bool todasLasTemporadas) {
            EColeccion<ESubida> col = new EColeccion<ESubida>("SUBIDAS");
            col.Filtros.Add(NPTipo, TipoCarrera.Subida);
            col.Filtros.Add(NPActivo, true);
            if (!todasLasTemporadas) {
                col.Filtros.Add(NPTemporada, ETemporada.Actual);
            }
            col.ColumnaMostrarRango(new string[] { NPNombre, NPAlias, NPFechaComienzo, NPFechaFin, NPCoeficiente, NPLocalidad, NPProvincia });
            if(!vacio)
                col.Cargar(asincronamente);
            return col;
        }

        public static EColeccion<ESubida> GetCollection(bool activo, bool todasLasTemporadas) {
            EColeccion<ESubida> col = new EColeccion<ESubida>("SUBIDAS");
            col.Filtros.Add(NPTipo, TipoCarrera.Subida);
            col.Filtros.Add(NPActivo, activo);
            if (!todasLasTemporadas) {
                col.Filtros.Add(NPTemporada, ETemporada.Actual);
            }
            col.ColumnaMostrarRango(new string[] { NPNombre, NPAlias, NPFechaComienzo, NPFechaFin, NPCoeficiente, NPLocalidad, NPProvincia });
            col.Cargar();
            return col;
        }

        #endregion

        public override bool NecesitaCopiloto() {
            return false;
        }

        private void ActualizarFases() {
            int numFasesActual = NumeroFasesActual;
            string nombres = "ABCDEFGHIJK";
            if (mNumeroFasesDeseado != 0 && numFasesActual < mNumeroFasesDeseado) {//hay que agregar tantas fases como diferencia entre estos valores.
                for (int cantidad = numFasesActual; cantidad < mNumeroFasesDeseado; cantidad++) {
                    CrearMangas(nombres[cantidad].ToString());
                }
            } else {
                if (mNumeroFasesDeseado != 0 && numFasesActual > mNumeroFasesDeseado) {//hay que quitar tantas fases hasta llegar a mNumeroFases
                    for (int cantidad = numFasesActual; cantidad > mNumeroFasesDeseado; cantidad--) {
                        EliminarFase(nombres[cantidad - 1].ToString());
                    }
                }
                
            }
            mNumeroFasesActual = mNumeroFasesDeseado;
            this.ActualizarMangas();
        }

        //Crea las mangas sin la de entrenos libres.
        private void CrearMangas(string nombreFase) {
            Mangas.Add(new EManga(TipoManga.EOficiales, nombreFase, this));
            Mangas.Add(new EManga(TipoManga.MOfcial, nombreFase, this));
            //if (HayEntrenosLibres) {
            //    Mangas.Add(new EManga(TipoManga.ELibres, nombreFase, this));
            //}
        }

        private void ActualizarMangas() {//se actualizan los entrenos libres
            bool tieneELibres = this.TieneEntrenosLibres();
            if (!mHayEntrenosLibres && tieneELibres) {//eliminar
                EColeccion<EManga> col = Mangas.Clone() as EColeccion<EManga>;
                foreach (EManga manga in col) {
                    if (manga.TipoManga == TipoManga.ELibres) {
                        Mangas.Remove(manga);
                    }
                }
            } else if (mHayEntrenosLibres && !tieneELibres) {//crear
                string nombres = "ABCDEFGHIJK";
                int numFasesActual = NumeroFasesActual;
                for (int num = 0; num < numFasesActual; num++) {
                    Mangas.Add(new EManga( TipoManga.ELibres, nombres[num].ToString(), this));
                }
            }
        }

        private bool TieneEntrenosLibres() {
            EColeccion<EManga> col = Mangas;
            foreach (EManga m in col) {
                if (m.TipoManga == TipoManga.ELibres) {
                    return true;
                }
            }
            return false;
        }

        public void EliminarFase(string nombreFase) {//elimina las mangas del nombre de fase que se pasa como parmetro
            int indice = 0;
            while (indice < Mangas.Count) {
                EManga manga = Mangas[indice];
                if (manga.NombreFase.Equals(nombreFase)) {
                    Mangas.Remove(manga);
                } else {
                    indice++;
                }
            }                
        }

        public override bool EstaSucio {
            get {
                return base.EstaSucio || (mMangas != null && mMangas.EstaSucio) || (NumeroFasesActual != NumeroFasesDeseado);
            }
        }

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar Subida")) {
                int indice = 0;
                EColeccion<EPilotoCarrera> colInscrito = PilotosCarrera;
                while (indice < colInscrito.Count) {
                    colInscrito[indice].Eliminar();
                    indice = indice + 1;
                }
                indice = 0;
                EColeccion<EManga> colMangas = Mangas;
                while (indice < colMangas.Count){
                    colMangas[indice].Eliminar();
                    indice = indice + 1;
                }
                base.Eliminar();
                transa.Commit();
            }
        }
    }
}

