using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    [Tabla("TablasFijas")]
    public class ETablaFija : EIdBase {

        #region Constructores
        public ETablaFija() : base() { }

        public ETablaFija(int vId) : base(vId) { }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
        }

        public override bool EsMasculino {
            get { return false; } 
        }

        public override string NombreDeLaEntidad {
            get { return "TablaFija"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return ToString(); } 
        }
        #endregion

        #region Propiedades

        public const string NPPuesto = "Puesto";
        private int mPuesto;

        [Campo]
        public int Puesto {
            get {
                PrePropertyRead(NPPuesto);
                return mPuesto;
            }
            set {
                PrePropertyChanged(NPPuesto);
                mPuesto = value;
                if (mPuesto > 0) {
                    RemoveError(NPPuesto);
                } else {
                    AddError(new PropertyException(NPPuesto, this, "El puesto tiene que ser mayor que cero."));
                }
                OnPropertyChanged(NPPuesto);
            }
        }

        public const string NPPuntos = "Puntos";
        private int mPuntos;

        [Campo]
        public int Puntos {
            get {
                PrePropertyRead(NPPuntos);
                return mPuntos;
            }
            set {
                PrePropertyChanged(NPPuntos);
                mPuntos = value;
                if (mPuntos >= 0) {
                    RemoveError(NPPuntos);
                } else {
                    AddError(new PropertyException(NPPuntos, this, "Los puntos asignados no pueden ser negativos."));
                }
                OnPropertyChanged(NPPuntos);
            }
        }

        public const string NPCampeonatoPuntuable = "CampeonatoPuntuable";
        private ECampeonatoPuntuable mCampeonatoPuntuable;

        [Campo("CampeonatoPuntuableId"), Requerido]
        public ECampeonatoPuntuable CampeonatoPuntuable {
            get {
                PrePropertyRead(NPCampeonatoPuntuable);
                return mCampeonatoPuntuable;
            }
            set {
                PrePropertyChanged(NPCampeonatoPuntuable);
                mCampeonatoPuntuable = value;
                OnPropertyChanged(NPCampeonatoPuntuable);
            }
        }

        #endregion

        public override string ToString() {
            return "Puesto: " + Puesto.ToString() + " - Puntos: " + Puntos.ToString();
        }
    }
}

