using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("Tramos")]
    public class ETramo : ECronometrable {

        #region Constructores
        public ETramo() : base() { }

        public ETramo(int vId) : base(vId) { }

        public ETramo(ESector s)
            : base(){//s.Carrera) {
            Sector = s;
            base.Carrera = s.Rally;
        }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            base.EsAcumulable = true;
            Nombre = null;
            Descripcion = null;
            DescripcionLLegada = null;
            DescripcionSalida = null;
            Localizacion = null;
            Sector = null;
            base.Tipo = TipoCronometrable.Tramo;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Tramo")) {
                CrearClasificaciones();
                base.Carrera = Sector.Rally;
                base.GuardarCore();
                if (mSector != null && mSector.EstaSucio) {
                    mSector.Guardar();
                }
                if (colClasificacionesCreadas != null) {
                    colClasificacionesCreadas.Guardar();
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            //if (Sector != null) {
            //    Sector.Deshacer();
            //}
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return true; } 
        }

        public override string NombreDeLaEntidad {
            get { return "Tramo"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return this.ToString(); }
        }
        #endregion

        #region Propiedades

        public const string NPNombre = "Nombre";
        private string mNombre;

        [Campo(Tamanyo = 50), Requerido]
        public string Nombre {
            get {
                PrePropertyRead(NPNombre);
                return mNombre;
            }
            set {
                PrePropertyChanged(NPNombre);
                mNombre = value;
                //el nombre tiene que aparecer en el nombre del sector.
                if (!String.IsNullOrEmpty(mNombre) && this.Sector != null) {
                    Sector.CHIni = mNombre;
                }
                OnPropertyChanged(NPNombre);
            }
        }


        public const string NPLocalizacion = "Localizacion";
        private string mLocalizacion;

        [Campo(Tamanyo = 100)]
        public string Localizacion {
            get {
                PrePropertyRead(NPLocalizacion);
                return mLocalizacion;
            }
            set {
                PrePropertyChanged(NPLocalizacion);
                mLocalizacion = value;
                //la localizacin hay que ponerla en la localizacinInicio del sector.
                if (!String.IsNullOrEmpty(mLocalizacion) && this.Sector != null) {
                    mSector.LocalizacionIni = mLocalizacion;
                }
                OnPropertyChanged(NPLocalizacion);
            }
        }

        public const string NPDescripcion = "Descripcion";
        private string mDescripcion;

        [Campo(Tamanyo = 100)]
        public string Descripcion {
            get {
                PrePropertyRead(NPDescripcion);
                return mDescripcion;
            }
            set {
                PrePropertyChanged(NPDescripcion);
                mDescripcion = value;
                OnPropertyChanged(NPDescripcion);
            }
        }

        public const string NPDescripcionSalida = "DescripcionSalida";
        private string mDescripcionSalida;

        [Campo(Tamanyo = 100)]
        public string DescripcionSalida {
            get {
                PrePropertyRead(NPDescripcionSalida);
                return mDescripcionSalida;
            }
            set {
                PrePropertyChanged(NPDescripcionSalida);
                mDescripcionSalida = value;
                OnPropertyChanged(NPDescripcionSalida);
            }
        }

        public const string NPDescripcionLLegada = "DescripcionLLegada";
        private string mDescripcionLLegada;

        [Campo(Tamanyo = 100)]
        public string DescripcionLLegada {
            get {
                PrePropertyRead(NPDescripcionLLegada);
                return mDescripcionLLegada;
            }
            set {
                PrePropertyChanged(NPDescripcionLLegada);
                mDescripcionLLegada = value;
                OnPropertyChanged(NPDescripcionLLegada);
            }
        }

        public const string NPSector = "Sector";
        private ESector mSector;

        [Campo("SectorId"), Requerido]
        public ESector Sector {
            get {
                PrePropertyRead(NPSector);
                return mSector;
            }
            set {
                PrePropertyChanged(NPSector);
                mSector = value;
                OnPropertyChanged(NPSector);
            }
        }

        //public override ECarrera Carrera {
        //    get {
        //        if (mSector != null) {
        //            return mSector.Rally;
        //        }
        //        return null;
        //    }
        //}
        #endregion

        #region Colecciones
        public static EColeccion<ETramo> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<ETramo> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<ETramo> col = new EColeccion<ETramo>();
            //TODO: Ponemos las propiedades que queremos mostrar en el listado.
            col.ColumnaMostrarRango(new string[] {  });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }
        #endregion

        //HACK
        public static ETramo DevolverTramo(ESector sector) {
            //se puede hacer de dos formas, voy a hacerla recorriendo la coleccin.
            //EColeccion<ETramo> col = ETramo.GetCollection();
            //foreach (ETramo tramo in col) {
            //    if (tramo.Sector.Equals(sector)) {
            //        return tramo;
            //    }
            //}
            //return null;
            ETramo tramo = new ETramo();
            tramo.Cargar(NPSector, sector);
            if (tramo.Existe) {
                return tramo;
            }
            return null;
        }

        public override string ToString() {
            return "Tramo: " + Nombre + ". " + Localizacion;
        }

        private EColeccion<EClasificacionCarreraCampeonato> colClasificacionesCreadas = null;
        private void CrearClasificaciones() {
            if (EsNuevo) {
                colClasificacionesCreadas = new EColeccion<EClasificacionCarreraCampeonato>();
                EColeccion<EPilotoCarrera> colInscritos = this.Carrera.PilotosCarrera;
                foreach (EPilotoCarrera inscrito in colInscritos) {
                    EClasificacionTramo clasificacion = this.ClasificacionesTramo.AddNew();
                    clasificacion.PilotoCarrera = inscrito;
                    EColeccion<EClasificacionTotalRally> colClasifRally = this.Sector.Rally.ClasificacionesRally;
                    bool encontrado = false;
                    int indice = 0;
                    while (!encontrado && indice < colClasifRally.Count) {
                        if (colClasifRally[indice].PilotoCarrera.Equals(inscrito)) {
                            clasificacion.ClasificacionRally = colClasifRally[indice];
                            encontrado = true;
                        } else {
                            indice = indice + 1;
                        }
                    }
                    //Creamos las clasificaciones carrera Campeonato
                    foreach (ECampeonatoInscrito ci in inscrito.CampeonatosInscrito) {
                        EClasificacionCarreraCampeonato clasifCampeonato = new EClasificacionCarreraCampeonato();
                        clasifCampeonato.ClasificacionCarrera = clasificacion;
                        clasifCampeonato.Campeonato = ci.Campeonato.Campeonato.Campeonato;
                        colClasificacionesCreadas.Add(clasifCampeonato);
                        EClasificacionCarreraCampeonato clasifGrupo = new EClasificacionCarreraCampeonato();
                        clasifGrupo.ClasificacionCarrera = clasificacion;
                        clasifGrupo.Campeonato = ci.Campeonato.Campeonato;
                        colClasificacionesCreadas.Add(clasifGrupo);
                        EClasificacionCarreraCampeonato clasifClase = new EClasificacionCarreraCampeonato();
                        clasifClase.Campeonato = ci.Campeonato;
                        clasifClase.ClasificacionCarrera = clasificacion;
                        colClasificacionesCreadas.Add(clasifClase);
                    }
                }
            }
        }

        public override void Eliminar() {
            using (AmbitoTransa transa3 = this.CrearAmbitoTransa("Eliminar Tramo")) {
                using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar Tramo")) {
                    using (AmbitoTransa transa2 = this.CrearAmbitoTransa("Eliminar cronometrable.")) {
                        int indice2 = 0;
                        while (this.Sanciones.Count > indice2) {
                            Sanciones[indice2].Eliminar();
                            indice2 = indice2 + 1;
                        }
                        transa2.Commit();
                    }
                    int indice = 0;
                    while (ClasificacionesTramo.Count > indice) {
                        EClasificacionTramo obj = ClasificacionesTramo[indice];
                        //if (!obj.EstaVacio) {
                        //    throw new Exception("No se puede eliminar el tramo.\nContiene clasificaciones.");
                        //}
                        obj.Eliminar();
                        indice = indice + 1;
                    }
                    base.Eliminar();
                    transa.Commit();
                }
                //Actualizamos clasificacion del rally.
                this.Sector.Rally.ActualizarClasificacion();
                this.Sector.Rally.Guardar();
                transa3.Commit();
            }
        }

        public bool EstaVacio {
            get {
                foreach (EClasificacionTramo clasificacion in ClasificacionesTramo) {
                    if (!clasificacion.EstaVacio) {
                        return false;
                    }
                }
                return Sanciones.Count == 0;
            }
        }

        //internal override void ActualizarClasificacion() {
        //    EColeccion<EClasificacionTramo> col = this.ClasificacionesTramo;
        //    col.ApplySort(EClasificacionCarrera.NPTiempoEfectivo, ListSortDirection.Ascending);
        //    EClasificacionCarrera primero = null;
        //    int puestoActual = 1;
        //    for (int indice = 0; indice < col.Count; indice++) {
        //        EClasificacionCarrera clasificaconCarrera = col[indice] as EClasificacionCarrera;
        //        if (clasificaconCarrera.HaAbandonado() || !clasificaconCarrera.EsMostrable()) {
        //            clasificaconCarrera.Puesto = 0;
        //        } else {
        //            if (puestoActual == 1) {
        //                primero = clasificaconCarrera;
        //                primero.TiempoAnterior = null;
        //                //primero.ClasificacionAnterior = null;
        //            } else {
        //                clasificaconCarrera.TiempoAnterior = (col[indice - 1] as EClasificacionCarrera).TiempoEfectivo;
        //                //cc.ClasificacionAnterior = col[indice - 1] as EClasificacionCarrera;
        //            }
        //            clasificaconCarrera.Puesto = puestoActual;
        //            clasificaconCarrera.TiempoPrimero = primero.TiempoEfectivo;
        //            //cc.ClasificacionPrimero = primero;
        //            puestoActual = puestoActual + 1;
        //        }
        //    }
        //}

        //public override EClasificacionCarrera GetClasificacion(EPilotoCarrera inscrito) {
        //    return BuscarClasificacion(this.ClasificacionesTramo, inscrito);
        //}
    }
}
