using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;
using System.Collections.ObjectModel;
using System.Drawing;

namespace Negocio {

    public enum TipoClasificacionCarrera {
        Ninguno = 0,
        Manga = 1,
        Rally = 2,
        Tramo = 3
    }

    [Tabla("ClasificacionesCarrera")]
    public abstract class EClasificacionCarrera : EIdBase, IClasificacion {

        #region Constructores

        public EClasificacionCarrera() : base() { }

        public EClasificacionCarrera(int vId) : base(vId) { }

        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            TipoClasificacion = TipoClasificacionCarrera.Ninguno;
            Puesto = 0;
            mHoraLLegada = "00";
            mMinutosLLegada = "00";
            mSegundosLLegada = "00";
            mMilesimasLLegada = "000";
            mHoraSalida = "00";
            mMinutosSalida = "00";
            mSegundosSalida = "00";
            mMilesimasSalida = "000";
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar ClasificacionCarrera")) {
                base.GuardarCore();
                //if (!EsNuevo) {
                //    ActualizarClasificacionesCampeonato();
                //}
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mClasificacionesCarreraCampeonatos = null;
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return false; }  
        }

        public override string NombreDeLaEntidad {
            get { return "ClasificacionTotal"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPPuesto; }
        }

        #endregion

        #region Propiedades

        public const string NPTipoClasificacion = "TipoClasificacion";
        private TipoClasificacionCarrera mTipoClasificacion;

        [Campo]
        public TipoClasificacionCarrera TipoClasificacion {
            get {
                PrePropertyRead(NPTipoClasificacion);
                return mTipoClasificacion;
            }
            set {
                PrePropertyChanged(NPTipoClasificacion);
                mTipoClasificacion = value;
                OnPropertyChanged(NPTipoClasificacion);
            }
        }

        public const string NPPuesto = "Puesto";
        private int mPuesto;

        [Campo]
        public int Puesto {
            get {
                PrePropertyRead(NPPuesto);
                return mPuesto;
            }
            set {
                PrePropertyChanged(NPPuesto);
                mPuesto = value;
                OnPropertyChanged(NPPuesto);
            }
        }

        public const string NPAbandono = "Abandono";
        private bool mAbandono;

        [Campo]
        public virtual bool Abandono {
            get {
                PrePropertyRead(NPAbandono);
                return mAbandono;
            }
            set {
                PrePropertyChanged(NPAbandono);
                mAbandono = value;
                if (!mAbandono && !String.IsNullOrEmpty(MotivoAbandono)) {
                    mMotivoAbandono = "";
                }
                OnPropertyChanged(NPAbandono);
            }
        }

        public const string NPMotivoAbandono = "MotivoAbandono";
        private string mMotivoAbandono;

        [Campo(Tamanyo = 100)]
        public string MotivoAbandono {
            get {
                PrePropertyRead(NPMotivoAbandono);
                return mMotivoAbandono;
            }
            set {
                PrePropertyChanged(NPMotivoAbandono);
                mMotivoAbandono = value;
                OnPropertyChanged(NPMotivoAbandono);
            }
        }

        public const string NPTiempoRealSalida = "TiempoRealSalida";
        private TimeSpan mTiempoRealSalida;
        private bool mTiempoRealSalidaCargado = false;

        public virtual TimeSpan TiempoRealSalida {
            get {
                if (!mTiempoRealSalidaCargado) {
                    int minutos;
                    int segundos;
                    int milesimas;
                    int hora;
                    if (Int32.TryParse(MinutosSalida, out minutos) && Int32.TryParse(SegundosSalida, out segundos) && Int32.TryParse(MilesimasSalida, out milesimas) && Int32.TryParse(HoraSalida, out hora)) {
                        mTiempoRealSalidaCargado = true;
                        mTiempoRealSalida = new TimeSpan(0, hora, minutos, segundos, milesimas);
                    } else {
                        mTiempoRealSalida = new TimeSpan();
                    }
                }
                return mTiempoRealSalida;
            }
            internal set {
                TimeSpan mTiempoRealAnterior = mTiempoRealSalida;
                mTiempoRealSalida = value;
                if (mTiempoRealSalidaCargado && this.EstaCargado) {
                    //Actualizar tiempo efectivo.
                    if (mTiempoRealSalida.CompareTo(TiempoRealLLegada) > 0) {
                        AddError(new PropertyException(NPTiempoRealSalida, this, "El tiempo de salida no puede ser mayor que el tiempo de llegada."));
                    } else {
                        RemoveError(NPTiempoRealSalida);
                        RemoveError(NPTiempoRealLLegada);
                        TimeSpan mTiempoReal = TiempoRealLLegada.Subtract(mTiempoRealSalida);
                        TiempoEfectivo = TiempoEfectivo.Subtract(mTiempoRealAnterior);
                        TiempoEfectivo = TiempoEfectivo.Add(mTiempoReal);
                    }
                }
            }
        }


        public const string NPTiempoRealLLegada = "TiempoRealLLegada";
        private TimeSpan mTiempoRealLLegada;
        private bool mTiempoRealLLegadaCargado = false;

        public virtual TimeSpan TiempoRealLLegada {
            get {
                if (!mTiempoRealLLegadaCargado) {
                    int minutos;
                    int segundos;
                    int milesimas;
                    int hora;
                    if (Int32.TryParse(MinutosLLegada, out minutos) && Int32.TryParse(SegundosLLegada, out segundos) && Int32.TryParse(MilesimasLLegada, out milesimas) && Int32.TryParse(HoraLLegada, out hora)) {
                        mTiempoRealLLegadaCargado = true;
                        mTiempoRealLLegada = new TimeSpan(0, hora, minutos, segundos, milesimas);
                    } else {
                        mTiempoRealLLegada = new TimeSpan();
                    }
                }
                return mTiempoRealLLegada;
            }
            internal set {
                TimeSpan mTiempoRealAnterior = mTiempoRealLLegada;
                mTiempoRealLLegada = value;
                if (mTiempoRealLLegadaCargado && this.EstaCargado) {
                    //actualizar tiempo efectivo.
                    if (TiempoRealSalida.CompareTo(mTiempoRealLLegada) > 0) {
                        AddError(new PropertyException(NPTiempoRealLLegada, this, "El tiempo de salida no puede ser mayor que el tiempo de llegada."));
                    } else {
                        RemoveError(NPTiempoRealSalida);
                        RemoveError(NPTiempoRealLLegada);
                        TimeSpan mTiempoReal = mTiempoRealLLegada.Subtract(TiempoRealSalida);
                        TiempoEfectivo = TiempoEfectivo.Subtract(mTiempoRealAnterior);
                        TiempoEfectivo = TiempoEfectivo.Add(mTiempoReal);
                    }
                }
            }
        }

        public const string NPTiempoReal = "TiempoReal";
        public TimeSpan TiempoReal {
            get {
                return TiempoRealLLegada.Subtract(TiempoRealSalida);
            }
        }

        public string TiempoRealToString {
            get {
                return CToString.ToString(TiempoReal);
            }
        }

        public const string NPTiempoEfectivo = "TiempoEfectivo";
        private TimeSpan mTiempoEfectivo;
        private bool mTiempoEfectivoCargado = false;

        public virtual TimeSpan TiempoEfectivo {
            get {
                if (!mTiempoEfectivoCargado && mTiempoRealLLegadaCargado && this.EstaCargado) {
                    mTiempoEfectivo = GetTiempoEfectivo();
                } else {
                    mTiempoEfectivo = new TimeSpan();
                }
                return mTiempoEfectivo;
            }
            set {
                mTiempoEfectivo = value;
                //if (this.EstaCargado) {
                //    ActualizarClasificacion();
                //}
            }
        }

        #region Tiempo

        public const string NPHoraSalida = "HoraSalida";
        private string mHoraSalida;

        [Campo("HoraSalida", Tamanyo = 2)]
        public string HoraSalida {
            get {
                PrePropertyRead(NPHoraSalida);
                return mHoraSalida;
            }
            set {
                PrePropertyChanged(NPHoraSalida);
                mHoraSalida = value;
                if (!String.IsNullOrEmpty(mHoraSalida) && mHoraSalida.Length < 2) {
                    mHoraSalida = "0" + mHoraSalida;
                }
                //comprobar que es un numero y est entre 00 y 23
                int hora;
                if (Int32.TryParse(mHoraSalida, out hora) && hora >= 0 && hora <= 23) {
                    RemoveError(NPHoraSalida);
                    //agregar estas horas a tiempo.
                    TiempoRealSalida = new TimeSpan(0, hora, TiempoRealSalida.Minutes, TiempoRealSalida.Seconds, TiempoRealSalida.Milliseconds);
                } else {
                    AddError(new PropertyException(NPHoraSalida, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 23."));
                }
                OnPropertyChanged(NPHoraSalida);
            }
        }

        public const string NPMinutosSalida = "MinutosSalida";
        private string mMinutosSalida;

        [Campo("MinutosSalida", Tamanyo = 2)]
        public string MinutosSalida {
            get {
                PrePropertyRead(NPMinutosSalida);
                return mMinutosSalida;
            }
            set {
                PrePropertyChanged(NPMinutosSalida);
                mMinutosSalida = value;
                if (!String.IsNullOrEmpty(mMinutosSalida) && mMinutosSalida.Length < 2) {
                    mMinutosSalida = "0" + mMinutosSalida;
                }
                //comprobar que es un numero y est entre 00 y 59
                int minutos;
                if (Int32.TryParse(mMinutosSalida, out minutos) && minutos >= 0 && minutos <= 59) {
                    RemoveError(NPMinutosSalida);
                    //agregar estos minutos a tiempo.
                    TiempoRealSalida = new TimeSpan(0, TiempoRealSalida.Hours, minutos, TiempoRealSalida.Seconds, TiempoRealSalida.Milliseconds);
                } else {
                    AddError(new PropertyException(NPMinutosSalida, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                }
                OnPropertyChanged(NPMinutosSalida);
            }
        }

        public const string NPSegundosSalida = "SegundosSalida";
        private string mSegundosSalida;

        [Campo("SegundosSalida", Tamanyo = 2)]
        public string SegundosSalida {
            get {
                PrePropertyRead(NPSegundosSalida);
                return mSegundosSalida;
            }
            set {
                PrePropertyChanged(NPSegundosSalida);
                mSegundosSalida = value;
                if (!String.IsNullOrEmpty(mSegundosSalida) && mSegundosSalida.Length < 2) {
                    mSegundosSalida = "0" + mSegundosSalida;
                }
                //comprobar que es un numero y est entre 00 y 59
                int segundos;
                if (Int32.TryParse(mSegundosSalida, out segundos) && segundos >= 0 && segundos <= 59) {
                    RemoveError(NPSegundosSalida);
                    //agregar a tiempo
                    TiempoRealSalida = new TimeSpan(0, TiempoRealSalida.Hours, TiempoRealSalida.Minutes, segundos, TiempoRealSalida.Milliseconds);
                } else {
                    AddError(new PropertyException(NPSegundosSalida, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                }
                OnPropertyChanged(NPSegundosSalida);
            }
        }

        public const string NPMilesimasSalida = "MilesimasSalida";
        private string mMilesimasSalida;

        [Campo("MilesimasSalida", Tamanyo = 3)]
        public string MilesimasSalida {
            get {
                PrePropertyRead(NPMilesimasSalida);
                return mMilesimasSalida;
            }
            set {
                PrePropertyChanged(NPMilesimasSalida);
                mMilesimasSalida = value;
                if (!String.IsNullOrEmpty(mMilesimasSalida) && mMilesimasSalida.Length < 3) {
                    if (mMilesimasSalida.Length < 2) {
                        mMilesimasSalida = "00" + mMilesimasSalida;
                    } else {
                        mMilesimasSalida = "0" + mMilesimasSalida;
                    }
                }
                //comprobar que es un numero y est entre 000 y 999
                int milesimas;
                if (Int32.TryParse(mMilesimasSalida, out milesimas) && milesimas >= 0 && milesimas <= 999) {
                    RemoveError(NPMilesimasSalida);
                    //agregar a tiempo.
                    TiempoRealSalida = new TimeSpan(0, TiempoRealSalida.Hours, TiempoRealSalida.Minutes, TiempoRealSalida.Seconds, milesimas);
                } else {
                    AddError(new PropertyException(NPMilesimasSalida, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 999."));
                }
                OnPropertyChanged(NPMilesimasSalida);
            }
        }

        public const string NPHoraLLegada = "HoraLLegada";
        private string mHoraLLegada;

        [Campo("HoraLLegada", Tamanyo = 2)]
        public string HoraLLegada {
            get {
                PrePropertyRead(NPHoraLLegada);
                return mHoraLLegada;
            }
            set {
                PrePropertyChanged(NPHoraLLegada);
                mHoraLLegada = value;
                if (!String.IsNullOrEmpty(mHoraLLegada) && mHoraLLegada.Length < 2) {
                    mHoraLLegada = "0" + mHoraLLegada;
                }
                //comprobar que es un numero y est entre 00 y 23
                int hora;
                if (Int32.TryParse(mHoraLLegada, out hora) && hora >= 0 && hora <= 23) {
                    RemoveError(NPHoraLLegada);
                    //agregar estas horas a tiempo.
                    TiempoRealLLegada = new TimeSpan(0, hora, TiempoRealLLegada.Minutes, TiempoRealLLegada.Seconds, TiempoRealLLegada.Milliseconds);
                } else {
                    AddError(new PropertyException(NPHoraLLegada, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 23."));
                }
                OnPropertyChanged(NPHoraLLegada);
            }
        }

        public const string NPMinutosLLegada = "MinutosLLegada";
        private string mMinutosLLegada;

        [Campo(Tamanyo = 2)]
        public virtual string MinutosLLegada {
            get {
                PrePropertyRead(NPMinutosLLegada);
                return mMinutosLLegada;
            }
            set {
                PrePropertyChanged(NPMinutosLLegada);
                mMinutosLLegada = value;
                if (!String.IsNullOrEmpty(mMinutosLLegada) && mMinutosLLegada.Length < 2) {
                    mMinutosLLegada = "0" + mMinutosLLegada;
                }
                //comprobar que es un numero y est entre 00 y 59
                int minutos;
                if (Int32.TryParse(mMinutosLLegada, out minutos) && minutos >= 0 && minutos <= 59) {
                    RemoveError(NPMinutosLLegada);
                    //agregar estos minutos a tiempo.
                    TiempoRealLLegada = new TimeSpan(0, TiempoRealLLegada.Hours, minutos, TiempoRealLLegada.Seconds, TiempoRealLLegada.Milliseconds);
                } else {
                    AddError(new PropertyException(NPMinutosLLegada, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                }
                OnPropertyChanged(NPMinutosLLegada);
            }
        }

        public const string NPSegundosLLegada = "SegundosLLegada";
        private string mSegundosLLegada;

        [Campo(Tamanyo = 2)]
        public virtual string SegundosLLegada {
            get {
                PrePropertyRead(NPSegundosLLegada);
                return mSegundosLLegada;
            }
            set {
                PrePropertyChanged(NPSegundosLLegada);
                mSegundosLLegada = value;
                if (!String.IsNullOrEmpty(mSegundosLLegada) && mSegundosLLegada.Length < 2) {
                    mSegundosLLegada = "0" + mSegundosLLegada;
                }
                //comprobar que es un numero y est entre 00 y 59
                int segundos;
                if (Int32.TryParse(mSegundosLLegada, out segundos) && segundos >= 0 && segundos <= 59) {
                    RemoveError(NPSegundosLLegada);
                    //Agregar a tiempo de llegada.
                    TiempoRealLLegada = new TimeSpan(0, TiempoRealLLegada.Hours, TiempoRealLLegada.Minutes, segundos, TiempoRealLLegada.Milliseconds);
                } else {
                    AddError(new PropertyException(NPSegundosLLegada, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                }
                OnPropertyChanged(NPSegundosLLegada);
            }
        }

        public const string NPMilesimasLLegada = "MilesimasLLegada";
        private string mMilesimasLLegada;

        [Campo(Tamanyo = 3)]
        public virtual string MilesimasLLegada {
            get {
                PrePropertyRead(NPMilesimasLLegada);
                return mMilesimasLLegada;
            }
            set {
                PrePropertyChanged(NPMilesimasLLegada);
                mMilesimasLLegada = value;
                if (!String.IsNullOrEmpty(mMilesimasLLegada) && mMilesimasLLegada.Length < 3) {
                    if (mMilesimasLLegada.Length < 2) {
                        mMilesimasLLegada = "00" + mMilesimasLLegada;
                    } else {
                        mMilesimasLLegada = "0" + mMilesimasLLegada;
                    }
                }
                //comprobar que es un numero y est entre 000 y 999
                int milesimas;
                if (Int32.TryParse(mMilesimasLLegada, out milesimas) && milesimas >= 0 && milesimas <= 999) {
                    RemoveError(NPMilesimasLLegada);
                    //agregar a tiempo.
                    TiempoRealLLegada = new TimeSpan(0, TiempoRealLLegada.Hours, TiempoRealLLegada.Minutes, TiempoRealLLegada.Seconds, milesimas);
                } else {
                    AddError(new PropertyException(NPMilesimasLLegada, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 999."));
                }
                OnPropertyChanged(NPMilesimasLLegada);
            }
        }

        #endregion

        public const string NPPilotoCarrera = "PilotoCarrera";
        private EPilotoCarrera mPilotoCarrera;

        [Campo("PilotoCarreraId"), Requerido]
        public EPilotoCarrera PilotoCarrera {
            get {
                PrePropertyRead(NPPilotoCarrera);
                return mPilotoCarrera;
            }
            set {
                PrePropertyChanged(NPPilotoCarrera);
                mPilotoCarrera = value;
                OnPropertyChanged(NPPilotoCarrera);
            }
        }

        public const string NPTiempoAnterior = "TiempoAnterior";
        private TimeSpan? mTiempoAnterior;
        private bool mTiempoAnteriorCargado = false;

        public TimeSpan? TiempoAnterior {
            get {
                if (!mTiempoAnteriorCargado) {
                    mTiempoAnterior = GetTiempo(this.Puesto - 1);
                    mTiempoAnteriorCargado = true;
                }
                return mTiempoAnterior;
            }
            set {
                mTiempoAnterior = value;
                mTiempoAnteriorCargado = true;
            }
        }

        public const string NPTiempoPrimero = "TiempoPrimero";
        private TimeSpan? mTiempoPrimero;
        private bool mTiempoPrimeroCargado = false;

        public TimeSpan? TiempoPrimero {
            get {
                if (!mTiempoPrimeroCargado) {
                    mTiempoPrimero = GetTiempo(1);
                    mTiempoPrimeroCargado = true;
                }
                return mTiempoPrimero;
            }
            set {
                mTiempoPrimero = value;
                mTiempoPrimeroCargado = true;
            }
        }

        //public const string NPClasificacionPrimero = "ClasificacionPrimero";
        //private EClasificacionCarrera mClasificacionPrimero;

        //[Campo("ClasificacionPrimeroId", Tamanyo = 50)]
        //public EClasificacionCarrera ClasificacionPrimero {
        //    get {
        //        PrePropertyRead(NPClasificacionPrimero);
        //        return mClasificacionPrimero;
        //    }
        //    set {
        //        PrePropertyChanged(NPClasificacionPrimero);
        //        mClasificacionPrimero = value;
        //        //TODO: Validacin del dato: AddError()
        //        OnPropertyChanged(NPClasificacionPrimero);
        //    }
        //}

        //public const string NPClasificacionAnterior = "ClasificacionAnterior";
        //private EClasificacionCarrera mClasificacionAnterior;

        //[Campo("ClasificacionAnteriorId", Tamanyo = 50)]
        //public EClasificacionCarrera ClasificacionAnterior {
        //    get {
        //        PrePropertyRead(NPClasificacionAnterior);
        //        return mClasificacionAnterior;
        //    }
        //    set {
        //        PrePropertyChanged(NPClasificacionAnterior);
        //        mClasificacionAnterior = value;
        //        //TODO: Validacin del dato: AddError()
        //        OnPropertyChanged(NPClasificacionAnterior);
        //    }
        //}

        #region Impresion

        public const string NPTiempoEfectivoToString = "TiempoEfectivoToString";
        public string TiempoEfectivoToString {
            get {
                return CToString.ToString(TiempoEfectivo);
            }
        }

        public const string NPDorsal = "Dorsal";
        public string Dorsal {
            get {
                return PilotoCarrera.Dorsal.ToString();
            }
        }

        public const string NPPiloto = "Piloto";
        public string Piloto {
            get {
                return PilotoCarrera.Piloto.ToString();
            }
        }

        public const string NPVehiculo = "Vehiculo";
        public string Vehiculo {
            get {
                return PilotoCarrera.Vehiculo.ToString();
            }
        }

        public const string NPDifTiempoAnteriorToString = "DifTiempoAnteriorToString";
        public string DifTiempoAnteriorToString {
            get {
                if (TiempoAnterior.HasValue) {
                    return CToString.ToString(TiempoEfectivo.Subtract(TiempoAnterior.Value));
                }
                return "";
            }
        }

        public const string NPDifTiempoPrimeroToString = "DifTiempoPrimeroToString";
        public string DifTiempoPrimeroToString {
            get {
                if (TiempoPrimero.HasValue) {
                    return CToString.ToString(TiempoEfectivo.Subtract(TiempoPrimero.Value));
                }
                return "";
            }
        }

        public const string NPLogoCarrera = "LogoCarrera";

        public Image LogoCarrera {
            get {
                return PilotoCarrera.Carrera.Logo;
            }
        }

        public const string NPNombreClasificacion = "NombreClasificacion";
        public abstract string NombreClasificacion { get; }

        public const string NPNombreCarrera = "NombreCarrera";

        public string NombreCarrera {
            get {
                return PilotoCarrera.Carrera.ToString();
            }
        }

        public const string NPEscuderia = "Escuderia";
        public string Escuderia {
            get {
                if (PilotoCarrera.Piloto.Escuderia != null) {
                    return PilotoCarrera.Piloto.Escuderia.ToString();
                }
                return "";
            }
        }

        /// <summary>
        /// Devuelve el motivo del inscrito si ha abandonado y si tiene sancin de descalificacin 
        /// tambin se le atribuye.
        /// </summary>
        public string MotivoToString {
            get {
                string resultado = MotivoAbandono;
                EColeccion<ESancion> colSanciones = this.PilotoCarrera.Sanciones;
                bool encontrado = false;
                int indice = 0;
                while (!encontrado && indice < colSanciones.Count) {
                    ESancion sancion = colSanciones[indice];
                    if (sancion.TipoSancion == TipoSancion.Descalificacion) {
                        if (!String.IsNullOrEmpty(resultado)) {
                            resultado = resultado + "\n";
                        }
                        resultado = resultado + "Descalificacin: " + sancion.Motivo;
                        encontrado = true;
                    }
                    indice = indice + 1;
                }
                return resultado;
            }
        }
        #endregion

        #endregion

        #region ClasificacionesCarreraCampeonatos
        public const string NPClasificacionesCarreraCampeonatos = "ClasificacionesCarreraCampeonatos";
        protected IEColeccion mClasificacionesCarreraCampeonatos = null;

        ///// <summary>Devuelve una coleccin con todos los ClasificacionesCarreraCampeonatos.</summary>
        //public EColeccion<EClasificacionCarreraCampeonato> ClasificacionesCarreraCampeonatos {
        //    get {
        //        if(mClasificacionesCarreraCampeonatos == null) {
        //            mClasificacionesCarreraCampeonatos = new EColeccion<EClasificacionCarreraCampeonato>();
        //            if(!this.EsNuevo) {
        //                mClasificacionesCarreraCampeonatos.Filtros.Add(EClasificacionCarreraCampeonato.NPClasificacionCarrera, this);
        //                mClasificacionesCarreraCampeonatos.Cargar();
        //            }
        //            mClasificacionesCarreraCampeonatos.AddingNew += new AddingNewEventHandler(mClasificacionesCarreraCampeonatos_AddingNew);
        //        }
        //        return mClasificacionesCarreraCampeonatos;
        //    }
        //}

        //private void mClasificacionesCarreraCampeonatos_AddingNew(object sender, AddingNewEventArgs e) {
        //    EClasificacionCarreraCampeonato entidad = new EClasificacionCarreraCampeonato();
        //    entidad.ClasificacionCarrera = this;
        //    e.NewObject = entidad;
        //}

        /// <summary>Devuelve una coleccin con todos los ClasificacionesCarreraCampeonatos.</summary>
        public abstract IEColeccion ClasificacionesCarreraCampeonatos { get;}

        #endregion

        #region Colecciones

        public static EColeccion<EClasificacionCarrera> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EClasificacionCarrera> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EClasificacionCarrera> col = new EColeccion<EClasificacionCarrera>();
            col.ColumnaMostrarRango(new string[] {NPPuesto, NPMinutosLLegada, NPSegundosLLegada});
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }

        #endregion

        protected abstract void ActualizarClasificacion();

        public abstract TimeSpan? GetTiempo(int puesto);

        public abstract TimeSpan? GetTiempo(int puesto, ECampeonato campeonato);

        public abstract int GetPuntos(int puesto, ECampeonato campeonato);

        public EClasificacionCarreraCampeonato GetClasificacionCarreraCampeonato(ECampeonato campeonato) {
            EColeccion<EClasificacionCarreraCampeonato> col = this.ClasificacionesCarreraCampeonatos as EColeccion<EClasificacionCarreraCampeonato>;
            foreach (EClasificacionCarreraCampeonato ccc in col) {
                if (ccc.Campeonato.Equals(campeonato)) {
                    return ccc;
                }
            }
            return null;
        }

        public EClasificacionCarreraCampeonatoConPuntos GetClasificacionCarreraCampeonatoConPuntos(ECampeonato campeonato) {
            if (campeonato == null || campeonato.Nivel != 2) {
                return null;
            }
            IEColeccion col = this.ClasificacionesCarreraCampeonatos;
            foreach (ESinClaveBase esb in col) {
                EClasificacionCarreraCampeonatoConPuntos conpuntos = esb as EClasificacionCarreraCampeonatoConPuntos;
                if (conpuntos != null && conpuntos.Campeonato.Equals(campeonato)) {
                    return conpuntos;
                }
            }
            return null;
        }

        /// <summary>
        /// Crea Clasificaciones Campeonato y actualiza la clasificacion para todas las clasificaciones Campeonato.
        /// </summary>
        protected abstract void ActualizarClasificacionesCampeonato();

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar ClasificacionCarrera")) {
                int indice = 0;
                EColeccion<EClasificacionCarreraCampeonato> col = ClasificacionesCarreraCampeonatos as EColeccion<EClasificacionCarreraCampeonato>;
                while (col.Count > indice) {
                    EClasificacionCarreraCampeonatoConPuntos objHerencia = col[indice] as EClasificacionCarreraCampeonatoConPuntos;
                    if (objHerencia != null) {
                        objHerencia.Eliminar();
                    } else {
                        EClasificacionCarreraCampeonato obj = col[indice] as EClasificacionCarreraCampeonato;
                        if (obj != null) {
                            obj.Eliminar();
                        }
                    }
                    indice = indice + 1;
                }
                base.Eliminar();
                transa.Commit();
            }
        }

        /// <summary>
        /// Devuelve el tiempo efectivo de esta clasificacin, que resulta el tiempo real que ha realizado el inscrito
        /// restandole las sanciones que haya podido tener, si las sanciones superan el tiempo real, devuelve un tiempo 0.
        /// </summary>
        /// <returns></returns>
        protected abstract TimeSpan GetTiempoEfectivo();

        public abstract bool EstaVacio { get;}

        #region IClasificacion

        public abstract bool HaAbandonado();

        public bool EsMostrable() {
            return this.PilotoCarrera.Estado == TipoEstado.Autorizado;
        }
        #endregion
    }
}