using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("ClasificacionesCarreraCampeonato")]
    public class EClasificacionCarreraCampeonato : EIdBase, IClasificacion {

        #region Constructores

        public EClasificacionCarreraCampeonato() : base() { }

        public EClasificacionCarreraCampeonato(int vId) : base(vId) { }

        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            
        }

        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "ClasificacionCarreraCampeonato"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return ""; }
        }
        #endregion

        #region Propiedades

        public const string NPPuesto = "Puesto";
        private int mPuesto;

        [Campo, Requerido]
        public int Puesto {
            get {
                PrePropertyRead(NPPuesto);
                return mPuesto;
            }
            set {
                PrePropertyChanged(NPPuesto);
                mPuesto = value;
                OnPropertyChanged(NPPuesto);
            }
        }
        
        public const string NPCampeonato = "Campeonato";
        private ECampeonato mCampeonato;

        [Campo("CampeonatoId"), Requerido]
        public virtual ECampeonato Campeonato {
            get {
                PrePropertyRead(NPCampeonato);
                return mCampeonato;
            }
            set {
                PrePropertyChanged(NPCampeonato);
                mCampeonato = value;
                //if (EstaOK()) {
                //    RemoveError(NPCampeonato);
                //} else if (this.EstaCargado) {
                //    RemoveError(NPCampeonato);
                //    AddError(new PropertyException(NPCampeonato, this, "El campeonato asignado no es correcto."));
                //}
                OnPropertyChanged(NPCampeonato);
            }
        }

        public const string NPClasificacionCarrera = "ClasificacionCarrera";
        private EClasificacionCarrera mClasificacionCarrera;

        [Campo("ClasificacionCarreraId"), Requerido]
        public EClasificacionCarrera ClasificacionCarrera {
            get {
                PrePropertyRead(NPClasificacionCarrera);
                return mClasificacionCarrera;
            }
            set {
                PrePropertyChanged(NPClasificacionCarrera);
                mClasificacionCarrera = value;
                //if (EstaOK()) {
                //    RemoveError(NPCampeonato);
                //} else if (this.EstaCargado) {
                //    RemoveError(NPCampeonato);
                //    AddError(new PropertyException(NPCampeonato, this, "El campeonato asignado no es correcto. (ClasificacionCarrera)"));
                //}
                OnPropertyChanged(NPClasificacionCarrera);
            }
        }

        public const string NPTiempo = "Tiempo";

        /// <summary>
        /// TiempoEfectivo de la clasificacionCarrera correspondiente.
        /// </summary>
        public TimeSpan Tiempo {
            get {
                if (ClasificacionCarrera == null) {
                    return new TimeSpan();
                }
                return ClasificacionCarrera.TiempoEfectivo;
            }
        }

        public const string NPTiempoAnterior = "TiempoAnterior";
        private TimeSpan? mTiempoAnterior;
        private bool mTiempoAnteriorCargado = false;

        public TimeSpan? TiempoAnterior {
            get {
                if (!mTiempoAnteriorCargado) {
                    mTiempoAnterior = GetTiempo(this.Puesto - 1);
                    mTiempoAnteriorCargado = true;
                }
                return mTiempoAnterior;
            }
            set {
                mTiempoAnterior = value;
                mTiempoAnteriorCargado = true;
            }
        }

        public const string NPTiempoPrimero = "TiempoPrimero";
        private TimeSpan? mTiempoPrimero;
        private bool mTiempoPrimeroCargado = false;

        public TimeSpan? TiempoPrimero {
            get {
                if (!mTiempoPrimeroCargado) {
                    mTiempoPrimero = GetTiempo(1);
                    mTiempoPrimeroCargado = true;
                }
                return mTiempoPrimero;
            }
            set {
                mTiempoPrimero = value;
                mTiempoPrimeroCargado = true;
            }
        }

        #region Impresion

        public const string NPDorsal = "Dorsal";
        public string Dorsal {
            get {
                return ClasificacionCarrera.Dorsal;
            }
        }

        public const string NPPiloto = "Piloto";
        public string Piloto {
            get {
                return ClasificacionCarrera.Piloto;
            }
        }

        public const string NPVehiculo = "Vehiculo";
        public string Vehiculo {
            get {
                return ClasificacionCarrera.Vehiculo;
            }
        }

        public const string NPTiempoToString = "TiempoToString";
        public string TiempoToString {
            get {
                return CToString.ToString(Tiempo);
            }
        }

        public const string NPDifTiempoAnteriorToString = "DifTiempoAnteriorToString";
        public string DifTiempoAnteriorToString {
            get {
                if (TiempoAnterior.HasValue) {
                    return CToString.ToString(Tiempo.Subtract(TiempoAnterior.Value));
                }
                return "";
            }
        }

        public const string NPDifTiempoPrimeroToString = "DifTiempoPrimeroToString";
        public string DifTiempoPrimeroToString {
            get {
                if (TiempoPrimero.HasValue) {
                    return CToString.ToString(Tiempo.Subtract(TiempoPrimero.Value));
                }
                return "";
            }
        }

        public const string NPMotivoAbandono = "MotivoAbandono";
        public string MotivoAbandono {
            get {
                return ClasificacionCarrera.MotivoAbandono;
            }
        }

        public const string NPEscuderia = "Escuderia";
        public string Escuderia {
            get {
                return ClasificacionCarrera.Escuderia;
            }
        }

        public const string NPNombreCarrera = "NombreCarrera";
        public string NombreCarrera {
            get {
                return ClasificacionCarrera.NombreCarrera;
            }
        }

        public const string NPNombreClasificacion = "NombreClasificacion";
        public string NombreClasificacion {
            get {
                if (mCampeonato != null) {
                    string res = "";
                    if (mCampeonato.Nivel == 2) {//es un grupo.
                        res = mCampeonato.Campeonato.Campeonato.ToString() + " - Grupo " + mCampeonato.Campeonato.ToString() + " - Clase " + mCampeonato.ToString();
                    } else if (mCampeonato.Nivel == 1) {
                        res = mCampeonato.Campeonato.ToString() + " - Grupo " + mCampeonato.ToString();
                    } else {
                        res = mCampeonato.ToString();
                    }
                    string s = ClasificacionCarrera.NombreClasificacion;
                    s = s.Replace("Clasificacin General", res);
                    return s;
                }
                return "";
            }
        }

        #endregion

        #endregion

        #region Colecciones

        public static EColeccion<EClasificacionCarreraCampeonato> GetCollection(ECampeonato campeonato, ERally rally) {
            EColeccion<EClasificacionCarreraCampeonato> col = new EColeccion<EClasificacionCarreraCampeonato>();
            EColeccion<EClasificacionTotalRally> colRally = rally.ClasificacionesRally;
            foreach (EClasificacionTotalRally clasificacion in colRally) {
                EClasificacionCarreraCampeonato clasificacionCarreraCamp = clasificacion.GetClasificacionCarreraCampeonato(campeonato);
                if (clasificacionCarreraCamp != null) {
                    col.Add(clasificacionCarreraCamp);
                }
            }
            return col;
        }

        public static EColeccion<EClasificacionCarreraCampeonato> GetCollection(ECampeonato campeonato, ECronometrable cronometrable) {
            IEColeccion colAux = null;
            if (cronometrable.Tipo == TipoCronometrable.Manga) {
                colAux = cronometrable.ClasificacionesManga;
            } else if (cronometrable.Tipo == TipoCronometrable.Tramo) {
                colAux = cronometrable.ClasificacionesTramo;
            }
            EColeccion<EClasificacionCarreraCampeonato> col = new EColeccion<EClasificacionCarreraCampeonato>();
            foreach (ESinClaveBase clasificacion in colAux) {
                EClasificacionCarrera clasificacionCarrera = clasificacion as EClasificacionCarrera;
                EClasificacionCarreraCampeonato clasifCarreraCamp = null;
                if (clasificacionCarrera != null) {
                    clasifCarreraCamp = clasificacionCarrera.GetClasificacionCarreraCampeonato(campeonato);
                }
                if (clasifCarreraCamp != null) {
                    col.Add(clasifCarreraCamp);
                }
            }
            return col;
        }

        #endregion

        private TimeSpan? GetTiempo(int puesto) {
            if (ClasificacionCarrera != null && Campeonato != null) {
                return ClasificacionCarrera.GetTiempo(puesto, Campeonato);
            }
            return null;
        }

        internal static void ActualizarClasificacion(EColeccion<EClasificacionCarreraCampeonato> col) {
            if (col != null) {
                col.ApplySort(EClasificacionCarreraCampeonato.NPTiempo, ListSortDirection.Ascending);
                EClasificacionCarreraCampeonato primero = null;
                int puestoActual = 1;
                for (int indice = 0; indice < col.Count; indice++) {
                    EClasificacionCarreraCampeonato clasificaconCarreraCampeonato = col[indice] as EClasificacionCarreraCampeonato;
                    if (clasificaconCarreraCampeonato.HaAbandonado() || !clasificaconCarreraCampeonato.EsMostrable()) {
                        clasificaconCarreraCampeonato.Puesto = 0;
                    } else {
                        if (puestoActual == 1) {
                            primero = clasificaconCarreraCampeonato;
                            primero.TiempoAnterior = null;
                            //primero.ClasificacionAnterior = null;
                        } else {
                            clasificaconCarreraCampeonato.TiempoAnterior = (col[indice - 1] as EClasificacionCarreraCampeonato).Tiempo;
                            //cc.ClasificacionAnterior = col[indice - 1] as EClasificacionCarrera;
                        }
                        clasificaconCarreraCampeonato.Puesto = puestoActual;
                        clasificaconCarreraCampeonato.TiempoPrimero = primero.Tiempo;
                        //cc.ClasificacionPrimero = primero;
                        puestoActual = puestoActual + 1;
                    }
                }
            }
        }

        private bool EstaOK() {
            if (mClasificacionCarrera != null && mCampeonato != null) {
                EColeccion<ECampeonatoInscrito> col = ClasificacionCarrera.PilotoCarrera.CampeonatosInscrito;
                foreach (ECampeonatoInscrito cInscrito in col) {
                    if (mCampeonato.Equals(cInscrito.Campeonato) || cInscrito.ClasePertenece(mCampeonato)) {
                        return true;
                    }
                }
            }
            return false;
        }

        #region IClasificacion

        public bool HaAbandonado() {
            return ClasificacionCarrera.HaAbandonado();
        }

        public bool EsMostrable() {
            return this.ClasificacionCarrera.EsMostrable();
        }
        #endregion
    }
}

