using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("ClasificacionesCarreraCampeonatosConPuntos")]
    public class EClasificacionCarreraCampeonatoConPuntos : EClasificacionCarreraCampeonato {

        #region Constructores

        public EClasificacionCarreraCampeonatoConPuntos() : base() { }

        public EClasificacionCarreraCampeonatoConPuntos(int vId) : base(vId) { }

        #endregion

        #region Sobrecargas de EBase

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar clasificacion con puntos")) {
                if (Campeonato == null || Campeonato.Nivel != 2) {
                    throw new Exception("El campeonato asignado debe ser una clase.");
                }
                base.GuardarCore();
                transa.Commit();
            }
        }

        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
        }


        public override bool EsMasculino {
            get { return false; }  
        }

        public override string NombreDeLaEntidad {
            get { return "ClasificacionCarreraCampeonatoConPuntos"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return base.NPToString; } 
        }
        #endregion

        #region Propiedades

        public const string NPPuntos = "Puntos";
        private int mPuntos;

        [Campo]
        public int Puntos {
            get {
                PrePropertyRead(NPPuntos);
                return mPuntos;
            }
            set {
                PrePropertyChanged(NPPuntos);
                mPuntos = value;
                OnPropertyChanged(NPPuntos);
            }
        }

        public const string NPPuntosPrimero = "PuntosPrimero";
        private int? mPuntosPrimero;
        private bool mPuntosPrimeroCargado = false;

        public int? PuntosPrimero {
            get {
                if (!mPuntosPrimeroCargado) {
                    mPuntosPrimero = GetPuntos(1);
                    mPuntosPrimeroCargado = true;
                }
                return mPuntosPrimero;
            }
            set {
                mPuntosPrimero = value;
                mPuntosPrimeroCargado = true;
            }
        }

        public const string NPPuntosAnterior = "PuntosAnterior";
        private int? mPuntosAnterior;
        private bool mPuntosAnteriorCargado = false;

        public int? PuntosAnterior {
            get {
                if (!mPuntosAnteriorCargado) {
                    mPuntosAnteriorCargado = true;
                    mPuntosAnterior = GetPuntos(this.Puesto - 1);
                }
                return mPuntosAnterior;
            }
            set {
                mPuntosAnterior = value;
                mPuntosAnteriorCargado = true;
            }
        }

        #region Impresion

        public const string NPDifPuntosAnterior = "DifPuntosAnterior";
        public string DifPuntosAnterior {
            get {
                if (PuntosAnterior.HasValue) {
                    int diferencia = PuntosAnterior.Value - Puntos;
                    return diferencia.ToString();
                }
                return "..........";
            }
        }

        public const string NPDifPuntosPrimero = "DifPuntosPrimero";
        public string DifPuntosPrimero {
            get {
                if (PuntosPrimero.HasValue) {
                    int diferencia = PuntosPrimero.Value - Puntos;
                    return diferencia.ToString();
                }
                return "..........";
            }
        }

        #endregion

        #endregion

        #region Colecciones

        /// <summary>
        /// devuelve una coleccin de clasificaciones con puntos.
        /// </summary>
        /// <param name="campeonato"></param>es una clase.
        /// <param name="rally"></param>
        /// <returns></returns>
        public static EColeccion<EClasificacionCarreraCampeonatoConPuntos> GetCollection(ECampeonato campeonato, ERally rally) {
            if (campeonato != null && campeonato.Nivel == 2) {
                EColeccion<EClasificacionTotalRally> colRally = rally.ClasificacionesRally;
                EColeccion<EClasificacionCarreraCampeonatoConPuntos> col = new EColeccion<EClasificacionCarreraCampeonatoConPuntos>();
                foreach (EClasificacionTotalRally clasficacion in colRally) {
                    EClasificacionCarreraCampeonatoConPuntos obj = clasficacion.GetClasificacionCarreraCampeonatoConPuntos(campeonato);
                    if (obj != null) {
                        col.Add(obj);
                    }
                }
                return col;
            }
            return null;
        }

        /// <summary>
        /// devuelve una coleccin de clasificaciones con puntos.
        /// </summary>
        /// <param name="campeonato"></param>es una clase.
        /// <param name="cronometrable"></param>El cronometrable tiene que ser una manga de tipo manga oficial.
        /// <returns></returns>
        public static EColeccion<EClasificacionCarreraCampeonatoConPuntos> GetCollection(ECampeonato campeonato, ECronometrable cronometrable) {
            EManga manga = new EManga(cronometrable.Id);
            if (!manga.Existe) {
                return null;
            }
            if (campeonato != null && campeonato.Nivel == 2 && manga.TipoManga == TipoManga.MOfcial) {
                EColeccion<EClasificacionCarreraCampeonatoConPuntos> col = new EColeccion<EClasificacionCarreraCampeonatoConPuntos>();
                EColeccion<EClasificacionManga> colClasifManga = cronometrable.ClasificacionesManga;
                foreach (EClasificacionManga clasifManga in colClasifManga) {
                    EClasificacionCarreraCampeonatoConPuntos obj = clasifManga.GetClasificacionCarreraCampeonato(campeonato) as EClasificacionCarreraCampeonatoConPuntos;
                    if (obj != null) {
                        col.Add(obj);
                    }
                }
                return col;
            }
            return null;
        }

        #endregion

        private int? GetPuntos(int puesto) {
            if (puesto > 0 && ClasificacionCarrera != null) {
                return ClasificacionCarrera.GetPuntos(puesto, Campeonato);
            }
            return null;
        }

        internal static void ActualizarClasificacion(EColeccion<EClasificacionCarreraCampeonatoConPuntos> col) {
            if (col != null && col.Count > 0) {
                col.ApplySort(EClasificacionCarreraCampeonato.NPTiempo, ListSortDirection.Ascending);
                int indice = 0;
                EClasificacionCarreraCampeonatoConPuntos primero = null;
                while (indice < col.Count) {
                    EClasificacionCarreraCampeonatoConPuntos clasificacion = col[indice];
                    if (indice == 0) {
                        primero = clasificacion;
                    } else {
                        clasificacion.TiempoAnterior = col[indice - 1].Tiempo;
                    }
                    clasificacion.Puesto = indice + 1;
                    clasificacion.TiempoPrimero = primero.Tiempo;
                    ECampeonatoPuntuable campeonatoPuntuable;
                    campeonatoPuntuable = new ECampeonatoPuntuable(clasificacion.Campeonato.Campeonato.Campeonato, clasificacion.ClasificacionCarrera.PilotoCarrera.Carrera);
                    if (campeonatoPuntuable.Existe) {
                        clasificacion.Puntos = campeonatoPuntuable.CalculadorPuntos.DevolverPuntos(clasificacion.Puesto, col.Count);
                    } else {
                        clasificacion.Puntos = 0;
                    }
                    if (indice == 0) {
                        clasificacion.PuntosAnterior = null;
                    } else {
                        clasificacion.PuntosAnterior = col[indice - 1].Puntos;
                    }
                    clasificacion.PuntosPrimero = primero.Puntos;
                    indice = indice + 1;
                }
            }
        }
    }
}

