using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;
using System.Collections.ObjectModel;

namespace Negocio {

    [Tabla("ClasificacionesCarrera")]
    public class EClasificacionManga : EClasificacionCarrera {

        #region Constructores
        public EClasificacionManga() : base() { }

        public EClasificacionManga(int vId) : base(vId) { }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            TipoClasificacion = TipoClasificacionCarrera.Manga;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Clasificacion Manga")) {
                base.GuardarCore();
                //if (mColeccionClasificacionesCampeonato != null) {
                //    foreach (EColeccion<EClasificacionCarreraCampeonato> col in mColeccionClasificacionesCampeonato) {
                //        if (col != null) {
                //            col.Guardar();
                //        }
                //    }
                //}
                transa.Commit();
            }
        }

        public override void Deshacer() {
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return false; }  
        }

        public override string NombreDeLaEntidad {
            get { return "ClasificacionManga"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return base.NPToString; }
        }
        #endregion

        #region Propiedades

        public const string NPCronometrable = "Cronometrable";
        private ECronometrable mCronometrable;

        [Campo("CronometrableId"), Requerido]
        public ECronometrable Cronometrable {
            get {
                PrePropertyRead(NPCronometrable);
                return mCronometrable;
            }
            set {
                PrePropertyChanged(NPCronometrable);
                mCronometrable = value;
                OnPropertyChanged(NPCronometrable);
            }
        }

        public override string NombreClasificacion {
            get {
                return Cronometrable.ToString() + " - Clasificacin General";
            }
        }
        #endregion

        #region ClasificacionesCarreraCampeonatos

        public override IEColeccion ClasificacionesCarreraCampeonatos {
            get {
                if (mClasificacionesCarreraCampeonatos == null) {
                    EManga manga = new EManga(Cronometrable.Id);
                    EColeccion<EClasificacionCarreraCampeonatoConPuntos> mcol = new EColeccion<EClasificacionCarreraCampeonatoConPuntos>();
                    if (manga.TipoManga == TipoManga.MOfcial) {//con puntos
                        mcol.Filtros.Add(EClasificacionCarreraCampeonatoConPuntos.NPClasificacionCarrera, this);
                        mcol.Cargar();
                    }
                    mClasificacionesCarreraCampeonatos = new EColeccion<EClasificacionCarreraCampeonato>();
                    mClasificacionesCarreraCampeonatos.Filtros.Add(EClasificacionCarreraCampeonato.NPClasificacionCarrera, this);
                    mClasificacionesCarreraCampeonatos.Cargar();
                    foreach (EClasificacionCarreraCampeonatoConPuntos obj in mcol) {
                        int indice = 0;
                        bool terminado = false;
                        while (!terminado && indice < mClasificacionesCarreraCampeonatos.Count) {
                            EClasificacionCarreraCampeonato aeliminar = mClasificacionesCarreraCampeonatos[indice] as EClasificacionCarreraCampeonato;
                            if (aeliminar.Id == obj.Id) {
                                terminado = true;
                                mClasificacionesCarreraCampeonatos.Remove(aeliminar);
                                mClasificacionesCarreraCampeonatos.Add(obj);
                            } else {
                                indice = indice + 1;
                            }
                        }
                    }
                }
                return mClasificacionesCarreraCampeonatos;
            }
        }

        #endregion

        #region Colecciones
        public static EColeccion<EClasificacionManga> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EClasificacionManga> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EClasificacionManga> col = new EColeccion<EClasificacionManga>();
            //TODO: Ponemos las propiedades que queremos mostrar en el listado.
            col.ColumnaMostrarRango(new string[] {  });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }
        #endregion

        protected override void ActualizarClasificacion() {
            if (Cronometrable != null) {
                Cronometrable.ActualizarClasificacion();
            }
        }

        public override TimeSpan? GetTiempo(int puesto){
            if (Cronometrable != null) {
                return Cronometrable.GetTiempo(puesto);
            }
            return null;
        }

        public override TimeSpan? GetTiempo(int puesto, ECampeonato campeonato) {
            if (puesto > 0) {
                EColeccion<EClasificacionCarreraCampeonato> col = EClasificacionCarreraCampeonato.GetCollection(campeonato, Cronometrable);
                if (col != null) {
                    foreach (EClasificacionCarreraCampeonato clasificacion in col) {
                        if (clasificacion.Puesto == puesto) {
                            return clasificacion.Tiempo;
                        }
                    }
                }
            }
            return null;
        }

        public override int GetPuntos(int puesto, ECampeonato campeonato) {
            EManga manga = new EManga(Cronometrable.Id);
            if (manga.Existe && manga.TipoManga == TipoManga.MOfcial) {
                EColeccion<EClasificacionCarreraCampeonatoConPuntos> col = EClasificacionCarreraCampeonatoConPuntos.GetCollection(campeonato, manga);
                if (puesto > 0 && puesto <= col.Count) {
                    col.ApplySort(EClasificacionCarreraCampeonatoConPuntos.NPPuesto, ListSortDirection.Ascending);
                    return col[puesto - 1].Puntos;
                }
            }
            return -1;
        }

        protected override void ActualizarClasificacionesCampeonato() {
            IEColeccion col = this.ClasificacionesCarreraCampeonatos;
            EManga manga = new EManga(Cronometrable.Id);
            foreach (ESinClaveBase obj in col) {

                //if (manga.TipoManga == TipoManga.MOfcial) {
                //    EClasificacionCarreraCampeonatoConPuntos clasifconpuntos = obj as EClasificacionCarreraCampeonatoConPuntos;
                //    EColeccion<EClasificacionCarreraCampeonatoConPuntos> colAGuardar = EClasificacionCarreraCampeonatoConPuntos.GetCollection(clasifconpuntos.Campeonato, manga);
                //    EClasificacionCarreraCampeonatoConPuntos.ActualizarClasificacion(colAGuardar);
                //    colAGuardar.Guardar();
                //} else {
                //    EClasificacionCarreraCampeonato clasif = obj as EClasificacionCarreraCampeonato;
                //    EColeccion<EClasificacionCarreraCampeonato> colAGuardar = EClasificacionCarreraCampeonato.GetCollection(clasif.Campeonato, this.Cronometrable);
                //    EClasificacionCarreraCampeonato.ActualizarClasificacion(colAGuardar);
                //    colAGuardar.Guardar();
                //}
            }
        }

        protected override TimeSpan GetTiempoEfectivo() {
            //tiempo real ms el tiempo de las sanciones para el cronometrable del inscrito.
            TimeSpan mSumaSanciones = new TimeSpan();
            EColeccion<ESancion> colSanciones = this.PilotoCarrera.Sanciones;
            colSanciones.Cargar();
            foreach (ESancion sancion in colSanciones) {
                if (sancion.TipoSancion == TipoSancion.Temporal && 
                    sancion.Cronometrable != null && sancion.Cronometrable.Equals(this.Cronometrable)) {

                    mSumaSanciones = mSumaSanciones.Add(sancion.Tiempo.Value);
                }
            }
            return TiempoReal.Add(mSumaSanciones);
        }

        public override bool EstaVacio {
            get {
                return TiempoEfectivo.Minutes == 0 && TiempoEfectivo.Seconds == 0 && TiempoEfectivo.Milliseconds == 0;
            }
        }

        public override bool HaAbandonado() {
            EColeccion<ESancion> colSanciones = this.PilotoCarrera.Sanciones;
            bool encontrado = false;
            int indice = 0;
            while (!encontrado && indice < colSanciones.Count) {
                ESancion sancion = colSanciones[indice];
                encontrado = sancion.TipoSancion == TipoSancion.Descalificacion && sancion.Cronometrable.Equals(this.Cronometrable);
                indice = indice + 1;
            }
            return Abandono || encontrado;
        }
    }
}

