using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;
using System.Collections.ObjectModel;

namespace Negocio {

    [Tabla("ClasificacionesCarrera")]
    public class EClasificacionTotalRally : EClasificacionCarrera {

        #region Constructores
        public EClasificacionTotalRally() : base() { }

        public EClasificacionTotalRally(int vId) : base(vId) { }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            TipoClasificacion = TipoClasificacionCarrera.Rally;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar ClasificacionTotalRally.")) {
                base.GuardarCore();
                //if (mClasificacionTramos != null)      //Si se ha cargado
                //    mClasificacionTramos.Guardar();   //guardar si hemos cambiado algo
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mClasificacionTramos = null;
            base.Deshacer();
        }


        public override bool EsMasculino {
            get { return false; }  
        }

        public override string NombreDeLaEntidad {
            get { return "ClasificacionTotalRally"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return base.NPToString; } 
        }
        #endregion

        #region Propiedades

        public const string NPRally = "Rally";
        private ERally mRally;

        [Campo("RallyId")]
        public ERally Rally {
            get {
                PrePropertyRead(NPRally);
                return mRally;
            }
            set {
                PrePropertyChanged(NPRally);
                mRally = value;
                OnPropertyChanged(NPRally);
            }
        }

        public override string NombreClasificacion {
            get { return Rally.ToString() + " - Clasificacin General"; }
        }

        #endregion

        #region ClasificacionesCarreraCampeonatos

        public override IEColeccion ClasificacionesCarreraCampeonatos {
            get {
                if (mClasificacionesCarreraCampeonatos == null) {
                    EColeccion<EClasificacionCarreraCampeonatoConPuntos> mcol = new EColeccion<EClasificacionCarreraCampeonatoConPuntos>();
                    mcol.Filtros.Add(EClasificacionCarreraCampeonatoConPuntos.NPClasificacionCarrera, this);
                    mcol.Cargar();
                    mClasificacionesCarreraCampeonatos = new EColeccion<EClasificacionCarreraCampeonato>();
                    mClasificacionesCarreraCampeonatos.Filtros.Add(EClasificacionCarreraCampeonato.NPClasificacionCarrera, this);
                    mClasificacionesCarreraCampeonatos.Cargar();
                    foreach (EClasificacionCarreraCampeonatoConPuntos obj in mcol) {
                        int indice = 0;
                        bool terminado = false;
                        while (!terminado && indice < mClasificacionesCarreraCampeonatos.Count) {
                            EClasificacionCarreraCampeonato aeliminar = mClasificacionesCarreraCampeonatos[indice] as EClasificacionCarreraCampeonato;
                            if (aeliminar.Id == obj.Id) {
                                terminado = true;
                                mClasificacionesCarreraCampeonatos.Remove(aeliminar);
                                mClasificacionesCarreraCampeonatos.Add(obj);
                            } else {
                                indice = indice + 1;
                            }
                        }
                    }
                    
                }
                return mClasificacionesCarreraCampeonatos;
            }
        }

        #endregion

        #region ClasificacionTramos
        public const string NPClasificacionTramos = "ClasificacionTramos";
        private EColeccion<EClasificacionTramo> mClasificacionTramos = null;

        /// <summary>Devuelve una coleccin con todos los ClasificacionTramos.</summary>
        public EColeccion<EClasificacionTramo> ClasificacionTramos {
            get {
                if(mClasificacionTramos == null) {
                    mClasificacionTramos = new EColeccion<EClasificacionTramo>();
                    if(!this.EsNuevo) {
                        mClasificacionTramos.Filtros.Add(EClasificacionTramo.NPClasificacionRally, this);
                        mClasificacionTramos.Cargar();
                    }
                    mClasificacionTramos.AddingNew += new AddingNewEventHandler(mClasificacionTramos_AddingNew);
                    mClasificacionTramos.ListChanged += new ListChangedEventHandler(mClasificacionTramos_ListChanged);
                }
                return mClasificacionTramos;
            }
        }

        private void mClasificacionTramos_AddingNew(object sender, AddingNewEventArgs e) {
            EClasificacionTramo entidad = new EClasificacionTramo();
            entidad.ClasificacionRally = this;
            e.NewObject = entidad;
        }

        private void mClasificacionTramos_ListChanged(object sender, ListChangedEventArgs e) {
            if (!ClasificacionTramos.EsValido)
                AddError(new PropertyException(NPClasificacionTramos, this, ClasificacionTramos.AlgunError.Message, ClasificacionTramos.AlgunError));
            else {
                RemoveError(NPClasificacionTramos);
                //TODO: actualizar el tiempo
                TimeSpan mTiempoTramos = new TimeSpan();
                foreach (EClasificacionTramo clasificacion in mClasificacionTramos) {
                    mTiempoTramos = mTiempoTramos.Add(clasificacion.TiempoReal);
                }
                //agregamos el tiempo en los minutos, segundos y milsimas.
                if (!mTiempoTramos.Equals(TiempoReal)) {
                    MinutosLLegada = mTiempoTramos.Minutes.ToString();
                    SegundosLLegada = mTiempoTramos.Seconds.ToString();
                    MilesimasLLegada = mTiempoTramos.Milliseconds.ToString();
                    //actualizar las clasificaciones de los campeonatos.
                    this.ActualizarClasificacion();
                    this.Rally.ClasificacionesRally.Guardar();
                    this.ActualizarClasificacionesCampeonato();
                }
            }
            OnPropertyCollectionChanged(NPClasificacionTramos);
        }

        #endregion

        #region Colecciones
        public static EColeccion<EClasificacionTotalRally> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EClasificacionTotalRally> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EClasificacionTotalRally> col = new EColeccion<EClasificacionTotalRally>();
            col.ColumnaMostrarRango(new string[] { });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }
        #endregion

        protected override void ActualizarClasificacion() {
            Rally.ActualizarClasificacion();
        }

        /// <summary>
        /// Actualiza y guarda las clasificaciones de campeonato
        /// </summary>
        protected override void ActualizarClasificacionesCampeonato() {
            //EColeccion<EClasificacionCarreraCampeonatoConPuntos> col = this.ClasificacionesCarreraCampeonatos as EColeccion<EClasificacionCarreraCampeonatoConPuntos>;
            //foreach (EClasificacionCarreraCampeonato clasifCarreraCamp in col) {
            //    EColeccion<EClasificacionCarreraCampeonato> colAGuardar = EClasificacionCarreraCampeonatoConPuntos.GetCollection(clasifCarreraCamp.Campeonato, this.Rally);
            //    EClasificacionCarreraCampeonatoConPuntos.ActualizarClasificacion(colAGuardar);
            //    colAGuardar.Guardar();
            //}
        }

        public override System.TimeSpan? GetTiempo(int puesto) {
            if (Rally != null) {
                return Rally.GetTiempo(puesto);
            }
            return null;
        }

        public override TimeSpan? GetTiempo(int puesto, ECampeonato campeonato) {
            if (puesto > 0) {
                EColeccion<EClasificacionCarreraCampeonato> col = EClasificacionCarreraCampeonato.GetCollection(campeonato, Rally);
                if (col != null) {
                    foreach (EClasificacionCarreraCampeonato clasificacion in col) {
                        if (clasificacion.Puesto == puesto) {
                            return clasificacion.Tiempo;
                        }
                    }
                }
            }
            return null;
        }

        public override int GetPuntos(int puesto, ECampeonato campeonato) {
            EColeccion<EClasificacionCarreraCampeonatoConPuntos> col = EClasificacionCarreraCampeonatoConPuntos.GetCollection(campeonato, Rally);
            if (puesto > 0 && puesto < col.Count) {
                col.ApplySort(EClasificacionCarreraCampeonatoConPuntos.NPPuesto, ListSortDirection.Ascending);
                return col[puesto - 1].Puntos;
            }
            return -1;
        }

        protected override TimeSpan GetTiempoEfectivo() {
            //tiempo real ms las sanciones de los sectores y sanciones de carrera completa.
            TimeSpan mSumaSanciones = new TimeSpan();
            foreach (ESancion sancion in this.PilotoCarrera.Sanciones) {
                if (sancion.TipoSancion == TipoSancion.Temporal) //&& 
                    //((sancion.Cronometrable != null && sancion.Cronometrable.Tipo == TipoCronometrable.Sector) || 
                    //sancion.Cronometrable == null)) {
                {
                    mSumaSanciones = mSumaSanciones.Add(sancion.Tiempo.Value);
                }
            }
            return TiempoReal.Add(mSumaSanciones);
        }

        public override bool EstaVacio {
            get {
                return TiempoEfectivo.Minutes == 0 && TiempoEfectivo.Seconds == 0 && TiempoEfectivo.Milliseconds == 0;
            }
        }

        public override bool HaAbandonado() {
            EColeccion<ESancion> colSanciones = this.PilotoCarrera.Sanciones;
            bool encontrado = false;
            int indice = 0;
            while (!encontrado && indice < colSanciones.Count) {
                encontrado = colSanciones[indice].TipoSancion == TipoSancion.Descalificacion;
                indice = indice + 1;
            }
            return Abandono || encontrado;
        }
    }
}

