using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System.Collections.ObjectModel;
using System;

namespace Negocio {

    [Tabla("ClasificacionesCarrera")]
    public class EClasificacionTramo : EClasificacionCarrera {

        #region Constructores
        public EClasificacionTramo() : base() { }

        public EClasificacionTramo(int vId) : base(vId) { }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            TipoClasificacion = TipoClasificacionCarrera.Tramo;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar clasificacion tramo")) {
                //if (this.ClasificacionRally != null && this.Abandono != this.ClasificacionRally.Abandono) {
                //    this.ClasificacionRally.MotivoAbandono = this.MotivoAbandono;
                //    this.ClasificacionRally.Abandono = this.Abandono;
                //    mGuardarTiempoClasificacionRally = true;
                //}
                base.GuardarCore();
                if (mGuardarClasificacionRally && mClasificacionRally != null) {
                    mClasificacionRally.Guardar();
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return false; } 
        }

        public override string NombreDeLaEntidad {
            get { return "ClasificacionTramo"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return base.NPToString; } 
        }

        #endregion

        #region Propiedades

        public const string NPCronometrable = "Cronometrable";
        private ECronometrable mCronometrable;

        [Campo("CronometrableId", Tamanyo = 50)]
        public ECronometrable Cronometrable {
            get {
                PrePropertyRead(NPCronometrable);
                return mCronometrable;
            }
            set {
                PrePropertyChanged(NPCronometrable);
                mCronometrable = value;
                OnPropertyChanged(NPCronometrable);
            }
        }

        
        public const string NPClasificacionRally = "ClasificacionRally";
        private EClasificacionTotalRally mClasificacionRally;

        [Campo("ClasificacionRallyId")]
        public EClasificacionTotalRally ClasificacionRally {
            get {
                PrePropertyRead(NPClasificacionRally);
                return mClasificacionRally;
            }
            set {
                PrePropertyChanged(NPClasificacionRally);
                mClasificacionRally = value;
                if (this.EsNuevo && mClasificacionRally.Abandono) {
                    this.Abandono = mClasificacionRally.Abandono;
                    this.MotivoAbandono = mClasificacionRally.MotivoAbandono;
                }
                OnPropertyChanged(NPClasificacionRally);
            }
        }

        public override TimeSpan TiempoRealLLegada {
            get {
                return base.TiempoRealLLegada;
            }
            internal set {
                TimeSpan mTiempoRealAnterior = base.TiempoRealLLegada;
                base.TiempoRealLLegada = value;
                if (this.EstaCargado) {
                    TimeSpan mTiempoRealClasificacionRally = this.ClasificacionRally.TiempoRealLLegada;
                    mTiempoRealClasificacionRally = mTiempoRealClasificacionRally.Subtract(mTiempoRealAnterior);
                    mTiempoRealClasificacionRally = mTiempoRealClasificacionRally.Add(value);
                    if (!mTiempoRealClasificacionRally.Equals(this.ClasificacionRally.TiempoRealLLegada)) {
                        this.ClasificacionRally.HoraLLegada = mTiempoRealClasificacionRally.Hours.ToString();
                        this.ClasificacionRally.MinutosLLegada = mTiempoRealClasificacionRally.Minutes.ToString();
                        this.ClasificacionRally.SegundosLLegada = mTiempoRealClasificacionRally.Seconds.ToString();
                        this.ClasificacionRally.MilesimasLLegada = mTiempoRealClasificacionRally.Milliseconds.ToString();
                        mGuardarClasificacionRally = true;
                    }
                }
            }
        }

        public override TimeSpan TiempoRealSalida {
            get {
                return base.TiempoRealSalida;
            }
            internal set {
                TimeSpan mTiempoRealAnterior = base.TiempoRealSalida;
                base.TiempoRealSalida = value;
                if (this.EstaCargado) {
                    TimeSpan mTiempoRealClasificacionRally = this.ClasificacionRally.TiempoRealSalida;
                    mTiempoRealClasificacionRally = mTiempoRealClasificacionRally.Subtract(mTiempoRealAnterior);
                    mTiempoRealClasificacionRally = mTiempoRealClasificacionRally.Add(value);
                    if (!mTiempoRealClasificacionRally.Equals(this.ClasificacionRally.TiempoRealLLegada)) {
                        this.ClasificacionRally.HoraSalida = mTiempoRealClasificacionRally.Hours.ToString();
                        this.ClasificacionRally.MinutosSalida = mTiempoRealClasificacionRally.Minutes.ToString();
                        this.ClasificacionRally.SegundosSalida = mTiempoRealClasificacionRally.Seconds.ToString();
                        this.ClasificacionRally.MilesimasSalida = mTiempoRealClasificacionRally.Milliseconds.ToString();
                        mGuardarClasificacionRally = true;
                    }
                }
            }
        }

        private bool mGuardarClasificacionRally = false;

        public override string NombreClasificacion {
            get { return Cronometrable.ToString() + " - Clasificacin General"; }
        }

        public override bool Abandono {
            get {
                return this.ClasificacionRally.Abandono;
            }
            set {
                base.Abandono = value;
                this.ClasificacionRally.Abandono = value;
                if (value) {
                    if (String.IsNullOrEmpty(ClasificacionRally.MotivoAbandono)) {
                        ClasificacionRally.MotivoAbandono = this.Cronometrable.ToString();
                    } else {
                        ClasificacionRally.MotivoAbandono += ", " + this.Cronometrable.ToString();
                    }
                } else if (!String.IsNullOrEmpty(ClasificacionRally.MotivoAbandono)){
                    ClasificacionRally.MotivoAbandono.Replace(Cronometrable.ToString(), "");
                }
                mGuardarClasificacionRally = true;
            }
        }

        #endregion

        #region ClasificacionesCarreraCampeonatos

        public override IEColeccion ClasificacionesCarreraCampeonatos {
            get {
                if (mClasificacionesCarreraCampeonatos == null) {
                    mClasificacionesCarreraCampeonatos = new EColeccion<EClasificacionCarreraCampeonato>();
                    mClasificacionesCarreraCampeonatos.Filtros.Add(EClasificacionCarreraCampeonato.NPClasificacionCarrera, this);
                    mClasificacionesCarreraCampeonatos.Cargar();
                    mClasificacionesCarreraCampeonatos.AddingNew += new AddingNewEventHandler(mClasificacionesCarreraCampeonatos_AddingNew);
                }
                return mClasificacionesCarreraCampeonatos;
            }
        }

        private void mClasificacionesCarreraCampeonatos_AddingNew(object sender, AddingNewEventArgs e) {
            EClasificacionCarreraCampeonato entidad = new EClasificacionCarreraCampeonato();
            entidad.ClasificacionCarrera = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Colecciones
        public static EColeccion<EClasificacionTramo> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EClasificacionTramo> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EClasificacionTramo> col = new EColeccion<EClasificacionTramo>();
            col.ColumnaMostrarRango(new string[] {  });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }
        #endregion

        protected override void ActualizarClasificacion() {
            if (Cronometrable != null) {
                Cronometrable.ActualizarClasificacion();
            }
        }

        protected override void ActualizarClasificacionesCampeonato() {
            EColeccion<EClasificacionCarreraCampeonato> col = this.ClasificacionesCarreraCampeonatos as EColeccion<EClasificacionCarreraCampeonato>;
            foreach (EClasificacionCarreraCampeonato clasifCarreraCamp in col) {
                EColeccion<EClasificacionCarreraCampeonato> colAGuardar = EClasificacionCarreraCampeonato.GetCollection(clasifCarreraCamp.Campeonato, this.Cronometrable);
                EClasificacionCarreraCampeonato.ActualizarClasificacion(colAGuardar);
                colAGuardar.Guardar();
            }
        }

        public override System.TimeSpan? GetTiempo(int puesto) {
            if (Cronometrable != null) {
                return Cronometrable.GetTiempo(puesto);
            }
            return null;
        }

        public override System.TimeSpan? GetTiempo(int puesto, ECampeonato campeonato) {
            if (puesto > 0) {
                EColeccion<EClasificacionCarreraCampeonato> col = EClasificacionCarreraCampeonato.GetCollection(campeonato, Cronometrable);
                if (col != null) {
                    foreach (EClasificacionCarreraCampeonato clasificacion in col) {
                        if (clasificacion.Puesto == puesto) {
                            return clasificacion.Tiempo;
                        }
                    }
                }
            }
            return null;
        }

        public override int GetPuntos(int puesto, ECampeonato campeonato) {
            return -1;
        }

        public override void Eliminar() {
            //TODO: antes de eliminar hay que hacer q esta clasificacion del tramo elimine su tiempo de
            //la clasificacion rally correspondiente. Y actualizar la clasificacin.
            //TODO: al cambiar el tiempo tambien hay que actualizar las clasificaciones de campeonatos de 
            //la clasificacion del rally.
            if (mClasificacionRally != null) {
                mClasificacionRally.ClasificacionTramos.Remove(this);
                ClasificacionRally.Guardar();
            }
            base.Eliminar();
        }

        protected override TimeSpan GetTiempoEfectivo() {
            //tiempo real ms las sanciones del cronometrable para el inscrito.
            TimeSpan mSumaSanciones = new TimeSpan();
            EColeccion<ESancion> colSanciones = this.PilotoCarrera.Sanciones;
            colSanciones.Cargar();
            foreach (ESancion sancion in colSanciones) {
                if (sancion.TipoSancion == TipoSancion.Temporal &&
                    sancion.Cronometrable != null && sancion.Cronometrable.Equals(this.Cronometrable)) {

                    mSumaSanciones = mSumaSanciones.Add(sancion.Tiempo.Value);
                }
            }
            return TiempoReal.Add(mSumaSanciones);
        }

        public override bool HaAbandonado() {
            return ClasificacionRally.HaAbandonado();
        }

        public override bool EstaVacio {
            get {
                return TiempoEfectivo.Minutes == 0 && TiempoEfectivo.Seconds == 0 && TiempoEfectivo.Milliseconds == 0;
            }
        }
    }
}

