using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;
using System.Drawing;

namespace Negocio {

    public enum TipoSancion {
        Descalificacion = 1,
        Economica,
        Temporal
    }

    [Tabla("Sanciones")]
    public class ESancion : EIdBase, IColor {
        #region Constructores
        public ESancion() : base() { }

        public ESancion(int vId) : base(vId) { }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            TipoSancion = TipoSancion.Economica;
            Importe = 0.0;
            Motivo = null;
            PilotoCarrera = null;
            Cronometrable = null;
            mMinutos = "00";
            mHora = "00";
            mSegundos = "00";
            mMilesimas = "000";
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Sancin")) {
                if (!this.EsNuevo) {
                    throw new Exception("Una sancin una vez creada no se puede modificar. Elimnela y cree otra nueva.");
                }
                if (this.TipoSancion == TipoSancion.Temporal && (String.IsNullOrEmpty(Minutos) || String.IsNullOrEmpty(Segundos) || String.IsNullOrEmpty(Milesimas))) {
                    throw new Exception("Si la sancin es de tiempo, hay que establecer el tiempo sancionado.");
                }
                if (mTipoSancion == TipoSancion.Descalificacion) {
                    foreach (ESancion s in mPilotoCarrera.Sanciones) {
                        if (!s.Equals(this) && s.TipoSancion == TipoSancion.Descalificacion) {
                            throw new Exception("No se puede descalificar dos veces al mismo inscrito.");
                        }
                    }
                }
                base.GuardarCore();
                if (mTipoSancion == TipoSancion.Temporal) {
                    if (mCronometrable != null) {
                        mCronometrable.ActualizarClasificacion();
                        mCronometrable.Guardar();
                    } else {
                        ERally rally = new ERally(mPilotoCarrera.Carrera.Id);
                        rally.ActualizarClasificacion();
                        rally.Guardar();
                    }
                } else if (mTipoSancion == TipoSancion.Descalificacion) {
                    if (mCronometrable != null) {
                        //EClasificacionCarrera clasificacion = mCronometrable.GetClasificacion(mPilotoCarrera);
                        //clasificacion.MotivoAbandono = mMotivo;
                        //clasificacion.Abandono = true;
                        mCronometrable.ActualizarClasificacion();
                        mCronometrable.Guardar();
                    } else {
                        ERally rally = new ERally(mPilotoCarrera.Carrera.Id);
                        //EClasificacionTotalRally clasificacion = rally.GetClasificacion(mPilotoCarrera);
                        //clasificacion.MotivoAbandono = mMotivo;
                        //clasificacion.Abandono = true;
                        foreach (ECronometrable crono in rally.Cronometrables) {
                            if (crono.Tipo == TipoCronometrable.Tramo) {
                                crono.ActualizarClasificacion();
                                crono.Guardar();
                            }
                        }
                        rally.ActualizarClasificacion();
                        rally.Guardar();
                    }
                }
                transa.Commit();
            }
        }

        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "Sancion"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPTipoSancion; }
        }
        #endregion

        #region Propiedades

        public const string NPTipoSancion = "TipoSancion";
        private TipoSancion mTipoSancion;
        private TipoSancion mTipoSancionAnterior = TipoSancion.Economica;

        [Campo, Requerido]
        public TipoSancion TipoSancion {
            get {
                PrePropertyRead(NPTipoSancion);
                return mTipoSancion;
            }
            set {
                mTipoSancionAnterior = mTipoSancion;
                PrePropertyChanged(NPTipoSancion);
                mTipoSancion = value;
                OnPropertyChanged(NPTipoSancion);
            }
        }

        public const string NPTiempo = "Tiempo";
        private TimeSpan? mTiempo;
        private bool mTiempoCargado = false;

        public TimeSpan? Tiempo {
            get {
                if (!mTiempoCargado) {
                    int minutos;
                    int segundos;
                    int milesimas;
                    int hora;
                    if (Int32.TryParse(Hora, out hora) && Int32.TryParse(Minutos, out minutos) && Int32.TryParse(Segundos, out segundos) && Int32.TryParse(Milesimas, out milesimas)) {
                        mTiempoCargado = true;
                        mTiempo = new TimeSpan(0, hora, minutos, segundos, milesimas);
                    } else {
                        mTiempo = new TimeSpan();
                    }
                }
                return mTiempo;
            }
            set {
                mTiempo = value;
            }
        }

        #region Tiempo

        public const string NPHora = "Hora";
        private string mHora;

        [Campo(Tamanyo = 2)]
        public string Hora {
            get {
                PrePropertyRead(NPHora);
                return mHora;
            }
            set {
                PrePropertyChanged(NPHora);
                mHora = value;
                if (!String.IsNullOrEmpty(mHora) && mHora.Length < 2) {
                    mHora = "0" + mHora;
                }
                //comprobar que es un numero y est entre 00 y 23
                int hora;
                if (mHora != null) {
                    if (Int32.TryParse(mHora, out hora) && hora >= 0 && hora <= 23) {
                        RemoveError(NPHora);
                        //agregar estos minutos a tiempo.
                        Tiempo = new TimeSpan(0, hora, Tiempo.Value.Minutes, Tiempo.Value.Seconds, Tiempo.Value.Milliseconds);
                    } else {
                        AddError(new PropertyException(NPHora, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 23."));
                    }
                }
                OnPropertyChanged(NPHora);
            }
        }

        public const string NPMinutos = "Minutos";
        private string mMinutos;

        [Campo(Tamanyo = 2)]
        public string Minutos {
            get {
                PrePropertyRead(NPMinutos);
                return mMinutos;
            }
            set {
                PrePropertyChanged(NPMinutos);
                mMinutos = value;
                if (!String.IsNullOrEmpty(mMinutos) && mMinutos.Length < 2) {
                    mMinutos = "0" + mMinutos;
                }
                //comprobar que es un numero y est entre 00 y 59
                int minutos;
                if (mMinutos != null) {
                    if (Int32.TryParse(mMinutos, out minutos) && minutos >= 0 && minutos <= 59) {
                        RemoveError(NPMinutos);
                        //agregar estos minutos a tiempo.
                        Tiempo = new TimeSpan(0, Tiempo.Value.Hours, minutos, Tiempo.Value.Seconds, Tiempo.Value.Milliseconds);
                    } else {
                        AddError(new PropertyException(NPMinutos, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                    }
                }
                OnPropertyChanged(NPMinutos);
            }
        }

        public const string NPSegundos = "Segundos";
        private string mSegundos;

        [Campo(Tamanyo = 2)]
        public string Segundos {
            get {
                PrePropertyRead(NPSegundos);
                return mSegundos;
            }
            set {
                PrePropertyChanged(NPSegundos);
                mSegundos = value;
                if (!String.IsNullOrEmpty(mSegundos) && mSegundos.Length < 2) {
                    mSegundos = "0" + mSegundos;
                }
                //comprobar que es un numero y est entre 00 y 59
                if (mSegundos != null) {
                    int segundos;
                    if (Int32.TryParse(mSegundos, out segundos) && segundos >= 0 && segundos <= 59) {
                        RemoveError(NPSegundos);
                        Tiempo = new TimeSpan(0, Tiempo.Value.Hours, Tiempo.Value.Minutes, segundos, Tiempo.Value.Milliseconds);

                    } else {
                        AddError(new PropertyException(NPSegundos, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                    }
                }
                OnPropertyChanged(NPSegundos);
            }
        }

        public const string NPMilesimas = "Milesimas";
        private string mMilesimas;

        [Campo(Tamanyo = 3)]
        public string Milesimas {
            get {
                PrePropertyRead(NPMilesimas);
                return mMilesimas;
            }
            set {
                PrePropertyChanged(NPMilesimas);
                mMilesimas = value;
                if (!String.IsNullOrEmpty(mMilesimas) && mMilesimas.Length < 3) {
                    if (mMilesimas.Length < 2) {
                        mMilesimas = "00" + mMilesimas;
                    } else {
                        mMilesimas = "0" + mMilesimas;
                    }
                }
                //comprobar que es un numero y est entre 000 y 999
                if (mMilesimas != null) {
                    int milesimas;
                    if (Int32.TryParse(mMilesimas, out milesimas) && milesimas >= 0 && milesimas <= 999) {
                        RemoveError(NPMilesimas);
                        Tiempo = new TimeSpan(0, Tiempo.Value.Hours, Tiempo.Value.Minutes, Tiempo.Value.Seconds, milesimas);
                    } else {
                        AddError(new PropertyException(NPMilesimas, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 999."));
                    }
                }
                OnPropertyChanged(NPMilesimas);
            }
        }

        #endregion

        public const string NPImporte = "Importe";
        private double mImporte;

        [Campo]
        public double Importe {
            get {
                PrePropertyRead(NPImporte);
                return mImporte;
            }
            set {
                PrePropertyChanged(NPImporte);
                mImporte = value;
                OnPropertyChanged(NPImporte);
            }
        }

        public const string NPMotivo = "Motivo";
        private string mMotivo;

        [Campo(Tamanyo = 255), Requerido]
        public string Motivo {
            get {
                PrePropertyRead(NPMotivo);
                return mMotivo;
            }
            set {
                PrePropertyChanged(NPMotivo);
                mMotivo = value;
                OnPropertyChanged(NPMotivo);
            }
        }

        public const string NPPilotoCarrera = "PilotoCarrera";
        private EPilotoCarrera mPilotoCarrera;

        [Campo("PilotoCarreraId"), Requerido]
        public EPilotoCarrera PilotoCarrera {
            get {
                PrePropertyRead(NPPilotoCarrera);
                return mPilotoCarrera;
            }
            set {
                PrePropertyChanged(NPPilotoCarrera);
                mPilotoCarrera = value;
                if (mPilotoCarrera != null && mCronometrable == null && mPilotoCarrera.Carrera.Tipo == TipoCarrera.Subida) {
                    AddError(new PropertyException(NPPilotoCarrera, this, "El cronometrable debe ser obligatorio."));
                } else {
                    RemoveError(NPPilotoCarrera);
                }
                OnPropertyChanged(NPPilotoCarrera);
            }
        }

        public const string NPCronometrable = "Cronometrable";
        private ECronometrable mCronometrable;
        private ECronometrable mCronometrableAnterior = null;

        [Campo("CronometrableId")]
        public ECronometrable Cronometrable {
            get {
                PrePropertyRead(NPCronometrable);
                return mCronometrable;
            }
            set {
                PrePropertyChanged(NPCronometrable);
                mCronometrableAnterior = mCronometrable;
                mCronometrable = value;
                if (mCronometrable == null && mPilotoCarrera != null && mPilotoCarrera.Carrera.Tipo == TipoCarrera.Subida) {
                    AddError(new PropertyException(NPCronometrable, this, "El cronometrable debe ser obligatorio."));
                } else {
                    RemoveError(NPPilotoCarrera);
                    RemoveError(NPCronometrable);
                }
                OnPropertyChanged(NPCronometrable);
            }
        }

        #endregion

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar Sancin")) {
                base.Eliminar();
                if (mTipoSancion == TipoSancion.Temporal) {
                    if (mCronometrable != null) {
                        mCronometrable.ActualizarClasificacion();
                        mCronometrable.Guardar();
                    } else {
                        ERally rally = new ERally(mPilotoCarrera.Carrera.Id);
                        rally.ActualizarClasificacion();
                        rally.Guardar();
                    }
                } else if (mTipoSancion == TipoSancion.Descalificacion) {
                    if (mCronometrable != null) {
                        //EClasificacionCarrera clasificacion = mCronometrable.GetClasificacion(mPilotoCarrera);
                        //clasificacion.MotivoAbandono = "";
                        //clasificacion.Abandono = false;
                        mCronometrable.ActualizarClasificacion();
                        mCronometrable.Guardar();
                    } else {
                        ERally rally = new ERally(mPilotoCarrera.Carrera.Id);
                        //EClasificacionTotalRally clasificacion = rally.GetClasificacion(mPilotoCarrera);
                        //clasificacion.MotivoAbandono = "";
                        //clasificacion.Abandono = false;
                        foreach (ECronometrable crono in rally.Cronometrables) {
                            if (crono.Tipo == TipoCronometrable.Tramo) {
                                crono.ActualizarClasificacion();
                                crono.Guardar();
                            }
                        }
                        rally.ActualizarClasificacion();
                        rally.Guardar();
                    }
                }
                transa.Commit();
            }
        }

        #region IColor

        public System.Drawing.Color DevolverColor() {
            return DevolverColor(TipoSancion);
        }

        public System.Drawing.Color DevolverColor(object o) {
            TipoSancion tipo;
            try {
                tipo = (TipoSancion)o;
            } catch (Exception ex) {
                throw new InvalidCastException(ex.Message);
            }
            switch (tipo) {
                case TipoSancion.Descalificacion:
                    return Color.Red;
                    break;
                default:
                    return Color.White;
                    break;
            }
        }

        #endregion
    }
}

