using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("TiemposPilotoSector")]
    public class ETiempoPilotoSector : EIdBase {
        #region Constructores
        public ETiempoPilotoSector() : base() { }

        public ETiempoPilotoSector(int vId) : base(vId) { }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            mMinutosLLegada = "00";
            mMilesimasLLegada = "000";
            mSegundosLLegada = "00";
            mHoraLLegada = "00";
            mMinutosSalida = "00";
            mMilesimasSalida = "000";
            mSegundosSalida = "00";
            mHoraSalida = "00";
            Cronometrable = null;
            PilotoCarrera = null;
        }

        public override bool EsMasculino {
            get { return true; }  
        }

        public override string NombreDeLaEntidad {
            get { return "TiempoPilotoSector"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPId; } 
        }
        #endregion

        #region Propiedades

        public const string NPTiempoSalida = "TiempoSalida";
        private TimeSpan mTiempoSalida;
        public bool mTiempoSalidaCargado = false;

        public TimeSpan TiempoSalida {
            get {
                if (!mTiempoSalidaCargado) {
                    int minutos;
                    int segundos;
                    int milesimas;
                    int hora;
                    if (Int32.TryParse(MinutosSalida, out minutos) && Int32.TryParse(SegundosSalida, out segundos) && Int32.TryParse(MilesimasSalida, out milesimas) && Int32.TryParse(HoraSalida, out hora)) {
                        mTiempoSalidaCargado = true;
                        mTiempoSalida = new TimeSpan(0, hora, minutos, segundos, milesimas);
                    } else {
                        mTiempoSalida = new TimeSpan();
                    }
                }
                return mTiempoSalida;
            }
            internal set {
                TimeSpan mTiempoAnterior = mTiempoSalida;
                mTiempoSalida = value;
                if (mTiempoSalidaCargado && this.EstaCargado) {
                    //Actualizar tiempo efectivo.
                    if (mTiempoSalida.CompareTo(TiempoLLegada) > 0) {
                        AddError(new PropertyException(NPTiempoSalida, this, "El tiempo de salida no puede ser mayor que el tiempo de llegada."));
                    } else {
                        RemoveError(NPTiempoSalida);
                        RemoveError(NPTiempoLLegada);
                        //TimeSpan mTiempoReal = TiempoLLegada.Subtract(mTiempoSalida);
                        //Tiempo = Tiempo.Subtract(mTiempoAnterior);
                        //Tiempo = Tiempo.Add(mTiempoReal);
                    }
                }
            }
        }

        public const string NPTiempoLLegada = "TiempoLLegada";
        private TimeSpan mTiempoLLegada;
        private bool mTiempoLLegadaCargado = false;

        public TimeSpan TiempoLLegada {
            get {
                if (!mTiempoLLegadaCargado) {
                    int minutos;
                    int segundos;
                    int milesimas;
                    int hora;
                    if (Int32.TryParse(MinutosLLegada, out minutos) && Int32.TryParse(SegundosLLegada, out segundos) && Int32.TryParse(MilesimasLLegada, out milesimas) && Int32.TryParse(HoraLLegada, out hora)) {
                        mTiempoLLegadaCargado = true;
                        mTiempoLLegada = new TimeSpan(0, hora, minutos, segundos, milesimas);
                    } else {
                        mTiempoLLegada = new TimeSpan();
                    }
                }
                return mTiempoLLegada;
            }
            internal set {
                TimeSpan mTiempoAnterior = mTiempoLLegada;
                mTiempoLLegada = value;
                if (mTiempoLLegadaCargado && this.EstaCargado) {
                    //Actualizar tiempo efectivo.
                    if (mTiempoLLegada.CompareTo(TiempoSalida) < 0) {
                        AddError(new PropertyException(NPTiempoLLegada, this, "El tiempo de salida no puede ser mayor que el tiempo de llegada."));
                    } else {
                        RemoveError(NPTiempoSalida);
                        RemoveError(NPTiempoLLegada);
                        //TimeSpan mTiempoReal = mTiempoLLegada.Subtract(TiempoSalida);
                        //Tiempo = Tiempo.Subtract(mTiempoAnterior);
                        //Tiempo = Tiempo.Add(mTiempoReal);
                    }
                }
            }
        }

        public const string NPTiempo = "Tiempo";
        //private TimeSpan mTiempo;
        //private bool mTiempoCargado = false;

        public TimeSpan Tiempo {
            get {
                return TiempoLLegada.Subtract(TiempoSalida);
            }
            //private set {
            //    mTiempo = value;
            //    mTiempoCargado = true;
            //}
        }

        #region Tiempo

        public const string NPHoraSalida = "HoraSalida";
        private string mHoraSalida;

        [Campo(Tamanyo = 2)]
        public string HoraSalida {
            get {
                PrePropertyRead(NPHoraSalida);
                return mHoraSalida;
            }
            set {
                PrePropertyChanged(NPHoraSalida);
                mHoraSalida = value;
                if (!String.IsNullOrEmpty(mHoraSalida) && mHoraSalida.Length < 2) {
                    mHoraSalida = "0" + mHoraSalida;
                }
                //comprobar que es un numero y est entre 00 y 59
                int hora;
                if (Int32.TryParse(mHoraSalida, out hora) && hora >= 0 && hora <= 23) {
                    RemoveError(NPHoraSalida);
                    //agregar estos minutos a tiempo.
                    TiempoSalida = new TimeSpan(0, hora, TiempoSalida.Minutes, TiempoSalida.Seconds, TiempoSalida.Milliseconds);
                } else {
                    AddError(new PropertyException(NPHoraSalida, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 23."));
                }
                OnPropertyChanged(NPHoraSalida);
            }
        }

        public const string NPMinutosSalida = "MinutosSalida";
        private string mMinutosSalida;

        [Campo(Tamanyo = 2)]
        public string MinutosSalida {
            get {
                PrePropertyRead(NPMinutosSalida);
                return mMinutosSalida;
            }
            set {
                PrePropertyChanged(NPMinutosSalida);
                mMinutosSalida = value;
                if (!String.IsNullOrEmpty(mMinutosSalida) && mMinutosSalida.Length < 2) {
                    mMinutosSalida = "0" + mMinutosSalida;
                }
                //comprobar que es un numero y est entre 00 y 59
                int minutos;
                if (Int32.TryParse(mMinutosSalida, out minutos) && minutos >= 0 && minutos <= 59) {
                    RemoveError(NPMinutosSalida);
                    //agregar estos minutos a tiempo.
                    TiempoSalida = new TimeSpan(0, TiempoSalida.Hours, minutos, TiempoSalida.Seconds, TiempoSalida.Milliseconds);
                } else {
                    AddError(new PropertyException(NPMinutosSalida, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                }
                OnPropertyChanged(NPMinutosSalida);
            }
        }

        public const string NPSegundosSalida = "SegundosSalida";
        private string mSegundosSalida;

        [Campo(Tamanyo = 2)]
        public string SegundosSalida {
            get {
                PrePropertyRead(NPSegundosSalida);
                return mSegundosSalida;
            }
            set {
                PrePropertyChanged(NPSegundosSalida);
                mSegundosSalida = value;
                if (!String.IsNullOrEmpty(mMinutosSalida) && mSegundosSalida.Length < 2) {
                    mSegundosSalida = "0" + mSegundosSalida;
                }
                //comprobar que es un numero y est entre 00 y 59
                int segundos;
                if (Int32.TryParse(mSegundosSalida, out segundos) && segundos >= 0 && segundos <= 59) {
                    RemoveError(NPSegundosSalida);
                    //agregar estos minutos a tiempo.
                    TiempoSalida = new TimeSpan(0, TiempoSalida.Hours, TiempoSalida.Minutes, segundos, TiempoSalida.Milliseconds);
                } else {
                    AddError(new PropertyException(NPSegundosSalida, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                }
                OnPropertyChanged(NPSegundosSalida);
            }
        }

        public const string NPMilesimasSalida = "MilesimasSalida";
        private string mMilesimasSalida;

        [Campo(Tamanyo = 3)]
        public string MilesimasSalida {
            get {
                PrePropertyRead(NPMilesimasSalida);
                return mMilesimasSalida;
            }
            set {
                PrePropertyChanged(NPMilesimasSalida);
                mMilesimasSalida = value;
                if (!String.IsNullOrEmpty(mMilesimasSalida) && mMilesimasSalida.Length < 3) {
                    if (mMilesimasSalida.Length < 2) {
                        mMilesimasSalida = "00" + mMilesimasSalida;
                    } else {
                        mMilesimasSalida = "0" + mMilesimasSalida;
                    }
                }
                //comprobar que es un numero y est entre 00 y 999
                int milesimas;
                if (Int32.TryParse(mMilesimasSalida, out milesimas) && milesimas >= 0 && milesimas <= 999) {
                    RemoveError(NPMilesimasSalida);
                    //agregar estos minutos a tiempo.
                    //Tiempo = new TimeSpan(0, 0, minutos, Tiempo.Seconds, Tiempo.Milliseconds);
                    TiempoSalida = new TimeSpan(0,TiempoSalida.Hours, TiempoSalida.Minutes, TiempoSalida.Seconds, milesimas);
                } else {
                    AddError(new PropertyException(NPSegundosSalida, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 999."));
                }
                OnPropertyChanged(NPMilesimasSalida);
            }
        }

        public const string NPHoraLLegada = "HoraLLegada";
        private string mHoraLLegada;

        [Campo(Tamanyo = 2)]
        public string HoraLLegada {
            get {
                PrePropertyRead(NPHoraLLegada);
                return mHoraLLegada;
            }
            set {
                PrePropertyChanged(NPHoraLLegada);
                mHoraLLegada = value;
                if (!String.IsNullOrEmpty(mHoraLLegada) && mHoraLLegada.Length < 2) {
                    mHoraLLegada = "0" + mHoraLLegada;
                }
                //comprobar que es un numero y est entre 00 y 59
                int hora;
                if (Int32.TryParse(mHoraLLegada, out hora) && hora >= 0 && hora <= 23) {
                    RemoveError(NPHoraLLegada);
                    //agregar estos minutos a tiempo.
                    TiempoLLegada = new TimeSpan(0, hora, TiempoLLegada.Minutes, TiempoLLegada.Seconds, TiempoLLegada.Milliseconds);
                } else {
                    AddError(new PropertyException(NPHoraLLegada, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 23."));
                }
                OnPropertyChanged(NPHoraLLegada);
            }
        }

        public const string NPMinutosLLegada = "MinutosLLegada";
        private string mMinutosLLegada;

        [Campo(Tamanyo = 2)]
        public string MinutosLLegada {
            get {
                PrePropertyRead(NPMinutosLLegada);
                return mMinutosLLegada;
            }
            set {
                PrePropertyChanged(NPMinutosLLegada);
                mMinutosLLegada = value;
                if (!String.IsNullOrEmpty(mMinutosLLegada) && mMinutosLLegada.Length < 2) {
                    mMinutosLLegada = "0" + mMinutosLLegada;
                }
                //comprobar que es un numero y est entre 00 y 59
                int minutos;
                if (Int32.TryParse(mMinutosLLegada, out minutos) && minutos >= 0 && minutos <= 59) {
                    RemoveError(NPMinutosLLegada);
                    //agregar estos minutos a tiempo.
                    TiempoLLegada = new TimeSpan(0, TiempoLLegada.Hours, minutos, TiempoLLegada.Seconds, TiempoLLegada.Milliseconds);
                } else {
                    AddError(new PropertyException(NPMinutosLLegada, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                }
                OnPropertyChanged(NPMinutosLLegada);
            }
        }

        public const string NPSegundosLLegada = "SegundosLLegada";
        private string mSegundosLLegada;

        [Campo(Tamanyo = 2)]
        public string SegundosLLegada {
            get {
                PrePropertyRead(NPSegundosLLegada);
                return mSegundosLLegada;
            }
            set {
                PrePropertyChanged(NPSegundosLLegada);
                mSegundosLLegada = value;
                if (!String.IsNullOrEmpty(mSegundosLLegada) && mSegundosLLegada.Length < 2) {
                    mSegundosLLegada = "0" + mSegundosLLegada;
                }
                //comprobar que es un numero y est entre 00 y 59
                int segundos;
                if (Int32.TryParse(mSegundosLLegada, out segundos) && segundos >= 0 && segundos <= 59) {
                    RemoveError(NPSegundosLLegada);
                    TiempoLLegada = new TimeSpan(0, TiempoLLegada.Hours, TiempoLLegada.Minutes, segundos, TiempoLLegada.Milliseconds);
                } else {
                    AddError(new PropertyException(NPSegundosLLegada, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 59."));
                }
                OnPropertyChanged(NPSegundosLLegada);
            }
        }

        public const string NPMilesimasLLegada = "MilesimasLLegada";
        private string mMilesimasLLegada;

        [Campo(Tamanyo = 3)]
        public string MilesimasLLegada {
            get {
                PrePropertyRead(NPMilesimasLLegada);

                return mMilesimasLLegada;
            }
            set {
                PrePropertyChanged(NPMilesimasLLegada);
                mMilesimasLLegada = value;
                if (!String.IsNullOrEmpty(mMilesimasLLegada) && mMilesimasLLegada.Length < 3) {
                    if (mMilesimasLLegada.Length < 2) {
                        mMilesimasLLegada = "00" + mMilesimasLLegada;
                    } else {
                        mMilesimasLLegada = "0" + mMilesimasLLegada;
                    }
                }
                //comprobar que es un numero y est entre 000 y 999
                int milesimas;
                if (Int32.TryParse(mMilesimasLLegada, out milesimas) && milesimas >= 0 && milesimas <= 999) {
                    RemoveError(NPMilesimasLLegada);
                    TiempoLLegada = new TimeSpan(0, TiempoLLegada.Hours, TiempoLLegada.Minutes, TiempoLLegada.Seconds, milesimas);
                } else {
                    AddError(new PropertyException(NPMilesimasLLegada, this, "El formato no es correcto./nTiene que ser un nmero entre 0 y 999."));
                }
                OnPropertyChanged(NPMilesimasLLegada);
            }
        }

        #endregion

        public const string NPPilotoCarrera = "PilotoCarrera";
        private EPilotoCarrera mPilotoCarrera;

        [Campo("PilotoCarreraId"), Requerido]
        public EPilotoCarrera PilotoCarrera {
            get {
                PrePropertyRead(NPPilotoCarrera);
                return mPilotoCarrera;
            }
            set {
                PrePropertyChanged(NPPilotoCarrera);
                mPilotoCarrera = value;
                OnPropertyChanged(NPPilotoCarrera);
            }
        }

        public const string NPCronometrable = "Cronometrable";
        private ECronometrable mCronometrable;

        [Campo("CronometrableId", Tamanyo = 50), Requerido]
        public ECronometrable Cronometrable {
            get {
                PrePropertyRead(NPCronometrable);
                return mCronometrable;
            }
            set {
                PrePropertyChanged(NPCronometrable);
                mCronometrable = value;
                if (mCronometrable != null && mCronometrable.Tipo != TipoCronometrable.Sector) {
                    AddError(new PropertyException(NPCronometrable, this, "Hay que asignarle un sector."));
                }
                OnPropertyChanged(NPCronometrable);
            }
        }

        public const string NPTiempoEstimado = "TiempoEstimado";
        private TimeSpan mTiempoEstimado;
        private bool mTiempoEstimadoCargado = false;

        public TimeSpan TiempoEstimado {
            get {
                if (!mTiempoEstimadoCargado) {
                    mTiempoEstimado = GetTiempoEstimado();
                    mTiempoEstimadoCargado = true;
                }
                return mTiempoEstimado;
            }
        }

        public const string NPDiferenciaTiempo = "DiferenciaTiempo";

        public TimeSpan DiferenciaTiempo {
            get {
                return Tiempo.Subtract(TiempoEstimado);
            }
        }

        #region ToString

        public const string NPDiferenciaToString = "DiferenciaToString";
        public string DiferenciaToString {
            get {
                return CToString.ToString(DiferenciaTiempo);
            }
        }

        public const string NPTiempoEstimadoToString = "TiempoEstimadoToString";
        public string TiempoEstimadoToString {
            get {
                return CToString.ToString(TiempoEstimado);
            }
        }

        public string TiempoToString {
            get {
                return CToString.ToString(Tiempo);
            }
        }

        #endregion

        #endregion

        /// <summary>
        /// Este mtodo devuelve el tiempo que se debe tardar en el sector dado por el cronometrable.
        /// </summary>
        private ESector mSector = null;
        private TimeSpan GetTiempoEstimado() {
            if (Cronometrable != null && Cronometrable.Tipo != TipoCronometrable.Sector) {
                throw new InvalidOperationException("Error en la captura de datos para el tiempo estimado.");
            }
            mSector = new ESector(Cronometrable.Id);
            if (mSector == null || mSector.EsNuevo) {
                throw new InvalidOperationException("Error en la captura de datos para el tiempo estimado.");
            }
            if (Cronometrable.Tipo != TipoCronometrable.Sector) {
                throw new InvalidOperationException("Error en la captura de datos para el tiempo estimado.");
            }
            return mSector.GetTiempoPredeterminado();
        }

        public bool EstaVacio {
            get {
                return Tiempo.Minutes == 0 && Tiempo.Seconds == 0 && Tiempo.Milliseconds == 0;
            }
        }
    }
}

