using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    [Tabla("Personas")]
    public class ECopiloto : EParticipante {

        #region Constructores
        public ECopiloto() : base() { }

        public ECopiloto(int vId) : base(vId) { }

        public ECopiloto(string nif) : base(nif) { }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            EsCopiloto = true;
        }

        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "Copiloto"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return base.ToString(); }
        }
        #endregion

        #region Propiedades

        public const string NPActivo = "Activo";
        private bool mActivo;

        [Campo("ActivoCopiloto")]
        public override bool Activo {
            get {
                PrePropertyRead(NPActivo);
                return mActivo;
            }
            set {
                PrePropertyChanged(NPActivo);
                mActivo = value;
                OnPropertyChanged(NPActivo);
            }
        }

        #endregion

        #region LicenciasCopiloto
        public const string NPLicenciasCopiloto = "LicenciasCopiloto";
        private EColeccion<ELicenciaCopiloto> mLicenciasCopiloto = null;

        /// <summary>Devuelve una coleccin con todos los LicenciasCopiloto.</summary>
        public EColeccion<ELicenciaCopiloto> LicenciasCopiloto {
            get {
                if(mLicenciasCopiloto == null) {
                    mLicenciasCopiloto = new EColeccion<ELicenciaCopiloto>();
                    if(!this.EsNuevo) {
                        //Licencias como piloto
                        EColeccion<ELicenciaCopiloto> colLicCopilotos = new EColeccion<ELicenciaCopiloto>();
                        colLicCopilotos.Filtros.Add(ELicenciaCopiloto.NPCopiloto, this);
                        colLicCopilotos.Filtros.Add(Profundizar(typeof(ELicenciaCopiloto), ELicenciaCopiloto.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.Copilotos);
                        colLicCopilotos.Cargar();
                        //Licencias como participante
                        EColeccion<ELicenciaCopiloto> colLicParticipante = new EColeccion<ELicenciaCopiloto>();
                        colLicParticipante.Filtros.Add(ELicenciaCopiloto.NPCopiloto, this);
                        colLicParticipante.Filtros.Add(Profundizar(typeof(ELicenciaCopiloto), ELicenciaCopiloto.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.Participante);
                        colLicParticipante.Cargar();
                        //Licencias como Otros
                        EColeccion<ELicenciaCopiloto> colLicOtros = new EColeccion<ELicenciaCopiloto>();
                        colLicOtros.Filtros.Add(ELicenciaCopiloto.NPCopiloto, this);
                        colLicOtros.Filtros.Add(Profundizar(typeof(ELicenciaCopiloto), ELicenciaCopiloto.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.Otros);
                        colLicOtros.Cargar();
                        //Agregar todos los elementos a la coleccin principal.
                        foreach (ELicenciaCopiloto lic in colLicCopilotos) {
                            mLicenciasCopiloto.Add(lic);
                        }
                        foreach (ELicenciaCopiloto lic in colLicParticipante) {
                            mLicenciasCopiloto.Add(lic);
                        }
                        foreach (ELicenciaCopiloto lic in colLicOtros) {
                            mLicenciasCopiloto.Add(lic);
                        }
                    }
                    mLicenciasCopiloto.AddingNew += new AddingNewEventHandler(mLicenciasCopiloto_AddingNew);
                    mLicenciasCopiloto.ColumnaMostrarRango(new string[] { ELicenciaCopiloto.NPLicencia, ELicenciaCopiloto.NPNumero, ELicenciaCopiloto.NPFechaCaducidad });
                }
                return mLicenciasCopiloto;
            }
        }

        private void mLicenciasCopiloto_AddingNew(object sender, AddingNewEventArgs e) {
            ELicenciaCopiloto entidad = new ELicenciaCopiloto();
            entidad.Copiloto = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Colecciones
        public static EColeccion<ECopiloto> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<ECopiloto> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<ECopiloto> col = new EColeccion<ECopiloto>("COPILOTOS");
            col.Filtros.Add(NPEsCopiloto,true);
            col.ColumnaMostrarRango(new string[] { NPNif, NPNombre, NPApellidos, NPFechaNacimiento, NPDireccion });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }

        /// <summary>
        /// Devuelve una coleccin de copilotos activos o no activos segn el valor
        /// pasado como parmetro.
        /// </summary>
        /// <param name="activo"></param>
        /// <returns></returns>
        public static EColeccion<ECopiloto> GetCollection(bool activo) {
            EColeccion<ECopiloto> col = new EColeccion<ECopiloto>("COPILOTOS");
            col.Filtros.Add(NPEsCopiloto, true);
            col.Filtros.Add(NPActivo, activo);
            col.ColumnaMostrarRango(new string[] { NPNif, NPNombre, NPApellidos, NPFechaNacimiento, NPDireccion });
            col.Cargar();
            return col;
        }

        #endregion

        #region Carreras Participadas

        private EColeccion<EPilotoCarrera> mCarrerasParticipadas = null;
        public override EColeccion<EPilotoCarrera> CarrerasParticipadas {
            get {
                if (mCarrerasParticipadas == null) {
                    EColeccion<EPilotoCarrera> col = new EColeccion<EPilotoCarrera>();
                    col.Filtros.Add(EPilotoCarrera.NPCopiloto, this);
                    col.Cargar();
                    mCarrerasParticipadas = col;
                }
                return mCarrerasParticipadas;
            }
        }

        #endregion

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar piloto")) {
                int indice = 0;
                EColeccion<ELicenciaCopiloto> colLicencias = LicenciasCopiloto;
                while (indice < colLicencias.Count) {
                    colLicencias[indice].Eliminar();
                    indice = indice + 1;
                }
                base.Eliminar();
                transa.Commit();
            }
        }

        public override bool EstaSucio {
            get {
                return base.EstaSucio || (mLicenciasCopiloto != null && mLicenciasCopiloto.EstaSucio);
            }
        }
    }
}

