﻿using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("Direcciones")]
    public class EDireccion : EIdBase {

        #region Constructores
        /// <summary>Crea una nueva dirección (de forma vaga).</summary>
        public EDireccion() : base() { }

        /// <summary>Carga una dirección existente (de forma vaga).</summary>
        /// <param name="vId">Id de la dirección a cargar.</param>
        public EDireccion(int vId) : base(vId) { }

        /// <summary>Carga una dirección existente por su dirección (de forma vaga).</summary>
        /// <param name="nif">Domicilio de la dirección a cargar.</param>
        public EDireccion(string domicilio) : this() {
            Cargar(NPDomicilio, domicilio);
        }

        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            Domicilio = null;
            Web = null;
            Fax = null;
            Localidad = null;
            Provincia = null;
            Movil = null;
            Telefono = null;
            Pais = null;
        }

        public override bool EsMasculino {
            get { return false; } 
        }

        public override string NombreDeLaEntidad {
            get { return "Dirección"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString
        {
            get { return NPDomicilio; }
        }
        #endregion

        #region Propiedades

        public const string NPDomicilio = "Domicilio";
        private string mDomicilio;

        [Campo(Tamanyo = 200), Unico]
        public string Domicilio {
            get {
                PrePropertyRead(NPDomicilio);
                return mDomicilio;
            }
            set {
                PrePropertyChanged(NPDomicilio);
                mDomicilio = value;
                OnPropertyChanged(NPDomicilio);
            }
        }

        public const string NPCodigoPostal = "CodigoPostal";
        private string mCodigoPostal;

        [Campo]
        public string CodigoPostal
        {
            get
            {
                PrePropertyRead(NPCodigoPostal);
                return mCodigoPostal;
            }
            set
            {
                PrePropertyChanged(NPCodigoPostal);
                mCodigoPostal = value;
                //es un numero.
                try {
                    if (!String.IsNullOrEmpty(mCodigoPostal)) {
                        int num = Convert.ToInt32(mCodigoPostal);
                        if (mCodigoPostal.Length != 5 || num <= 0) {
                            AddError(new PropertyException(NPCodigoPostal, this, "El código postal si se escribe tiene que ser un número de 5 dígitos mayor que cero."));
                        } else {
                            RemoveError(NPCodigoPostal);
                        }
                    } else {
                        RemoveError(NPCodigoPostal);
                    }
                } catch (Exception) {
                    RemoveError(NPCodigoPostal);
                    AddError(new PropertyException(NPCodigoPostal, this, "El código postal, si se escribe, tiene que ser un número de 5 dígitos mayor que cero."));
                }
                OnPropertyChanged(NPCodigoPostal);
            }
        }

        public const string NPLocalidad = "Localidad";
        private string mLocalidad;

        [Campo(Tamanyo = 70)]
        public string Localidad
        {
            get
            {
                PrePropertyRead(NPLocalidad);
                return mLocalidad;
            }
            set
            {
                PrePropertyChanged(NPLocalidad);
                mLocalidad = value;
                OnPropertyChanged(NPLocalidad);
            }
        }

        public const string NPProvincia = "Provincia";
        private string mProvincia;

        [Campo(Tamanyo = 50)]
        public string Provincia
        {
            get
            {
                PrePropertyRead(NPProvincia);
                return mProvincia;
            }
            set
            {
                PrePropertyChanged(NPProvincia);
                mProvincia = value;
                OnPropertyChanged(NPProvincia);
            }
        }

        public const string NPPais = "Pais";
        private string mPais;

        [Campo(Tamanyo = 50)]
        public string Pais
        {
            get
            {
                PrePropertyRead(NPPais);
                return mPais;
            }
            set
            {
                PrePropertyChanged(NPPais);
                mPais = value;
                OnPropertyChanged(NPPais);
            }
        }

        public const string NPTelefono = "Telefono";
        private string mTelefono;

        [Campo(Tamanyo = 50)]
        public string Telefono
        {
            get
            {
                PrePropertyRead(NPTelefono);
                return mTelefono;
            }
            set
            {
                PrePropertyChanged(NPTelefono);
                mTelefono = value;
                try {
                    if (!String.IsNullOrEmpty(mTelefono)) {
                        int num = Convert.ToInt32(mTelefono);
                        if ((mTelefono.Length != 6 && !String.IsNullOrEmpty(mTelefono)) || num <= 0) {
                            AddError(new PropertyException(NPTelefono, this, "El teléfono, si se escribe, tiene que ser un número de 6 dígitos mayor que cero."));
                        } else {
                            RemoveError(NPTelefono);
                        }
                    } else {
                        RemoveError(NPTelefono);
                    }
                } catch (Exception) {
                    RemoveError(NPTelefono);
                    AddError(new PropertyException(NPTelefono, this, "El teléfono, si se escribe, tiene que ser un número de 6 dígitos mayor que cero."));
                }
                OnPropertyChanged(NPTelefono);
            }
        }

        public const string NPMovil = "Movil";
        private string mMovil;

        [Campo(Tamanyo = 50)]
        public string Movil
        {
            get
            {
                PrePropertyRead(NPMovil);
                return mMovil;
            }
            set
            {
                PrePropertyChanged(NPMovil);
                mMovil = value;
                try {
                    if (!String.IsNullOrEmpty(mMovil)) {
                        int num = Convert.ToInt32(mMovil);
                        if ((mMovil.Length != 6 && !String.IsNullOrEmpty(mMovil)) || num <= 0) {
                            AddError(new PropertyException(NPMovil, this, "El móvil, si se escribe, tiene que ser un número de 6 dígitos mayor que cero."));
                        } else {
                            RemoveError(NPMovil);
                        }
                    } else {
                        RemoveError(NPMovil);
                    }
                } catch (Exception) {
                    RemoveError(NPMovil);
                    AddError(new PropertyException(NPMovil, this, "El móvil, si se escribe, tiene que ser un número de 6 dígitos mayor que cero."));
                }
                OnPropertyChanged(NPMovil);
            }
        }

        public const string NPFax = "Fax";
        private string mFax;

        [Campo(Tamanyo = 50)]
        public string Fax
        {
            get
            {
                PrePropertyRead(NPFax);
                return mFax;
            }
            set
            {
                PrePropertyChanged(NPFax);
                mFax = value;
                try {
                    if (!String.IsNullOrEmpty(mFax)) {
                        int num = Convert.ToInt32(mFax);
                        if ((mFax.Length != 6 && !String.IsNullOrEmpty(mFax)) || num <= 0) {
                            AddError(new PropertyException(NPFax, this, "El fax, si se escribe, tiene que ser un número de 6 dígitos mayor que cero."));
                        } else {
                            RemoveError(NPFax);
                        }
                    } else {
                        RemoveError(NPFax);
                    }
                } catch (Exception) {
                    RemoveError(NPFax);
                    AddError(new PropertyException(NPFax, this, "El fax, si se escribe, tiene que ser un número de 6 dígitos mayor que cero."));
                }
               OnPropertyChanged(NPFax);
            }
        }

        public const string NPWeb = "Web";
        private string mWeb;

        [Campo(Tamanyo = 50)]
        public string Web
        {
            get
            {
                PrePropertyRead(NPWeb);
                return mWeb;
            }
            set
            {
                PrePropertyChanged(NPWeb);
                mWeb = value;
                OnPropertyChanged(NPWeb);
            }
        }

        #endregion

        #region Personas
        public const string NPPersonas = "Personas";
        private EColeccion<EPersona> mPersonas = null;

        /// <summary>Devuelve una colección con todos las Personas.</summary>
        public EColeccion<EPersona> Personas {
            get {
                if(mPersonas == null) {
                    mPersonas = new EColeccion<EPersona>();
                    if(!this.EsNuevo) {
                        mPersonas.Filtros.Add(EPersona.NPDireccion, this);
                        mPersonas.Cargar();
                    }
                }
                return mPersonas;
            }
        }

       #endregion

        #region Escuderias
        public const string NPEscuderias = "Escuderias";
        private EColeccion<EEscuderia> mEscuderias = null;

        /// <summary>Devuelve una colección con todos los Escuderias.</summary>
        public EColeccion<EEscuderia> Escuderias {
            get {
                if(mEscuderias == null) {
                    mEscuderias = new EColeccion<EEscuderia>();
                    if(!this.EsNuevo) {
                        mEscuderias.Filtros.Add(EEscuderia.NPDireccion, this);
                        mEscuderias.Cargar();
                    }
                    mEscuderias.AddingNew += new AddingNewEventHandler(mEscuderias_AddingNew);
                    mEscuderias.ListChanged += new ListChangedEventHandler(mEscuderias_ListChanged);
                }
                return mEscuderias;
            }
        }

        private void mEscuderias_AddingNew(object sender, AddingNewEventArgs e) {
            EEscuderia entidad = new EEscuderia();
            entidad.Direccion = this;
            e.NewObject = entidad;
        }

        private void mEscuderias_ListChanged(object sender, ListChangedEventArgs e) {
            if(!Escuderias.EsValido)
                AddError(new PropertyException(NPEscuderias, this, Escuderias.AlgunError.Message, Escuderias.AlgunError));
            else
                RemoveError(NPEscuderias);
            OnPropertyCollectionChanged(NPEscuderias);
        }
        #endregion

        /// <summary>
        /// una direccion es vacía cuando todos sus campos son vacíos.
        /// </summary>
        /// <returns></returns>
        public bool EstaVacio() {
            return String.IsNullOrEmpty(mDomicilio) &&
                String.IsNullOrEmpty(mFax) && String.IsNullOrEmpty(mLocalidad) && String.IsNullOrEmpty(mMovil) &&
                String.IsNullOrEmpty(mPais) && String.IsNullOrEmpty(mProvincia) && String.IsNullOrEmpty(mTelefono) &&
                String.IsNullOrEmpty(mWeb) && String.IsNullOrEmpty(mCodigoPostal);
        }

        /// <summary>
        /// Elimina la primera aparición de una dirección que no tiene referenciada ni personas ni escuderias.
        /// </summary>
        /// <param name="idDirecc"></param>
        internal static void EliminarDireccionVacia(int idDirecc) {
            EDireccion direccion = new EDireccion(idDirecc);
            if (!direccion.EsNuevo && direccion.Personas.Count == 0 && direccion.Escuderias.Count == 0) {
                direccion.Eliminar();
            } 
        }
    }
}

