using System.ComponentModel;
using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    public enum TipoEscuderia {
        Marca = 1,
        Club,
        Escuderia,
        Otras
    }

    [Tabla("Escuderias")]
    public class EEscuderia : EIdBase, IAltaBaja {

        #region Constructores

        public EEscuderia() : base() { }

        public EEscuderia(int vId) : base(vId) { }

        public EEscuderia(string nombre) : base() {
            this.Cargar(NPNombre, nombre);    
        }

        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            Nombre = null;
            Tipo = TipoEscuderia.Otras;
            NombreContacto = null;
            Direccion = null;
            Activo = true;
        }


        protected override void GuardarCore() {
            bool hayQueEliminar = false;
            int idDirecc = 0;
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar Escuderia")) {
                if (Direccion != null && Direccion.EstaVacio()) {
                    hayQueEliminar = Direccion.Personas.Count == 0 && Direccion.Escuderias.Count == 1;
                    idDirecc = Direccion.Id;
                    Direccion = null;
                } else if (mDireccion != null) {
                    Direccion.Guardar();
                }
                base.GuardarCore();
                //if (mParticipantes != null)
                //    mParticipantes.Guardar();
                if (mLicenciaEscuderia != null)
                    mLicenciaEscuderia.Guardar();
                //if (mCarreras != null)
                //    mCarreras.Guardar();
                if (mLogoCargada)
                    CentroImagenesE.Actual.AddImagen(this, Logo);
                transa.Commit();
            }
            if (hayQueEliminar) {
                //Eliminar la direccion anterior.
                EDireccion.EliminarDireccionVacia(idDirecc);
            }
        }

        public override void Deshacer() {
            mCarreras = null;
            mParticipantes = null;
            mLicenciaEscuderia = null;
            mLogoCargada = false;
            if (Direccion != null) {
                Direccion.Deshacer();
            }
            base.Deshacer();
        }


        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "Escuderia"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return this.ToString(); } 
        }
        #endregion

        #region Propiedades

        public const string NPNombre = "Nombre";
        private string mNombre;

        [Campo(Tamanyo = 80), Requerido, Unico]
        public string Nombre {
            get {
                PrePropertyRead(NPNombre);
                return mNombre;
            }
            set {
                PrePropertyChanged(NPNombre);
                mNombre = value;
                OnPropertyChanged(NPNombre);
            }
        }

        public const string NPTipo = "Tipo";
        private TipoEscuderia mTipo;

        [Campo("Tipo")]
        public TipoEscuderia Tipo {
            get {
                PrePropertyRead(NPTipo);
                return mTipo;
            }
            set {
                PrePropertyChanged(NPTipo);
                mTipo = value;
                OnPropertyChanged(NPTipo);
            }
        }

        public const string NPNombreContacto = "NombreContacto";
        private string mNombreContacto;

        [Campo(Tamanyo = 80)]
        public string NombreContacto {
            get {
                PrePropertyRead(NPNombreContacto);
                return mNombreContacto;
            }
            set {
                PrePropertyChanged(NPNombreContacto);
                mNombreContacto = value;
                OnPropertyChanged(NPNombreContacto);
            }
        }

        public const string NPEmail = "Email";
        private string mEmail;

        [Campo(Tamanyo = 100)]
        public string Email {
            get {
                PrePropertyRead(NPEmail);
                return mEmail;
            }
            set {
                PrePropertyChanged(NPEmail);
                mEmail = value;
                OnPropertyChanged(NPEmail);
            }
        }

        public const string NPDireccion = "Direccion";
        private EDireccion mDireccion;

        [Campo("DireccionId")]
        public EDireccion Direccion {
            get {
                PrePropertyRead(NPDireccion);
                return mDireccion;
            }
            set {
                PrePropertyChanged(NPDireccion);
                mDireccion = value;
                OnPropertyChanged(NPDireccion);
            }
        }

        public const string NPLogo = "Logo";
        private System.Drawing.Image mLogo;
        private bool mLogoCargada = false;

        public System.Drawing.Image Logo {
            get {
                if (!mLogoCargada) {
                    mLogo = CentroImagenesE.Actual.GetImagen(this);
                    mLogoCargada = true;
                }
                return mLogo;
            }
            set {
                mLogo = value;
                mLogoCargada = true;
                OnPropertyChanged(NPLogo);
            }
        }

        public const string NPActivo = "Activo";
        private bool mActivo;

        [Campo]
        public bool Activo {
            get {
                PrePropertyRead(NPActivo);
                return mActivo;
            }
            set {
                PrePropertyChanged(NPActivo);
                mActivo = value;
                OnPropertyChanged(NPActivo);
            }
        }

        #endregion

        #region LicenciaEscuderia
        public const string NPLicenciaEscuderia = "LicenciaEscuderia";
        private EColeccion<ELicenciaEscuderia> mLicenciaEscuderia = null;

        /// <summary>Devuelve una coleccin con todos los LicenciaEscuderia.</summary>
        public EColeccion<ELicenciaEscuderia> LicenciaEscuderia {
            get {
                if(mLicenciaEscuderia == null) {
                    mLicenciaEscuderia = new EColeccion<ELicenciaEscuderia>();
                    if(!this.EsNuevo) {
                        EColeccion<ELicenciaEscuderia> colLicEscuderia = new EColeccion<ELicenciaEscuderia>();
                        colLicEscuderia.Filtros.Add(ELicenciaEscuderia.NPEscuderia, this);
                        colLicEscuderia.Filtros.Add(Profundizar(typeof(ELicenciaEscuderia), ELicenciaEscuderia.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.EMC);
                        colLicEscuderia.Cargar();
                        //Licencias como Otros
                        EColeccion<ELicenciaEscuderia> colLicOtros = new EColeccion<ELicenciaEscuderia>();
                        colLicOtros.Filtros.Add(ELicenciaEscuderia.NPEscuderia, this);
                        colLicOtros.Filtros.Add(Profundizar(typeof(ELicenciaEscuderia), ELicenciaEscuderia.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.Otros);
                        colLicOtros.Cargar();
                        //Agregar todos los elementos a la coleccin principal.
                        foreach (ELicenciaEscuderia lic in colLicEscuderia) {
                            mLicenciaEscuderia.Add(lic);
                        }
                        foreach (ELicenciaEscuderia lic in colLicOtros) {
                            mLicenciaEscuderia.Add(lic);
                        }
                    }
                    mLicenciaEscuderia.AddingNew += new AddingNewEventHandler(mLicenciaEscuderia_AddingNew);
                    mLicenciaEscuderia.ColumnaMostrarRango(new string[] { ELicenciaEscuderia.NPLicencia, ELicenciaEscuderia.NPNumero });
                }
                return mLicenciaEscuderia;
            }
        }

        private void mLicenciaEscuderia_AddingNew(object sender, AddingNewEventArgs e) {
            ELicenciaEscuderia entidad = new ELicenciaEscuderia();
            entidad.Escuderia = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Participantes
        public const string NPParticipantes = "Participantes";
        private EColeccion<EParticipante> mParticipantes = null;

        /// <summary>Devuelve una coleccin con todos los Participantes.</summary>
        public EColeccion<EParticipante> Participantes {
            get {
                if(mParticipantes == null) {
                    mParticipantes = new EColeccion<EParticipante>();
                    if(!this.EsNuevo) {
                        EColeccion<EPiloto> colPilotos = EPiloto.GetCollection();
                        EColeccion<ECopiloto> colCopilotos = ECopiloto.GetCollection();
                        colPilotos.Filtros.Add(EParticipante.NPEscuderia, this);
                        colCopilotos.Filtros.Add(EParticipante.NPEscuderia, this);
                        colPilotos.Cargar();
                        colCopilotos.Cargar();
                        foreach (EPiloto piloto in colPilotos) {
                            mParticipantes.Add(piloto);
                        }
                        int maxPilotos = mParticipantes.Count;
                        foreach (ECopiloto copiloto in colCopilotos) {
                            int indice = 0;
                            bool terminado = false;
                            while (!terminado && indice < maxPilotos) {
                                terminado = copiloto.Id == mParticipantes[indice].Id;
                                indice = indice + 1;
                            }
                            if (!terminado) {
                                mParticipantes.Add(copiloto);
                            }
                        }
                    }
                    mParticipantes.ColumnaMostrarRango(new string[] { EParticipante.NPEsPiloto, EParticipante.NPEsCopiloto, EParticipante.NPNif, EParticipante.NPNombre, EParticipante.NPApellidos });
                }
                return mParticipantes;
            }
        }

        #endregion

        #region Carreras
        public const string NPCarreras = "Carreras";
        private EColeccion<ECarrera> mCarreras = null;

        /// <summary>Devuelve una coleccin con todos los Carreras que ha organizado.</summary>
        public EColeccion<ECarrera> Carreras {
            get {
                if(mCarreras == null) {
                    mCarreras = new EColeccion<ECarrera>();
                    if(!this.EsNuevo) {
                        mCarreras.Filtros.Add(ECarrera.NPEscuderiaOrganiza, this);
                        mCarreras.Cargar();
                    }
                    mCarreras.ColumnaMostrarRango(new string[] { ECarrera.NPTipo, ECarrera.NPNombre, ECarrera.NPCoeficiente, ECarrera.NPFechaComienzo });
                }
                return mCarreras;
            }
        }

        #endregion

        #region Colecciones
        public static EColeccion<EEscuderia> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EEscuderia> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EEscuderia> col = new EColeccion<EEscuderia>("ESCUDERAS");
            col.Filtros.Add(NPActivo, true);
            col.ColumnaMostrarRango(new string[] {NPTipo, NPNombre, NPNombreContacto, NPEmail ,NPDireccion });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }

        public static EColeccion<EEscuderia> GetCollection(bool activo) {
            EColeccion<EEscuderia> col = new EColeccion<EEscuderia>("ESCUDERAS");
            col.Filtros.Add(NPActivo, activo);
            col.ColumnaMostrarRango(new string[] { NPTipo, NPNombre, NPNombreContacto, NPEmail, NPDireccion });
            col.Cargar();
            return col;
        }

        #endregion

        #region IBaja

        public void DarAltaBaja(bool alta) {
            this.Activo = alta;
        }

        #endregion

        public override bool EstaSucio {
            get {
                return base.EstaSucio || (mLicenciaEscuderia != null && mLicenciaEscuderia.EstaSucio) || (mParticipantes != null && mParticipantes.EstaSucio) || (mCarreras != null && mCarreras.EstaSucio) || (mDireccion != null && mDireccion.EstaSucio);
            }
        }

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar Escuderia")) {
                int indice = 0;
                EColeccion<ELicenciaEscuderia> colLicencias = LicenciaEscuderia;
                while (indice < colLicencias.Count) {
                    colLicencias[indice].Eliminar();
                    indice = indice + 1;
                }
                //Los participantes;
                EColeccion<EParticipante> colParticpantes = Participantes;
                foreach (EParticipante participante in colParticpantes) {
                    participante.Escuderia = null;
                    participante.Guardar();
                }
                EColeccion<ECarrera> colCarreras = Carreras;
                foreach (ECarrera carrera in colCarreras) {
                    carrera.EscuderiaOrganiza = null;
                    carrera.Guardar();
                }
                base.Eliminar();
                transa.Commit();
            }
            
        }

        /// <summary>
        /// Devuelve el nombre de la escudera.
        /// </summary>
        /// <returns></returns>
        public override string ToString() {
            return Nombre;
        }
    }
}

