﻿using System.ComponentModel;
using Nibi.Dal;
using Nibi.Negocio;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System;

namespace Negocio
{
    public enum TipoDirigido{
        Ninguno = 0,
        [Description("Escudería, Marca, Club")]
        EMC = 1,
        [Description("Pilotos y Copilotos")]
        Participante,
        Pilotos,
        Copilotos,
        Oficial,
        [Description("Oficial en Prácticas")]
        OPracticas,
        Otros
    }

    [Tabla("Licencias")]
    public class ELicencia : EIdBase, IAltaBaja {

        #region Constructores
        public ELicencia() : base() { }

        public ELicencia(int vId) : base(vId) { }

        public ELicencia(string cod) : base(){
            this.Cargar(NPCodigo, cod);
        }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo(){
            Codigo = null;
            Nombre = null;
            TipoDirigido = TipoDirigido.Ninguno;
            Documentacion = null;
            Importe = 0.0;
            Activo = true;
        }

        public override bool EsMasculino{
            get { return false; }
        }

        public override string NombreDeLaEntidad{
            get { return "Licencia"; }
        }

        public override bool Modificable(string nombreProp){
            if (!EstaCargado) return true;

            switch (nombreProp)
            {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString{
            get { return NPCodigo; }
        }
        #endregion

        #region Propiedades

        public const string NPCodigo = "Codigo";
        private string mCodigo;

        [Campo(Tamanyo = 5), Requerido, Unico]
        public string Codigo
        {
            get
            {
                PrePropertyRead(NPCodigo);
                return mCodigo;
            }
            set
            {
                PrePropertyChanged(NPCodigo);
                mCodigo = value;
                OnPropertyChanged(NPCodigo);
            }
        }

        public const string NPNombre = "Nombre";
        private string mNombre;

        [Campo(Tamanyo = 60), Requerido]
        public string Nombre {
            get {
                PrePropertyRead(NPNombre);
                return mNombre;
            }
            set {
                PrePropertyChanged(NPNombre);
                mNombre = value;
                OnPropertyChanged(NPNombre);
            }
        }


        public const string NPTipoDirigido = "TipoDirigido";
        private TipoDirigido mTipoDirigido;

        [Campo("Tipo"), Requerido]
        public TipoDirigido TipoDirigido
        {
            get
            {
                PrePropertyRead(NPTipoDirigido);
                return mTipoDirigido;
            }
            set
            {
                PrePropertyChanged(NPTipoDirigido);
                mTipoDirigido = value;
                OnPropertyChanged(NPTipoDirigido);
            }
        }

        public const string NPDocumentacion = "Documentacion";
        private string mDocumentacion;

        [Campo(Tamanyo = 255)]
        public string Documentacion
        {
            get
            {
                PrePropertyRead(NPDocumentacion);
                return mDocumentacion;
            }
            set
            {
                PrePropertyChanged(NPDocumentacion);
                mDocumentacion = value;
                OnPropertyChanged(NPDocumentacion);
            }
        }

        public const string NPImporte = "Importe";
        private double mImporte;

        [Campo]
        public double Importe
        {
            get
            {
                PrePropertyRead(NPImporte);
                return mImporte;
            }
            set
            {
                PrePropertyChanged(NPImporte);
                mImporte = value;
                OnPropertyChanged(NPImporte);
            }
        }

        public const string NPActivo = "Activo";
        private bool mActivo;

        [Campo]
        public bool Activo {
            get {
                PrePropertyRead(NPActivo);
                return mActivo;
            }
            set {
                PrePropertyChanged(NPActivo);
                mActivo = value;
                OnPropertyChanged(NPActivo);
            }
        }

        #endregion

        #region LicenciasNumeradas
        private const string NPLicenciasNumeradas = "LicenciasNumeradas";
        private EColeccion<ELicenciaNumerada> mLicenciasNumeradas = null;

        /// <summary>Devuelve una colección con todos los LicenciasNumeradas.</summary>
        private EColeccion<ELicenciaNumerada> LicenciasNumeradas {
            get {
                if(mLicenciasNumeradas == null) {
                    mLicenciasNumeradas = new EColeccion<ELicenciaNumerada>();
                    if(!this.EsNuevo) {
                        mLicenciasNumeradas.Filtros.Add(ELicenciaNumerada.NPLicencia, this);
                        mLicenciasNumeradas.Cargar();
                    }
                }
                return mLicenciasNumeradas;
            }
        }

        #endregion

        #region Colecciones

        public static EColeccion<ELicencia> GetCollection(){
            return GetCollection(false, false);
        }

        public static EColeccion<ELicencia> GetCollection(bool vacio, bool asincronamente){
            EColeccion<ELicencia> col = new EColeccion<ELicencia>("LICENCIAS");
            col.Filtros.Add(NPActivo, true);
            col.ColumnaMostrarRango(new string[] { NPCodigo, NPNombre, NPDocumentacion, NPImporte, NPTipoDirigido });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }

        public static EColeccion<ELicencia> GetCollection(TipoDirigido t) {
            EColeccion<ELicencia> col = new EColeccion<ELicencia>();
            col.Filtros.Add(NPActivo, true);
            col.Filtros.Add(NPTipoDirigido, t);
            col.ColumnaMostrarRango(new string[] { NPCodigo, NPNombre, NPDocumentacion, NPImporte, NPTipoDirigido });
            col.Cargar();
            return col;
        }

        public static EColeccion<ELicencia> GetCollection(bool activo) {
            EColeccion<ELicencia> col = new EColeccion<ELicencia>("LICENCIAS");
            col.Filtros.Add(NPActivo, activo);
            col.ColumnaMostrarRango(new string[] { NPCodigo, NPNombre, NPDocumentacion, NPImporte, NPTipoDirigido });
            col.Cargar();
            return col;
        }

        public static Collection<KeyValuePair<ELicencia, string>> GetCollectionCombo(TipoDirigido t) {
            EColeccion<ELicencia> col = GetCollection(t);
            Collection<KeyValuePair<ELicencia, string>> colP = new Collection<KeyValuePair<ELicencia, string>>();
            foreach (ELicencia lic in col) {
                colP.Add(new KeyValuePair<ELicencia, string>(lic, lic.Codigo));
            }
            return colP;
        }

        #endregion

        #region IBaja

        public void DarAltaBaja(bool alta) {
            Activo = alta;
        }

        #endregion

        public override void Eliminar() {
            if (LicenciasNumeradas.Count > 0) {
                throw new Exception("La licencia no se puede eliminar por que hay personas y/o entidades que la poseen.");
            }
            base.Eliminar();
        }
    }
}

