using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    public enum TipoCargoOficial {
        Ninguno = 0,
        [Description("Comisario Deportivo")]
        CDeportivo = 1,
        [Description("Director de Carrera")]
        DCarrera,
        [Description("Secretario de Carrera")]
        SCarrera,
        [Description("Jefe de rea")]
        JArea,
        [Description("Comisario Tcnico")]
        CTecnico,
        Controlador,
        Cronometrador,
        [Description("Comisario de Ruta")]
        CRuta,
        [Description("Cronometrador en Prcticas")]
        CronoPracticas,
        [Description("Comisario de Ruta en Prcitcas")]
        CRutaPracticas
    }

    [Tabla("LicenciasNumeradas")]
    public class ELicenciaOficial : ELicenciaNumerada {

        #region Constructores
        public ELicenciaOficial() : base() { }

        public ELicenciaOficial(int vId) : base(vId) { }

        public ELicenciaOficial(string num) : base(num) { }

        public ELicenciaOficial(EOficial of, TipoCargoOficial tco) : base() {
            Cargar(new string[] {NPOficial, NPCargo }, new object[] {of, tco });
        }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            Oficial = null;
            Cargo = 0;
        }

        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "Licencia Oficial"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return base.NPToString; } 
        }
        #endregion

        #region Propiedades

        public const string NPOficial = "Oficial";
        private EOficial mOficial;

        [Campo("PersonaId"), Requerido]
        public EOficial Oficial {
            get {
                PrePropertyRead(NPOficial);
                return mOficial;
            }
            set {
                PrePropertyChanged(NPOficial);
                mOficial = value;
                OnPropertyChanged(NPOficial);
            }
        }

        public const string NPCargo = "Cargo";
        private TipoCargoOficial mCargo;

        [Campo, Requerido]
        public TipoCargoOficial Cargo {
            get {
                PrePropertyRead(NPCargo);
                return mCargo;
            }
            set {
                PrePropertyChanged(NPCargo);
                mCargo = value;
                OnPropertyChanged(NPCargo);
            }
        }

        #endregion
    }
}
