using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("Personas")]
    public class EOficial : EPersona {

        #region Constructores

        public EOficial() : base() { }

        public EOficial(int vId) : base(vId) { }

        public EOficial(string nif) : base(nif) { }

        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            EsOficial = true;
        }

        public override bool EsMasculino {
            get { return true; }
        }

        public override string NombreDeLaEntidad {
            get { return "Oficial"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return base.Modificable(nombreProp);
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return base.NPToString; } 
        }
        #endregion

        #region Propiedades

        public const string NPActivo = "Activo";
        private bool mActivo;

        [Campo("ActivoOficial")]
        public override bool Activo {
            get {
                PrePropertyRead(NPActivo);
                return mActivo;
            }
            set {
                PrePropertyChanged(NPActivo);
                mActivo = value;
                OnPropertyChanged(NPActivo);
            }
        }

        public const string NPEsOficial = "EsOficial";
        private bool mEsOficial;

        [Campo]
        public bool EsOficial {
            get {
                PrePropertyRead(NPEsOficial);
                return mEsOficial;
            }
            set {
                PrePropertyChanged(NPEsOficial);
                mEsOficial = value;
                OnPropertyChanged(NPEsOficial);
            }
        }

        #endregion

        #region LicenciasOficial
        public const string NPLicenciasOficial = "LicenciasOficial";
        private EColeccion<ELicenciaOficial> mLicenciasOficial = null;

        /// <summary>Devuelve una coleccin con todos las LicenciasOficial de un oficial.</summary>
        public EColeccion<ELicenciaOficial> LicenciasOficial {
            get {
                if(mLicenciasOficial == null) {
                    mLicenciasOficial = new EColeccion<ELicenciaOficial>();
                    if(!this.EsNuevo) {
                        //Licencias como oficial
                        EColeccion<ELicenciaOficial> colLicOficial = new EColeccion<ELicenciaOficial>();
                        colLicOficial.Filtros.Add(ELicenciaOficial.NPOficial, this);
                        colLicOficial.Filtros.Add(Profundizar(typeof(ELicenciaOficial), ELicenciaOficial.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.Oficial);
                        colLicOficial.Cargar();
                        //Licencias como oficial en prcticas
                        EColeccion<ELicenciaOficial> colLicOPracticas = new EColeccion<ELicenciaOficial>();
                        colLicOPracticas.Filtros.Add(ELicenciaOficial.NPOficial, this);
                        colLicOPracticas.Filtros.Add(Profundizar(typeof(ELicenciaOficial), ELicenciaOficial.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.OPracticas);
                        colLicOPracticas.Cargar();
                        //Licencias como Otros
                        EColeccion<ELicenciaOficial> colLicOtros = new EColeccion<ELicenciaOficial>();
                        colLicOtros.Filtros.Add(ELicenciaOficial.NPOficial, this);
                        colLicOtros.Filtros.Add(Profundizar(typeof(ELicenciaOficial), ELicenciaOficial.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.Otros);
                        colLicOtros.Cargar();
                        //Agregar todos los elementos a la coleccin principal.
                        foreach (ELicenciaOficial lic in colLicOficial) {
                            mLicenciasOficial.Add(lic);
                        }
                        foreach (ELicenciaOficial lic in colLicOPracticas) {
                            mLicenciasOficial.Add(lic);
                        }
                        foreach (ELicenciaOficial lic in colLicOtros) {
                            mLicenciasOficial.Add(lic);
                        }
                    }
                    mLicenciasOficial.AddingNew += new AddingNewEventHandler(mLicenciasOficial_AddingNew);
                    mLicenciasOficial.ColumnaMostrarRango(new string[] { ELicenciaOficial.NPLicencia, ELicenciaOficial.NPNumero, ELicenciaOficial.NPCargo });
                }
                return mLicenciasOficial;
            }
        }

        private void mLicenciasOficial_AddingNew(object sender, AddingNewEventArgs e) {
            ELicenciaOficial entidad = new ELicenciaOficial();
            entidad.Oficial = this;
            e.NewObject = entidad;
        }

        #endregion

        #region OficialCarreras
        public const string NPOficialCarreras = "OficialCarreras";
        private EColeccion<ECargo> mOficialCarreras = null;

        /// <summary>Devuelve una coleccin con todos los OficialCarreras.</summary>
        public EColeccion<ECargo> OficialCarreras {
            get {
                if(mOficialCarreras == null) {
                    mOficialCarreras = new EColeccion<ECargo>();
                    if(!this.EsNuevo) {
                        mOficialCarreras.Filtros.Add(ECargo.NPOficial, this);
                        mOficialCarreras.Cargar();
                    }
                    mOficialCarreras.AddingNew += new AddingNewEventHandler(mOficialCarreras_AddingNew);
                }
                return mOficialCarreras;
            }
        }

        private void mOficialCarreras_AddingNew(object sender, AddingNewEventArgs e) {
            ECargo entidad = new ECargo();
            entidad.Oficial = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Colecciones

        public static EColeccion<EOficial> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EOficial> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EOficial> col = new EColeccion<EOficial>("OFICIALES");
            col.Filtros.Add(NPEsOficial, true);
            col.ColumnaMostrarRango(new string[] { NPNif, NPNombre, NPApellidos, NPFechaNacimiento, NPDireccion });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }

        /// <summary>
        /// Devuelve una coleccin de oficiales activos o no activos segn el valor
        /// pasado como parmetro.
        /// </summary>
        /// <param name="activo"></param>
        /// <returns></returns>
        public static EColeccion<EOficial> GetCollection(bool activo) {
            EColeccion<EOficial> col = new EColeccion<EOficial>("OFICIALES");
            col.Filtros.Add(NPEsOficial, true);
            col.Filtros.Add(NPActivo, activo);
            col.ColumnaMostrarRango(new string[] { NPNif, NPNombre, NPApellidos, NPFechaNacimiento, NPDireccion });
            col.Cargar();
            return col;
        }

        #endregion

        private ELicenciaOficial licenciaActual = null;
        public bool PuedeEjercer(TipoCargoOficial tco) {
            foreach(ELicenciaOficial lic in this.LicenciasOficial) {
                if(lic.Cargo.Equals(tco)) {
                    licenciaActual = lic;
                    return true;
                }
            }
            licenciaActual = null;
            return false;
        }

        /// <summary>
        /// Devuelve una licencia del oficial segn el cargo pasado como parmetro.
        /// Si no existe la licencia devuelve null
        /// </summary>
        /// <param name="tco"></param>
        /// Tipo cargo de Oficial 
        /// <returns></returns>
        /// 
        public ELicenciaOficial Licencia(TipoCargoOficial tco){
            if(licenciaActual != null && licenciaActual.Cargo.Equals(tco)) {
                return licenciaActual;
            }
            foreach(ELicenciaOficial lic in this.LicenciasOficial) {
                if(lic.Cargo.Equals(tco)) {
                    licenciaActual = lic;
                    return lic;
                }
            }
            licenciaActual = null;
            return null;
        }

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar piloto")) {
                if (OficialCarreras.Count > 0) {
                    throw new Exception("No se puede eliminar el oficial.\nParticipa (o ha participado) en carreras.");
                }
                int indice = 0;
                EColeccion<ELicenciaOficial> colLicencias = LicenciasOficial;
                while (indice < colLicencias.Count) {
                    colLicencias[indice].Eliminar();
                    indice = indice + 1;
                }
                base.Eliminar();
                transa.Commit();
            }
        }

        public override bool EstaSucio {
            get {
                return base.EstaSucio || ((mLicenciasOficial != null && mLicenciasOficial.EstaSucio) || (mOficialCarreras != null && mOficialCarreras.EstaSucio));
            }
        }
    }
}