using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("Personas")]
    public abstract class EParticipante : EPersona, IQuitar {

        #region Constructores
        public EParticipante() : base() { }

        public EParticipante(int vId) : base(vId) { }

        public EParticipante(string nif) : base(nif) { }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
        }

        public override bool EsMasculino {
            get { return true; } 
        }

        public override string NombreDeLaEntidad {
            get { return "Participante"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPNif; } 
        }
        #endregion

        #region Propiedades

        public const string NPEsPiloto = "EsPiloto";
        private bool mEsPiloto;

        [Campo]
        public bool EsPiloto {
            get {
                PrePropertyRead(NPEsPiloto);
                return mEsPiloto;
            }
            set {
                PrePropertyChanged(NPEsPiloto);
                mEsPiloto = value;
                OnPropertyChanged(NPEsPiloto);
            }
        }

        public const string NPEsCopiloto = "EsCopiloto";
        private bool mEsCopiloto;

        [Campo]
        public bool EsCopiloto {
            get {
                PrePropertyRead(NPEsCopiloto);
                return mEsCopiloto;
            }
            set {
                PrePropertyChanged(NPEsCopiloto);
                mEsCopiloto = value;
                OnPropertyChanged(NPEsCopiloto);
            }
        }

        public const string NPEscuderia = "Escuderia";
        private EEscuderia mEscuderia;

        [Campo("EscuderiaId")]
        public EEscuderia Escuderia {
            get {
                PrePropertyRead(NPEscuderia);
                return mEscuderia;
            }
            set {
                PrePropertyChanged(NPEscuderia);
                mEscuderia = value;
                OnPropertyChanged(NPEscuderia);
            }
        }

        #endregion

        public abstract EColeccion<EPilotoCarrera> CarrerasParticipadas { get;}
        
        #region Colecciones

        public static EColeccion<EParticipante> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EParticipante> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EParticipante> colP = GetCollectionPilotos();
            int maximoPilotos = colP.Count;
            EColeccion<EParticipante> colC = GetCollectionCopilotos();
            foreach (ECopiloto copiloto in colC) {
                int indice = 0;
                bool terminado = false;
                while (!terminado && indice < maximoPilotos) {
                    terminado = copiloto.Id == colP[indice].Id;
                    indice = indice + 1;
                }
                if (!terminado) {
                    colP.Add(copiloto);
                }
            }
            colP.ColumnaMostrarRango(new string[] { NPNombre, NPApellidos });
            colP.Titulo = "PARTICIPANTES";
            return colP;
        }

        public static EColeccion<EParticipante> GetCollectionPilotos() {
            return GetCollectionPilotos(false, false);
        }

        public static EColeccion<EParticipante> GetCollectionPilotos(bool vacio, bool asincronamente) {
            EColeccion<EParticipante> col = new EColeccion<EParticipante>("PARTICIPANTES (PILOTOS)");
            EColeccion<EPiloto> colPilotos = EPiloto.GetCollection();
            foreach (EPiloto piloto in colPilotos) {
                col.Add(piloto);
            }
            col.ColumnaMostrarRango(new string[] { NPNombre, NPApellidos });
            return col;
        }

        public static EColeccion<EParticipante> GetCollectionCopilotos() {
            return GetCollectionCopilotos(false, false);
        }

        public static EColeccion<EParticipante> GetCollectionCopilotos(bool vacio, bool asincronamente) {
            EColeccion<EParticipante> col = new EColeccion<EParticipante>("PARTICIPANTES (COPILOTOS)");
            EColeccion<ECopiloto> colPilotos = ECopiloto.GetCollection();
            foreach (ECopiloto piloto in colPilotos) {
                col.Add(piloto);
            }
            col.ColumnaMostrarRango(new string[] { NPNombre, NPApellidos });
            return col;
        }

        /// <summary>
        /// Devuelve una coleccin con los participantes que no estn ya en la coleccin pasada como parmetro.
        /// </summary>
        /// <param name="mColeccion"></param>
        /// <returns></returns>
        public static EColeccion<EParticipante> GetCollection(EColeccion<EParticipante> mColeccion) {
            //HACK: cuidado con los nombres de entidad.
            EColeccion<EParticipante> col = EParticipante.GetCollection();
            if (mColeccion != null) {
                foreach (EParticipante participante in mColeccion) {
                    EParticipante partAEliminar = null;
                    int indice = 0;
                    bool terminado = false;
                    while (!terminado && indice < col.Count) {
                        partAEliminar = col[indice];
                        terminado = partAEliminar.Id == participante.Id;
                        indice = indice + 1;
                    }
                    col.Remove(partAEliminar);
                }
            }
            col.ColumnaMostrarRango(new string[] {EParticipante.NPEsPiloto, EParticipante.NPEsCopiloto, EParticipante.NPNif ,EParticipante.NPNombre, EParticipante.NPApellidos });
            return col;
        }

        #endregion

        #region IQuitar

        public void Quitar() {
            Escuderia = null;
        }

        #endregion
    }
}

