using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    public enum TipoPermiso {
        Administrador,
        Secretario,
        Cronometrador,
        Publico
    }

    [Tabla("Permisos")]
    public class EPermiso : EIdBase {

        #region Constructores

        public EPermiso() : base() { }

        public EPermiso(int vId) : base(vId) { }

        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            Tipo = TipoPermiso.Publico;
        }

        public override bool EsMasculino {
            get { return true; } 
        }

        public override string NombreDeLaEntidad {
            get { return "Permiso"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPTipo.ToString(); } //TODO: Cambiar NPNombre si es necesario.
        }
        #endregion

        #region Propiedades

        public const string NPTipo = "Tipo";
        private TipoPermiso mTipo;

        [Campo, Requerido]
        public TipoPermiso Tipo {
            get {
                PrePropertyRead(NPTipo);
                return mTipo;
            }
            set {
                PrePropertyChanged(NPTipo);
                mTipo = value;
                bool ok = true;
                if (mUsuario != null) {
                    EColeccion<EPermiso> col = mUsuario.Permisos;
                    int indice = 0;
                    while (!ok && indice < col.Count) {
                        EPermiso permiso = col[indice];
                        ok = !(permiso.Tipo == this.Tipo && !permiso.Equals(this));
                    }
                    if (!ok) {
                        AddError(new PropertyException(NPTipo, this, "El usuario ya tiene este permiso, elija otro."));
                    } else {
                        RemoveError(NPTipo);
                    }
                } else {
                    RemoveError(NPTipo);
                }
                OnPropertyChanged(NPTipo);
            }
        }

        public const string NPUsuario = "Usuario";
        private EUsuario mUsuario;

        [Campo("UsuarioId", Tamanyo = 50)]
        public EUsuario Usuario {
            get {
                PrePropertyRead(NPUsuario);
                return mUsuario;
            }
            set {
                PrePropertyChanged(NPUsuario);
                mUsuario = value;
                OnPropertyChanged(NPUsuario);
            }
        }

        #endregion

        /// <summary>
        /// comparacin con el permiso pasado como parmetro. Si este permiso tiene ms privilegios
        /// devuelve 1, si tiene los mismos devuelve 0, si tiene menos devuelve -1. (En caso de error
        /// devuelve -2).
        /// </summary>
        /// <param name="p"></param>
        /// <returns></returns>
        internal int Comparar(EPermiso p) {
            if (p.Tipo.Equals(this.Tipo)) {
                return 0;
            }
            switch (Tipo) {
                case TipoPermiso.Administrador:
                    return 1;
                    break;
                case TipoPermiso.Secretario:
                    if (p.Tipo == TipoPermiso.Administrador) {
                        return -1;
                    } else {
                        return 1;
                    }
                    break;
                case TipoPermiso.Cronometrador:
                    if (p.Tipo.Equals(TipoPermiso.Administrador) || p.Tipo.Equals(TipoPermiso.Secretario)) {
                        return -1;
                    } else {
                        return 1;
                    }
                    break;
                case TipoPermiso.Publico:
                    return -1;
                    break;
            }
            return -2;
        }

        public override void Eliminar() {
            //if (Usuario != null && Usuario.Permisos.Count == 1) {
            //    throw new Exception("Solo le queda un permiso y no se puede eliminar.");
            //}
            //TODO: inventar el actualizar el usuario para que salga null y no entrar en el throw.
            base.Eliminar();
        }

        /// <summary>
        /// Devuelve true si esta entidad tiene permiso igual o con mayor privilegio
        /// que el permiso pasado como parmetro.
        /// </summary>
        /// <param name="tp"></param>
        /// <returns></returns>
        public bool PermisoIgualOSuperior(TipoPermiso tp) {
            switch (tp) {
                case TipoPermiso.Administrador:
                    return this.Tipo == TipoPermiso.Administrador;
                    break;
                case TipoPermiso.Secretario:
                    return this.Tipo == TipoPermiso.Administrador || this.Tipo == TipoPermiso.Secretario;
                    break;
                case TipoPermiso.Cronometrador:
                    return this.Tipo == TipoPermiso.Cronometrador || this.Tipo == TipoPermiso.Administrador;
                    break;
                case TipoPermiso.Publico:
                    return true;
                    break;
                default:
                    return false;
            }
        }
    }
}

