using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;
using System.Drawing;

namespace Negocio {

    [Tabla("Personas")]
    public abstract class EPersona : EIdBase, IAltaBaja {

        #region Constructores

        public EPersona() : base() { }

        public EPersona(int vId) : base(vId) { }

        public EPersona(string nif):this(){
            this.Cargar(NPNif, nif);
        }

        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            Nif = null;
            Nombre = null;
            Apellidos = null;
            FechaNacimiento = DateTime.Now;
            Direccion = null;
            Activo = true;
        }


        protected override void GuardarCore() {
            bool hayQueEliminar = false;
            int idDirecc = 0;
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar persona")) {
                //comprobamos la direccion actual
                if (Direccion != null && Direccion.EstaVacio()) {
                    hayQueEliminar = Direccion.Personas.Count == 1 && Direccion.Escuderias.Count == 0;
                    idDirecc = Direccion.Id;
                    Direccion = null;
                } else if (mDireccion != null) {
                    Direccion.Guardar();
                }
                base.GuardarCore();
                if (mFotoCargada) {
                    CentroImagenesE.Actual.AddImagen(this, Foto);
                }
                transa.Commit();
            }
            if (hayQueEliminar) {
                //Eliminar la direccion anterior.
                EDireccion.EliminarDireccionVacia(idDirecc);
            }
        }

        public override void Deshacer() {
            mFotoCargada = false;
            mFoto = null;
            if (Direccion != null) {
                Direccion.Deshacer();
            }
            base.Deshacer();
        }


        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "Persona"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return ToString(); }
        }
        #endregion

        #region Propiedades

        public const string NPNif = "Nif";
        private string mNif;

        [Campo(Tamanyo = 50)]
        public string Nif
        {
            get
            {
                PrePropertyRead(NPNif);
                return mNif;
            }
            set
            {
                PrePropertyChanged(NPNif);
                mNif = value;
                OnPropertyChanged(NPNif);
            }
        }

        public const string NPNombre = "Nombre";
        private string mNombre;

        [Campo(Tamanyo = 50), Requerido]
        public string Nombre
        {
            get
            {
                PrePropertyRead(NPNombre);
                return mNombre;
            }
            set
            {
                PrePropertyChanged(NPNombre);
                mNombre = value;
                OnPropertyChanged(NPNombre);
            }
        }

        public const string NPApellidos = "Apellidos";
        private string mApellidos;

        [Campo(Tamanyo = 80), Requerido]
        public string Apellidos
        {
            get
            {
                PrePropertyRead(NPApellidos);
                return mApellidos;
            }
            set
            {
                PrePropertyChanged(NPApellidos);
                mApellidos = value;
                OnPropertyChanged(NPApellidos);
            }
        }

        public const string NPFechaNacimiento = "FechaNacimiento";
        private DateTime mFechaNacimiento;

        [Campo]
        public DateTime FechaNacimiento
        {
            get
            {
                PrePropertyRead(NPFechaNacimiento);
                return mFechaNacimiento;
            }
            set
            {
                PrePropertyChanged(NPFechaNacimiento);
                mFechaNacimiento = value;
                OnPropertyChanged(NPFechaNacimiento);
            }
        }

        public const string NPEmail = "Email";
        private string mEmail;

        [Campo(Tamanyo = 100)]
        public virtual string Email {
            get {
                PrePropertyRead(NPEmail);
                return mEmail;
            }
            set {
                PrePropertyChanged(NPEmail);
                mEmail = value;
                OnPropertyChanged(NPEmail);
            }
        }

        public const string NPDireccion = "Direccion";
        private EDireccion mDireccion;

        [Campo("DireccionId")]
        public EDireccion Direccion
        {
            get
            {
                PrePropertyRead(NPDireccion);
                return mDireccion;
            }
            set
            {
                PrePropertyChanged(NPDireccion);
                mDireccion = value;
                OnPropertyChanged(NPDireccion);
            }
        }


        public const string NPFoto = "Foto";
        private Image mFoto, mFotoAnterior;
        private bool mFotoCargada = false;
        private enum TipoFoto {
            Ninguno,
            Piloto,
            Copiloto,
            Oficial
        }
        private TipoFoto tipoFoto;

        public System.Drawing.Image Foto {
            get {
                if (!mFotoCargada) {
                    Image fotoPiloto = CentroImagenesE.Actual.GetImagen(new EPiloto(this.Id));
                    Image fotoCopiloto = null;
                    Image fotoOficial = null;
                    if (fotoPiloto != null) {
                        mFoto = fotoPiloto;
                        tipoFoto = TipoFoto.Piloto;
                    } else if ((fotoCopiloto = CentroImagenesE.Actual.GetImagen(new ECopiloto(this.Id))) != null) {
                        mFoto = fotoCopiloto;
                        tipoFoto = TipoFoto.Copiloto;
                    } else if ((fotoOficial = CentroImagenesE.Actual.GetImagen(new EOficial(this.Id))) != null) {
                        mFoto = fotoOficial;
                        tipoFoto = TipoFoto.Oficial;
                    } else {
                        mFoto = null;
                        tipoFoto = TipoFoto.Ninguno;
                    }
                    mFotoAnterior = mFoto;
                    mFotoCargada = true;
                }
                return mFoto;
            }
            set {
                mFoto = value;
                mFotoCargada = true;
                OnPropertyChanged(NPFoto);
            }
        }


        public abstract bool Activo { get; set;}

        #endregion
        
        /// <summary>
        /// Devuelve el nombre y apellidos de la persona.
        /// </summary>
        /// <returns></returns>
        public override string ToString() {
            return Nombre + ", " + Apellidos;
        }

        public override void Eliminar() {
            base.Eliminar();
        }

        #region IBaja

        public void DarAltaBaja(bool alta) {
            Activo = alta;
        }

        #endregion
    }
}

