using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("Personas")]
    public class EPiloto : EParticipante {

        #region Constructores
        public EPiloto() : base() { }

        public EPiloto(int vId) : base(vId) { }

        public EPiloto(string nif) : base(nif) { }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            EsPiloto = true;
        }


        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar piloto")) {
                base.GuardarCore();
                if (mVehiculos != null)
                    mVehiculos.Guardar();
                if (mLicenciasPiloto != null)
                    mLicenciasPiloto.Guardar();
                //if (mPilotoCarreras != null)
                //    mPilotoCarreras.Guardar();
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mVehiculos = null;
            mLicenciasPiloto = null;
            mPilotoCarreras = null;
            base.Deshacer();
        }


        public override bool EsMasculino {
            get { return true; }  
        }

        public override string NombreDeLaEntidad {
            get { return "Piloto"; }
        }

        public override bool Modificable(string nombreProp) {
            if(!EstaCargado) return true;

            switch(nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return base.ToString(); } 
        }
        #endregion

        #region Propiedades

        public const string NPActivo = "Activo";
        private bool mActivo;

        [Campo("ActivoPiloto")]
        public override bool Activo {
            get {
                PrePropertyRead(NPActivo);
                return mActivo;
            }
            set {
                PrePropertyChanged(NPActivo);
                mActivo = value;
                OnPropertyChanged(NPActivo);
            }
        }

        #endregion

        #region Vehiculos
        public const string NPVehiculos = "Vehiculos";
        private EColeccion<EVehiculo> mVehiculos = null;

        /// <summary>Devuelve una coleccin con todos los Vehiculos.</summary>
        public EColeccion<EVehiculo> Vehiculos {
            get {
                if(mVehiculos == null) {
                    mVehiculos = new EColeccion<EVehiculo>();
                    if(!this.EsNuevo) {
                        mVehiculos.Filtros.Add(EVehiculo.NPPiloto, this);
                        mVehiculos.Cargar();
                    }
                    mVehiculos.AddingNew += new AddingNewEventHandler(mVehiculos_AddingNew);
                    mVehiculos.ColumnaMostrarRango(new string[] {EVehiculo.NPMatricula, EVehiculo.NPMarca, EVehiculo.NPModelo});
                }
                return mVehiculos;
            }
        }

        private void mVehiculos_AddingNew(object sender, AddingNewEventArgs e) {
            EVehiculo entidad = new EVehiculo();
            entidad.Piloto = this;
            e.NewObject = entidad;
        }

        #endregion

        #region LicenciasPiloto
        public const string NPLicenciasPiloto = "LicenciasPiloto";
        private EColeccion<ELicenciaPiloto> mLicenciasPiloto = null;

        /// <summary>Devuelve una coleccin con todos los LicenciasPiloto.</summary>
        public EColeccion<ELicenciaPiloto> LicenciasPiloto {
            get {
                if(mLicenciasPiloto == null) {
                    mLicenciasPiloto = new EColeccion<ELicenciaPiloto>();
                    if(!this.EsNuevo) {
                        //Licencias como piloto
                        EColeccion<ELicenciaPiloto> colLicPilotos = new EColeccion<ELicenciaPiloto>();
                        colLicPilotos.Filtros.Add(ELicenciaPiloto.NPPiloto, this);
                        colLicPilotos.Filtros.Add(Profundizar(typeof(ELicenciaPiloto), ELicenciaPiloto.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.Pilotos);
                        colLicPilotos.Cargar();
                        //Licencias como participante
                        EColeccion<ELicenciaPiloto> colLicParticipante = new EColeccion<ELicenciaPiloto>();
                        colLicParticipante.Filtros.Add(ELicenciaPiloto.NPPiloto, this);
                        colLicParticipante.Filtros.Add(Profundizar(typeof(ELicenciaPiloto), ELicenciaPiloto.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.Participante);
                        colLicParticipante.Cargar();
                        //Licencias como Otros
                        EColeccion<ELicenciaPiloto> colLicOtros = new EColeccion<ELicenciaPiloto>();
                        colLicOtros.Filtros.Add(ELicenciaPiloto.NPPiloto, this);
                        colLicOtros.Filtros.Add(Profundizar(typeof(ELicenciaPiloto), ELicenciaPiloto.NPLicencia, ELicencia.NPTipoDirigido, null), TipoDirigido.Otros);
                        colLicOtros.Cargar();
                        //Agregar todos los elementos a la coleccin principal.
                        foreach (ELicenciaPiloto lic in colLicPilotos) {
                            mLicenciasPiloto.Add(lic);
                        }
                        foreach (ELicenciaPiloto lic in colLicParticipante) {
                            mLicenciasPiloto.Add(lic);
                        }
                        foreach (ELicenciaPiloto lic in colLicOtros) {
                            mLicenciasPiloto.Add(lic);
                        }
                    }
                    mLicenciasPiloto.AddingNew += new AddingNewEventHandler(mLicenciasPiloto_AddingNew);
                    mLicenciasPiloto.ColumnaMostrarRango(new string[] { ELicenciaPiloto.NPLicencia, ELicenciaPiloto.NPNumero, ELicenciaPiloto.NPFechaCaducidad });
                }
                return mLicenciasPiloto;
            }
        }

        private void mLicenciasPiloto_AddingNew(object sender, AddingNewEventArgs e) {
            ELicenciaPiloto entidad = new ELicenciaPiloto();
            entidad.Piloto = this;
            e.NewObject = entidad;
        }

        #endregion

        #region PilotoCarreras
        public const string NPPilotoCarreras = "PilotoCarreras";
        private EColeccion<EPilotoCarrera> mPilotoCarreras = null;

        /// <summary>Devuelve una coleccin con todos los PilotoCarreras.</summary>
        public EColeccion<EPilotoCarrera> PilotoCarreras {
            get {
                if(mPilotoCarreras == null) {
                    mPilotoCarreras = new EColeccion<EPilotoCarrera>();
                    if(!this.EsNuevo) {
                        mPilotoCarreras.Filtros.Add(EPilotoCarrera.NPPiloto, this);
                        mPilotoCarreras.Cargar();
                    }
                    mPilotoCarreras.AddingNew += new AddingNewEventHandler(mPilotoCarreras_AddingNew);
                }
                return mPilotoCarreras;
            }
        }

        private void mPilotoCarreras_AddingNew(object sender, AddingNewEventArgs e) {
            EPilotoCarrera entidad = new EPilotoCarrera();
            entidad.Piloto = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Colecciones
        public static EColeccion<EPiloto> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EPiloto> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EPiloto> col = new EColeccion<EPiloto>("PILOTOS");
            col.Filtros.Add(NPEsPiloto, true);
            col.ColumnaMostrarRango(new string[] { NPNif, NPNombre, NPApellidos, NPFechaNacimiento, NPDireccion });
            if(!vacio)
                col.Cargar(asincronamente);
            return col;
        }

        /// <summary>
        /// Devuelve una coleccin de pilotos activos o no activos segn el valor
        /// pasado como parmetro.
        /// </summary>
        /// <param name="activo"></param>
        /// <returns></returns>
        public static EColeccion<EPiloto> GetCollection(bool activo) {
            EColeccion<EPiloto> col = new EColeccion<EPiloto>("PILOTOS");
            col.Filtros.Add(NPEsPiloto, true);
            col.Filtros.Add(NPActivo, activo);
            col.ColumnaMostrarRango(new string[] { NPNif, NPNombre, NPApellidos, NPFechaNacimiento, NPDireccion });
            col.Cargar();
            return col;
        }

        #endregion

        #region CarrerasParticipadas

        public override EColeccion<EPilotoCarrera> CarrerasParticipadas {
            get {
                EColeccion<EPilotoCarrera> col = PilotoCarreras;
                return col; 
            }
        }

        #endregion

        public override void Eliminar() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Eliminar piloto")) {
                if (PilotoCarreras.Count > 0) {
                    throw new Exception("No se puede eliminar el piloto.\nTiene inscripciones en carreras.");
                }
                int indice = 0;
                EColeccion<ELicenciaPiloto> colLicencias = LicenciasPiloto;
                while (indice < colLicencias.Count) {
                    colLicencias[indice].Eliminar();
                    indice = indice + 1;
                }
                indice = 0;
                EColeccion<EVehiculo> colVehiculos = Vehiculos;
                while (indice < colVehiculos.Count) {
                    colVehiculos[indice].Eliminar();
                    indice = indice + 1;
                }
                base.Eliminar();
                transa.Commit();
            }
        }

        public override bool EstaSucio {
            get {
                return base.EstaSucio || ((mLicenciasPiloto != null && mLicenciasPiloto.EstaSucio) || (mVehiculos != null && mVehiculos.EstaSucio) || mPilotoCarreras != null && mPilotoCarreras.EstaSucio);
            }
        }
    }
}

