using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    public enum TipoEstadoTemporada {
        EnCurso = 1,
        Finalizada = 2
    }

    [Tabla("Temporadas")]
    public class ETemporada : EIdBase {
        #region Constructores
        public ETemporada() : base() { }

        public ETemporada(int vId) : base(vId) { }
        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            Anho = 0;
        }

        public override bool EsMasculino {
            get { return false; }
        }

        public override string NombreDeLaEntidad {
            get { return "Temporada"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return NPAnho; }
        }
        #endregion

        #region Propiedades

        public const string NPAnho = "Anho";
        private int mAnho;

        [Campo, Unico]
        public int Anho {
            get {
                PrePropertyRead(NPAnho);
                return mAnho;
            }
            set {
                PrePropertyChanged(NPAnho);
                mAnho = value;
                OnPropertyChanged(NPAnho);
            }
        }

        public const string NPEstadoTemporada = "EstadoTemporada";
        private TipoEstadoTemporada mEstadoTemporada;

        [Campo("Estado")]
        public TipoEstadoTemporada EstadoTemporada {
            get {
                PrePropertyRead(NPEstadoTemporada);
                return mEstadoTemporada;
            }
            set {
                PrePropertyChanged(NPEstadoTemporada);
                mEstadoTemporada = value;
                OnPropertyChanged(NPEstadoTemporada);
            }
        }

        public const string NPActual = "Actual";
        private static ETemporada mActual;

        public static ETemporada Actual {
            get {
                return mActual;
            }
            set {
                mActual = value;
            }
        }

        #endregion

        #region Carreras
        
        public const string NPCarreras = "Carreras";
        private EColeccion<ECarrera> mCarreras = null;

        /// <summary>Devuelve una coleccin con todos las Carreras.</summary>
        public EColeccion<ECarrera> Carreras {
            get {
                if(mCarreras == null) {
                    mCarreras = new EColeccion<ECarrera>();
                    //if(!this.EsNuevo) {
                    //    mCarreras.Filtros.Add(ECarrera.NPTemporada, this);
                    //    mCarreras.Cargar();
                    //}
                    EColeccion<ESubida> colSubidas = new EColeccion<ESubida>();
                    colSubidas.Filtros.Add(ESubida.NPTemporada, this);
                    colSubidas.Filtros.Add(ESubida.NPTipo, TipoCarrera.Subida);
                    colSubidas.Cargar();
                    foreach (ESubida s in colSubidas) {
                        mCarreras.Add(s);
                    }
                    EColeccion<ERally> colRallys = new EColeccion<ERally>();
                    colRallys.Filtros.Add(ERally.NPTemporada, this);
                    colRallys.Filtros.Add(ERally.NPTipo, TipoCarrera.Rally);
                    colRallys.Cargar();
                    foreach (ERally r in colRallys) {
                        mCarreras.Add(r);
                    }
                }
                return mCarreras;
            }
        }

        #endregion

        #region Colecciones
        public static EColeccion<ETemporada> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<ETemporada> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<ETemporada> col = new EColeccion<ETemporada>();
            col.ColumnaMostrarRango(new string[] { NPAnho, NPEstadoTemporada });
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }
        #endregion

        public static ETemporada GetTemporadaActual() {
            ETemporada temporada = new ETemporada();
            temporada.Cargar(NPEstadoTemporada, TipoEstadoTemporada.EnCurso);
            return temporada;
        }

        /// <summary>
        /// Crea una nueva temporada cerrando todas las carreras de la temporada actual.
        /// </summary>
        public static void NuevaTemporada() {
            //using (AmbitoTransa transa = new AmbitoTransa("Nueva Temporada")) {
                ETemporada actual = ETemporada.Actual;
                actual.EstadoTemporada = TipoEstadoTemporada.Finalizada;
                EColeccion<ECarrera> colCarreras = actual.Carreras;
                foreach (ECarrera carrera in colCarreras) {
                    if (!carrera.EstaCerrada) {
                        carrera.Cerrar();
                    }
                }
                actual.Guardar();
                ETemporada.Actual = new ETemporada();
                ETemporada.Actual.Anho = actual.Anho + 1;
                ETemporada.Actual.EstadoTemporada = TipoEstadoTemporada.EnCurso;
                ETemporada.Actual.Guardar();
                //transa.Commit();
            //}
        }
    }
}
