using System.ComponentModel;
using System.Security.Cryptography;
using System.Text;
using Nibi.Dal;
using Nibi.Negocio;
using System;

namespace Negocio {

    [Tabla("Personas")]
    public class EUsuario : EPersona {

        #region Constructores

        public EUsuario() : base() { }

        public EUsuario(int vId) : base(vId) { }

        public EUsuario(string nif) : base(nif) { }

        #endregion

        #region Sobrecargas de EBase

        protected override void InicializarObjetoNuevo() {
            base.InicializarObjetoNuevo();
            Email = null;
            Nick = null;
            Actual = null;
            Nombre = null;
            Apellidos = null;
            Contrasenha = null;
            EsUsuario = true;
        }

        protected override void GuardarCore() {
            using (AmbitoTransa transa = this.CrearAmbitoTransa("Guardar entidad")) {
                if (String.IsNullOrEmpty(Nick)) {
                    throw new Exception("El nick de usuario no puede estar vaco.");
                }
                mContrasenha = this.ObtenerPass(mContrasenha);
                base.GuardarCore();
                if (mPermisos != null) {
                    mPermisos.Guardar();
                }
                //Preguntamos primero si es el primer usuario que ser crea.
                EColeccion<EUsuario> mColeccion = GetCollection();
                if (mColeccion.Count == 1 && mColeccion[0].Equals(this) && Permisos.Count == 0) {
                    EPermiso permisoAdmin = Permisos.AddNew();
                    permisoAdmin.Tipo = TipoPermiso.Administrador;
                    permisoAdmin.Guardar();
                }else if (Permisos.Count == 0) {
                    //No puede tener ningun permiso, si es as se crea uno de publico.
                    EPermiso permisoPublico = Permisos.AddNew();
                    permisoPublico.Tipo = TipoPermiso.Publico;
                    permisoPublico.Guardar();
                }
                transa.Commit();
            }
        }

        public override void Deshacer() {
            mPermisos = null;
            base.Deshacer();
        }

        public override bool EsMasculino {
            get { return true; } 
        }

        public override string NombreDeLaEntidad {
            get { return "Usuario"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return base.NPToString; } 
        }
        #endregion

        #region Propiedades

        public const string NPActivo = "Activo";
        private bool mActivo;

        [Campo("ActivoUsuario")]
        public override bool Activo {
            get {
                PrePropertyRead(NPActivo);
                return mActivo;
            }
            set {
                PrePropertyChanged(NPActivo);
                mActivo = value;
                OnPropertyChanged(NPActivo);
            }
        }

        public const string NPActual = "Actual";
        private static EUsuario mActual;

        public static EUsuario Actual {
            get {
                return mActual;
            }
            set {
                mActual = value;
            }
        }

        public const string NPNick = "Nick";
        private string mNick;
        private const int MINLENGTHNICK = 4;

        [Campo(Tamanyo = 50), Requerido, Unico]
        public string Nick {
            get {
                PrePropertyRead(NPNick);
                return mNick;
            }
            set {
                PrePropertyChanged(NPNick);
                mNick = value;
                string[] comprobar = null;
                if (!String.IsNullOrEmpty(mNick)) {
                    comprobar = value.Split(' ', '`', '/', ',', '(', ')');
                }
                if (!String.IsNullOrEmpty(mNick) && mNick.Length < MINLENGTHNICK) {
                    AddError(new PropertyException(NPNick, this, "El nick debe tener al menos " + MINLENGTHNICK + " caracteres."));
                } else if (comprobar != null && comprobar.Length > 1) {
                    RemoveError(NPNick);
                    AddError(new PropertyException(NPNick, this, "El nick no puede tener los siguientes caracteres: ( ' ', '`', '/', ',' '(', ')' )."));
                } else if (String.IsNullOrEmpty(mNick)){
                    RemoveError(NPNick);
                    AddError(new PropertyException(NPNick, this, "El nick es obligatorio."));
                } else {
                    RemoveError(NPNick);
                }
                OnPropertyChanged(NPNick);
            }
        }

        public const string NPContrasenha = "Contrasenha";
        private string mContrasenha;
        private const int MINLENGTHPASS = 4;

        [Campo(Tamanyo = 50), Requerido]
        public string Contrasenha {
            get {
                PrePropertyRead(NPContrasenha);
                return mContrasenha;
            }
            set {
                PrePropertyChanged(NPContrasenha);
                mContrasenha = value;
                string[] comprobar = null;
                if (!String.IsNullOrEmpty(mContrasenha)) {
                    comprobar = value.Split(' ', '`', '/', ',', '(', ')');
                }
                if (!String.IsNullOrEmpty(mContrasenha) && mContrasenha.Length < MINLENGTHPASS) {
                    AddError(new PropertyException(NPContrasenha, this, "El password debe tener al menos " + MINLENGTHPASS + " caracteres."));
                } else if (comprobar != null && comprobar.Length > 1) {
                    RemoveError(NPContrasenha);
                    AddError(new PropertyException(NPContrasenha, this, "El password no puede tener los siguientes caracteres: ( ' ', '`', '/', ',' '(', ')' )."));
                } else {
                    RemoveError(NPContrasenha);
                }
                OnPropertyChanged(NPContrasenha);
            }
        }

        [Campo, Requerido]
        public override string Email {
            get {
                return base.Email;
            }
            set {
                base.Email = value;
                if (String.IsNullOrEmpty(Email)) {
                    AddError(new PropertyException(NPEmail, this, "El E-mail es obligatorio."));
                } else {
                    RemoveError(NPEmail);
                }
            }
        }

        public const string NPEsUsuario = "EsUsuario";
        private bool mEsUsuario;

        [Campo]
        public bool EsUsuario {
            get {
                PrePropertyRead(NPEsUsuario);
                return mEsUsuario;
            }
            set {
                PrePropertyChanged(NPEsUsuario);
                mEsUsuario = value;
                OnPropertyChanged(NPEsUsuario);
            }
        }

        #endregion

        #region Permisos
        public const string NPPermisos = "Permisos";
        private EColeccion<EPermiso> mPermisos = null;

        /// <summary>Devuelve una coleccin con todos los Permisos.</summary>
        public EColeccion<EPermiso> Permisos {
            get {
                if(mPermisos == null) {
                    mPermisos = new EColeccion<EPermiso>();
                    if(!this.EsNuevo) {
                        mPermisos.Filtros.Add(EPermiso.NPUsuario, this);
                        mPermisos.ColumnaMostrarRango(new string[] { EPermiso.NPTipo });
                        mPermisos.Cargar();
                    }
                    mPermisos.AddingNew += new AddingNewEventHandler(mPermisos_AddingNew);
                }
                return mPermisos;
            }
        }

        private void mPermisos_AddingNew(object sender, AddingNewEventArgs e) {
            EPermiso entidad = new EPermiso();
            entidad.Usuario = this;
            e.NewObject = entidad;
        }

        #endregion

        #region Colecciones

        public static EColeccion<EUsuario> GetCollection() {
            return GetCollection(false, false);
        }

        public static EColeccion<EUsuario> GetCollection(bool vacio, bool asincronamente) {
            EColeccion<EUsuario> col = new EColeccion<EUsuario>("USUARIOS");
            col.Filtros.Add(NPEsUsuario, true);
            col.ColumnaMostrarRango(new string[] {NPNick, NPNif, NPNombre, NPApellidos});
            if (!vacio)
                col.Cargar(asincronamente);
            return col;
        }

        /// <summary>
        /// Devuelve una coleccin de usuarios activos o no activos segn el valor
        /// pasado como parmetro.
        /// </summary>
        /// <param name="activo"></param>
        /// <returns></returns>
        public static EColeccion<EUsuario> GetCollection(bool activo) {
            EColeccion<EUsuario> col = new EColeccion<EUsuario>("USUARIOS");
            col.Filtros.Add(NPEsUsuario, true);
            col.Filtros.Add(NPActivo, activo);
            col.ColumnaMostrarRango(new string[] { NPNif, NPNombre, NPApellidos, NPFechaNacimiento, NPDireccion });
            col.Cargar();
            return col;
        }

        #endregion

        public string ObtenerPass(string pass) {
            MD5 md5 = MD5CryptoServiceProvider.Create();
            byte[] dataMd5 = md5.ComputeHash(Encoding.Default.GetBytes(pass));
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < dataMd5.Length; i++)
                sb.AppendFormat("{0:x2}", dataMd5[i]);
            return sb.ToString();
        }

        public string NuevaContrasenha() {
            string nuevo = this.ObtenerPass(this.Nick + this.Nick);
            Random random = new Random(0);
            int pos1 = random.Next(nuevo.Length);
            int pos2 = random.Next(nuevo.Length);
            if (pos1 > pos2) {
                int aux = pos2;
                pos2 = pos1;
                pos1 = aux;
            }
            string resultado = nuevo.Substring(pos1, pos2 - pos1);
            if (resultado.Length < MINLENGTHPASS) {
                resultado += nuevo.Substring(0, MINLENGTHPASS);
            }
            return resultado;
        }

        public EPermiso GetMayorPermiso() {
            EColeccion<EPermiso> col = Permisos;
            EPermiso permisoMayor = null;
            foreach (EPermiso p in col) {
                if (permisoMayor == null || permisoMayor.Comparar(p) == -1) {
                    permisoMayor = p;
                }
            }
            return permisoMayor;
        }
    }
}

