using System.ComponentModel;

using Nibi.Dal;
using Nibi.Negocio;

namespace Negocio {

    [Tabla("Vehiculos")]
    public class EVehiculo : EIdBase {
        #region Constructores
        public EVehiculo() : base() { }

        public EVehiculo(int vId) : base(vId) { }

        public EVehiculo(string matr) : this() {
            this.Cargar(NPMatricula, matr);
        }
        #endregion

        #region Sobrecargas de EBase
        protected override void InicializarObjetoNuevo() {
            Matricula = null;
            Marca = null;
            Modelo = null;
            Piloto = null;
        }

        public override bool EsMasculino {
            get { return true; } 
        }

        public override string NombreDeLaEntidad {
            get { return "Vehiculo"; }
        }

        public override bool Modificable(string nombreProp) {
            if (!EstaCargado) return true;

            switch (nombreProp) {
                default:
                    return true;
            }
        }

        [Browsable(false)]
        public override string NPToString {
            get { return this.ToString(); }
        }
        #endregion

        #region Propiedades

        public const string NPMatricula = "Matricula";
        private string mMatricula;

        [Campo(Tamanyo = 50), Requerido]
        public string Matricula {
            get {
                PrePropertyRead(NPMatricula);
                return mMatricula;
            }
            set {
                PrePropertyChanged(NPMatricula);
                mMatricula = value;
                OnPropertyChanged(NPMatricula);
            }
        }

        public const string NPMarca = "Marca";
        private string mMarca;

        [Campo(Tamanyo = 50), Requerido]
        public string Marca {
            get {
                PrePropertyRead(NPMarca);
                return mMarca;
            }
            set {
                PrePropertyChanged(NPMarca);
                mMarca = value;
                OnPropertyChanged(NPMarca);
            }
        }

        public const string NPModelo = "Modelo";
        private string mModelo;

        [Campo(Tamanyo = 50), Requerido]
        public string Modelo {
            get {
                PrePropertyRead(NPModelo);
                return mModelo;
            }
            set {
                PrePropertyChanged(NPModelo);
                mModelo = value;
                OnPropertyChanged(NPModelo);
            }
        }

        public const string NPCilindrada = "Cilindrada";
        private double mCilindrada;

        [Campo]
        public double Cilindrada {
            get {
                PrePropertyRead(NPCilindrada);
                return mCilindrada;
            }
            set {
                PrePropertyChanged(NPCilindrada);
                mCilindrada = value;
                if (mCilindrada < 0) {
                    AddError(new PropertyException(NPCilindrada, this, "La cilindrada no puede ser menor que 0."));
                } else {
                    RemoveError(NPCilindrada);
                }
                OnPropertyChanged(NPCilindrada);
            }
        }

        public const string NPPiloto = "Piloto";
        private EPiloto mPiloto;

        [Campo("PersonaId", Tamanyo = 50), Requerido]
        public EPiloto Piloto {
            get {
                PrePropertyRead(NPPiloto);
                return mPiloto;
            }
            set {
                PrePropertyChanged(NPPiloto);
                mPiloto = value;
                OnPropertyChanged(NPPiloto);
            }
        }

        #endregion

        public override string ToString() {
            return Marca + " - " + Modelo;
        }
    }
}
